﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include "Graphics.h"

namespace nexdemo
{

namespace
{
// ヒープ
nn::fnd::ExpHeap s_gxHeap;
// バッファ
uptr s_pBuffer = NULL;

// レンダラ
demo::RenderSystemDrawing s_renderSystem;
}

// 初期化処理を行います。
void Graphics::Initialize(size_t heapSize)
{
    // ヒープの初期化処理を行う
    s_gxHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);
    // ヒープを確保する
    s_pBuffer = reinterpret_cast<uptr>(s_gxHeap.Allocate(heapSize));

    // レンダラの初期化処理を行う
    s_renderSystem.Initialize(s_pBuffer, heapSize);
}

// 終了処理を行います。
void Graphics::Finalize()
{
    // レンダラの終了処理を行う
    s_renderSystem.Finalize();

    // ヒープを破棄する
    s_gxHeap.Free(reinterpret_cast<void*>(s_pBuffer));
    // ヒープの終了処理を行う
    s_gxHeap.Finalize();
}

// レンダラを取得します。
demo::RenderSystemDrawing* Graphics::GetRenderSystem()
{
    return &s_renderSystem;
}

} // namespace nexdemo
