﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <nn/gx.h>
#include <nn/math.h>

#include "demo/ShaderProgram/demo_ProgramObject.h"
#include "demo/ShaderProgram/demo_ColorFillProgram.h"
#include "demo/ShaderProgram/demo_FontProgram.h"
#include "demo/ShaderProgram/demo_DecalTextureProgram.h"

namespace demo
{

/*!
@internal

@brief    ShaderManagerクラスの最大シェーダバイナリ数
*/
const u32 MAX_SHADER_BINARY_NUM = 1;

/*!
@internal

@brief    ShaderManagerクラスの単一シェーダバイナリの最大シェーダ数
*/
const u32 MAX_SHADER_PROGRAM_NUM = 3;

/*!
@internal

@brief    基本的なシェーダを表す列挙型
*/
enum ShaderType
{
    COLOR_FILL_SHADER = 0,   //!< 頂点カラーシェーダ
    FONT_SHADER = 1,         //!< フォントシェーダ
    DECAL_TEXTURE_SHADER = 2, //!< デカールテクスチャシェーダ
    BASIC_SHADER_BINARY_SHADER_NUM = 3 //!< 基本的なシェーダの総数
};

/*!
@internal

@brief    基本的なシェーダのシェーダバイナリのインデックス
*/
const u32 BASIC_SHADER_BINARY_INDEX = 0;

/*!
    @internal

    @brief シェーダを管理するクラスです。
*/

class ShaderManager : private nn::util::NonCopyable<ShaderManager>
{
public:
    /*!
    @internal

    @brief    コンストラクタです。
    */
    ShaderManager(void);

    /*!
    @internal

    @brief    デストラクタです。
    */
    virtual ~ShaderManager(void);

public:
    /*!
    @internal

    @brief    初期処理を行います。
    */
    virtual bool Initialize(void);

    /*!
    @internal

    @brief    終了処理を行います。
    */
    virtual bool Finalize(void);
protected:
    virtual bool InitializeBasicShader(void);

public:
    /*!
    @internal

    @brief    指定したシェーダの頂点属性を取得します。
    @param[in]    shaderType    シェーダの種類
    @return       頂点属性
    */
    u32 GetVertexAttributes(const ShaderType shaderType);

    /*!
    @internal

    @brief    指定したシェーダのポインタを取得します。

    @param[in]    shaderType    シェーダの種類
    @return       シェーダプログラムのポインタ
    */
    ProgramObject* GetShaderProgram(const ShaderType shaderType);

protected:
    bool m_Initialized;
    u8 m_Padding[3];
    GLuint m_ShaderProgramIdArray[MAX_SHADER_BINARY_NUM][MAX_SHADER_PROGRAM_NUM];
    ProgramObject* m_ShaderProgramPtrArray[MAX_SHADER_BINARY_NUM][MAX_SHADER_PROGRAM_NUM];

protected:
    ColorFillProgram m_ColorFillProgram;
    FontProgram m_FontProgram;
    DecalTextureProgram m_DecalTextureProgram;
};

}

