#!/usr/bin/perl
#
# CTR-NEX
#   独自サーバ用認証トークン解析プログラムサンプル Ver 1.2
#
# ※利用方法
#   tokendec_ctr.pl key token
#      key   : 認証トークン鍵（文字表記）
#      token : 認証トークン
#

use strict;
use warnings;
use Crypt::CBC;
use Digest::SHA1 qw(sha1_hex);
use MIME::Base64 qw(decode_base64);

# 認証トークン鍵・認証トークンを引数から取得
if ($#ARGV < 1) {
  die "usage: tokendec_ctr.pl key token\n";
}
if ($ARGV[0] !~ m/^\w{16}$/xms) {
  die "invalid parameter (key)\n";
}
if ($ARGV[1] !~ m/^[\w\-\.\*]+$/xms) {
  die "invalid parameter (token)\n";
}
my $key   = $ARGV[0];
my $tmptoken = $ARGV[1];
$tmptoken =~ tr/\-\.\*/\/\+\=/;
my $token = decode_base64($tmptoken);


# AES復号
# CBCモード、鍵=128bit、IV=オール\0、パディング=PKCS#5
my $iv  = pack 'H32', '0' x 32; # IV
my $cipher = Crypt::CBC->new(
  -key         => $key,
  -keysize     => 16,
  -literal_key => 1,
  -cipher      => 'Crypt::Rijndael',
  -iv          => $iv,
  -padding     => 'standard',
  -prepend_iv  => 0,
);
my $dectoken = $cipher->decrypt($token);


# トークン解析
my ($pid, $friendcode, $servertype, $timestamp, $hash);
if ($dectoken =~ m/^\\u\\(\d{1,10})\\f\\(\d{12})\\s\\(\w{2})\\t\\(\d+)\\h\\(\w{8})$/xms) {
  $pid        = $1;
  $friendcode = $2;
  $servertype = $3;
  $timestamp  = $4;
  $hash       = $5;
} else {
  die "invalid token or key\n";
}


# 各値表示
print "PID         : $pid\n";
print "Friend Code : $friendcode\n";
print "Server Type : $servertype\n";
print "Timestamp   : $timestamp\n";
print "SHA-1 Hash  : $hash\n";


# ハッシュ値正常？
if ($dectoken =~ m/^(.*)\\h\\\w{8}$/xms) {
  my $calchash = substr sha1_hex($1), 0, 8;
  if ($calchash ne $hash) {
      die "invalid hash (correct hash=$calchash)\n";
  }
}


# 24時間以内？
my $now = time;
if ($now - $timestamp > 24 * 60 * 60) {
  die "token expired (now=$now, 24hours before=" . ($now - 24 * 60 * 60) . ")\n";
}


print "correct token!\n";

exit;

