﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include "CallContextTrainer.h"

namespace nex_demo_util
{

class JobAdd : public nn::nex::SingleStepJob
{
public:
    JobAdd(nn::nex::TimeInterval delayMs, nn::nex::qResult qresult, qInt32 operand1, qInt32 operand2, qInt32* pSum)
        : nn::nex::SingleStepJob()
        , m_delayMs(delayMs)
        , m_qresult(qresult)
        , m_startedTime(0)
        , m_operand1(operand1)
        , m_operand2(operand2)
        , m_pSum(pSum)
    {
    }

    virtual qBool OnStep(nn::nex::qResult* result)
    {
        if (m_startedTime == nn::nex::Time(0))
        {
            NEX_DEMO_LOG("Asynchronous call has been started. I will complete this task after %u ms.\n", m_delayMs);
            m_startedTime = nn::nex::SystemClock::GetTime();
        }

        if (nn::nex::SystemClock::GetTime() - m_startedTime < m_delayMs)
        {
            // まだ設定されたディレイ時間に達していない
            return false;
        }

        if (!m_qresult)
        {
            // 結果が設定されている場合はそれを返す。
            *result = m_qresult;
            return true;
        }

        NEX_DEMO_LOG("Hmm. %d + %d = %d !\n", m_operand1, m_operand2, m_operand1 + m_operand2);

        if (m_pSum)
        {
            *m_pSum = m_operand1 + m_operand2;
        }
        *result = QRESULT_SUCCESS(Core, Success);
        return true;
    }

    virtual void OnCancelled()
    {
        NEX_DEMO_LOG("Asynchronous call has been cancelled.\n");
    }
private:
    nn::nex::TimeInterval m_delayMs;
    nn::nex::qResult m_qresult;
    nn::nex::Time m_startedTime;
    qInt32 m_operand1;
    qInt32 m_operand2;
    qInt32* m_pSum;
};

qBool CallContextTrainer::Add(nn::nex::CallContext* pCallContext, qInt32 operand1, qInt32 operand2, qInt32* pSum)
{
    JobAdd* pJobAdd = qNew JobAdd(m_delayMs, m_qresult, operand1, operand2, pSum);
    return pJobAdd->Start(pCallContext, true);
}

class JobLog : public nn::nex::SingleStepJob
{
public:
    JobLog(nn::nex::TimeInterval delayMs, nn::nex::qResult qresult, const nn::nex::String& text)
        : nn::nex::SingleStepJob()
        , m_delayMs(delayMs)
        , m_qresult(qresult)
        , m_text(text)
    {
    }

    virtual qBool OnStep(nn::nex::qResult* result)
    {
        if (m_startedTime == nn::nex::Time(0))
        {
            NEX_DEMO_LOG("Asynchronous call has been started. I will complete this task after %u ms.\n", m_delayMs);
            m_startedTime = nn::nex::SystemClock::GetTime();
        }

        if (nn::nex::SystemClock::GetTime() - m_startedTime < m_delayMs)
        {
            // まだ設定されたディレイ時間に達していない
            return false;
        }

        if (!m_qresult)
        {
            // 結果が設定されている場合はそれを返す。
            *result = m_qresult;
            return true;
        }

        NEX_DEMO_LOG("%s\n", m_text.GetLiteralStringAs<qChar8>().CStr());
        *result = QRESULT_SUCCESS(Core, Success);
        return true;
    }

    virtual void OnCancelled()
    {
        NEX_DEMO_LOG("Asynchronous call has been cancelled.\n");
    }
private:
    nn::nex::TimeInterval m_delayMs;
    nn::nex::qResult m_qresult;
    nn::nex::Time m_startedTime;
    nn::nex::String m_text;
};

qBool CallContextTrainer::Log(nn::nex::CallContext* pCallContext, const nn::nex::String& text)
{
    JobLog* pJobLog = qNew JobLog(m_delayMs, m_qresult, text);
    return pJobLog->Start(pCallContext, true);
}

}   // end of namespace nex_demo_util
