﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include "DemoMem.h"

namespace nex_demo_util
{

DemoMemBase::DemoMemBase():
    m_memUsage(0),
    m_currentAllocCount(0),
    m_totalAllocCount(0),
    m_memMaxUsage(0),
    m_cs(nn::nex::LockMasks::Ignored)
{
}

void DemoMemBase::Initialize(void* memBlock, size_t memSize)
{
    InnerInitialize(memBlock, memSize);
#if defined(NEX_NNSDK_FAMILY)
    m_memUsage.exchange(0);
    m_currentAllocCount.exchange(0);
    m_totalAllocCount.exchange(0);
#else
    m_memUsage = 0;
    m_currentAllocCount = 0;
    m_totalAllocCount = 0;
#endif
    m_memMaxUsage = 0;
}

void DemoMemBase::Finalize()
{
    InnerFinalize();
}

qBool DemoMemBase::IsClean() const
{
    return m_memUsage == 0;
}

void* DemoMemBase::AllocFunction(size_t size)
{
    void* ret = InnerAlloc(size);
    if (!ret)
    {
        ret = InnerAlloc(size);
    }
    ++m_currentAllocCount;
    ++m_totalAllocCount;

#if defined(CTR_TARGET) || defined(CAFE_TARGET) || defined(NEX_NNSDK_FAMILY)
    m_memUsage += GetBlockSize(ret);
    if (m_memUsage > m_memMaxUsage)
    {
        m_memMaxUsage = m_memUsage;
    }
#endif

    return ret;
}

void DemoMemBase::FreeFunction(void* p)
{
    if (p == NULL)
    {
        return;
    }
    --m_currentAllocCount;
#if defined(CTR_TARGET) || defined(CAFE_TARGET)  || defined(NEX_NNSDK_FAMILY)
    m_memUsage -= GetBlockSize(p);
#endif
    InnerFree(p);
}

#if defined(NEX_NNSDK_FAMILY)
void* DemoMemBase::ReallocFunction(void* p, size_t size)
{
    if (p != NULL)
    {
        --m_currentAllocCount;
        m_memUsage -= GetBlockSize(p);
    }
    void* ret = InnerRealloc(p, size);
    
    if (ret != NULL)
    {
        ++m_currentAllocCount;
        ++m_totalAllocCount;
        m_memUsage += GetBlockSize(ret);
        if (m_memUsage > m_memMaxUsage)
        {
            m_memMaxUsage = m_memUsage;
        }
    }
    
    return ret;
}
#endif

size_t DemoMemBase::GetUsage()const
{
    return m_memUsage;
}
size_t DemoMemBase::GetMaxUsage()const
{
    return m_memMaxUsage;
}
size_t DemoMemBase::GetCurrentAllocCount()const
{
    return m_currentAllocCount;
}

size_t DemoMemBase::GetTotalAllocCount()const
{
    return m_totalAllocCount;
}

void DemoMemBase::Invalidate()
{
    InnerInvalidate();
}

}   // end of namespace nex_demo_util
