﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <nex.h>

#include "NexUtilDefs.h"
#include "NexUtilCallContext.h"
#include "NetInit.h"

#if defined(NEX_NNSDK_FAMILY)
#include "NnSdkNexUtil.h"
#elif defined(CTR_TARGET)
#include "CtrNexUtil.h"
#elif defined(CAFE_TARGET)
#include "CafeNexUtil.h"
#elif defined(WIN_FAMILY) || defined(UNIX_FAMILY)
#include "WinLinuxNexUtil.h"
#endif

namespace nex_demo_util
{

/// スリープする時間
static const qUnsignedInt32 SLEEP_DURATION = 16;

/// 推奨するログインタイムアウト時間(ミリ秒)
static const qUnsignedInt32 RECOMMENDED_LOGIN_TIMEOUT_MS = 60 * 1000;


/*!
   @brief アプリケーション定義のメモリアロケータを NEX に設定します。

   ヒープにメモリを確保し、メモリアロケータ関数を NEX に登録します。
   アロケータ関数を設定しない場合は std::malloc()、std::free() が使用されるため、通常は設定することを推奨します。
 */
void InitializeMemoryAllocator();

/*!
   @brief InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
 */
void FinalizeMemoryAllocator();

/*!
   @brief サンプルデモで使用するアプリケーション定義のメモリ確保関数です。

   @param size 確保するメモリのサイズ。
   @return 確保したメモリへのポインタ。必ず有効なアドレスを返す必要があります。
 */
void* AllocFunction(size_t size);

/*!
   @brief サンプルデモで使用するアプリケーション定義のメモリ解放関数です。
   
   @param pointer 解放するメモリのポインタ。
 */
void FreeFunction(void *pointer);

/*!
   @brief メモリ使用状況をデバッグログに表示します。
 */
void PrintMemoryUsage();

/*!
   @brief qResultのリターンコードをデバッグログに表示します。

   @param result リターンコードを表示する qResult オブジェクト。
   @param pHeaderText 追加で表示する文字列。
 */
void ReportQResultReturnCode(const nn::nex::qResult& result, const char* pHeaderText = NULL);

}   // end of namespace nex_demo_util
