﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#pragma once

#if defined(NEX_NNSDK_FAMILY)
#include <nn/nn_Log.h>
#include <nn/nn_Assert.h>
#elif defined(CTR_TARGET)
#include <nn.h>
#elif defined(CAFE_TARGET)
#include <cafe.h>
#endif

// 文字列の書式指定子。CTRとCafeではNEX_UNICODE_VERSIONの"%ls"が定義されます。
#ifdef NEX_UNICODE_VERSION
#define PRs "ls"
#define STRING_FORMAT "%ls"
#define STRING_FORMAT_SPECIFIER "ls"
#else
#define PRs "s"
#define STRING_FORMAT "%s"
#define STRING_FORMAT_SPECIFIER "s"
#endif

// size_tなどの書式指定子
#if defined(WIN_FAMILY)
#define NEX_SIZE_T_FORMAT    "%Iu"
#define NEX_SSIZE_T_FORMAT   "%Id"
#define NEX_PTRDIFF_T_FORMAT "%Id"
#define NEX_SIZE_T_FORMAT_SPECIFIER    "Iu"
#define NEX_SSIZE_T_FORMAT_SPECIFIER   "Id"
#define NEX_PTRDIFF_T_FORMAT_SPECIFIER "Id"
#else
#define NEX_SIZE_T_FORMAT    "%zu"
#define NEX_SSIZE_T_FORMAT   "%zd"
#define NEX_PTRDIFF_T_FORMAT "%td"
#define NEX_SIZE_T_FORMAT_SPECIFIER    "zu"
#define NEX_SSIZE_T_FORMAT_SPECIFIER   "zd"
#define NEX_PTRDIFF_T_FORMAT_SPECIFIER "td"
#endif

#ifdef NEX_STDINT_SUPPORT
#define NEX_INT64_FORMAT  "%" PRId64
#define NEX_UINT64_FORMAT "%" PRIu64
#define NEX_INT64_FORMAT_SPECIFIER  PRId64
#define NEX_UINT64_FORMAT_SPECIFIER PRIu64
#else
#define NEX_INT64_FORMAT  "%lld"
#define NEX_UINT64_FORMAT "%llu"
#define NEX_INT64_FORMAT_SPECIFIER  "lld"
#define NEX_UINT64_FORMAT_SPECIFIER "llu"
#endif

#ifdef NEX_USE_PRINCIPALID_64BIT
#define NEX_PID_FORMAT NEX_UINT64_FORMAT
#define NEX_PID_FORMAT_SPECIFIER NEX_UINT64_FORMAT_SPECIFIER
#else
#define NEX_PID_FORMAT "%u"
#define NEX_PID_FORMAT_SPECIFIER "u"
#endif

// 標準出力にログを出力します
#if defined(CTR_TARGET) || defined(NEX_NNSDK_FAMILY)
#define NEX_DEMO_LOG( ... ) NN_LOG(__VA_ARGS__)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_LOG( ... ) printf(__VA_ARGS__)
#else
#define NEX_DEMO_LOG( ... ) std::printf(__VA_ARGS__)
#endif

// デバッグビルドのとき、条件を満たさない場合に処理を停止します
// リリースビルドのときは条件を評価しません
#if defined(CTR_TARGET) || defined(NEX_NNSDK_FAMILY)
#define NEX_DEMO_ASSERT(expr) NN_ASSERT(expr)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_ASSERT(expr) ASSERT(expr)
#else
#define NEX_DEMO_ASSERT(expr) SYSTEMCHECK(expr)
#endif

// 処理を停止します
#if defined(NEX_NNSDK_FAMILY)
#define NEX_DEMO_PANIC( ... ) NN_ABORT(__VA_ARGS__)
#elif defined(CTR_TARGET)
#define NEX_DEMO_PANIC( ... ) NN_PANIC(__VA_ARGS__)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_PANIC( ... ) NEX_DEMO_LOG(__VA_ARGS__); OSHalt("Panic!")
#else
#define NEX_DEMO_PANIC( ... ) NEX_DEMO_LOG(__VA_ARGS__); NEX_EXIT(-1)
#endif

// デバッグビルドのとき、条件を満たさない場合に処理を停止します
// リリースビルドのときも条件を評価します。条件を満たさない場合でも処理は停止しません。
#ifdef NEX_RELEASE_VERSION
#define NEX_DEMO_VERIFY(expr) ((void)(expr))
#else
#define NEX_DEMO_VERIFY(expr) NEX_DEMO_ASSERT(expr)
#endif

// 配列で初期化された要素数を取得するマクロです
#define NEX_DEMO_SIZEOFARRAY(array) (sizeof(array) / sizeof(array[0]))
