﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#include <nex.h>
#include "NexUtil.h"
#include "NexUtilNetZ.h"

#if defined(CTR_TARGET)
#include <nn.h>
#include <nn/friends.h>
#elif defined(CAFE_TARGET)
#include <cafe.h>
#include <nn/act.h>
#include <nn/fp.h>
#endif

namespace nex_demo_util
{

/*!
   @brief NetZのStationに自分の名前を設定します。この名前が最近遊んだプレイヤーとして登録されます。
 */
void SetupLocalPlayerName()
{
#if defined(CAFE_TARGET)
    //アカウントIDを使用する
    char accountId[ACT_ACCOUNT_ID_SIZE];
    nn::Result result = nn::act::GetAccountId(accountId);

    if(result.IsSuccess())
    {
        // アプリケーションが使用する言語を指定
        nn::nex::Station::SetLocalPlayerName(accountId, nn::fp::LANGUAGE_JAPANESE);
    }
    else
    {
        nn::nex::Station::SetLocalPlayerName(NEX_T("DummyName"), nn::fp::LANGUAGE_JAPANESE);
    }
#elif defined(CTR_TARGET)
    //Miiの名前を使用する
    char16 screenName[nn::friends::SCREEN_NAME_SIZE];
    nn::Result result = nn::friends::GetMyScreenName(screenName);
    if(result.IsSuccess() && screenName[0] != 0) // Miiが設定されていない場合も除く
    {
        nn::nex::Station::SetLocalPlayerName(screenName, 0);
    }
    else
    {
        nn::nex::Station::SetLocalPlayerName(NEX_T("DummyName"), 0);
    }
#else
    nn::nex::Station::SetLocalPlayerName(NEX_T("DummyName"), 0);
#endif
}

}   // end of namespace nex_demo_util
