﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

//group=Error Code Enumerations
BEGIN_NAMESPACE_NEX


// Description: This enumeration defines the success codes that may be returned for the DOCore facility.
/*!
 @brief    DOCore モジュールのサクセスコードを定義する名前空間です。
 */
#ifdef SWIG
class DOCoreSuccessCode
{
public:
#else
namespace DOCoreSuccessCode
{
#endif

/*
Description: Enumerates the success codes for the DOCore facility that an operation may return.
Remarks: Details as to the cause of each SuccessCode are detailed in the documentation
of each individual $PRODUCTNAME$ method that may return a result.
*/

/*!
 @brief    DOCore モジュールのサクセスコード。
*/
enum SuccessCode
{
    // The call terminated successfully.
    Success         = 1,
    // The call has been postponed. A call may be postponed via CallMethodOperation::PostponeOperation.
    CallPostponed   = 2

};
#ifdef SWIG
};
#else
}
#endif

//group=Error Code Enumerations

// Description: This enumeration defines the error codes that may be returned for the DOCore facility.
/*!
 @brief    DOCore モジュールのエラーコードを定義する名前空間です。
 */
#ifdef SWIG
class DOCoreErrorCode
{
public:
#else
namespace DOCoreErrorCode
{
#endif

/*
Description: Enumerates the error codes for the DOCore facility that a call may return.
Remarks: Reasons as to the cause of each ErrorCode are detailed in the documentation
of each individual $PRODUCTNAME$ method that may return a result.
*/

/*!
 @brief    DOCore モジュールのエラーコード
*/
enum ErrorCode
{
    START = 1,
    // The target station of the call could not be reached.
    StationNotReached               = 1,  //!< 呼び出し対象のステーションに到達できませんでした。
    // The call was invoked but the target station was disconnected before a response was received.
    TargetStationDisconnect         = 2,  //!< 呼び出しは実行されましたが、応答が返る前に対象ステーションとの接続が途絶えました。
    // The call was invoked but the local station is leaving the session and a response
    // will never be received.
    LocalStationLeaving             = 3, //!< 呼び出しは実行されましたが、ローカルステーションはセッションを離脱中のため、応答を受け取れません。
    // The object on which the call was called could not be found on the target station of the call.
    ObjectNotFound                  = 4, //!< 呼び出し対象のオブジェクトが、対象ステーション中に見つかりません。
    // The object was found, but the role of the object was not
    // the one expected.  For example, this occurs if the flag
    // TargetObjectMustBeMaster was set but an RMC was called
    // on a Duplica instead of a Duplication master.
    InvalidRole                     = 5, //!< オブジェクトは見つかりましたが、その役割(Role)が想定と異なっています。例えば、RMCにTargetObjectMustBeMasterフラグがセットされていながら、複製マスターではなくデュプリカに対してRMCを呼び出したようなケースが該当します。
    // The call failed because it did not complete before the time-out (set for the call) expired.
    CallTimeout                     = 6, //!< 呼び出しに設定されたタイムアウト時間内に処理を完了できなかったため、呼び出しは失敗しました。
    // The method invocation of an RMC failed.  This can be caused by
    // conflicting versions of the DDL between the caller and the
    // callee. This outcome is only valid if the context is an RMCContext.
    RMCDispatchFailed               = 7, //!< RMCのメソッド実行は失敗しました。このエラーはRMCの呼び出し元と呼び出され側とでDDLバージョンが矛盾する場合に起こりえます。この結果は、コンテキストがRMCContextである場合にのみ有効です。
    // The migration of an object is in progress. This outcome is only valid if the context
    // is a MigrationContext.
    MigrationInProgress             = 8, //!< オブジェクトのマイグレーションが進行中です。この結果は、コンテキストがMigrationContextである場合にのみ有効です。
    // The object was found, but the TargetObjectMustHaveAuthority flag was set and the
    // object did not have authority. Refer to DuplicatedObject::HasAuthority for
    // more details about object authority.
    NoAuthority                     = 9, //!< オブジェクトは見つかりました。しかしTargetObjectMustHaveAuthorityフラグがセットされているにもかかわらず、オブジェクトにはその権限がありません。オブジェクトの権限についての詳細な情報は、DuplicatedObject::HasAuthorityを参照して下さい。
    // No target station was specified for this call.
    NoTargetStationSpecified        = 10, //!< この呼び出しの対象となるステーションが指定されていません。
    // The call to JoinSession failed.
    JoinFailed                      = 11, //!< JoinSessionへの呼び出しが失敗しました。
    // The call to JoinSession was denied.
    JoinDenied                      = 12, //!< JoinSessionへの呼び出しが拒絶されました。
    // JoinSession failed because the connectivity test failed.
    ConnectivityTestFailed          = 13, //!< 接続テストに失敗したため、JoinSessionが失敗しました。
    // The result of the call is unknown.
    Unknown                         = 14, //!< 不明な呼び出し結果です。
    // Upon tear down, some references or selection iterators are still pointing to duplicated
    // objects that are being destroyed.
    // Use DORef::TraceReferences() to find them.
    UnfreedReferences               = 15, //!< オブジェクトの破棄処理において、いくつかの参照またはセレクションイテレータが、破棄されようとしている複製オブジェクトをまだポイントしています。DORef::TraceReferences() を用いて、原因となっている参照もしくはセレクションイテレータを見つけ出して下さい。

    //n1769
    JobTerminationFailed            = 16, //!< JobTerminateFacadeにおいて、終了処理が完了できなかったことを表すエラーです。 深刻度は SEVERITY_CRITICAL に分類されますが、NEXのオブジェクト（NetZ／NGSなど）とメモリアロケーターの再初期化による対処も可とします。


    //n1769
    InvalidState                    = 17, //!< DuplicatedObjectクラス内部で発生しうるエラーです。発生してしまった場合は、NetZオブジェクトの初期化が必要です。

    //n1769
    FaultRecoveryFatal              = 18, //!< 障害回復処理の開始に失敗したことを表すエラーです。

    FaultRecoveryJobProcessFailed   = 19, //!< 障害回復処理実行中に失敗したことを表すエラーです。

    StationInconsistency = 20, //!< ステーション数がセッションマスターと違う状態が続いたことを表すエラーです。

    AbnormalMasterState = 21, //!< セッションマスターが二重に存在するなどして、復帰不可能な状態になったことを表すエラーです。

    VersionMismatch  = 22  //!< ホストとのアプリケーションもしくは、NetZのバージョンが異なり互換性がなく通信できないことを表すエラーです。
};
//note: 新規result追加時には、OnlineCore/src/Platform/Core/ResulDefine.hのNEX_ERROR_RESULT_DEFINEの変更も必要
//Wiki エラーコード追加時に行う作業 を参照
//アプリケーションがハンドリングするべきエラーを増やした場合には、commonmanualのoverall.rstの「アプリケーションがハンドリングするべきエラー一覧」に追加すること

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX


