﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX

//Group=Miscellaneous Types and Macros

// Description: This enumerates the Role that a duplicated object may have.
/*!
 * @brief   複製オブジェクトが持っている役割の列挙体です。
 */
enum Role
{
    /*
    A Duplication master is the controlling instance of a duplicated object.
    The Duplication master can update the values of
    its datasets, an operation that a Duplica cannot perform.
    See Also: Duplication master, ChangeMasterStationOperation
    */

    /*!
     * @brief   複製マスター(@ref DuplicatedObject::IsADuplicationMaster())です。
     *          複製マスターは、オブジェクトのコピー元で、変数などを更新する権限を持ちます。
     */
    DUPLICATION_MASTER = 0,

    /*
    A Duplica is a copy of a duplicated object. Its datasets are updated remotely, automatically,
    and transparently by $PRODUCTNAME$ whenever its Duplication master performs an update.
    See Also: Duplica, ChangeMasterStationOperation
    */

    /*!
     * @brief   デュプリカ(@ref DuplicatedObject::IsADuplica())です。
     *          デュプリカは、オブジェクトのデータをコピーした存在で、変数などの更新権限はありません。
     *          データは常に複製マスターと同期されます。
     */
    DUPLICA = 1,

    // This value is returned when the role of a duplicated object is not yet defined.
    /*!
     * @brief   複製オブジェクトの役割が未決定な状態であるときに返ります。
     */
    ROLE_NOT_DEFINED = 2
};

END_NAMESPACE_NEX

