﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <NetZ/src/Core/DOCore/DORole.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>

#include <NetZ/src/Core/DOCore/Selection.h>

BEGIN_NAMESPACE_NEX

//group=Data Logging

/*
Description: This class contains information about the local DuplicatedObject instances.
Remarks: This class offers the following call which lists all of the duplicated object instances present
on the local station at the time of the call:

            DOSelections::GetInstance()->Trace(TRACE_ALWAYS);

See Also: DuplicatedObject::Trace
*/
class DOSelections: public RootObject
{
    // Group=Methods
public:

    /*
    Remarks: A DOSelections object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ prior to session creation. To access the DOSelections object from within your game
    use the GetInstance method.
    */
    DOSelections();

    /*
    Remarks: A DOSelections object should never be directly destroyed. It is destroyed
    by $PRODUCTNAME$ when the session terminates.
    */
    virtual ~DOSelections();

    /*
    Description: Returns a pointer to the DOSelections.
    Returns: A reference to the DOSelections.
    Remarks: This method returns a pointer to the instance of DOSelections that can then
    be used to Trace the properties of all of the local duplicated objects as follows:

            DOSelections::GetInstance()->Trace(TRACE_ALWAYS);
    */
    static DOSelections* GetInstance();

    // {secret}
    qBool IsInUse(DOHandle hDO) const;

    /*
    Description: Traces the contents of this DOSelections object.
    Remarks: This method traces the DOHandle of each duplicated object listed is this DOSelections.
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, DOFilter* pFilter=NULL);

    // {secret}
    void ClearOnCorruption();

    //{secret}
    void RemoveFromAllSelections(DuplicatedObject* pDO);

    // {secret}
    Selection& AllDOs()
    {
        return m_setAllDOs;
    }
    // {secret}
    Selection& Masters()
    {
        return m_setMasters;
    }
    // {secret}
    Selection& Duplicas()
    {
        return m_setDuplicas;
    }

private:
    // DO集合の親玉
    SelectionMaster m_setAllDOs;
    SelectionMaster m_setMasters;
    SelectionMaster m_setDuplicas;

    // Station DO集合のキャッシュ
    SelectionCache m_setAllDOsStationCache;
    SelectionCache m_setMastersStationCache;
    SelectionCache m_setDuplicasStationCache;
};

END_NAMESPACE_NEX

