﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/StationOperation.h>
#include <NetZ/src/Core/DOCore/Station.h>

BEGIN_NAMESPACE_NEX

/*!
 * @brief   ステーションがセッションに参加するときに実行される操作です。
 *
 *          @ref Station::RegisterOperationBeginCallback 、@ref Station::RegisterOperationEndCallback で
 *          登録したコールバックで得られる @ref StationOperation の、@ref StationOperation::GetType が
 *          @ref Operation::JoinStation だった場合に、JoinStationOperation::DynamicCast を行うことで、
 *          本クラスへのポインタを取得できます。
 *
 *          通信切断のタイミングによっては、本クラスの参加処理開始コールバックが呼ばれても、参加処理完了コールバックが呼ばれないことがあります。
 *
 *          <br>
 *          【 Station::RegisterOperationBeginCallback で登録したコールバックでの JoinStationOperation の用途】<br>
 *          ・ステーションの参加処理開始を示します。<br>
 *          ・@ref GetTargetStation で取得できるステーションはまだ不完全です。<br>
 *          ・この時点から、@ref DirectStream のデータを受信する可能性があり、NEX内部のバッファに溜められて参照可能になりますが、参加処理完了後の受信データ参照を推奨します。<br>
 *          <br>
 *          <br>
 *          【 Station::RegisterOperationEndCallback で登録したコールバックでの JoinStationOperation の用途】<br>
 *          ・ステーションの参加処理完了を示します。<br>
 *          ・@ref GetTargetStation で取得できるステーションはすでに参加処理が完了しているので、アプリケーション独自のデータを DirectStreamやRMCで送信したい場合には、これ以降に行ってください。<br>
 *          ・タイミングによっては、このステーションからのパケットがこれより前に届くことがあることがあります。<br>
 */
class JoinStationOperation: public StationOperation
{
public:

    JoinStationOperation();
    virtual ~JoinStationOperation();

    NEX_IMPL_OPERATION_COMMON_METHODS(JoinStation);

private:
#ifdef NEX_TRACE_ENABLED
    virtual void TraceImpl(Event oEvent, qUnsignedInt64 uiTraceFlags)const;
#endif
};

END_NAMESPACE_NEX

