﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/LogicalClock.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DORef.h>

BEGIN_NAMESPACE_NEX

class Station;

// {secret}
#define LC_MASTER_OF_ORPHAN_DUPLICA     1
// {secret}
#define LC_FIRST_MASTER                 2

// Ideally, this should be a Ref<Station>, but this would cause circular
// references, this this refers to a class embedded in the DO class.

class MasterStationRef: public DORef
{
public:
    MasterStationRef();
    MasterStationRef(const MasterStationRef& oMasterStationRef);
    MasterStationRef(DOHandle hStation, LogicalClock lcVersion);

    virtual ~MasterStationRef();

    qBool NewMasterIsValid(const MasterStationRef& oMasterStationRef) const;

    MasterStationRef& operator=(const MasterStationRef& oMasterStationRef);

    Station* GetStation();
    const Station* GetStation() const;

    Station* operator->()
    {
        return GetStation();
    }
    const Station* operator->() const
    {
        return GetStation();
    }

    DOHandle GetStationHandle() const
    {
        return GetHandleValue();
    }
    const LogicalClock& GetVersion() const
    {
        return m_lcVersion;
    }

    LogicalClock& Version()
    {
        return m_lcVersion;
    }
    const LogicalClock& Version() const
    {
        return m_lcVersion;
    }

private:
    void Init();

private:

    LogicalClock m_lcVersion;
};

END_NAMESPACE_NEX


