﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Transport/Interface/Message.h>


BEGIN_NAMESPACE_NEX

class EndPoint;
class Station;

class MessageBundle: public RootObject
{
    friend class Station;
public:

    MessageBundle();
    virtual ~MessageBundle();

    qBool MustFlushBefore(Message* pMessage, qUnsignedInt32 uiFlags) const ;
    qBool MustFlushNow() const;

    qResult Send(EndPoint* pEndPoint);
    void Add(Message *pMessage, qUnsignedInt32 uiFlags);

    size_t GetSize() const
    {
        return m_oMessage.GetTotalSize();
    }

private:

    void Clear();
    qBool IsEmpty() const;
    void MarkBundleEnd();
    void UnmarkBundleEnd();

    qUnsignedInt32 GetSendingFlags()
    {
        return m_uiSendingFlags;
    }

    //n1589
    qBool FlagIsSet(qUnsignedInt16 uiFlag)
    {
        return (m_uiSendingFlags & uiFlag) != 0;
    }

private:
    qBool SizeWillBeExceeded(Message *pMessage) const;
    qBool SizeIsExceeded() const;
    qBool TimeIsExpired() const;

#if NEX_ENABLE_FIX_BUNDLE_MAXSIZE_OPTIMIZATION
    //n1769:RootTransportのDEFAULT_MTUと同じ値にしました。
    //n2964:Routingを考慮して1250バイトへ
    static const qUnsignedInt32 FIXED_MAXIMUM_SIZE = 1250;
    static qUnsignedInt32 GetMaximumSize()
    {
        return FIXED_MAXIMUM_SIZE;
    }
#else
    static qUnsignedInt32 GetMaximumSize();
#endif

private:

    Message         m_oMessage;
    Time            m_tiFirstAddedMsg;
    qUnsignedInt32  m_uiSendingFlags;
};


END_NAMESPACE_NEX

