﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/Operation.h>
#include <NetZ/src/Core/DOCore/Station.h>

BEGIN_NAMESPACE_NEX

/*!
 * @brief   ステーションに対する操作をカプセル化します。
 *
 * この操作は、@ref JoinStationOperation および @ref LeaveStationOperation の基底クラスです。
 *
 * @ref Station::RegisterOperationBeginCallback 、@ref Station::RegisterOperationEndCallback で
 * 登録したコールバックで本クラスへのポインタを取得できます。
 *
 * @see     Station::RegisterOperationBeginCallback, Station::RegisterOperationEndCallback
 */
class StationOperation: public Operation
{
    //Group=Methods
public:
    StationOperation(const qChar* pStationNameReference);
    virtual ~StationOperation();

    // {secret}
    void Begin(Station* pStation);
    // {secret}
    void End(Station* pStation);

    /*!
     * @brief   この操作の対象となるステーションを取得します。
     * @return  操作対象のステーションへのポインタ
     */
    const Station* GetTargetStation() const
    {
        return *m_refTargetStation;
    }

private:
    Station::Ref m_refTargetStation;
};

END_NAMESPACE_NEX

