﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX
// Group = Session Management

enum
{
    /*
    Summary:  The system is initializing.
    Remarks: This state can be used to determine if the session setup stage is still in progress or if the session
    has already started. To $PRODUCTNAME$ the states SYSTEM_STATE_INITIALIZING and SYSTEM_STATE_RUNNING are identical.
    In both states objects can be published and discovered and data can be updated.
    See Also: Session::SetSystemState, Session::GetSystemState
    */
    SYSTEM_STATE_INITIALIZING=0,

    /*
    Summary:  The system is running.
    Remarks: This state can be used to determine if the session setup stage is still in progress or if the session
    has already started. To $PRODUCTNAME$ the states SYSTEM_STATE_INITIALIZING and SYSTEM_STATE_RUNNING are identical.
    In both states objects can be published and discovered and data can be updated.
    See Also: Session::SetSystemState, Session::GetSystemState
    */
    SYSTEM_STATE_RUNNING=1,

    // # define SYSTEM_STATE_REMOVED        2           State number 2 has been removed. See [Bug 5962] for the reasons

    /*
    Description:  The session is terminating.
    Remarks:  The state SYSTEM_STATE_TERMINATING is the only state that $PRODUCTNAME$ interprets. When the session
    terminates certain operations, such session fault tolerance, are not performed. It does not make sense to perform
    the migration of the managed duplicated objects since there is no place to migrate as each station is leaving
    the session. Therefore, the system callbacks DuplicatedObject::ApproveEmigration and DuplicatedObject::ApproveFaultRecovery
    will not be called when a station leaves since the session is terminating.

    This state should only be set at the end of a session and usually only on the Session master. Once this state is set,
    you may still reset the state via Session::SetSystemState in order to return to one of the other system states.
    See Also: Session::SetSystemState, Session::GetSystemState
    */
    SYSTEM_STATE_TERMINATING=3
};

END_NAMESPACE_NEX

