﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupImpl.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h> /* For MSG_RELIABLE */
#include <OnlineCore/src/Transport/Interface/Message.h>

BEGIN_NAMESPACE_NEX

class UpdateProtocol: public RootObject
{
public:
    UpdateProtocol() {}
    virtual ~UpdateProtocol() {};

    virtual qBool UsesSpecialInnerUpdateLoop()
    {
        return false;
    }
    //n1589:memo:現状これが使用される
    // TODO - Will eventually disapear...
    virtual qUnsignedInt32 SpecialInnerUpdateLoop(DuplicatedObject* pDO,
            void* pDS, qByte byIndex, Time tTimestamp)
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(pDS);
        DUMMY_PARAM(byIndex);
        DUMMY_PARAM(tTimestamp);
        return 0;
    }

    // Called by the UpdatePolicy::Update

    virtual qByte GetCommunicationFlags(DuplicatedObject* pDO,
                                        void* pDS, qByte byIndex)=0;

    virtual void AddToMessage(DuplicatedObject* pDO,
                              void* pDS, qByte byIndex,
                              Time tTimestamp,
                              Message* pMessage, qBool bDiscovery)=0;

    virtual void ExtractFromMessage(DuplicatedObject* pDO,
                                    void* pDS, qByte byIndex,
                                    Message* pMessage,
                                    qBool bInitialRefresh)=0;
};

template<class DS>
class BasicUpdateProtocol: public UpdateProtocol
{
public:
    BasicUpdateProtocol()
    {
    }

    virtual qBool UsesSpecialInnerUpdateLoop()
    {
        return false;
    }

    virtual qByte GetCommunicationFlags(DuplicatedObject* pDO,
                                        void* pDS, qByte byIndex)
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(byIndex);
        if (static_cast<DS*>(pDS)->ReliableUpdate())
        {
            return EndPoint::MSG_RELIABLE;
        }
        else
        {
            return 0;
        }
    }

    virtual void AddToMessage(DuplicatedObject* pDO,
                              void* pDS, qByte byIndex,
                              Time tTimestamp,
                              Message* pMessage, qBool bDiscovery)
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(byIndex);
        static_cast<DS*>(pDS)->AddSourceTo(pMessage, tTimestamp, bDiscovery);
    }

    virtual void ExtractFromMessage(DuplicatedObject* /*pDO*/,
                                    void* pDS, qByte /*byIndex*/,
                                    Message* pMessage,
                                    qBool /*bInitialRefresh*/)
    {
        static_cast<DS*>(pDS)->SetRecvSubStreamID(pMessage->GetSubStreamID());
        static_cast<DS*>(pDS)->ExtractFrom(pMessage);
    }
};

template<class DS>
class LegacyUpdateProtocol: public UpdateProtocol
{
public:
    LegacyUpdateProtocol()
    {
    }

    virtual qBool UsesSpecialInnerUpdateLoop()
    {
        return true;
    }

    virtual qUnsignedInt32 SpecialInnerUpdateLoop(DuplicatedObject* pDO,
            void* pDS, qByte byIndex, Time tTimestamp)
    {
        // This one is special: it also adds old filters.
        return static_cast<DS*>(pDS)->UpdateLoop(pDO,pDS,byIndex,tTimestamp);
    }

    virtual qByte GetCommunicationFlags(DuplicatedObject* pDO,
                                        void* pDS, qByte byIndex)
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(byIndex);
        if (static_cast<DS*>(pDS)->ReliableUpdate())
        {
            return EndPoint::MSG_RELIABLE;
        }
        else
        {
            return 0;
        }
    }

    virtual void AddToMessage(DuplicatedObject* pDO,
                              void* pDS, qByte byIndex,
                              Time tTimestamp,
                              Message* pMessage, qBool bDiscovery)
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(byIndex);
        static_cast<DS*>(pDS)->AddSourceTo(pMessage, tTimestamp, bDiscovery);
    }

    virtual void ExtractFromMessage(DuplicatedObject* pDO,
                                    void* pDS, qByte byIndex,
                                    Message* pMessage,
                                    qBool bInitialRefresh)
    {
        DUMMY_PARAM(byIndex);
        static_cast<DS*>(pDS)->SetRecvSubStreamID(pMessage->GetSubStreamID());
        static_cast<DS*>(pDS)->ExtractFrom(pMessage, bInitialRefresh, pDO);
    }
};


END_NAMESPACE_NEX

