﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX

//group=DDL Output

/*
Description: Use this macro to generate the name of the base class from
which your own duplication space implementation inherits.
Remarks:
For example, if a duplicated object is declared in a DDL file as follows:

        dupspace InterestSpace;

The user-defined InterestSpace class is then declared as follows:

        class InterestSpace : public DUPSPACE(InterestSpace) {
            // User specific variables and methods including PSMatch definition
            virtual qBool DuplicationSpace::PSMatch(DuplicatedObject * pPublisher,
                                                DuplicatedObject * pSubscriber) {
            //User Implementation of PSMatch function for this space
            }
        };
*/
#define DUPSPACE(USERSPACE) _DUPSPACE_##USERSPACE

#define DUPSPACEID(USERSPACE)   _DUPSPACE_##USERSPACE::GetDupSpaceID()

#define _DUPSPACE_DeclareGeneralMethods(SPACE)      \
    virtual const qChar* GetClassNameString();          \
static DupSpace::ID GetDupSpaceID();                \
static SPACE* GetInstance();

#define _DUPSPACE_ImplementGeneralMethods(SPACE)                                        \
    SPACE g_o##SPACE##Instance;                                                         \
    const qChar* _DUPSPACE_##SPACE::GetClassNameString() {return NEX_T(#SPACE);}                   \
    DupSpace::ID _DUPSPACE_##SPACE::GetDupSpaceID() {return g_o##SPACE##Instance.GetID();}  \
    SPACE* _DUPSPACE_##SPACE::GetInstance() {return (SPACE*)&g_o##SPACE##Instance;}

#define _DUPSPACE_NoUserCells()                                                     \
    virtual qBool CellMatch(DuplicatedObject* pPS, DuplicatedObject* pCell, qBool /*bOnRelevanceCheck*/) {      \
        return DefaultCellMatch(pPS, pCell);                                        \
    }                                                                               \
    virtual void CreateCells() {CreateDefaultCells();}                              \
    virtual qBool UsesDefaultCells() {return true;}

END_NAMESPACE_NEX

