﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/SystemComponent.h>
#include <OnlineCore/src/Core/OperationManager.h>
#include <NetZ/src/Core/DupSpace/DuplicationSpaceTable.h>

BEGIN_NAMESPACE_NEX

//Group=DupSpaces

/*
Description: This class indicates to $PRODUCTNAME$ that an DupSpace will be used.
Remarks: If you wish to use an DupSpace, the DupSpaceExtension must be registered to $PRODUCTNAME$ via Register to
ensure that the system knows that it should use an DupSpace. If not the appropriate files will not be loaded
and you will end up with compiler errors.
*/
class DupSpaceExtension: public SystemComponent
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(DupSpaceExtension,SystemComponent);
    //group=Methods

    DupSpaceExtension();
    virtual ~DupSpaceExtension();

    class DupSpaceOperationCallback: public OperationCallback
    {
    public:
        DupSpaceOperationCallback();
        virtual void CallMethod(Operation* pOperation);
    };

    //{secret}
    virtual qBool BeginTermination();

    //{secret}
    virtual qBool BeginInitialization();

    /*
    Description: Registers an DupSpaceExtension to $PRODUCTNAME$.
    Returns: true if the DupSpaceExtension was successfully registered to $PRODUCTNAME$, false if not
    or an error occurred.
    Remarks: This method must be called after the $PRODUCTNAME$ object is created and before a Session
    is either created or joined. For example,

            // Create $PRODUCTNAME$ object
            $PRODUCTNAME$ *g_p$PRODUCTNAME$ = qNew $PRODUCTNAME$;

            // Register the DupSpaceExtension
            DupSpaceExtension::Register

            // Create the Session
            if(!(Session::CreateSession())){
                // Process the session creation error
                // SystemError
            }

    See Also: Session::CreateSession, Session::JoinSession
    */
    static qBool Register();

private:
    DupSpaceOperationCallback m_oOperationCallbacks;
    DuplicationSpaceTable m_oDSTable;
};

END_NAMESPACE_NEX

