﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>

BEGIN_NAMESPACE_NEX

//group=Adapters

/*
Description: This class
Remarks:
*/
class AdapterEvent: public RefCountedObject
{
public:

    //group= Type Definitions

    typedef enum _Type
    {
        // Special value.
        InvalidEvent,
        // AdapterDOEvents:
        AddToStore,
        RoleChange,
        RemoveFromStore,

        // Adapter specific events.  Start at this value.
        AdapterSpecificEvent
    } Type;


    //group=Methods

    AdapterEvent(Type eEventType);
    virtual ~AdapterEvent();

    /*
    Description: Returns the Type of AdapterEvent.
    Returns: The Type of AdapterEvent may be one of InvalidEvent, AddToStore, RoleChange, RemoveFromStore, or
    AdapterSpecificEvent.
    See Also: Type, _Type
    */
    Type GetType() const
    {
        return m_eEventType;
    }
    const qChar* GetTypeString() const ;
    ICContext GetInstanceContext() const
    {
        return m_icContext;
    }

    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);

private:
    Type m_eEventType;
    // TODO: Ifdefs.
    //{secret}
    ICContext m_icContext;
};

END_NAMESPACE_NEX

