﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DatasetCGMacros.h>
#include <NetZ/src/Core/DOCore/StationUpdateFilter.h>
#include <NetZ/src/Core/DOCore/GlobalUpdateFilter.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class Station;

// This is the implementation of the station filter used by the custom dead reckoning extension.

template<class DS, class M>
class DeadReckoningFilter: public StationUpdateFilter
{
public:


    DeadReckoningFilter() {};
    ~DeadReckoningFilter() {};

    // This method is called when DuplicatedObject::Update is called
    virtual qBool UpdateRequired(DuplicatedObject* pDO,
                                 void* pOpaqueDS,
                                 Time tTimestamp,
                                 Station* pTargetStation)
    {
        DS* pCurrentValue = static_cast<DS*>(pOpaqueDS);

        M* pCurrentModel = pCurrentValue->GetRemoteDeadReckoningModel(pTargetStation);
        if (pCurrentModel==NULL)
        {
            // When no model is available for a station, it means
            // that no update is required.
            return false;
        }
#if 0
        QTRACE(D_TRACE_DEBUG,NEX_T("Model towards ") << hex << (pTargetStation->GetHandleValue()) << NEX_T(" at ") << hex << (
                   pCurrentModel) << NEX_T(".  last update = ") << dec << (
                   pCurrentModel->GetLastUpdate()) << NEX_T(", last update loop = ") << dec << (
                   pCurrentModel->GetLastUpdateLoop()) << NEX_T(", update loop number = ") << dec << (
                   GetUpdateLoopNumber()));
#endif
        if (pCurrentModel->GetLastUpdate()==GetUpdateLoopNumber())
        {
            // This model is re-used for another station, in
            // the same update loop.
            // Since an update was sent to the other station, let's
            // send one to this station as well. We ignore the call to
            // IsAccurateEnough- since the model was updated at the
            // last update, it is most probably accurate now, but
            // the current target never received the message.

            // The call to set the update info in the model will be
            // done in the 'UpdateSent' call - this ensures that
            // the value is set only if the target passes the other
            // filters.
#if 0
            QTRACE(D_TRACE_DEBUG,NEX_T("Sending an update (also) to station ") << hex << (pTargetStation->GetHandleValue()) << NEX_T(" at time ") << dec << ( tTimestamp));
#endif
            return true;
        }

        if (pCurrentModel->GetLastUpdateLoop() == GetUpdateLoopNumber())
        {
            // This model is re-used for another station, in
            // the same update loop.
            // Since NO update was sent to the other station, we won't send any
            // to this one.  We ignore the call to IsAccurateEnough(), since
            // the model was accurate the last time, is is most probably
            // still accurate.
#if 0
            QTRACE(D_TRACE_DEBUG,NEX_T("Skipping call to IsAccurateEnough() for station ") << hex << (pTargetStation->GetHandleValue()) << NEX_T(" at time ") << dec << ( tTimestamp));
#endif
            return false;
        }

        pCurrentModel->SetLastUpdateLoop(GetUpdateLoopNumber());

        // We test if the model is accurate enough.  For
        // convenience, the model is passed a ptr to the DO and to
        // the target station.  Some models may use this value to
        // compute the error thresholds.
        if (pCurrentModel->IsAccurateEnough(*pCurrentValue, tTimestamp, pDO, pTargetStation))
        {
            return false;
        }
        else
        {
#if 0
            QTRACE(D_TRACE_DEBUG,NEX_T("Sending an update to station ") << hex << (pTargetStation->GetHandleValue()) << NEX_T(" at time ") << dec << ( tTimestamp));
#endif
            return true;
        }
    }

    // This method is called when the duplication master sends an update
    virtual void UpdateSent(DuplicatedObject* pDO,
                            void* pOpaqueDS,
                            Time tTimestamp,
                            Station* pTargetStation)
    {
        DS* pCurrentValue = static_cast<DS*>(pOpaqueDS);
        M* pCurrentModel = pCurrentValue->GetRemoteDeadReckoningModel(pTargetStation);
        SYSTEMCHECK(pCurrentModel!=NULL); // Since in UpdateRequired, a model was found.

        if (pCurrentModel->GetLastUpdate()!=GetUpdateLoopNumber())
        {
            // This is a real update - we must update the model.
            // We update the last update info for the current model.
            pCurrentModel->SetLastUpdate(GetUpdateLoopNumber());
            SYSTEMCHECK(pCurrentModel->GetLastUpdate()==GetUpdateLoopNumber());
            pCurrentModel->UpdateModelOnMaster(*pCurrentValue, tTimestamp);
        }
    }
};


END_NAMESPACE_NEX


