﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

//n1589:memo:ObjDupDecl.h is included
#include <NetZ/src/Core/DOCore/ObjDupImpl.h>
#include <NetZ/src/Core/DOCore/UpdateContext.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/ObjDupProtocol.h>
#include <NetZ/src/Extensions/PHBDR/ErrorToleranceFunction.h>
#include <NetZ/src/Extensions/PHBDR/PHBDRVar.h>
#include <NetZ/src/Extensions/PHBDR/PHBDRModel.h>

BEGIN_NAMESPACE_NEX

#define DATASET_OP_REGISTER     0
#define DATASET_OP_UNREGISTER   1

// Group=Object Duplication Subsystem

#define IMPLEMENTAPREDICTION(PREDICTION, TYPE, RETURNTYPE)  \
    RETURNTYPE Predict##PREDICTION(PHBDRVAR(TYPE)& v, Time tFuture=DataSet::GetTimestamp()) {   \
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());                                                       \
        EDSCHECK_ROLE_IS_DUPLICA(0);                                                            \
        RETURNTYPE dValue, dRateOfChange, dAcceleration;                                        \
        v.PredictValues(*DataSetTemplate<SpecificDataSet>::GetPHBDRParameters(), \
                        DataSetTemplate<SpecificDataSet>::GetPHBDRModel()->GetTimeVector(),         \
                        &dValue,&dRateOfChange, &dAcceleration, tFuture);                   \
        return d##PREDICTION;                                                               \
    }

//n1589
#if defined(NEX_CLIENT_OPTIMIZATION)

//n1589:todo2:SIGNAL_SYSTEMERRORしてるので関数化しておきたい
#define IMPLEMENTLOOPBACKPREDICTION(PREDICTION, TYPE, RETURNTYPE) \
    RETURNTYPE PredictLoopback##PREDICTION(\
        PHBDRVAR(TYPE)& v,\
        Time tFuture = DataSet::GetTimestamp()\
    ){\
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());\
        if( !DataSetTemplate<SpecificDataSet>::HasLoopback() )\
        {\
            QLOG(EventLog::Warning, NEX_T("IMPLEMENTLOOPBACKPREDICTION:SYSTEMERROR_GEN_INVALID_OPERATION"));\
            SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_INVALID_OPERATION,0);\
            return 0;\
        }\
        EDSCHECK_ROLE_IS_DM(0);\
        RETURNTYPE dValue, dRateOfChange, dAcceleration;\
        v.PredictValues(\
            *DataSetTemplate<SpecificDataSet>::GetPHBDRParameters(),\
            DataSetTemplate<SpecificDataSet>::GetLoopbackModel()->GetTimeVector(),\
            &dValue,\
            &dRateOfChange,\
            &dAcceleration,\
            tFuture\
        );\
        return d##PREDICTION;\
    }

#else

#define IMPLEMENTLOOPBACKPREDICTION(PREDICTION, TYPE, RETURNTYPE)   \
    RETURNTYPE PredictLoopback##PREDICTION(PHBDRVAR(TYPE)& v, Time tFuture=DataSet::GetTimestamp()) {   \
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());                                                       \
        if (!DataSetTemplate<SpecificDataSet>::HasLoopback()) {SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_INVALID_OPERATION,0); return 0;} \
        EDSCHECK_ROLE_IS_DM(0);                                                         \
        RETURNTYPE dValue, dRateOfChange, dAcceleration;                                        \
        v.PredictValues(*DataSetTemplate<SpecificDataSet>::GetPHBDRParameters(), \
                        DataSetTemplate<SpecificDataSet>::GetLoopbackModel()->GetTimeVector(),          \
                        &dValue,&dRateOfChange, &dAcceleration, tFuture);                   \
        return d##PREDICTION;                                                               \
    }

#endif

#define IMPLEMENTPREDICTIONS(PREDICTION)        \
    IMPLEMENTAPREDICTION(PREDICTION, qDouble, qDouble)  \
    IMPLEMENTAPREDICTION(PREDICTION, qFloat, qFloat)    \
    IMPLEMENTAPREDICTION(PREDICTION, qInt32, qFloat)    \
    IMPLEMENTAPREDICTION(PREDICTION, qInt16, qFloat)    \
    IMPLEMENTAPREDICTION(PREDICTION, qInt8, qFloat) \
    IMPLEMENTLOOPBACKPREDICTION(PREDICTION, qDouble, qDouble)   \
    IMPLEMENTLOOPBACKPREDICTION(PREDICTION, qFloat, qFloat) \
    IMPLEMENTLOOPBACKPREDICTION(PREDICTION, qInt32, qFloat) \
    IMPLEMENTLOOPBACKPREDICTION(PREDICTION, qInt16, qFloat) \
    IMPLEMENTLOOPBACKPREDICTION(PREDICTION, qInt8, qFloat)

template <typename SpecificDataSet> class EDataSetTemplate: public DataSetTemplate<SpecificDataSet>
{
public:


    //---------------------------------------------------------------------------------------
    UpdateContext* LocateUCIfRequired(Station* pStation, DuplicatedObject* pDO, qByte byIndex)
    {
        if (UsesUpdateContext())
        {
            UpdateContext* pUC=pStation->UpdateContextes().Find(pDO->GetHandle(), byIndex);
            if (!pUC)
            {
                pUC=CreateUpdateContext();
                pStation->UpdateContextes().Add(pDO->GetHandle(), byIndex, pUC);
            }
            SYSTEMCHECK(pUC!=NULL);
            return pUC;
        }
        else
        {
            return NULL;
        }
    }

    //---------------------------------------------------------------------------------------
    qBool UsesUpdateContext()
    {
        return DataSetTemplate<SpecificDataSet>::s_erFunction.DistanceMatters();
    }


    // These methods have a reverse semantic for the returned
    // value, when a min or a max is not specified.  But this is
    // correct and makes sense.  The default is to specify that the
    // update delay is somewhere between min and max - thus min is
    // expired but not max.

    //---------------------------------------------------------------------------------------
    qBool MaximumUpdateDelayExpired(UpdateContext* pUC, const Time& tTimestamp)
    {
        if (DataSetTemplate<SpecificDataSet>::s_tiMaximumUpdateDelay>0 &&
                ((PHBDRModel*)pUC)->CurrentUpdateDelay(tTimestamp)>DataSetTemplate<SpecificDataSet>::s_tiMaximumUpdateDelay)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    //---------------------------------------------------------------------------------------
    qBool MinimumUpdateDelayExpired(UpdateContext* pUC, const Time& tTimestamp)
    {
        if (DataSetTemplate<SpecificDataSet>::s_tiMinimumUpdateDelay>0 &&
                ((PHBDRModel*)pUC)->CurrentUpdateDelay(tTimestamp)<DataSetTemplate<SpecificDataSet>::s_tiMinimumUpdateDelay)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    //---------------------------------------------------------------------------------------
    UpdateContext* CreateUpdateContext()
    {
        return (UpdateContext*) qNew typename SpecificDataSet::Buffer;
    }

    //---------------------------------------------------------------------------------------
    qBool SomeUpdatesRequired(qBool* pbForcedReliable, const Time& tTimestamp)
    {
        // Checks if this dataset nees to be updated on remote stations, and
        // whether the update should be sent reliably.
        if (DataSetTemplate<SpecificDataSet>::m_oBreak.ForceUpdateMessage())
        {
            // continuity breaks always cause a reliable update to be sent.
            *pbForcedReliable = DataSetTemplate<SpecificDataSet>::m_oBreak.IsReliable();
            return true;
        }
        if (UsesUpdateContext())
        {
            // distance-based DR means we need to check each station's update
            // context individually.
            return true;
        }
        if (MinimumUpdateDelayExpired(DataSetTemplate<SpecificDataSet>::GetPHBDRModel(), tTimestamp))
        {
            const qReal fSqError = DataSetTemplate<SpecificDataSet>::ComputeExtrapolationSquaredError(DataSetTemplate<SpecificDataSet>::GetPHBDRModel(), tTimestamp);
            const qReal fSqThreshold = DataSetTemplate<SpecificDataSet>::s_erFunction.SquaredValueAt(UNKNOWN_DISTANCE);
            // An update should be send... if the minimum
            // update delay is expired and the error is greater
            // than desired.
            if (fSqError > fSqThreshold)
            {
                return true;
            }
        }
        if (MaximumUpdateDelayExpired(DataSetTemplate<SpecificDataSet>::GetPHBDRModel(), tTimestamp) && !DataSetTemplate<SpecificDataSet>::m_dsBuffer.ModelIsReliable())
        {
            // If the model isn't reliable (contains points sent
            // using an unreliable channel), and the maximum update
            // delay is expired, then we send an update.
            *pbForcedReliable=true;
            return true;
        }
        return false;
    }

    //---------------------------------------------------------------------------------------
    void FinishedAllUpdates(Time tTimestamp, qBool bReliable)
    {
        DataSetTemplate<SpecificDataSet>::UpdateModel(DataSetTemplate<SpecificDataSet>::GetPHBDRModel(), tTimestamp, bReliable);
        DataSetTemplate<SpecificDataSet>::m_oBreak.Transition();
        DataSetTemplate<SpecificDataSet>::FinishedAllUpdates();
    }

    //---------------------------------------------------------------------------------------
    void FinishedAnUpdate(UpdateContext* pUC, Time tTimestamp, qBool bReliable)
    {
        if (pUC)
        {
            DataSetTemplate<SpecificDataSet>::UpdateModel((typename SpecificDataSet::Buffer*)pUC, tTimestamp, bReliable);
        }
    }

    //---------------------------------------------------------------------------------------
    qBool UpdateRequired(Station* pStation, UpdateContext* pUC, DuplicatedObject* pDO, qBool* pbForcedReliable, const Time& tTimestamp)
    {
        // Checks if this dataset may need to be updated on a given station, and
        // whether the update should be sent reliably.
        if (!DataSetTemplate<SpecificDataSet>::UpdateIsRequired(pDO, pStation))
        {
            // When overridden, this function returning false really means
            // 'UpdateIsNotAllowed'.  No matter what follows, we do not update.
            return false;
        }
        if (DataSetTemplate<SpecificDataSet>::m_oBreak.ForceUpdateMessage())
        {
            // continuity breaks always cause a reliable update to be sent.
            // NOTE: relies upon SomeUpdatesRequired() to have already set
            //       *pbForcedReliable to true!
            return true;
        }
        if (!pUC)    // pUC == NULL
        {
            // No update context (distance-based DR is off).  Minimum and maximum
            // update delays expiration already checked in SomeUpdatesRequired().
            return true;
        }
        if (MinimumUpdateDelayExpired(pUC, tTimestamp))
        {
            const qReal fSqError = DataSetTemplate<SpecificDataSet>::ComputeExtrapolationSquaredError((typename SpecificDataSet::Buffer*) pUC, tTimestamp);
            const qReal fSqThreshold = DataSetTemplate<SpecificDataSet>::s_erFunction.SquaredValueAt(pStation->ComputeDistanceTo(pDO->GetHandle()));
            // An update should be send... if the minimum update delay is
            // expired and the distance-based error is greater than desired.
            if (fSqError>fSqThreshold)
            {
                return true;
            }
        }
        if (MaximumUpdateDelayExpired(pUC, tTimestamp) && !((typename SpecificDataSet::Buffer*)pUC)->ModelIsReliable())
        {
            // If the model isn't reliable (contains points sent
            // using an unreliable channel), and the maximum update
            // delay is expired, then we send an update.
            *pbForcedReliable=true;
            return true;
        }
        return false;
    }

    //n1589:todo2:InnerUpdateLoop, move to cpp
    //---------------------------------------------------------------------------------------
    qUnsignedInt32 InnerUpdateLoop(DuplicatedObject* pDO, qByte byIndex, Time tTimestamp=DataSet::GetTimestamp())
    {
        // Called for every DataSet.
        qUnsignedInt32 uiNbSent=0;

        // This flag starts with the value returned by
        // ReliableUpdate.  However, when checking if some updates
        // are required, it could be set true anyway (but not
        // false).  As we update the model, we pass the current
        // information about reliability.  This is not perfectly
        // correct, since a check for updates to another stations
        // might set the flag true - but it does not matter, as it
        // is always correct to assume than a reliable message is
        // unreliable.
        qBool bReliable=DataSetTemplate<SpecificDataSet>::ReliableUpdate();
        if (SomeUpdatesRequired(&bReliable, tTimestamp))
        {
            QTRACE(
                TRACE_UPDATE,
                NEX_T("Updating DataSet ") << dec << (pDO->GetDatasetNameString(byIndex))
                << NEX_T(" (") << ( byIndex) << NEX_T(") of DO ")
                << hex << ((qUnsignedInt32)pDO->GetDOID()) << NEX_T(" (") << ( pDO->GetClassNameString()) << NEX_T(")")
            );

            // MULTICAST : Some stations can be updated MULTICAST
            Selection  oStationsToIncludeSelection(Selection::OwnsReference);
            qVector<UpdateContext  *> finishedUpdates;

            Selection::iterator itStationToUpdate = pDO->GetDuplicasStationLocations()->begin();
            const Selection::iterator itEnd = pDO->GetDuplicasStationLocations()->end();
            while (itStationToUpdate!=itEnd)
            {
                Station* pStation = static_cast<Station*>(itStationToUpdate->second);
                UpdateContext  *pUC=LocateUCIfRequired(pStation, pDO, byIndex);
                // If a station has a UpdateContext, so all the station has a different update context
                if(UpdateRequired(pStation, pUC, pDO, &bReliable, tTimestamp))
                {
                    oStationsToIncludeSelection.Add(itStationToUpdate->first);

                    finishedUpdates.push_back(pUC); //FinishedAnUpdateがここあったが、キュー長制限チェック後に移動
                }

                itStationToUpdate++;
            }

            if (!oStationsToIncludeSelection.empty())
            {

                Message *pMsg = BuildUpdateMessage(pDO, byIndex, tTimestamp);
                qUnsignedInt16 uiFlags = 0;
                if (bReliable)
                {
                    //n1589
                    QTRACE(D_TRACE_TRANSPORT_RELIABLE, NEX_T("EDataSetTemplate::InnerUpdateLoop:MSG_RELIABLE"));

                    uiFlags |= EndPoint::MSG_RELIABLE;
                }
#if NEX_ENABLE_UNRELIABLE_BC
                else
                {
                    TransportAdapter::ETransportMode transportMode = Network::GetTransportAdapter()->GetTransportMode();
                    if(transportMode == TransportAdapter::UnreliableBroadcast)
                    {
                        uiFlags |= EndPoint::MSG_BROADCAST;
                    }
                    else if(transportMode == TransportAdapter::UnreliableSendMulti)
                    {
                        uiFlags |= EndPoint::MSG_SEND_MULTI;
                    }
                }

#endif
                if(pDO->IsInvalidSubStreamID(&oStationsToIncludeSelection, DataSetTemplate<SpecificDataSet>::GetSubStreamID()))
                {
                    qDelete pMsg;
                    return false;
                }
                pMsg->SetSubStreamID(DataSetTemplate<SpecificDataSet>::GetSubStreamID());
                //UserDOかつReliableだったら、キュー長で送信を制限
                if(pDO->IsAUserDO() && pDO->IsOverReliableSendBufferLimit(&oStationsToIncludeSelection, uiFlags, DataSetTemplate<SpecificDataSet>::GetSubStreamID()))
                {
                    qDelete pMsg;
                    return QUNSIGNEDINT32_MAX;
                }

                for(qVector<UpdateContext *>::iterator it2 = finishedUpdates.begin();
                        it2 != finishedUpdates.end(); ++it2)
                {
                    FinishedAnUpdate(*it2, tTimestamp, bReliable);
                }

                pDO->SendToSomeDuplicas(&oStationsToIncludeSelection, pMsg, uiFlags);
                qDelete pMsg;
                uiNbSent++;
            }

            FinishedAllUpdates(tTimestamp, bReliable); // Only called if at least an update was performed...

        }
        return uiNbSent;
    }


    //---------------------------------------------------------------------------------------
    qUnsignedInt32 UpdateLoopWithUpdatedModel(DuplicatedObject* pDO, qByte byIndex, Time tTimestamp)
    {
        qUnsignedInt32 uiNbRecipient = 0;
        for (qInt i=0; i<3; i++)
        {
            DataSetTemplate<SpecificDataSet>::AddPredictedPoint(tTimestamp+TimeInterval(i));
            uiNbRecipient += InnerUpdateLoop(pDO, byIndex, tTimestamp+TimeInterval(i));
        }
        return uiNbRecipient;
    }

    //---------------------------------------------------------------------------------------
    qUnsignedInt32 UpdateLoop(DuplicatedObject* pDO, void* pDS, qByte byIndex, Time tTimestamp=DataSet::GetTimestamp())
    {
        //n1589:memo:このロックがないと送信処理で必要な排他ができない、Station::GetEndPointなど
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        DUMMY_PARAM(pDS);
#ifdef NEX_DEBUG_VERSION
        SYSTEMCHECK(pDO==DataSetTemplate<SpecificDataSet>::m_pDOContainer);
#endif
        qUnsignedInt32 uiNbRecipient = 0;
        if (DataSetTemplate<SpecificDataSet>::m_oBreak.UpdatedModel())
        {
            UpdateLoopWithUpdatedModel(pDO, byIndex, tTimestamp);
        }
        else
        {
            uiNbRecipient = InnerUpdateLoop(pDO, byIndex, tTimestamp);
        }
        return uiNbRecipient;
    }


    //---------------------------------------------------------------------------------------
    Message* BuildUpdateMessage(DuplicatedObject *pDO, qByte byIndex, Time tTimestamp)
    {
        // Build the Update Message
        Message *pMsg;
        qUnsignedInt32 uiID=pDO->GetHandle();
        qByte byZero=0;
        pMsg=ObjDupProtocol::GetInstance()->CreateUpdateMessage(&uiID,&byIndex);

        DataSetTemplate<SpecificDataSet>::AddSourceTo(pMsg, tTimestamp, false);
        // Since there might eventually be more than one dataset per message,
        // we end with a zero to indicate the end.
        *pMsg << byZero;
        return pMsg;
    }

    // Updated API
    //{secret}
    void OperationBegin(DOOperation* pOperation)
    {
        DataSetTemplate<SpecificDataSet>::OperationBegin(pOperation);
        UpdateContextMap::RemoveCorrespondingContextes(pOperation);
    }


    //---------------------------------------------------------------------------------------
    qBool IndicateContinuityBreak(qByte byBreak, qBool bReliable = true)
    {
        EDSCHECK_ROLE_IS_DM(false);
        DataSetTemplate<SpecificDataSet>::m_oBreak.SetReliable(bReliable);
        return DataSetTemplate<SpecificDataSet>::m_oBreak.Set(byBreak);
    }

    //---------------------------------------------------------------------------------------
    qBool SetModel(PHBDRVAR(qDouble)& v, qDouble dValue, qDouble dRateOfChange, qDouble dAcceleration,
                   Time tTimestamp=DataSet::GetTimestamp())
    {
        EDSCHECK_ROLE_IS_DM(false);
        v.SetModel(dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    //---------------------------------------------------------------------------------------
    qBool SetModel(PHBDRVAR(qInt32)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                   Time tTimestamp=DataSet::GetTimestamp())
    {
        EDSCHECK_ROLE_IS_DM(false);
        v.SetModel(dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    //---------------------------------------------------------------------------------------
    qBool SetModel(PHBDRVAR(qInt16)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                   Time tTimestamp=DataSet::GetTimestamp())
    {
        EDSCHECK_ROLE_IS_DM(false);
        v.SetModel(dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }
    //---------------------------------------------------------------------------------------
    qBool SetModel(PHBDRVAR(qInt8)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                   Time tTimestamp=DataSet::GetTimestamp())
    {
        EDSCHECK_ROLE_IS_DM(false);
        v.SetModel(dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    //---------------------------------------------------------------------------------------
    qBool SetModel(PHBDRVAR(qFloat)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                   Time tTimestamp=DataSet::GetTimestamp())
    {
        EDSCHECK_ROLE_IS_DM(false);
        v.SetModel(dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    //---------------------------------------------------------------------------------------
    qBool SetLocalCorrection(PHBDRVAR(qDouble)& v, qDouble dValue, qDouble dRateOfChange, qDouble dAcceleration,
                             Time tTimestamp=DataSet::GetTimestamp())
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        EDSCHECK_ROLE_IS_DUPLICA(false);
        v.SetLocalCorrection(DataSetTemplate<SpecificDataSet>::s_oPHBDRParameters, dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    //---------------------------------------------------------------------------------------
    qBool SetLocalCorrection(PHBDRVAR(qInt32)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                             Time tTimestamp=DataSet::GetTimestamp())
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        EDSCHECK_ROLE_IS_DUPLICA(false);
        v.SetLocalCorrection(DataSetTemplate<SpecificDataSet>::s_oPHBDRParameters, dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    //---------------------------------------------------------------------------------------
    qBool SetLocalCorrection(PHBDRVAR(qInt16)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                             Time tTimestamp=DataSet::GetTimestamp())
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        EDSCHECK_ROLE_IS_DUPLICA(false);
        v.SetLocalCorrection(DataSetTemplate<SpecificDataSet>::s_oPHBDRParameters, dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }
    //---------------------------------------------------------------------------------------
    qBool SetLocalCorrection(PHBDRVAR(qInt8)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                             Time tTimestamp=DataSet::GetTimestamp())
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        EDSCHECK_ROLE_IS_DUPLICA(false);
        v.SetLocalCorrection(DataSetTemplate<SpecificDataSet>::s_oPHBDRParameters, dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    //---------------------------------------------------------------------------------------
    qBool SetLocalCorrection(PHBDRVAR(qFloat)& v, qFloat dValue, qFloat dRateOfChange, qFloat dAcceleration,
                             Time tTimestamp=DataSet::GetTimestamp())
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        EDSCHECK_ROLE_IS_DUPLICA(false);
        v.SetLocalCorrection(DataSetTemplate<SpecificDataSet>::s_oPHBDRParameters, dValue, dRateOfChange, dAcceleration, tTimestamp);
        return true;
    }

    IMPLEMENTPREDICTIONS(Value);
    IMPLEMENTPREDICTIONS(RateOfChange);
    IMPLEMENTPREDICTIONS(Acceleration);
};


#define _DS_DECLARE_ERRORTOLERANCE() \
        static ErrorToleranceFunction s_erFunction; \
        static ErrorToleranceFunction* GetErrorToleranceFunction() {return &s_erFunction;}

#define _DS_IMPLEMENT_ERRORTOLERANCE(DS) ErrorToleranceFunction DATASET(DS)::s_erFunction;

#define _DS_DECLARE_PHBDRPARAMETERS()           \
        static PHBDRParameters s_oPHBDRParameters;  \
        static PHBDRParameters* GetPHBDRParameters() {return &s_oPHBDRParameters;}  \
        Buffer* GetPHBDRModel() {return &m_dsBuffer;}

#define _DS_DECLARE_MAXUPDATEDELAY()            \
        static TimeInterval s_tiMaximumUpdateDelay; \
        static void SetMaximumUpdateDelay(TimeInterval tiMaximumUpdateDelay) {s_tiMaximumUpdateDelay=tiMaximumUpdateDelay;} \
        static TimeInterval GetMaximumUpdateDelay() {return s_tiMaximumUpdateDelay;}

#define _DS_DECLARE_MINUPDATEDELAY()            \
        static TimeInterval s_tiMinimumUpdateDelay; \
        static void SetMinimumUpdateDelay(TimeInterval tiMinimumUpdateDelay) {s_tiMinimumUpdateDelay=tiMinimumUpdateDelay;} \
        static TimeInterval GetMinimumUpdateDelay() {return s_tiMinimumUpdateDelay;}

#define _DS_IMPLEMENT_MAXUPDATEDELAY(DS)    \
        TimeInterval DATASET(DS)::s_tiMaximumUpdateDelay=DEFAULT_MAXIMUM_UPDATE_DELAY;

#define _DS_IMPLEMENT_MINUPDATEDELAY(DS)    \
        TimeInterval DATASET(DS)::s_tiMinimumUpdateDelay=DEFAULT_MINIMUM_UPDATE_DELAY;


#define _DS_IMPLEMENT_PHBDRPARAMETERS(DS)   \
        PHBDRParameters DATASET(DS)::s_oPHBDRParameters;

#define _DS_DECLARE_CONTINUITY_BREAKS()     \
        ContinuityBreak m_oBreak;

END_NAMESPACE_NEX

