﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CommandLineParser.h>

BEGIN_NAMESPACE_NEX

class OSDK_API StdCmdLine : public RootObject
{
public:
    StdCmdLine();

    virtual ~StdCmdLine();

    enum Options
    {
        Create,
        Join,
        Enumerate,
        Help
    };


    // Returns one of the Options selected by the user.
    Options GetOption();

    // Analyses arguments passed to the function and fills the parser FLAGS
    // to initialise the session. Valid arguments are {'IP Address'} OR
    // {'-create [name]'}. After this call is made,
    // the user can use the functions to obtain the system's state.
    // Parameters :
    //      qInt iArgc  : The number of arguments in szArgv.
    //      qChar**     : A list of all arguments passed to the
    //                    parser.
    // Returns true if the command line is correctly formed.
    qBool Parse( qInt iArgc, qChar** szArgv );

    qBool Parse( qChar *szCmdLine );

    // Returns true if the arguments at the command line are well formed.
    qBool IsWellFormed();

    // Returns the name of the session as a string or NULL if no session name was specified.
    const qChar* GetSessionName();

    // Returns the IP address of the session as a string or NULL if no IP address was specified.
    const qChar* GetIPAddress();

private:

    CommandLineParser m_oParser;

};

END_NAMESPACE_NEX

