﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/qMultiMap.h>
#include <OnlineCore/src/Core/Core.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Core/MethodCallJob.h>
#include <OnlineCore/src/Core/SystemComponent.h>
#include <OnlineCore/src/Core/SystemSetting.h>
#include <OnlineCore/src/Core/CallContextRegister.h>


BEGIN_NAMESPACE_NEX

class CallContext;
class PeriodicJob;
class EndPoint;

class CallContextRegister: public SystemComponent
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(CallContextRegister,SystemComponent);

private:
    friend class Core;

    CallContextRegister();
    virtual ~CallContextRegister();

public:

    CallContext *GetCallContext(qUnsignedInt32 uiID);

    static CallContextRegister* GetInstance()
    {
        return Core::GetInstance()->GetCallContextRegister();
    }

    void CancelCallContexts(EndPoint* pConnection, ConnectionID idConnection, const qResult& result);
    // n2222 CallContextを関連づけるための関数を追加。関連づけたCallContextはまとめてキャンセルできます。
    void AssociateCallContext(qUnsignedInt32 parentCallContextId, qUnsignedInt32 childCallContextId);
    void GetAssociatedId(qUnsignedInt32 parentCallContextId, qVector<qUnsignedInt32>* associatedIds);

private:
    friend class CallContext;

    // {secret}
    void Stop();
    // {secret}
    void Start();

    virtual qBool BeginInitialization();
    virtual qBool BeginTermination();

    qUnsignedInt32 Register(CallContext *pContext);
    void Unregister(qUnsignedInt32 uiID);
    void CheckExpiredCalls(qInt iDummy);
    void CancelExpiredCalls();

public:

    void Trace(qUnsignedInt64 uiTraceFlags);

private:
    typedef qMap<qUnsignedInt32, CallContext *> ContextMap;
    typedef qMultiMap<qUnsignedInt32, qUnsignedInt32> AssociatedContextMultiMap;

    ContextMap m_mapCallContextes;
    AssociatedContextMultiMap m_associatedCallContextes;
    qUnsignedInt32 m_uiNextID;
    MethodCallJob<CallContextRegister, qInt, PeriodicJob>   *m_pCheckExpiredCallsJob;
    qVector<qUnsignedInt32> m_tempAllChildTimeoutedContexts;    // CancelExpiredCalls()用の一時的なvector。関数内で毎回生成するとメモリ確保が発生するためメンバに持つ。
    static SystemSetting    s_ssCheckExpiredCallsPeriod;
};

END_NAMESPACE_NEX

