﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>

BEGIN_NAMESPACE_NEX

// No specific protection.  The specific class is responsible to
// protect its members.

// This is an abstract CallPolicy.  CallPolicy are used to implement
// specific restrictions and checks on methods calls.  For instance
// (the only example at the moment), there is a SingleThreadCallPolicy
// that is used to enforce that a given method is never executed by
// more than one thread.

class CallPolicy: public RootObject
{
public:
    CallPolicy() {}
    virtual ~CallPolicy() {}

    // ProDG must see the inline declarations before their
    // use.
    // コール(CallPolicyの対象物)が許可されればtrueを、許可されない場合はfalseを返す。
    // trueが返された場合はコール終了後にEndCall()を呼び出さなければならない。
    // falseを返した場合はEndCall()を呼んではいけない。
    qBool BeginCall()
    {
        return BeginCallImpl();
    }
    void EndCall()
    {
        EndCallImpl();
    }

    class CallScope
    {
    public:
        CallScope(CallPolicy& refPolicy)
            : m_refPolicy(refPolicy)
            , m_canProceed(false)
        {
            m_canProceed = m_refPolicy.BeginCall();
        }
        ~CallScope()
        {
            if (CanProceed())
            {
                m_refPolicy.EndCall();
            }
        }
        qBool CanProceed() const
        {
            return m_canProceed;
        }
    private:
        CallScope(const CallScope&);
        CallScope& operator=(const CallScope&);
    private:
        CallPolicy& m_refPolicy;
        qBool m_canProceed;
    };

    virtual qUnsignedInt32 GetCurrentRecursionLevel() const=0;
    virtual qBool CurrentThreadIsCalling() const=0;
    virtual qBool OtherThreadIsCalling() const=0;
private:

    virtual qBool BeginCallImpl()=0;
    virtual void EndCallImpl()=0;
};

END_NAMESPACE_NEX

