﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>


BEGIN_NAMESPACE_NEX

//Group=Template Classes

/*
Description: This is the root class for all callbacks.
Remarks: This is the root class for all callbacks. User-defined callbacks should be defined via the
Callback class.
See Also: Callback
*/

/*!
 @brief    非同期処理終了時のコールバックのルートクラスです。

 ユーザー定義コールバックのルートクラスです。@ref CallContext::RegisterCompletionCallback() で、
 @ref CallbackRoot 継承クラスを登録すると、非同期処理完了時にCallbackRoot::CallObjectMethod()が
 呼び出された後、オブジェクトを破棄します。

 @see      CallContext
 */
class CallbackRoot: public RootObject
{
public:
    //group=Methods

    CallbackRoot() {}
    virtual ~CallbackRoot() {}

    /*
    Description: Calls the callback method of this CallbackRoot object.
    See Also: Callback
    */
    virtual void Call()
    {
        CallObjectMethod();
    }

private:
    /*!
     @brief    非同期処理終了時に呼び出されるコールバック関数の実体です。

     本関数をオーバーロードしてください。
     */
    virtual void CallObjectMethod()=0;
};

END_NAMESPACE_NEX

