﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Checksum/Interface/ChecksumDecl.h>
#include <OnlineCore/src/Core/Checksum/Interface/KeyedChecksumAlgorithm.h>

BEGIN_NAMESPACE_NEX
class Buffer;
class MD5;

//Group=Checksum Algorithms

/*
Description: Computes a checksum via the HMAC algorithm.
Remarks: HMAC is an algorithm that provides message authentication in which an
iterative cryptographic hash function is used with a secret key. For a HMAC, the
Message Authentication Code (MAC) algorithm is based on a hash function. In
$PRODUCTNAME$, the RSA MD5 algorithm is used as the hashing algorithm.
*/
class HMACChecksum: public KeyedChecksumAlgorithm
{
public:
    //Group=Methods

    HMACChecksum();
    virtual ~HMACChecksum();

    // Summary: Calculates the checksum using the HMAC algorithm.
    // Remarks: The $PRODUCTNAME$ HMAC implementation uses RSA MD5 as the hashing algorithm.
    virtual qBool ComputeChecksum(const Buffer &oBuffer, Buffer *pChecksum);

    virtual qBool ComputeChecksum(const qByte** pBuffer, const size_t nSize[], size_t length, SignatureBytes &signature);

    // {secret}
    virtual qUnsignedInt32 ComputeChecksumForTransportArray( const qByte** pBuffer, const size_t nSize[], size_t length);

    //Summary: Returns the checksum length.
    //Returns: The checksum length in bytes. The default value is 16-bits however,
    //it can be reduced via ChecksumAlgorithm::ReduceChecksumLength.
    //See Also: ChecksumAlgorithm::ReduceChecksumLength
    virtual qByte GetChecksumLength();

    //{secret}
    virtual void KeyHasChanged();
private:
    //{secret}
    unsigned char m_kipad[65]; /* inner padding */
    //{secret}
    unsigned char m_kopad[65]; /* outer padding */

    void ChecksumComputeHelper( const qByte** pBuffer, const size_t nSize[], size_t length, MD5 &oOuter);
};

END_NAMESPACE_NEX


