﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Encryption/Interface/EncryptionDecl.h>
#include <OnlineCore/src/Core/Encryption/Interface/EncryptionAlgorithm.h>

BEGIN_NAMESPACE_NEX
class CriticalSection;
class Buffer;

//group=Encryption

/*
Description: This class implements Net-Z's RC4 encryption algorithm.
Remarks: For details of this well-known encryption algorithm, refer to any good
encryption book.
*/
class RC4Encryption: public EncryptionAlgorithm
{
public:
    //group=Methods

    RC4Encryption();
    virtual ~RC4Encryption();

    //{secret}
    virtual void KeyHasChanged();

    //{secret}
    virtual qUnsignedInt32 Encrypt(const Buffer &oInBuffer, Buffer *pOutBuffer);
    //{secret}
    virtual qUnsignedInt32 Encrypt(Buffer *pInOutBuffer);

    //{secret}
    virtual qUnsignedInt32 Decrypt(const Buffer &oInBuffer, Buffer *pOutBuffer);
    //{secret}
    virtual qUnsignedInt32 Decrypt(Buffer *pInOutBuffer);

    //{secret}
    void Encrypt(qByte* pInData, size_t uiInSize);
    //{secret}
    void Decrypt(qByte* pInData, size_t uiInSize);

    //{secret}
    static Key GetDefaultKey();

#if NEX_ENABLE_USE_SECURE_KEY
    void SetReinitEverytime(qBool bReinitEverytime);
#endif

private:
    void EncryptDecrypt(qByte* pInData, size_t uiInSize);
    inline void SwapByte(qByte* a, qByte* b);
    void PrepareEncryption();
    void ReinitStateArray();

    qByte m_bState[256];
    qByte m_bStateCopy[256];
    qByte m_bKeyX;
    qByte m_bKeyY;


#if NEX_ENABLE_USE_SECURE_KEY
    qBool m_bReinitEverytime;
#endif

    CriticalSection *m_pCriticalSection;
};

END_NAMESPACE_NEX


