﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/DebugString.h>
#include <OnlineCore/src/Core/Job.h>

BEGIN_NAMESPACE_NEX

class PeriodicJob: public Job
{
    // Group=
public:
    PeriodicJob(JobType type, const DebugString& strPeriodicJobDescription);
    virtual ~PeriodicJob() {}

    TimeInterval GetPeriod() const
    {
        return m_tiPeriod;
    }
    void SetPeriod(TimeInterval tiPeriod)
    {
        m_tiPeriod=tiPeriod;
    }
    void SetFrequency(qUnsignedInt uiFrequency)
    {
        SetPeriod(1000/uiFrequency);
    }

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif

    virtual qBool SkipWaitDelayAtTermination();

    // Cancel the job now (never excute anymore).  Only valid if
    // the job is NOT executing
    void Cancel()
    {
        SetToCancel();
    }
    // Cancel the job at the next run (will execute one more, and
    // then stop).  Valid for any
    // state of the job.
    void CancelNext()
    {
        SetPeriod(0);
    }

    virtual void SetDefaultPostExecutionState();

private:
    TimeInterval m_tiPeriod;
};

END_NAMESPACE_NEX

