﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/PluginsDecl.h>

//Group=Transport Protocol


BEGIN_NAMESPACE_NEX

enum
{
//Summary: The plug-in is a transport plug-in.
    TRANSPORT_PLUGIN_TYPE=   0x1,
//Summary: The plug-in is a compression plug-in.
    COMPRESSION_PLUGIN_TYPE= 0x2,
//Summary: The plug-in is an encryption plug-in.
    ENCRYPTION_PLUGIN_TYPE=  0x3,
//Summary: The plug-in is a checksum plug-in.
    CHECKSUM_PLUGIN_TYPE=    0x4,
//Summary: The plug-in is an authentication plug-in.
    AUTHENTICATION_PLUGIN_TYPE=  0x5
};


class PluginObject;

//Group=Transport Protocol

//Description: Retrieves details regarding a particular plug-in.
//Remarks: This class is used to retrieve details regarding a particular plug-in
class Plugin: public RootObject
{

    //Group=Methods
public:
    Plugin();
    virtual ~Plugin();

    //Description: Returns the Global Universal ID of a given plug-in.
    //Returns: The Global Universal ID (GUID) of the plug-in.
    virtual const qChar           *GetGUID()=0;

    //Description: Returns the name of a given plug-in.
    //Returns: The name of the plug-in.
    virtual const qChar     *GetName()=0;

    //Description: Returns the author of a given plug-in.
    //Returns: The author of the plug-in.
    virtual const qChar     *GetAuthor()=0;

    //Description: Returns the version number of a given plug-in.
    //Returns: The version number of the plug-in.
    virtual qUnsignedInt32  GetVersion()=0;

    //{secret}
    virtual qBool           IsCertified()=0;

    //Description: Returns the type of a given plug-in.
    //Returns: The type of plug-in.
    //Remarks: The type of plug-in may be one of TRANSPORT_PLUGIN_TYPE, COMPRESSION_PLUGIN_TYPE, or
    //ENCRYPTION_PLUGIN_TYPE.
    virtual qUnsignedInt32  GetType()=0;


    //{secret}
    //  static Plugin *Load(qChar *szPluginFilename);
    //{secret}
    void Unload();

    //{secret}
    virtual qBool Activate()=0;
    //{secret}
    virtual void Release()=0;

#if 0
    //{secret}
    virtual qUnsignedInt32  GetStringFormat();
    //{secret}
    virtual qUnsignedInt32  GetTargetInfo();
#endif
    //{secret}
    virtual qBool           Initialize();
    //{secret}
    virtual void            ThreadAttach();
    //{secret}
    virtual void            ThreadDetach();
    //{secret}
    virtual void            Destroy();

    //{secret}
    void                    Trace(qUnsignedInt64 uiTraceFlags);

    //{secret}
    static Plugin   *GetInstance();
    //{secret}
    void                    SetLibrary(void *hLibrary);

    //{secret}
    virtual PluginObject            *CreatePluginObject()=0;
    //{secret}
    virtual void            DeletePluginObject(PluginObject *pPlugin)=0;

protected:
    static Plugin   *s_pInstance;
    void                    *m_hLibrary;
};

END_NAMESPACE_NEX

