﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/ThreadVariable.h>
#include <OnlineCore/src/Core/InstantiationContext.h>
#include <OnlineCore/src/Core/InstanceControl.h>


BEGIN_NAMESPACE_NEX

class PseudoSingleton : public InstanceControl
{
public:
    PseudoSingleton(ICType icInstanceType);
    virtual ~PseudoSingleton();

    class Scope
    {
    public:
        Scope(ICContext icNew)
        {
            m_icOld = GetCurrentContext();
            SetContextIfRequired(icNew);
        }
        ~Scope()
        {
            SetContextIfRequired(m_icOld);
        }

    private:
        ICContext m_icOld;
    };

    static qBool SetContext(ICContext icNewContext);
    static void SetContextIfRequired(ICContext icNewContext);

    static ICContext GetCurrentContext()
    {
#ifndef NEX_ENABLE_PSEUDOSINGLETON_OPTIMIZATION
        return s_bUseInstantiationContext?ICContext(*st_pIcContext):DEFAULT_CONTEXT;
#else
        return DEFAULT_CONTEXT;
#endif
    }

    static qBool UseInstantiationContext(size_t uiNbExtraContexts);
    static qBool UseNoInstantiationContext();
    static qBool UsingInstantiationContext();

private:

#ifndef NEX_ENABLE_PSEUDOSINGLETON_OPTIMIZATION

    static ThreadVariable<ICContext>* st_pIcContext;

#endif
    static qBool s_bUseInstantiationContext;
};

// For the TraceOutput.
// {secret}
extern qUnsignedInt32 GetCurrentContext();


END_NAMESPACE_NEX


