﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/ThreadVariable.h>

BEGIN_NAMESPACE_NEX

class SecurityContext: public RootObject
{
public:
    // moved here for inlining purposes.
    PrincipalID  GetPID() const
    {
        return m_pid;
    }
    void SetPID(PrincipalID pid)
    {
        m_pid=pid;
    }
    ConnectionID GetCID() const
    {
        return m_cid;
    }
    void SetCID(ConnectionID cid)
    {
        m_cid=cid;
    }

    qUnsignedInt32 GetPublicAddress() const
    {
        return m_publicAddress;
    }
    void SetPublicAddress(qUnsignedInt32 publicAddress)
    {
        m_publicAddress=publicAddress;
    }


    SecurityContext(PrincipalID pid=INVALID_PRINCIPALID, ConnectionID cid=INVALID_CONNECTIONID, qUnsignedInt32 publicAddress=0): m_pid(pid), m_cid(cid), m_publicAddress(publicAddress) {}
    SecurityContext(const SecurityContext& oOther):RootObject(), m_pid(oOther.m_pid), m_cid(oOther.m_cid), m_publicAddress(oOther.m_publicAddress) {}
    virtual ~SecurityContext() {}

    SecurityContext& operator=(const SecurityContext& oOther)
    {
        m_pid=oOther.m_pid;
        m_cid=oOther.m_cid;
        m_publicAddress=oOther.m_publicAddress;
        return *this;
    }


private:
    PrincipalID m_pid;
    ConnectionID m_cid;
    qUnsignedInt32 m_publicAddress;
};

END_NAMESPACE_NEX


