﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Core/SecurityContext.h>
#include <OnlineCore/src/Platform/Core/ThreadVariable.h>
#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

class SecurityContextManager: public RootObject
{
public:
    class Scope
    {
    public:
        Scope(PrincipalID pid, ConnectionID cid, qUnsignedInt32 publicAddress)
        {
            SecurityContextManager::GetInstance()->Push(pid,cid,publicAddress);
        }
        ~Scope()
        {
            SecurityContextManager::GetInstance()->Pop();
        }
    };

    SecurityContextManager();
    virtual ~SecurityContextManager();

    static SecurityContextManager* GetInstance();

    void Push(PrincipalID pid, ConnectionID cid, qUnsignedInt32 publicAddress=0);
    void Pop();

    PrincipalID  GetCurrentPID() const;
    ConnectionID GetCurrentCID() const;
    qUnsignedInt32 GetCurrentAddress() const;

    // TODO: remove these.
    static PrincipalID  StaticGetCurrentPID();
    static ConnectionID StaticGetCurrentCID();

private:
    ThreadVariable<qList<SecurityContext> > m_tvlstSecurityContextes;
};

END_NAMESPACE_NEX


