﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/DebugString.h>
#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/Job.h>

BEGIN_NAMESPACE_NEX

// TRACEが無効のときはステップ名を保持しない
#if defined(NEX_TRACE_ENABLED) || defined(NEX_JOB_TRACE_INFO_ENABLED)
#define JOBSTEP(STEP) Step(static_cast<StepMethod>(&STEP),NEX_T(#STEP))
#else
#define JOBSTEP(STEP) Step(static_cast<StepMethod>(&STEP),NEX_T(""))
#endif

class CallbackRoot;
class CallContext;
class SystemComponent;

// Group=Core Subsystem
/*!
    @brief 非同期に一連のステップを処理するクラスです。本クラスを直接使用しないで下さい。
*/
class StepSequenceJob: public Job
{
public:
    typedef void (StepSequenceJob::*StepMethod)();

    class Step: public RootObject
    {
    public:
        Step(): m_pStepMethod(NULL), m_szStepDescription(NULL) {}
        Step(void (StepSequenceJob::*pStepMethod)(), const qChar* szStepDescription):
            m_pStepMethod(pStepMethod), m_szStepDescription(szStepDescription) {}
        virtual ~Step() {};

        // Default copy constructor and assignement operators
        // are fine.

        const qChar* GetDescription() const
        {
            return m_szStepDescription;
        }

        void Execute(StepSequenceJob* pJob)
        {
            (pJob->*m_pStepMethod)();
        }

        //n1769
        bool SanityCheck(void) const;
        void Dump(void) const;

    private:
        void (StepSequenceJob::*m_pStepMethod)();
        const qChar* m_szStepDescription;
    };

    StepSequenceJob(JobType type, const DebugString& strJobDescription);
    virtual ~StepSequenceJob();

    void SetTraceFlags(qUnsignedInt64 uiTraceFlags)
    {
        m_uiTraceFlags = uiTraceFlags;
    }
    qUnsignedInt64 GetTraceFlags()
    {
        return m_uiTraceFlags;
    }

    CallbackRoot* CreateResumeCompletionCallback(Step* pcbNextStep);

    //WARNING: Make sure that Job state is Suspended as soon as you leave
    //         the step afterwards
    //Ex.:
    //      SetToSuspended();
    //      ResumeOnCallCompletion(&m_oJobLoginCC, qNew JOBSTEP(JobBackEndServicesLogin::ProcessAuthConnectionResult));
    //
    void ResumeOnCallCompletion(CallContext* pContext, Step* pcbNextStep);
    void ResumeOnCallCompletionOrAlreadyDone(CallContext* pContext, Step* pcbNextStep);

    void SetStep(const Step& oStep);
public:
    virtual void CheckExceptions() {};

    virtual String GetTraceInfo() const;

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif

    TimeInterval GetStepDuration() const
    {
        return TimeInterval(Time::GetTime()-m_tStepStart);
    }
    const qChar* GetStepDescription() const
    {
        return m_oNextStep.GetDescription();
    }

    virtual void Execute();

    static void SetJobStepExecutionBusyIntervalThreshold(LargeTimeInterval interval)
    {
        s_JobStepExecutionBusyIntervalThreshold = interval;
    }

protected:

    // Util method, to make it easier to Terminate a component.
    void TerminateComponent(SystemComponent* pComponent, const Step& oNextStep);

private:
    void ProcessCallResult(Step *pcbNextStep);
    virtual void AddActivity(const qChar* szSupplement);
    void AddStepActivity();

protected:
    // Used directly by the JobTerminateDOCore for now.
    Time m_tStepStart;

private:
    qUnsignedInt32 m_uiNbStepExecutions;
    Step m_oNextStep;
    qUnsignedInt64 m_uiTraceFlags;
    static LargeTimeInterval s_JobStepExecutionBusyIntervalThreshold;
};

END_NAMESPACE_NEX

