﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/CallPolicy.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>

BEGIN_NAMESPACE_NEX

// 二つ以上のスレッドからコールされた場合に続行させない(CallScope::CanProceed()がfalseを返す)ポリシー
// 再入は不可。
class ThreadSafeCallPolicy : public CallPolicy, public NonCopyable
{
public:
    ThreadSafeCallPolicy()
        : CallPolicy()
        , m_cs()
        , m_uiCurrentRecursionLevel(0)
        , m_uiMaximumRecursionLevel(1)
        , m_uiCurrentThread(NEX_INVALID_THREAD_ID)
    {
    }
    virtual ~ThreadSafeCallPolicy() {}

    void SetMaximumRecursionLevel(qUnsignedInt32 uiMaximumRecursionLevel)
    {
        m_uiMaximumRecursionLevel = uiMaximumRecursionLevel;
    }

    qUnsignedInt32 GetCurrentRecursionLevel() const
    {
        return m_uiCurrentRecursionLevel;
    }

    virtual qBool CurrentThreadIsCalling() const;
    virtual qBool OtherThreadIsCalling() const;

private:
    virtual qBool BeginCallImpl();
    virtual void EndCallImpl();

private:
    mutable CriticalSection m_cs;
    qUnsignedInt32 m_uiCurrentRecursionLevel;
    qUnsignedInt32 m_uiMaximumRecursionLevel;
    ThreadID m_uiCurrentThread;
};

END_NAMESPACE_NEX
