﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>

BEGIN_NAMESPACE_NEX

// メモリ確保をしないURIパーサです。メモリ確保しても良い場合はHTTPURLが便利です。
// <scheme>::/[<username>[:<password>]@]<host>[:<port>][/<path>][;<parameters>][?<query>][#<fragment>]]
//
// 制限事項
// ・pathより後の細かい取得は未実装
// ・pathの先頭の/は必須。
class UriParserNoAlloc : public RootObject
{
public:
    UriParserNoAlloc()
        : m_schemeBegin(NULL)
        , m_schemeEnd(NULL)
        , m_userNameBegin(NULL)
        , m_userNameEnd(NULL)
        , m_passwordBegin(NULL)
        , m_passwordEnd(NULL)
        , m_hostBegin(NULL)
        , m_hostEnd(NULL)
        , m_portBegin(NULL)
        , m_portEnd(NULL)
        , m_pathBegin(NULL)
    {
    }

    // URLをパースする
    // 指定したuriを呼び出し元で保持している間GetSchemeBegin()などを呼び出せます。
    // pathもしくはhostが存在しない場合はfalseを返す。
    qBool Parse(const qChar* uri);

    void Reset()
    {
        m_schemeBegin = NULL;
        m_schemeEnd = NULL;
        m_userNameBegin = NULL;
        m_userNameEnd = NULL;
        m_passwordBegin = NULL;
        m_passwordEnd = NULL;
        m_hostBegin = NULL;
        m_hostEnd = NULL;
        m_portBegin = NULL;
        m_portEnd = NULL;
        m_pathBegin = NULL;
    }

    const qChar* GetSchemeBegin() const { return m_schemeBegin; }
    const qChar* GetSchemeEnd() const { return m_schemeEnd; }
    const qChar* GetUserNameBegin() const { return m_userNameBegin; }
    const qChar* GetUserNameEnd() const { return m_userNameEnd; }
    const qChar* GetPasswordBegin() const { return m_passwordBegin; }
    const qChar* GetPasswordEnd() const { return m_passwordEnd; }
    const qChar* GetHostBegin() const { return m_hostBegin; }
    const qChar* GetHostEnd() const { return m_hostEnd; }
    const qChar* GetPortBegin() const { return m_portBegin; }
    const qChar* GetPortEnd() const { return m_portEnd; }
    const qChar* GetPathBegin() const { return m_pathBegin; }

    size_t GetSchemeLength() const { return m_schemeEnd - m_schemeBegin; }
    size_t GetUserNameLength() const { return m_userNameEnd - m_userNameBegin; }
    size_t GetPasswordLength() const { return m_passwordEnd - m_passwordBegin; }
    size_t GetHostLength() const { return m_hostEnd - m_hostBegin; }
    size_t GetHostAndPortLength() const { return m_pathBegin - m_hostBegin; }
    size_t GetPortLength() const { return m_portEnd - m_portBegin; }
private:
    const qChar* m_schemeBegin;
    const qChar* m_schemeEnd;
    const qChar* m_userNameBegin;
    const qChar* m_userNameEnd;
    const qChar* m_passwordBegin;
    const qChar* m_passwordEnd;
    const qChar* m_hostBegin;
    const qChar* m_hostEnd;
    const qChar* m_portBegin;
    const qChar* m_portEnd;
    const qChar* m_pathBegin;
};

END_NAMESPACE_NEX
