﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <functional>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

// バインドする引数と関数を保持するクラス
template <typename TFunctor>
class BoundUnaryFunction : public std::unary_function<typename TFunctor::argument_type, typename TFunctor::result_type>
{
    typedef BoundUnaryFunction<TFunctor> SelfType;

    TFunctor functor;
    typename SelfType::argument_type arg;

public:
    BoundUnaryFunction(const TFunctor& functor_, const typename SelfType::argument_type& arg_)
        : functor(functor_), arg(arg_)
    {
    }

    typename SelfType::result_type operator()()
    {
        return functor(arg);
    }
};

// std::unary_functionを継承した関数オブジェクトの第一引数をバインドする。
// std::bind1stはbinary_function(引数を二つ持つ関数)に対してしか適用できない。これを使うと引数を一つ持つ関数に対してバインドできる。
// 引用元 http://stackoverflow.com/questions/29080857/stl-stdbind1st-for-stdunary-function
template <typename TFunctor>
BoundUnaryFunction<TFunctor> BindUnaryFunction(const TFunctor& functor,
                                               const typename TFunctor::argument_type& arg)
{
    return BoundUnaryFunction<TFunctor>(functor, arg);
}

END_NAMESPACE_NEX
