﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/ParseTree.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

BEGIN_NAMESPACE_NEX

/**
    \ingroup group_onlinecore_ddl

    \brief This class defines some DDL generated methods.

    An instance of this class is generated by the DDL compiler for each application.
    For example, if your DDL is defined in the \c MyGame.ddl file, then the generated class is
    called MyGameDDLDeclarations.

    \sa DynamicRunTimeExtension (Net-Z class)
*/
class OSDK_API DDLDeclarations: public RootObject
{
public:
    DDLDeclarations(qBool bCoreDDLDeclarations=false);
    virtual ~DDLDeclarations();

    static void LoadAll();
    static void UnloadAll();
    static void ResetDOClassIDs();

protected:

    void RegisterIfRequired();
    void Unregister();

    static void UnregisterAll();

    void Load();
    void Unload();

    virtual void Init()=0;

    OSDK_DISABLE_EXPORT_WARNING(static PseudoGlobalVariable<qUnsignedInt32> s_uiBaseClassID);
    //static qUnsignedInt32 s_uiBaseClassID;

private:

    qInt m_iUseRefCount;
    qBool m_bRegistered;

    OSDK_DISABLE_EXPORT_WARNING(static PseudoGlobalVariable<DDLDeclarations*> s_pFirstDDLDecl);
    //static DDLDeclarations* s_pFirstDDLDecl;
    DDLDeclarations* m_pNextDDLDecl;

    qBool m_bCoreDDLDeclarations;
};

END_NAMESPACE_NEX

