﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/Declaration.h>
#include <OnlineCore/src/ParseTree/NameSpaceRef.h>
#include <OnlineCore/src/ParseTree/NameSpace.h>
#include <OnlineCore/src/ParseTree/PropertyDeclaration.h>

BEGIN_NAMESPACE_NEX

class OSDK_API DatasetDeclaration: public Declaration
{
public:
    DatasetDeclaration(const String& strName);
    virtual ~DatasetDeclaration();

    DECLARE_NAMESPACE_ITEM(DatasetDeclaration, Declaration);

    virtual void SetNameSpace(NameSpace *pNS);

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlag) const;
#endif

    qResult StreamIn(BinaryRepresentation *pStream) const;
    qResult StreamOut(BinaryRepresentation *pStream);

    virtual qBool IsEqual(const ParseTreeItem* pOther) const;

    virtual qBool ResolveReferences();

    virtual qBool MayHaveProperties() const
    {
        return true;
    }

    NameSpace& NS()
    {
        return m_nsDataset;
    }
    const NameSpace& NS() const
    {
        return m_nsDataset;
    }

    virtual void ForEachSubItem(void (*pf)(ParseTreeItem*))
    {
        Declaration::ForEachSubItem(pf);
        m_nsDataset.ForEachItem(pf);
    }

    qBool UsesExtrapolationFilter() const
    {
        return HasProperty(NEX_T("extrapolation_filter"));
    }
    qBool BufferedSet() const
    {
        return HasProperty(NEX_T("buffered"));
    }
    qBool LoopbackSet() const
    {
        return HasProperty(NEX_T("loopback"));
    }

    PropertyDeclaration* GetCustomVarProperty() const
    {
        return GetFirstPropertyByCategory(DSPROP_CUSTOM_VAR);
    }
    PropertyDeclaration* GetCustomDSDeclProperty() const
    {
        return GetFirstPropertyByCategory(DSPROP_CUSTOM_DSDECL);
    }

    void ApplyDefaultProperties();

private:
    NameSpace m_nsDataset;
};

END_NAMESPACE_NEX

