﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>

#define BEGIN_NS_ITERATION(decl,doclasstype) \
                                             { \
                                             NameSpace::Iterator    o##doclasstype##It; \
                                             doclasstype        *p##doclasstype; \
                                             for (o##doclasstype##It = (decl)->NS().Begin(); \
                                             o##doclasstype##It != (decl)->NS().End();  \
                                             ++o##doclasstype##It) { \
                                             p##doclasstype = doclasstype::DynamicCast(*o##doclasstype##It);\
                                             if( p##doclasstype == NULL ){\
                                             continue;\
                                             }

#define END_NS_ITERATION }}


BEGIN_NAMESPACE_NEX

class ParseTreeItem;
class NameSpaceItem;
class BinaryRepresentation;

class OSDK_API NameSpace: public RootObject
{
public:
    NameSpace();
    ~NameSpace();

    qBool ForEachItem(void (*pf)(ParseTreeItem*));
    qBool ForEachItem(qBool (*pf)(ParseTreeItem*));

    // TODO.  This involves exposing the STL iterators.
    typedef qList<NameSpaceItem*>::iterator Iterator;
    typedef qList<NameSpaceItem*>::const_iterator ConstIterator;

    Iterator Begin()
    {
        return m_lstItems.begin();
    }
    ConstIterator Begin() const
    {
        return m_lstItems.begin();
    }

    Iterator End()
    {
        return m_lstItems.end();
    }
    ConstIterator End() const
    {
        return m_lstItems.end();
    }

    void Clear();
    qBool AddItem(NameSpaceItem *pNewItem, qBool bUseCurrentRef=true);
    qBool RemoveItem(const String& strItemToDelete);
    NameSpaceItem* GetItem(const String& strItem);
    const NameSpaceItem* GetItem(const String& strItem) const;

#if defined(NEX_TRACE_ENABLED)
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;
#endif
    void CheckCoherence();

    qResult StreamIn(BinaryRepresentation *pStream, qBool (*pfDeclarationFilter)(NameSpaceItem*)=NULL) const;
    qResult StreamOut(BinaryRepresentation *pStream, qBool bExpandCurrentTree=false);

    // Operators...
    qBool operator==(const NameSpace& oOther) const;
    NameSpace& operator=(const NameSpace& oOther);

    void SetGlobalNS(NameSpace* pGlobalNS)
    {
        m_pGlobalNS=pGlobalNS;
    }
    NameSpace* GetGlobalNS()
    {
        return m_pGlobalNS;
    }

    qBool ResolveReferences();

    void Dump();

private:
    OSDK_DISABLE_EXPORT_WARNING(qList<NameSpaceItem*> m_lstItems);
    // Every namespace keeps a ptr to the global namespace.
    NameSpace* m_pGlobalNS;
};

END_NAMESPACE_NEX

