﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/NameSpace.h>
#include <OnlineCore/src/ParseTree/DOClassDeclaration.h>
#include <OnlineCore/src/ParseTree/DatasetDeclaration.h>
#include <OnlineCore/src/ParseTree/TypeDeclaration.h>
#include <OnlineCore/src/ParseTree/ProtocolDeclaration.h>

#include <OnlineCore/src/ParseTree/BinaryRepresentation.h>

BEGIN_NAMESPACE_NEX

class OSDK_API ParseTree: public RootObject
{
public:
    ParseTree();
    ~ParseTree();

    typedef BinaryRepresentation Stream;

    // Operators...
    qBool operator==(const ParseTree& oOther) const;
    qBool operator!=(const ParseTree& oOther) const
    {
        return !((*this)==oOther);
    }

#if defined(NEX_TRACE_ENABLED)
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;
#endif

    void DeclareBuiltInUnit(const String& strUnitName, const String& strUnitDir);

    NameSpace& NS()
    {
        return m_nsGlobal;
    }

    qResult StreamIn(BinaryRepresentation *pStream, qBool (*pfDeclarationFilter)(NameSpaceItem*)=NULL) const;
    qResult StreamOut(BinaryRepresentation *pStream, qBool bExpandCurrentTree=false, qBool bPerformVersionCheck=true);
    void CheckCoherence()
    {
        m_nsGlobal.CheckCoherence();
    }
#ifdef NEX_FILE_SUPPORT
    qResult StreamFromFile(const String &strFileName, qBool bExpandCurrentTree, qBool bPerformVersionCheck);
#endif

    void InitRootClass(const String& strRootClass);

    DDLUnitDeclaration* GetCurrentDDLUnit() const;
    void PushDDLUnit(DDLUnitDeclaration* pDDLUnit);
    void PopDDLUnit();

private:
    OSDK_DISABLE_EXPORT_WARNING(qList<DDLUnitDeclaration*> m_lstDDLUnits);

    NameSpace m_nsGlobal;
};

END_NAMESPACE_NEX

