﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/NameSpaceItem.h>
#include <OnlineCore/src/ParseTree/NameSpaceRef.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX

class Declaration;

class OSDK_API Variable: public NameSpaceItem
{
public:
    Variable(const String& strName, Declaration* pTypeDeclaration,
             qUnsignedInt uiArraySize=0);
    virtual ~Variable();

    DECLARE_NAMESPACE_ITEM(Variable, NameSpaceItem);

#ifdef NEX_TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;
#endif

    qResult StreamIn(BinaryRepresentation *pStream) const;
    qResult StreamOut(BinaryRepresentation *pStream);

    virtual qBool IsEqual(const ParseTreeItem* pOther) const;

    Declaration* GetTypeDeclaration() const;
    String GetTypeName(qBool bComplete=true) const;

    virtual qBool ResolveReferences();

    qBool IsAnArray() const;
    qUnsignedInt GetArraySize() const
    {
        return m_uiArraySize;
    }

    virtual String GetDeclarationString() const;

    qBool IsATemplate() const;

private:
    NameSpaceRef m_refTypeDeclaration;
    qUnsignedInt m_uiArraySize; // 0 if not an array.
};

END_NAMESPACE_NEX

