﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

/* NOTE: This file is read in two different contextes: Imakefile and CPP files. */
/* Regular CPP comments cannot be used in it */

#pragma once

#include <OnlineCore/src/Platform/Core/Recognition.h>

// GCCのバージョンを取得
#if defined(__GNUC__) && defined(UNIX_FAMILY) && !defined(__clang__)
#include <features.h>
#if __GNUC_PREREQ(4, 4)
#define NEX_GNUC_EQUAL_4_4_OR_HIGHER
#endif
#endif

#if !defined(NON_SUPPORT_B_TARGET) && !defined(NON_SUPPORT_A_TARGET)
#define NEX_CPP_EXCEPTION_SUPPORT
#endif

#if (defined(WIN32_TARGET) && !defined(NEX_SIGLO_FAMILY)) || defined(UNIX_FAMILY)
#define NEX_MULTIPROCESS_SUPPORT
#endif

/*
This is used when we need to write methods for all basic types, to
indicate if qChar16 should be done as well, or it overlaps with
one of the other basic types */

/*
*SigloNX
qChar -> char 組み込み型
qWildChar -> wchar_t 組み込み型(4バイト)
qChar16 -> char16_t 組み込み型

define NEX_QCHAR16_IS_NOT_WCHAR_T, NEX_QCHAR16_IS_NOT_UINT16_T, NEX_WCHAR_4BYTES

*VS2013までのSigloWin
qChar -> char 組み込み型
qWildChar -> wchar_t 組み込み型(2バイト)
qChar16 -> char16_t unsigned shortのtypedef

define NEX_QCHAR16_IS_NOT_WCHAR_T

*VS2015からののSigloWin
qChar -> char 組み込み型
qWildChar -> wchar_t 組み込み型(2バイト)
qChar16 -> char16_t  組み込み型(2バイト)

define NEX_QCHAR16_IS_NOT_WCHAR_T, NEX_QCHAR16_IS_NOT_UINT16_T


*普通のWin32/CTR/Cafe
qChar -> char 組み込み型
qWildChar -> wchar_t 組み込み型(2バイト)
qChar16 -> wchar_t 組み込み型(2バイト)

define NEX_QCHAR16_IS_NOT_UINT16_T

*Linux
qChar -> char 組み込み型
qWildChar -> wchar_t 組み込み型(4バイト)
qChar16 -> unsigned short

define NEX_QCHAR16_IS_NOT_WCHAR_T, NEX_WCHAR_4BYTES

 */

#if defined(NEX_SIGLO_FAMILY)
// qChar16 = char16_t
// ただし、NXとVS2015だと、char16_tが組み込み型になる。VS2013以下は、char16_t = uint16_t
#if defined(NEX_NX_TARGET) || (defined(_MSC_VER) && _MSC_VER >= 1900)
#define NEX_QCHAR16_IS_NOT_UINT16_T
#endif

#if defined(NEX_NX_TARGET)
#define NEX_WCHAR_4BYTES
#endif

#define NEX_QCHAR16_IS_NOT_WCHAR_T

#elif defined(WIN_FAMILY) || defined(CTR_UNICODE) || defined(CAFE_UNICODE)
// このターゲットでは、VS2015以降以外、 qChar16 = wchar_t VS2015以降は、qChar16 = char16_t
#define NEX_QCHAR16_IS_NOT_UINT16_T
#if defined(_MSC_VER) && _MSC_VER >= 1900
#define NEX_QCHAR16_IS_NOT_WCHAR_T
#endif

#elif defined(LINUX_TARGET)
// Linuxでは、qChar16 = uint16_t
#define NEX_WCHAR_4BYTES
#define NEX_QCHAR16_IS_NOT_WCHAR_T
#endif

// stdint.hをサポートしているかどうか
// CTRとCafeは非サポート。VS2010以下は非サポート
#if !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !(defined(_MSC_VER) && _MSC_VER <= 1600)
#define NEX_STDINT_SUPPORT
#endif

#if defined(WIN_FAMILY) || defined(NEX_SIGLO_FAMILY) || defined(CTR_UNICODE) || defined(UNIX_FAMILY) || defined(CAFE_UNICODE)
#define NEX_UNICODE_SUPPORT
#define NEX_SNPRINTF_SUPPORT
#endif

#if defined(WIN32_TARGET) && !defined(NEX_SIGLO_FAMILY)
#define NEX_CODEPAGE_SUPPORT
#endif

#if !defined(NEX_SIGLO_FAMILY) && !defined(NEX_USE_PRINCIPALID_64BIT)
#define NEX_GET_MII_SUPPORT
#endif


#if defined(WII_TARGET) || defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
#define NEX_SNPRINTF_SUPPORT
#endif

#if defined(WIN_FAMILY) && !defined(NON_SUPPORT_D_FAMILY)
#define NEX_REGISTRY_SUPPORT
#endif

#if defined(WIN32_TARGET) || defined(UNIX_FAMILY)
#define NEX_ENV_VARIABLE_SUPPORT
#endif

#if defined(WIN32_TARGET)
#define NEX_STACK_TRACER_SUPPORT
#endif

#if defined(WIN32_TARGET) || defined(UNIX_FAMILY)
#define NEX_FILE_SUPPORT
#define NEX_FILE_STREAM_SUPPORT     /* ofstream, ifstream, etc. */
#endif

#define NEX_LEAKTRACKER_SUPPORT

#if !defined(NON_SUPPORT_B_TARGET) && !defined(NON_SUPPORT_A_TARGET) && !defined(NON_SUPPORT_C_TARGET) && !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !defined(CAFEIOP_TARGET)
#define NEX_THREAD_SCRAMBLER_SUPPORT
#endif

#ifdef DYNAMIC_LIBRARY_VERSION
#define DYNAMIC_LIBRARY_SUPPORT
#endif

#ifdef FAST_CALL_VERSION
/* With the FastCall version of the Win32 library, we do not support
 * DLL's */
#undef DYNAMIC_LIBRARY_SUPPORT
#undef NEX_LEAKTRACKER_SUPPORT
#endif

#define NEX_NAMESPACE_SUPPORT
#define NEX_WATERMARK_SUPPORT

#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
#undef NEX_WATERMARK_SUPPORT
#endif

/* n2494 */
#if defined(CTR_SHIPPING_VERSION)
#undef NEX_WATERMARK_SUPPORT
#endif

#ifdef WIN32_TARGET
#define NEX_FP_EXCEPTION_SUPPORT    /* Floating-point support */
#endif

/* Configure NEX_ALLOCATOR_SUPPORT */

#define NEX_ALLOCATOR_SUPPORT

#if defined(NEX_ALLOCATOR_SUPPORT) && defined(__MWERKS__)
#define NEX_STD_TREE_PAIR_ALLOCATOR_SUPPORT
#endif

#if defined(NEX_ALLOCATOR_SUPPORT) && defined(MACOSX_TARGET)
#define NEX_STD_TREE_PAIR_ALLOCATOR_SUPPORT
#endif

#if defined(NEX_ALLOCATOR_SUPPORT) && (defined(CTR_TARGET) || defined(CAFE_TARGET) || defined(CAFEIOP_TARGET) || defined(NEX_NX_TARGET))
#define NEX_STD_TREE_PAIR_ALLOCATOR_SUPPORT
#endif


#ifdef WIN32_TARGET
#define NEX_PYTHON_SUPPORT
#endif

#if defined(WIN32_TARGET)
#define NEX_EXCEPTION_HANDLER_SUPPORT
#endif

#if defined(WIN32_TARGET) || defined(LINUX_TARGET) || defined(MACOSX_TARGET)
#define NEX_TARGET_IS_HOST
#endif

/* Configure NEX_SERVICE_HOST_SUPPORT */

#if defined(WIN32_TARGET) && defined(NEX_ANSI_VERSION)
#define NEX_SERVICE_HOST_SUPPORT
#endif

#if defined(LINUX_TARGET)
#define NEX_SERVICE_HOST_SUPPORT
#endif

#ifdef CRYSTALD_BUILD
#undef NEX_SERVICE_HOST_SUPPORT
#endif

#if defined(VS2005_COMPILER)
/* We will not support 2005 for the backend */
#undef NEX_SERVICE_HOST_SUPPORT
#endif

/*
 TODO.  This macro is still used in the code.  The code portions it
 controls should probably be deleted or fixed and enabled [bug 5563]
 This variable controls if config files can be used to read the license keys
 (eg. for the ddl compiler).

 FILE_VARIABLE_SUPPORT

*/

/*
  Thread Capabilities - Enables setting the thread affinity of
  $PRODUCTNAME$ processes.  If the list of target platforms changes, we must change the
  documentation in the Core::SetDefaultThreadAffinity method
  appropriately.
 */

#if defined(WIN32_TARGET) || defined(NON_SUPPORT_E_TARGET) || defined(CAFE_TARGET)
#define NEX_THREAD_AFFINITY_SUPPORT
#endif

/*
 * STL Capabilities - Used for the STL extensions.
 */

#if !defined(WIN32_TARGET) || (defined(DOT_NET_COMPILER) && !defined(STL_PORT))
#define NEX_STL_REBIND_SUPPORT
#endif

/* Defines if the STL defines std::iterator correctly or not */

#ifndef SN_COMPILER
#define NEX_STD_BASE_ITERATOR_SUPPORT
#endif


/*
 * Services Capabilities
 */

#ifdef WIN32_TARGET
#if !defined(LIVE_FAMILY)
#define NEX_WINDOWS_MFC_SUPPORT
#endif
#endif

#if (defined(VS2008_COMPILER) && defined(UNREAL_BUILD)) || defined(VS_EXPRESS_COMPILER)
#undef NEX_WINDOWS_MFC_SUPPORT
#endif

#if (defined(VS2010_COMPILER) && defined(UNREAL_BUILD)) || defined(VS_EXPRESS_COMPILER)
#undef NEX_WINDOWS_MFC_SUPPORT
#endif

/*
 * FileStorage support
 */

#if defined(WIN32_TARGET) || defined(NON_SUPPORT_B_TARGET) || defined(NON_SUPPORT_D_FAMILY)
#define NEX_FILESTORAGE_SUPPORT
#endif

/*
 * PSAPI - to access process information, in the ControlProtocol.
 */

#ifdef WIN32_TARGET
#define NEX_PSAPI_SUPPORT
#endif

#if defined(VS2005_COMPILER) || defined(VS2008_COMPILER) || defined(VS2010_COMPILER)
/* TODO: This should be fixed */
#undef NEX_PSAPI_SUPPORT
#endif

/*
 * Console IO support
 */
#define NEX_CONIO_SUPPORT
#if defined(NON_SUPPORT_D_FAMILY) || defined(NON_SUPPORT_B_TARGET) || defined(NON_SUPPORT_A_TARGET) || defined(NON_SUPPORT_C_TARGET) || defined(NINTENDO_FAMILY)
#undef NEX_CONIO_SUPPORT
#endif

#if defined(LINUX_TARGET) || defined(MACOSX_TARGET)
/* It is supported... but the syntax isn't equivalent.  It is easier to simply disable it for now. */
#undef NEX_CONIO_SUPPORT
#endif

#if defined(NON_SUPPORT_D_FAMILY) || defined(WIN32_TARGET)
#define NEX_DIRECTX_SUPPORT
#endif

#if defined(NON_SUPPORT_B_TARGET) || defined(WIN32_TARGET) || defined(NON_SUPPORT_D_FAMILY) || defined(NON_SUPPORT_A_TARGET) || defined(NON_SUPPORT_C_TARGET) || defined(NINTENDO_FAMILY)
/* The NON_SUPPORT_A and Wii target is there for specific clients that request this, it is not really supported */
#define NEX_AVSTREAM_SUPPORT
#endif

#if defined(WIN32_TARGET) && !defined(USE_DX10)
#define NEX_AVSTREAM_SUPPORT
#endif


#if defined(NEX_DEBUG_VERSION) || defined(TRACE_BLOCKING_CALLS) || defined(NEX_DEVELOPMENT_VERSION)
/** this is to used in CriticalSection to store thread info (i.e. ownership)*/
#define NEX_LOCK_DEBUG_ENABLED
#endif

/*
 * ConsoleUI support.  For Spark's text sample
 */

#if defined(WIN32_TARGET) && !defined(USE_DX10)
#define NEX_CONSOLEUI_SUPPORT
#endif

/*
 * ContentSharing support (only a subset of platforms for now)
 */
#if defined(NON_SUPPORT_C_TARGET) || defined(LIVE_FAMILY) || defined(WIN32_TARGET)
#define NEX_CONTENT_SHARING_SUPPORT
#endif


/*
 * SFTP support: a subset of platforms having NEX_CONTENT_SHARING_SUPPORT supports SFTP.
 */
#if defined(NON_SUPPORT_C_TARGET) || (defined(WIN32_TARGET) && !defined(WINLIVE_TARGET))
#define NEX_SFTP_SUPPORT
#endif

// std::atomicが利用可能かどうか。
// clangなら__ATOMIC_SEQ_CSTが定義されている。GCCなら4.4以上なら使える。
#if (defined(_MSC_VER) && _MSC_VER >= 1800) || (defined(__GNUC__) && defined(__ATOMIC_SEQ_CST)) || (defined(__GNUC__) && defined(NEX_GNUC_EQUAL_4_4_OR_HIGHER))
#define NEX_C11_ATOMIC_SUPPORT
#endif

// Condition::WaitForEvent()でタイムアウトの指定がサポートされているか。
// Cafeではタイムアウトを指定できないので、タイムアウトが必要な箇所ではエミュレーション版(EmulatedCondition)を使う。
#ifndef CAFE_TARGET
#define NEX_CONDITION_TIMEOUT_SUPPORT
#endif

/*
 * For some custom builds, we might want to disable SFTP in some circumstance.
 */
#if defined(NO_NEX_SFTP_SUPPORT)
#undef NEX_SFTP_SUPPORT
#endif


/*
 * NATCheck support
 */
#define NEX_RV_NAT_SUPPORT


/*
 * Endianness
 */

#if !defined(LINUX_TARGET) && !defined(MACOSX_TARGET)
#define NN_NEX_BIG_ENDIAN 1
#define NN_NEX_LITTLE_ENDIAN 2
#if defined(NON_SUPPORT_E_TARGET) || defined(NON_SUPPORT_C_TARGET) || defined(WII_TARGET) || defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
#define NN_NEX_BYTE_ORDER NN_NEX_BIG_ENDIAN
#else
#define NN_NEX_BYTE_ORDER NN_NEX_LITTLE_ENDIAN
#endif
#endif

/*
Little Endian (Intel)
0xfffffdb3 (2 Bytes Int)
0xb3    0xfd
0xffd993dd (4 Bytes Int)
0xdd    0x93    0xd9    0xff
0xf9fe63b7f8e89162 (8 Bytes Int)
0x62    0x91    0xe8    0xf8    0xb7    0x63    0xfe    0xf9


Big Endian (PowerPC)
lv2(3): 0xfffffdb3 (2 Bytes Int)
lv2(3): 0xfd    0xb3
lv2(3): 0xffd993dd (4 Bytes Int)
lv2(3): 0xff    0xd9    0x93    0xdd
lv2(3): 0xf9fe63b7f8e89162 (8 Bytes Int)
lv2(3): 0xf9    0xfe    0x63    0xb7    0xf8    0xe8    0x91    0x62
*/


/*
 * Obsolete API support - still supporting client API with error code returned as qBool/qResult
 */
#ifndef NO_OBSOLETE_API_SUPPORT     /* e.g. Samples */
#define OBSOLETE_API_SUPPORT
#endif

#ifdef NO_NEX_QCHAR16_IS_NOT_UINT16_T
#undef NEX_QCHAR16_IS_NOT_UINT16_T
#endif

#ifdef NO_NEX_FILE_STREAM_SUPPORT
#undef NEX_FILE_STREAM_SUPPORT
#endif

#define NEX_ROUTING_SUPPORT

#if defined(NEX_SIGLO_FAMILY)
// nn::sslを使うプラットフォームを定義
#define NEX_NNSSL_SUPPORT
#endif

// TCPでの通信をサポートするプラットフォームを定義
// CTRは3つ以上のTCPソケットを作れないので除外する。
#if !defined(CTR_TARGET) && !defined(CAFEIOP_TARGET)
#define NEX_TRANSPORT_PROTOCOL_TYPE_TCP_SUPPORT
#endif

// WebSocketでの通信をサポートするプラットフォームを定義
// CTRとCafeはWebSocketに非対応。
#if !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !defined(CAFEIOP_TARGET)
#define NEX_TRANSPORT_PROTOCOL_TYPE_WEB_SOCKET_SUPPORT
#endif

// メモリアロケータが常にスレッドセーフと仮定して良いかどうか
#if defined(WIN_FAMILY) || defined(UNIX_FAMILY)
#define NEX_ALLOCATOR_IS_ALWAYS_THREAD_SAFE
#endif

#ifdef LIVE_VERSION
// libwebsocketsを使えるかどうか
#define NEX_LIBWEBSOCKETS_SUPPORT
#define NEX_WEBSOCKET_CLIENT_SUPPORT
// libwebsocketsを使う場合はメモリアロケータがスレッドセーフである必要がある
#ifndef NEX_ALLOCATOR_IS_ALWAYS_THREAD_SAFE
#error libwebsockets can be used for using thread safe allocator only.
#endif

#else   // LIVE_VERSION
// Wslayを使えるかどうか
#if !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !defined(CAFEIOP_TARGET)
#define NEX_WEBSOCKET_CLIENT_SUPPORT
#define NEX_WEBSOCKET_CLIENT_IS_DEFAULT
#endif
#endif  // LIVE_VERSION

// epollを使えるかどうか
#ifdef LINUX_TARGET
#define NEX_EPOLL_SUPPORT
#endif

// NGS でブロックリストを保持しているかどうか
// CTR は該当する
// Cafe は NNAS のレプリなので該当しない
// NX は AA BaaS なので該当しない
// Linux と Server は PID が 64 ビット(=AAUser)時は該当しない
#if !defined(CAFE_TARGET) && !defined(NEX_SIGLO_FAMILY) && !defined(NEX_USE_PRINCIPALID_64BIT)
#define NEX_BLOCKLIST_OWNED_BY_NGS
#endif

// NNAS と同期した NFS のレプリでブロックリストを保持しているかどうか
// CTR は該当しない
// Cafe は該当する
// NX は AA BaaS なので該当しない
// Linux と Server は PID が 64 ビット(=AAUser)時は該当しない
#if !defined(CTR_TARGET) && !defined(NEX_SIGLO_FAMILY) && !defined(NEX_USE_PRINCIPALID_64BIT)
#define NEX_BLOCKLIST_OWNED_BY_NFS
#endif

// AA BaaS と動機した Kodama のレプリでブロックリストを保持しているかどうか
// CTR は該当しない
// Cafe は NNAS のレプリなので該当しない
// NX は 該当する
// Linux と Server は PID が 64 ビット(=AAUser)時に該当する
#if defined(NEX_SIGLO_FAMILY) || defined(NEX_USE_PRINCIPALID_64BIT)
#define NEX_BLOCKLIST_OWNED_BY_URS
#endif

// C++11のムーブセマンティクスが使用可能かどうか
// Visual Studioは2015でも__cplusplusが199711なのでVSバージョンで比較する
// _MSC_VER >= 1600 はVS2010以降を指す。
#if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600)
#define NEX_MOVE_SEMANTICS_SUPPORT
#define NEX_STD_MOVE(a) std::move(a)
#define NEX_STD_FORWARD(type, value) std::forward<type>(value)
#else
#define NEX_STD_MOVE(a) (a)
#define NEX_STD_FORWARD(type, value) (value)
#endif

// std::vector::emplace_back()などの配置挿入がサポートされているか
// Visual Studioは2015でも__cplusplusが199711なのでVSバージョンで比較する
// _MSC_VER >= 1800 はVS2013以降を指す。(実際にはVS2010から使えるが不完全なので除外する)
#if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1800)
#define NEX_PLACEMENT_INSERT_SUPPORT
#endif

// 可変長引数テンプレートがサポートされているか
// Visual Studioは2015でも__cplusplusが199711なのでVSバージョンで比較する
// _MSC_VER >= 1800 はVS2013以降を指す。
#if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1800)
#define NEX_VARIADIC_TEMPLATES_SUPPORT
#endif

// ラムダ式やstd::bindが使用可能かどうか
// Visual Studioは2015でも__cplusplusが199711なのでVSバージョンで比較する
// _MSC_VER >= 1600 はVS2010以降を指す。
#if __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600)
#define NEX_LAMBDA_AND_FUNCTION_FEATURE_SUPPORT
#endif

// C++11のnoexceptを使用可能かどうか
// Visual Studioでは2015以降でnoexceptをサポートしているが、例外処理モデルによって警告が出るので常にthrow()を使う。
#if __cplusplus >= 201103L && !defined(_MSC_VER)
#define NEX_NOEXCEPT_SUPPORT
#define NEX_NOEXCEPT noexcept
#else
#define NEX_NOEXCEPT throw()
#endif

// 任天堂独自採番の国コードがサポートされているか
#if defined(CTR_TARGET) || defined(CAFE_TARGET)
#define NEX_NINTENDO_COUNTRY_CODE_SUPPORT
#endif
