﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>

//n1769
// NN_HW_TICKS_PER_SECONDの値を丸写し。
#ifdef CTR_TARGET
#define CTR_TICKS_PER_SECOND (67027964 * 4)
#endif

BEGIN_NAMESPACE_NEX

//==============================================================================
// The high resolution clock provides time values in microseconds (tick values).
//
// This is "raw" time with some bad properties: It is not starting at zero and
// can roll. The values could even be non consecutive in specific contexts (i.e.
// if using tick counts and the thread changes from one CPU core to another).
//==============================================================================
class HighResolutionClock: public RootObject
{
public:
    HighResolutionClock();
    virtual ~HighResolutionClock();

    // {secret}
    static qUnsignedInt64 GetTickCount();

#if defined(NEX_SIGLO_FAMILY)
    static qUnsignedInt64 GetFrequency()
    {
        return nn::os::GetSystemTickFrequency();
    }
#elif defined(UNIX_FAMILY)
    static qUnsignedInt64 GetFrequency()
    {
        return 1000000;
    }
#elif defined(WII_TARGET) || defined(CAFE_TARGET)
    static qUnsignedInt64 GetFrequency()
    {
        return qUnsignedInt64(OS_TIMER_CLOCK);
    }
#elif defined(CAFEIOP_TARGET)
    static qUnsignedInt64 GetFrequency()
    {
        // IOSTime64 は microseconds オーダー
        return 1000000ULL;
    }
#elif defined(CTR_TARGET)
    static qUnsignedInt64 GetFrequency()
    {
        return qUnsignedInt64(CTR_TICKS_PER_SECOND);
    }
#else
    static qUnsignedInt64 GetFrequency();
#endif

    // On platforms where high resolution can always be used, this maps to GetTime.
    static Time GetLowResolutionTime(); // Resolution in milliseconds

    // WARNING: Never use GetHighResolutionTime directly, but HighResolutionChrono instead.
    // TODO:R18 - Make this private?
    static  HighResolutionTime GetHighResolutionTime()
    {   
        return (GetTickCount() * qUnsignedInt64(1000000)) / GetFrequency();
    }

    static Time GetTime() // Resolution in milliseconds
    {
        return (GetTickCount() * qUnsignedInt64(1000)) / GetFrequency();
    }
};


END_NAMESPACE_NEX

