﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Platform/Core/LogEntry.h>

BEGIN_NAMESPACE_NEX

//group=Log Devices

/*
Description: The root class for all logging devices.
Remarks: This is the base class for all logging devices. The logging devices are the
classes responsible for outputting log strings to a given device. Sub class this class or a
class that inherits from this class to create your own custom logging device.
*/

/*!
 * @brief LogDeviceの基底クラスです。
 *
 * カスタムLogDeviceを作るためには、このクラスを継承する必要があります。
 */
class LogDevice : public RootObject
{
public:
    //group=Methods

    /*!
     * @brief デストラクタです。
     */
    virtual ~LogDevice() {};

    /*
    Description: Defines how log entries are outputted to a given LogDevice.
    Remarks: You must overwrite this function and use it to define how the supplied LogEntry
    is outputted to the specific implemented LogDevice.
    Parameters:
    oLogEntry: The LogEntry to be outputted to the LogDevice.
    */
    //virtual void Output(const LogEntry &oLogEntry) = 0;

    /*!
     * @brief 文字列をLogDeviceから出力します。
     *
     * @param[in] pLog 出力文字列です。
     */
    virtual void Output(const qChar *pLog) = 0;
};

END_NAMESPACE_NEX


