﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/LogDevice.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>

BEGIN_NAMESPACE_NEX

/*
指定したメモリにログを書き込む。
リングバッファとして書き込むので、古いログは上書きされる。
asciiで書き込む。 wcharの場合asciiに変換して書き込み。
一回のログでは最大256文字まで。
ログはqChar8で書き込むので、UNICODEバージョンではascii変換後が256バイトより多くならないように。
バッファのラストは区切り文字delimiterが書き込まれる。deilmiterは4文字以内で。
*/
class LogDeviceMemory : public LogDevice
{
public:
    // 一回のログ最大長
    static const qInt32 MAX_ONE_LOG_LEN = 256;

public:
    LogDeviceMemory(qChar8* pBuffer, qInt32 bufferSize, const qChar8* delimiter, qChar8** ppLogTop=NULL);
    virtual ~LogDeviceMemory();

    virtual void Output(const qChar *pLog);

    /*! @internal
    @brief ログを書き込むバッファの設定。

    @param[inout] pBuffer 書き込み先。
    @param[in] bufferSize そのサイズ。
    @param[out] ログ書き込みごとに、ログの先頭を示すアドレスの書き込み先。NULL指定で無視。 ログが書き込まれるたびに更新される。
    */
    void SetBuffer(qChar8* pBuffer, qInt32 bufferSize, qChar8** ppLogTop=NULL);
    /*! @internal
    @brief バッファのアドレス取得。

    @return バッファのアドレス。
    */
    qChar8* GetBuffer()
    {
        return m_pBuffer;
    }
    /*! @internal
    @brief 現在のログの先頭アドレス。 リングバッファなのに注意。

    @return 現在のログの先頭アドレス。
    */
    qChar8* GetLogTop()
    {
        return m_pBuffer + m_offsetTop;
    }


    // ログ先頭位置のオフセット
    size_t GetOffsetTop()
    {
        return m_offsetTop;
    }
    // ログ書き込み位置のオフセット
    size_t GetOffsetWrite()
    {
        return m_offsetWrite;
    }

private:
    qChar8* m_pBuffer;
    size_t m_bufferSize;
    const qChar8* m_delimiter;
    qChar8** m_ppLogTop;
    ForcedCriticalSection m_cs;

    size_t m_offsetTop;
    size_t m_offsetWrite;
    qBool m_rotated; // ログが一周した

    qChar8 m_tmpBuffer[ MAX_ONE_LOG_LEN+8 ]; // 一時領域 改行とデリミタ
};

END_NAMESPACE_NEX


