﻿
/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once
#include <string>
#include <OnlineCore/src/Platform/Core/Defs.h>
BEGIN_NAMESPACE_NEX

// General Performance Counters - Exposed in Operations Reference Manual
// General Performance Counters - Exposed to user in SandboxProbe


//Group=Performance Counters

/******************************************************************************
*  Found in Platform\Core\EventLog.cpp
******************************************************************************/

/*
Description:  Counts the warning events
Remarks: This counter returns the number of warning events that occur per second
*/
#define  WARNING_PERF_C_STR  NEX_T("Warning Event")

/*
Description:  Counts the error events
Remarks: Counts the number of error events that occur per second
*/
#define ERROR_PERF_C_STR   NEX_T("Error Event")

/*
Description:  Counts the number of fatal events
Remarks: Counts the number of fatal events that occur per second
*/
#define FATAL_PERF_C_STR  NEX_T("Fatal Event")


/******************************************************************************
*  Found in Plugins\Transport\Interface\TransportPerfCounters.h
*******************************************************************************/

/*
Description:  Counts the number of unknown transport requests per second
Remarks: Counts the number of calls that were not completed as the result of an unknown destination
*/
#define TRANSPORT_UNKNOWN_PERF_C_STR  NEX_T("<transport unknown>")


// Found in ObjDup/DataSetPerfCounters.cpp These perf counters do not appear in the SandboxProbe.

/*
Description:  Counts the number of multicast updates that triggered messages
Remarks: Counts the number of multicast updates that triggered messages
*/
#define UPDATE_MULT_CAST_PERF_C_STR  NEX_T("UpdatesThatTrigeredMessages")

/*
Description:  Counts the number of update messages sent
Remarks: Counts the number of update messages sent
*/
#define UPDATE_MESSAGE_PERF_C_STR  NEX_T("MessagesSent")

/******************************************************************************
*  Found in  Plugins/Transport/Interface/TransportPerfCounters.h
*******************************************************************************/

/*
Description:  Counts the number of bits sent per second by the transport layer
Remarks: Counts the number of bits sent per second
*/
#define TRANSPORT_BIT_SENT_PERF_C_STR   NEX_T("bits sent")

/*
Description:  Counts the number of bits received per second by the transport layer
Remarks: Counts the number of bits received per second
*/
#define TRANSPORT_BITS_REC_PERF_C_STR  NEX_T("bits received")

/*
Description:  Counts the total number of packets sent per second by the transport layer
Remarks: Counts the total number of (UDP) packets sent per second
*/
#define TRANSPORT_PACKETS_SENT_PERF_C_STR  NEX_T("packets sent")

/*
Description:  Counts the total number of packets received per second by the transport layer
Remarks: Counts the total number of (UDP) packets received per second
*/
#define TRANSPORT_PACKETS_REC_PERF_C_STR  NEX_T("packets received")

/*
Description:  Counts the total number of transport packets lost over one second
Remarks: Counts the total number of transport packets lost over one second.
Estimated packet loss is from statistics of TCP error recovery.
*/
#define TRANSPORT_PACKETS_LOST_PERF_C_STR  NEX_T("packets lost")

/*
Description:  Counts the number of unreliable packets transmitted per second
Remarks: Counts the number of unreliable (UDP) packets transmitted per second.
Unreliable packets may arrive out of order, or not at all.
*/
#define TRANSPORT_UNRELIABLE_TRANS_PERF_C_STR   NEX_T("unreliable packets xmit")

/*
Description:  Counts the number of reliable (PRUDP) packets transmitted per second
Remarks: Counts the number of reliable (PRUDP) packets transmitted  (to other nodes) per second
*/
#define TRANSPORT_RELIABLE_TRANS_PERF_C_STR  NEX_T("reliable packets xmit")

/*
Description:  Counts the number of reliable (PRUDP) packets retransmitted per second
Remarks: Counts the number of reliable (PRUDP) packets retransmitted (to other nodes) per second
*/
#define TRANSPORT_RELIABLE_RETRANS_PERF_C_STR  NEX_T("reliable packets rexmit")

/*
Description:  Counts the number of packets routed per second by the transport layer
Remarks: Counts the number of routed packets retransmitted per second
*/
#define TRANSPORT_ROUTED_PACKETS_PERF_C_STR NEX_T("Routed packets xmit")

/*
Description:  Counts the number of routed packets in bits per second
Remarks: Counts the number of routed packets in bits per second
*/
#define TRANSPORT_ROUTED_BPS_PERF_C_STR NEX_T("Routed bits xmit")

/*
Description:  Counts the number of packets forwarded per second
Remarks: Counts the number of packets forwarded per second
*/
#define TRANSPORT_FORWARDED_PACKETS_PERF_C_STR NEX_T("Forwarded packets xmit")

/*
Description:  Counts the number forwarded packets in bits per second
Remarks: Counts the number of forwarded packets in bits per second
*/
#define TRANSPORT_FORWARDED_BPS_PERF_C_STR NEX_T("Forwarded bits xmit")


#define TRANSPORT_SYNCACHE_ENTRY_FULL_COUNT_C_STR NEX_T("Syncache entry full count")

#define TRANSPORT_SYNCACHE_UNMATCH_PACKETS_PERF_C_STR NEX_T("Syncache unmatch packets ")

// Found in  Services/Foundation/Service/CallProfiler.cpp


/*
Description:  Counts the number of slow calls, calls whose execution
time is >= 5 seconds (as defined by the system setting
CallProfiler::m_tiSlowCalls)
*/
#define SLOW_CALLS_PERF_C_STR   NEX_T("CallProfiler/SlowCalls")

/******************************************************************************
*  Found in  Protocol/ServerProtocol.cpp
******************************************************************************/

/*
Description: Counts the number of non blocking requests issued per second
Remarks: Counts the number of non blocking requests issued per second
*/
#define NON_BLOCK_REQ_PERF_C_STR   NEX_T("Non Blocking Requests")

/*
Description:  Counts the number of blocking requests issued per second
Remarks: Counts the number of blocking requests issued per second
*/
#define BLOCK_REQ_PERF_C_STR  NEX_T("Blocking Requests")

/*
Description:  Counts the total number of slow requests issued per second. "Slow"
 is a configurable threshold for processing calls. See the system setting
 quazal.CallProfiler/SlowCallThreshold
Remarks: Counts the total number of slow requests issued per second.
*/
#define SLOW_REQ_TOTAL_PERF_C_STR  NEX_T("Slow Requests - Total")

/*
Description:  Counts the number of slow requests executed per second. Slow"
 is a configurable threshold for processing calls. See the system setting
 quazal.CallProfiler/SlowCallThreshold
Remarks: Counts the number of slow requests executed per second
*/
#define SLOW_REQ_EXE_PERF_C_STR  NEX_T("Slow Requests - Execution")


/******************************************************************************
// Found in  Services/Foundation/Service/ServerStreamManager.h
******************************************************************************/

/*
Description:  Counts the number of messages that the ServerStream Manager has dispatched
Remarks: Counts the number of messages that the ServerStream Manager has dispatched
*/
#define SERVER_STREAM_MSG_REC_PERF_C_STR  NEX_T("ServerStreamManager/MessagesDispatched")

/*
Description:  Counts the number of ServerStream Manager disconnections per second
Remarks: Counts the number of ServerStream Manager disconnections per second
*/
#define SERVER_STREAM_DISCONNECT_PERF_C_STR  NEX_T("ServerStreamManager/Disconnections")
/*

Description:  Counts the number of ServerStream Manager faults per second
Remarks: Counts the number of ServerStream Manager faults per second
*/
#define SERVER_STREAM_DISCONNECT_FAULT_PERF_C_STR  NEX_T("ServerStreamManager/Faults")

/*
Description:  Counts the number of user requests received since the sandbox started
Remarks: Counts the number of user requests received since the sandbox started
*/
#define SERVER_STREAM_USER_REQ_REC_PERF_C_STR  NEX_T("UserRequestsReceived")


/******************************************************************************
// Found in  Services/SecureConnection/Service/SecureConnectionManager.cpp
******************************************************************************/

/*
Description:  Counts the number of ServerStream Manager faults per second
Remarks: Counts the number of ServerStream Manager faults per second
*/
#define SECURE_CONN_RATE_PERF_C_STR  NEX_T("BackEnd Connection")

//n2361 TestConnectivity呼び出し回数チェック
#define SECURE_CONN_TESTCONNECTIVITY_PERF_C_STR  NEX_T("SecureConnectionServer::TestConnectivity/Calls")

/*
Description:  Counts the number of minutes users have been logged in since the sandbox started
Remarks: Counts the total number of minutes users have been logged in since the sandbox started
*/
#define SECURE_CONN_USER_LOGGED_IN_TIME_PERF_C_STR  NEX_T("UserLoggedInTime")

/******************************************************************************
// Found in  RVPackages/Siege/Tools/Agent/SiegeAgent.cpp
******************************************************************************/

/*
Description:  Counts the login rate per second
Remarks: Counts the login rate per second
*/
#define SIEGE_AGENT_LOGIN_RATE_PERF_C_STR  NEX_T("LoginRate")
/*

Description:  Counts the logout rate per second
Remarks: Counts the logout rate per second
*/
#define SIEGE_AGENG_LOGOUT_RATE_PERF_C_STR   NEX_T("LogoutRate")

/*
Description:  Counts the number of calls issued on the RV daemon per second
Remarks: Counts the number of calls issued on the RV daemon per second
*/
#define SIEGE_AGENG_CALL_RATE_PERF_C_STR  NEX_T("CallRate")


// The following is never defined. It is used only to expose the cache PerfCounters with
// the other PerfCounters.

#ifdef FOR_DOC_PURPOSE

/******************************************************************************
*  Found in the Cache Module. Visible in SandboxProbe.
******************************************************************************/

/*
Description: Counts the current number of cache hits.
Remarks: Counts the current number of cache hits (successful attempts to access the cache).
This perfcounter originates in the python Cache class on the backend and is displayed in the
SandboxProbe. You can also view the stat using the Cache.Cache.GetHitPerfCounter() method.
Note: <CacheName> is the cache /name/ defined using the Cache.Cache.LocalCache or Cache.Cache.RemoteCache
classes. That is, each cache maintains its own statistics.
*/
#define  CACHE_HIT_PERF_C_STR NEX_T("CacheHit/<CacheName>")

/*
Description: Counts the current number of cache misses.
Remarks: Counts the current number of cache misses (unsuccessful attempts to access the cache).
This perfcounter originates in the python Cache class on the backend and is displayed in the
SandboxProbe. You can also view the stat using the Cache.Cache.GetMissPerfCounter() method.
Note: <CacheName> is the cache /name/ defined using the Cache.Cache.LocalCache or Cache.Cache.RemoteCache
classes. That is, each cache maintains its own statistics.
*/
#define  CACHE_MISS_PERF_C_STR NEX_T("CacheMiss/<CacheName>")

/*
Description: Counts the current number of error conditions encounterd by the cache.
Remarks: Counts the current number of error conditions (unexpected exceptions thrown by the cache).
This perfcounter originates in the python Cache class on the backend and is displayed in the
SandboxProbe. You can also view its statistics using the Cache.Cache.GetErrorPerfCounter() method.
Note: <CacheName> is the cache /name/ defined using the Cache.Cache.LocalCache or Cache.Cache.RemoteCache
classes. That is, each cache maintains its own statistics.
*/
#define  CACHE_ERROR_PERF_C_STR NEX_T("CacheError/<CacheName>")

#endif


/*
Description:  Counts the number of INSERT into the table deletions per second
Remarks: Counts the number of INSERT into the table deletions per second
*/
#define DELETIONS_INSERT_RECORD_DISCONNECTIONS_COMP NEX_T("DeletionsInsert_RecordDisconnections_COMP")
#define DELETIONS_INSERT_RECORD_DISCONNECTIONS_MM NEX_T("DeletionsInsert_RecordDisconnections_MM")
#define DELETIONS_INSERT_UnregisterGatherings_JobDeleteUnreferencedEvents NEX_T("DeletionsInsert_UnregisterGatherings_JobDeleteUnreferencedEvents")

/*
Description:  Counts the number of SELECT into the table deletions per second
Remarks: Counts the number of SELECT into the table deletions per second
*/
#define DELETIONS_FETCH_GetPendingDeletions_JobDeleteCompetitionEvents NEX_T("DeletionsFetch_GetPendingDeletions_JobDeleteCompetitionEvents")
#define DELETIONS_FETCH_GetPendingDeletions_JobProcessEventDisconnections NEX_T("DeletionsFetch_GetPendingDeletions_JobProcessEventDisconnections")
#define DELETIONS_FETCH_Not_Used_Competition NEX_T("DeletionsFetch_Not_Used_Competition")
#define DELETIONS_FETCH_DELETE_UNREFERENCED_EVENTS NEX_T("DeletionsFetch_DeleteUnreferencedEvents")
#define DELETIONS_FETCH_PENDING_DELETIONS NEX_T("DeletionsFetch_Pending_Deletions")

/*
Description:  Counts the number of DELETE into the table deletions per second
Remarks: Counts the number of DELETE into the table deletions per second
*/
#define DELETIONS_DELETE_DeleteFromDeletions NEX_T("DeletionsDelete_DeleteFromDeletions")
#define DELETIONS_DELETE_DeleteGathering NEX_T("DeletionsDelete_DeleteGathering")
#define DELETIONS_DELETE_DeleteFromDeletionsExt NEX_T("DeletionsDelete_DeleteFromDeletionsExt")
#define DELETIONS_DELETE_UnregisterGatherings_UserCall NEX_T("DeletionsDelete_UnregisterGatherings_UserCall")
#define DELETIONS_DELETE_UnregisterGathering_ParticipationPolicy NEX_T("DeletionsDelete_UnregisterGathering_ParticipationPolicy")
#define DELETIONS_DELETE_UnregisterGathering NEX_T("DeletionsDelete_UnregisterGathering")
#define DELETIONS_DELETE_DeleteFromDeletions_JobProcessEventDisconnections NEX_T("DeletionsDelete_DeleteFromDeletions_JobProcessEventDisconnections")
#define DELETIONS_DELETE_DeleteFromDeletions_JobDeleteCompetitionEvents NEX_T("DeletionsDelete_DeleteFromDeletions_JobDeleteCompetitionEvents")


/// // [NEX1][LINUS] BEGIN - Names for RDV perf counters // ///

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = URL), calls whose execution time is >= 0.5 seconds (as defined by the system setting CallProfiler::m_UrlSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_UrlSourceSlowCalls
*/
#define URL_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/UrlSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = Message), calls whose execution time is >= 5 seconds (as defined by the system setting CallProfiler::m_MessageSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_MessageSourceSlowCalls
*/
#define MESSAGE_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/MessageSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = Background), calls whose execution time is >= 5 seconds (as defined by the system setting CallProfiler::m_BackgroundJobSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_BackgroundJobSourceSlowCalls
*/
#define BACKGROUNDJOB_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/BackgroundJobSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = FirstUser), calls whose execution time is >= 5 seconds (as defined by the system setting CallProfiler::m_FirstUserControlScriptSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_FirstUserControlScriptSourceSlowCalls
*/
#define FIRST_USER_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/FirstUserControlScriptSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = SecondUser), calls whose execution time is >= 5 seconds (as defined by the system setting CallProfiler::m_SecondUserControlScriptSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_SecondUserControlScriptSourceSlowCalls
*/
#define SECOND_USER_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/SecondUserControlScriptSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = TimeCriticalSystem), calls whose execution time is >= 5 seconds (as defined by the system setting CallProfiler::m_TimeCriticalSystemControlScriptSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_TimeCriticalSystemControlScriptSourceSlowCalls
*/
#define TIME_CRITICAL_SYSTEM_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/TimeCriticalSystemControlScriptSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = TimeInsensitiveSystem), calls whose execution time is >= 60 seconds (as defined by the system setting CallProfiler::m_TimeInsensitiveSystemControlScriptSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_TimeInsensitiveSystemControlScriptSourceSlowCalls
*/
#define TIME_INSENSITIVE_SYSTEM_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/TimeInsensitiveSystemControlScriptSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of slow calls (Source = TrackingSystem), calls whose execution time is >= 180 seconds (as defined by the system setting CallProfiler::m_TrackingSystemControlScriptSourceSlowCalls).
    \sa PerfCounter::GetName, CallProfiler::m_TrackingSystemControlScriptSourceSlowCalls
*/
#define TRACKING_SYSTEM_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/TrackingSystemControlScriptSource_SlowCallThreshold")

/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of SELECT into the table deletions per second.
    \sa PerfCounter::GetName
*/
#define DELETIONS_FETCH_GetPendingDeletions_JobDeleteGatherings NEX_T("DeletionsFetch_GetPendingDeletions_JobDeleteGatherings")


/**
    \ingroup group_onlinecore_platform
    \brief This counter counts the number of SELECT into the table deletions per second.
    \sa PerfCounter::GetName
*/
#define DELETIONS_FETCH_GetPendingDeletions_JobProcessDisconnections NEX_T("DeletionsFetch_GetPendingDeletions_JobProcessDisconnections")


class PerfCounterNames
{
public:
    PerfCounterNames():
        STR_DELETIONS_FETCH_DELETE_UNREFERENCED_EVENTS(DELETIONS_FETCH_DELETE_UNREFERENCED_EVENTS),
        STR_DELETIONS_INSERT_RECORD_DISCONNECTIONS_COMP(DELETIONS_INSERT_RECORD_DISCONNECTIONS_COMP),
        STR_SECURE_CONN_USER_LOGGED_IN_TIME_PERF(SECURE_CONN_USER_LOGGED_IN_TIME_PERF_C_STR),
        STR_DELETIONS_FETCH_PENDING_DELETIONS(DELETIONS_FETCH_PENDING_DELETIONS),
        STR_DELETIONS_DELETE_DeleteFromDeletions(DELETIONS_DELETE_DeleteFromDeletions),
        STR_DELETIONS_DELETE_DeleteGathering(DELETIONS_DELETE_DeleteGathering),
        STR_DELETIONS_INSERT_RECORD_DISCONNECTIONS_MM(DELETIONS_INSERT_RECORD_DISCONNECTIONS_MM),
        STR_DELETIONS_DELETE_UnregisterGathering(DELETIONS_DELETE_UnregisterGathering),
        STR_DELETIONS_DELETE_DeleteFromDeletionsExt(DELETIONS_DELETE_DeleteFromDeletionsExt),
        STR_DELETIONS_FETCH_GetPendingDeletions_JobDeleteGatherings(DELETIONS_FETCH_GetPendingDeletions_JobDeleteGatherings),  //[NEX][LINUS][For RDV]
        STR_DELETIONS_FETCH_GetPendingDeletions_JobProcessDisconnections(DELETIONS_FETCH_GetPendingDeletions_JobProcessDisconnections), //[NEX][LINUS][For RDV]
        STR_DELETIONS_DELETE_UnregisterGathering_ParticipationPolicy(DELETIONS_DELETE_UnregisterGathering_ParticipationPolicy)
    {}

    ~PerfCounterNames(){}

    const qChar* STR_DELETIONS_FETCH_DELETE_UNREFERENCED_EVENTS;
    const qChar* STR_DELETIONS_INSERT_RECORD_DISCONNECTIONS_COMP;
    const qChar* STR_SECURE_CONN_USER_LOGGED_IN_TIME_PERF;
    const qChar* STR_DELETIONS_FETCH_PENDING_DELETIONS;
    const qChar* STR_DELETIONS_DELETE_DeleteFromDeletions;
    const qChar* STR_DELETIONS_DELETE_DeleteGathering;
    const qChar* STR_DELETIONS_INSERT_RECORD_DISCONNECTIONS_MM;
    const qChar* STR_DELETIONS_DELETE_UnregisterGathering;
    const qChar* STR_DELETIONS_DELETE_DeleteFromDeletionsExt;
    const qChar* STR_DELETIONS_FETCH_GetPendingDeletions_JobDeleteGatherings; //[NEX][LINUS][For RDV]
    const qChar* STR_DELETIONS_FETCH_GetPendingDeletions_JobProcessDisconnections; //[NEX][LINUS][For RDV]
    const qChar* STR_DELETIONS_DELETE_UnregisterGathering_ParticipationPolicy;


#ifdef NOT_YET
#define TRANSPORT_UNKNOWN_PERF_C_STR  NEX_T("<transport unknown>")
#define UPDATE_MULT_CAST_PERF_C_STR  NEX_T("UpdatesThatTrigeredMessages")
#define UPDATE_MESSAGE_PERF_C_STR  NEX_T("MessagesSent")
#define TRANSPORT_BITS_SENT_PERF_C_STR   NEX_T("bits sent")
#define TRANSPORT_BITS_REC_PERF_C_STR  NEX_T("bits received")
#define TRANSPORT_PACKETS_SENT_PERF_C_STR  NEX_T("packets sent")
#define TRANSPORT_PACKETS_REC_PERF_C_STR  NEX_T("packets received")
#define TRANSPORT_PACKETS_LOST_PERF_C_STR  NEX_T("packets lost")
#define TRANSPORT_UNRELIABLE_TRANS_PERF_C_STR   NEX_T("unreliable packets xmit")
#define TRANSPORT_RELIABLE_TRANS_PERF_C_STR  NEX_T("reliable packets xmit")
#define TRANSPORT_RELIABLE_RETRANS_PERF_C_STR  NEX_T("reliable packets rexmit")
#define TRANSPORT_ROUTED_PACKETS_PERF_C_STR NEX_T("Routed packets xmit")
#define TRANSPORT_ROUTED_BPS_PERF_C_STR NEX_T("Routed bits xmit")
#define TRANSPORT_FORWARDED_PACKETS_PERF_C_STR NEX_T("Forwarded packets xmit")
#define TRANSPORT_FORWARDED_BPS_PERF_C_STR NEX_T("Forwarded bits xmit")
#define TRANSPORT_FORWARDED_MTUDROPPED_PERF_C_STR NEX_T("MTU Dropped packets")
#define SLOW_CALLS_PERF_C_STR   NEX_T("CallProfiler/SlowCalls")
#define URL_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/UrlSource_SlowCallThreshold")
#define MESSAGE_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/MessageSource_SlowCallThreshold")
#define BACKGROUNDJOB_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/BackgroundJobSource_SlowCallThreshold")
#define FIRST_USER_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/FirstUserControlScriptSource_SlowCallThreshold")
#define SECOND_USER_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/SecondUserControlScriptSource_SlowCallThreshold")
#define TIME_CRITICAL_SYSTEM_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/TimeCriticalSystemControlScriptSource_SlowCallThreshold")
#define TIME_INSENSITIVE_SYSTEM_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/TimeInsensitiveSystemControlScriptSource_SlowCallThreshold")
#define TRACKING_SYSTEM_CONTROL_SCRIPT_SOURCE_SLOW_CALLS_PERF_C_STR NEX_T("CallProfiler/TrackingSystemControlScriptSource_SlowCallThreshold")
#define NON_BLOCK_REQ_PERF_C_STR   NEX_T("Non Blocking Requests")
#define BLOCK_REQ_PERF_C_STR  NEX_T("Blocking Requests")
#define SLOW_REQ_TOTAL_PERF_C_STR  NEX_T("Slow Requests - Total")
#define SLOW_REQ_EXE_PERF_C_STR  NEX_T("Slow Requests - Execution")
#define SERVER_STREAM_MSG_REC_PERF_C_STR  NEX_T("ServerStreamManager/MessagesDispatched")
#define SERVER_STREAM_DISCONNECT_PERF_C_STR  NEX_T("ServerStreamManager/Disconnections")
#define SERVER_STREAM_DISCONNECT_FAULT_PERF_C_STR  NEX_T("ServerStreamManager/Faults")
#define SERVER_STREAM_USER_REQ_REC_PERF_C_STR  NEX_T("UserRequestsReceived")
#define SECURE_CONN_RATE_PERF_C_STR  NEX_T("BackEnd Connection")
#define SECURE_CONN_USER_LOGGED_IN_TIME_PERF_C_STR  NEX_T("UserLoggedInTime")
#define SIEGE_AGENT_LOGIN_RATE_PERF_C_STR  NEX_T("LoginRate")
#define SIEGE_AGENG_LOGOUT_RATE_PERF_C_STR   NEX_T("LogoutRate")
#define SIEGE_AGENG_CALL_RATE_PERF_C_STR  NEX_T("CallRate")
#define  CACHE_HIT_PERF_C_STR NEX_T("CacheHit/<CacheName>")
#define  CACHE_MISS_PERF_C_STR NEX_T("CacheMiss/<CacheName>")
#define  CACHE_ERROR_PERF_C_STR NEX_T("CacheError/<CacheName>")
#define DELETIONS_INSERT_RECORD_DISCONNECTIONS_COMP NEX_T("DeletionsInsert_RecordDisconnections_COMP")
#define DELETIONS_INSERT_UnregisterGatherings_JobDeleteUnreferencedEvents NEX_T("DeletionsInsert_UnregisterGatherings_JobDeleteUnreferencedEvents")
#define DELETIONS_FETCH_GetPendingDeletions_JobDeleteCompetitionEvents NEX_T("DeletionsFetch_GetPendingDeletions_JobDeleteCompetitionEvents")
#define DELETIONS_FETCH_GetPendingDeletions_JobProcessEventDisconnections NEX_T("DeletionsFetch_GetPendingDeletions_JobProcessEventDisconnections")
#define DELETIONS_FETCH_Not_Used_Competition NEX_T("DeletionsFetch_Not_Used_Competition")
#define DELETIONS_FETCH_DELETE_UNREFERENCED_EVENTS NEX_T("DeletionsFetch_DeleteUnreferencedEvents")
#define DELETIONS_DELETE_UnregisterGathering NEX_T("DeletionsDelete_UnregisterGathering")
#define DELETIONS_DELETE_DeleteFromDeletions_JobProcessEventDisconnections NEX_T("DeletionsDelete_DeleteFromDeletions_JobProcessEventDisconnections")
#define DELETIONS_DELETE_DeleteFromDeletions_JobDeleteCompetitionEvents NEX_T("DeletionsDelete_DeleteFromDeletions_JobDeleteCompetitionEvents")
#endif
};
/// // [NEX1][LINUS] END - Names for RDV perf counters // ///

END_NAMESPACE_NEX

