﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>

BEGIN_NAMESPACE_NEX

//group=Rendez-Vous Core

/*
Description: A unique positive integer used to identify each Principal in the lobby.
Remarks: A Principal is a Process (service, administrator, client /etc./ ) that can log in to
the lobby. Depending on the Group to which the Principal belongs, the Principal may be able to
maintain several simultaneous lobby log-ins.

PrincipalIDs ranging from 1 to 99 are reserved for Quazal processes and cannot be
assigned by the developer. IDs above and including 100 can be assigned by the developer however,
IDs from 100 to 999 are reserved for server processes.
*/

/*!
 @brief    クライアントを識別するユニークな ID です。

 フレンドサーバーに初回接続したときに割り振られ、CTR本体に保存される完全固有のIDです。
 
 */
#ifdef NEX_USE_PRINCIPALID_64BIT
typedef qUnsignedInt64 PrincipalID;
#else
typedef qUnsignedInt32 PrincipalID;
#endif

/*!
 @brief    無効な PrincipalID です。
 */
#define INVALID_PRINCIPALID         0

END_NAMESPACE_NEX



