﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>


BEGIN_NAMESPACE_NEX

class StringConversion: public RootObject
{
public:

    // Unicode
#ifdef NEX_UNICODE_VERSION
    static void Char8_2T(const qChar8* szASrc, qChar* szTDest, size_t uiDestLength);
    static void T2Char8(const qChar* szTSrc, qChar8* szADest, size_t uiDestLength);
    static void T2Char8Alloc(const qChar* szTSrc, qChar8** pszADest);

    static void T2Char16(const qChar* szTSrc, qChar16* szWDest, size_t uiDestLength);
    static void Char16_2T(const qChar16* szWSrc, qChar* szTDest, size_t uiDestLength);

    static size_t GetUtf8ToTBufferSize( const qChar8* szASrc );
#endif
    // ANSI
#ifdef NEX_ANSI_VERSION
    static void Char8_2T(const qChar8* szASrc, qChar* szTDest, size_t uiDestLength);
    static void T2Char8(const qChar* szTSrc, qChar8* szADest, size_t uiDestLength);
    static void T2Char8Alloc(const qChar* szTSrc, qChar8** pszADest);

#ifdef NEX_UNICODE_SUPPORT
    static void T2Char16(const qChar* szTSrc, qChar16* szWDest, size_t uiDestLength);
    static void Char16_2T(const qChar16* szWSrc, qChar* szTDest, size_t uiDestLength);
#endif
#endif // end of NEX_UNICODE_VERSION

    static void Utf8ToT(const qChar8* szASrc, qChar* szTDest, size_t uiDestLength);
    static void TToUtf8(const qChar* szTSrc, qChar8* szADest, size_t uiDestLength);

    static size_t GetTToUtf8BufferSize( const qChar* szTSrc );
    static void Char8ToUtf8Alloc( const qChar8* szASrc, qChar8** szADest);
    static void WideCharToUtf8Alloc( const qWideChar* szWSrc, qChar8** szADest);
    static void Utf8ToWideCharAlloc( const qChar8* szASrc, qWideChar** szWDest);

    static void T2Char8Release(qChar8* szADest);

    static size_t Utf8ToWideChar( const qChar8* szASrc, qWideChar* szWDest, size_t uiDestLength);
    static size_t WideCharToUtf8( const qWideChar* szWSrc, qChar8* szADest, size_t uiDestLength);
    static size_t GetNeedUtf8BufferSize( const qWideChar* szSrc );
#ifdef NEX_QCHAR16_IS_NOT_WCHAR_T
    static size_t Utf8ToChar16( const qChar8* szASrc, qChar16* szWDest, size_t uiDestLength);
    static size_t Char16ToUtf8( const qChar16* szWSrc, qChar8* szADest, size_t uiDestLength);
    static size_t GetNeedUtf8BufferSize( const qChar16* szSrc );
#endif
    static size_t GetNeedWideCharBufferSize( const qChar8* szSrc );

    static qBool Encode(const qByte *szByteBuffer, size_t uiBufferSize, qChar *szEncodedString, size_t uiStringSize);
    static qBool Decode(const qChar *szEncodedString, qByte *szByteBuffer, size_t uiBufferSize);

    static void StringToHex(const qChar8* szASrc, qByte* pbDest, size_t uiDestLength);

    static const qChar base64_chars[];
    static const qByte base64_values[];
    static qUnsignedInt32 HexToString(const qByte *pbSrc, qChar8* szADest, size_t uiSrcLength);
    static void ByteArrayToBase64(const qByte* pbSrc, size_t uiSrcLength, qChar* szADest, size_t* puiDestLength);
    static qBool Base64ToByteArray(const qChar* szASrc, size_t uiSrcLength, qByte* pbDest, size_t* puiDestLength);

#ifdef NEX_UNICODE_SUPPORT
    static void StringToHex(const qWideChar* szWSrc, qByte* pbDest, size_t uiDestLength);
    static qUnsignedInt32 HexToString(const qByte *pbSrc, qWideChar* szWDest, size_t uiSrcLength);
#endif

private:
    static qByte GetCharValue(qChar8 cNumber);
#ifdef NEX_UNICODE_SUPPORT
    static qByte GetCharValue(qWideChar cNumber);
#endif
};

END_NAMESPACE_NEX

