﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once
#ifdef CTR_TARGET // [NEX1][LINUS]


#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#include <nn/Result.h>


BEGIN_NAMESPACE_NEX

//n1769 NEX P2Pのリザルトコードを作り出すマクロです。
#define NN_DEFINE_NEX_RESULT_CONST(name, level, summary, description)  NN_DEFINE_RESULT_CONST(name, level, summary, Result::MODULE_NN_NEX, description)

//n1589:memo:0x80 = 128
const u32 NEX_ERROR_BIT   = 0x80;  //!< エラーレベルであることを示すビットです。
//n1589:memo:0x40 = 64
const u32 NEX_WARNING_BIT = 0x40;  //!< 警告レベルであることを示すビットです。
//n1769
const u32 NEX_DESCRIPTION_MASK = 0x3f;

//n1769 Descriptionの限界個数は64個です。8ビットの内、上2ビットは↑で
//      WarningとErrorの区別に使っているためです。
//      新しいエラー定数を追加する際は、必ず最後に追加していって下さい。

//---------------------------------------------------------------------------
//  NEX P2PにおけるDescriptionです。ドキュメントはnn::Result型の方に記述します。
//---------------------------------------------------------------------------
#ifndef FOR_DOC_PURPOSE
enum Description
{
    DESCRIPTION_VOID                       =                     0, //!< 無効であることを示すためのシンボルです。

    DESCRIPTION_OUT_OF_BOUNDARY            = NEX_ERROR_BIT   |   1, //!< SYSTEMERROR_GEN_INDEX_OOB に対応。                  Nintendo問い合わせ。
    DESCRIPTION_DUPLICATED_KEY             = NEX_WARNING_BIT |   2, //!< SYSTEMERROR_GEN_DUPLICATED_KEY に対応。             無視可能。
    DESCRIPTION_ITEM_NOT_FOUND             = NEX_WARNING_BIT |   3, //!< SYSTEMERROR_GEN_ITEM_NOT_FOUND に対応。             無視可能。
    DESCRIPTION_END_OF_COLLECTION          = NEX_WARNING_BIT |   4, //!< SYSTEMERROR_GEN_END_OF_COLLECTION に対応。          無視可能。
    DESCRIPTION_OBJ_NOT_INIT               = NEX_ERROR_BIT   |   5, //!< SYSTEMERROR_GEN_OBJ_NOT_INIT に対応。               Nintendo問い合わせ。
    DESCRIPTION_NOT_IMPLEMENTED            = NEX_ERROR_BIT   |   6, //!< SYSTEMERROR_GEN_NOT_IMPLEMENTED に対応。            Nintendo問い合わせ。
    DESCRIPTION_TIMEOUT                    = NEX_ERROR_BIT   |   7, //!< SYSTEMERROR_GEN_TIMEOUT に対応。                    タイムアウト。
    //  DESCRIPTION_CHILDPROCESS_FAILED        = NEX_WARNING_BIT |   8, //!< SYSTEMERROR_GEN_CHILDPROCESS_FAILED に対応。        CTR環境では使用されない(NEX_MULTIPROCESS_SUPPORTではない)ので、除去。
    DESCRIPTION_INVALID_OPERATION          = NEX_ERROR_BIT   |   9, //!< SYSTEMERROR_GEN_INVALID_OPERATION に対応。          Description: An operation was called in an invalid context.
    DESCRIPTION_MULTIPLE_INSTANCES         = NEX_ERROR_BIT   |  10, //!< SYSTEMERROR_GEN_MULTIPLE_INSTANCES に対応。         Description: An attempt was made at instantiating more than one $PRODUCTNAME$ object in the same process. NetZインスタンスを複数作ろうとした場合のエラー。
    //  DESCRIPTION_EXPIRED                    = NEX_ERROR_BIT   |  11, //!< SYSTEMERROR_GEN_EXPIRED に対応。                    Description: An attempt to use an expired version of $PRODUCTNAME$ was made.  CTRでは使われないようなので、除去。
    DESCRIPTION_OUT_OF_RANGE               = NEX_ERROR_BIT   |  12, //!< SYSTEMERROR_GEN_OUT_OF_RANGE に対応。               Nintendo問い合わせ。Description: A method was called with an out-of-range parameter.
    DESCRIPTION_INVALID_PARAMETER          = NEX_ERROR_BIT   |  13, //!< SYSTEMERROR_GEN_INVALID_PARAMETER に対応。          Description: An invalid parameter, which is typically NULL, was passed to a method. おそらくアプリ側のミス。
    //  DESCRIPTION_INVALID_PRODUCT_KEY        = NEX_ERROR_BIT   |  14, //!< SYSTEMERROR_GEN_INVALID_PRODUCT_KEY に対応。        Description: The product key used for $PRODUCTNAME$ does not exist or is invalid.  CTRでは使われないようなので、除去。
    DESCRIPTION_BUFFER_EXTRACTION_OVERFLOW = NEX_ERROR_BIT   |  15, //!< SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW に対応。 ライブラリとアプリのどちらに責任があるかはケースバイケース。
    DESCRIPTION_INVALID_WAIT               = NEX_ERROR_BIT   |  16, //!< SYSTEMERROR_GEN_INVALID_WAIT に対応。               アプリ側のミス。シングルスレッドでタイムアウト値としてゼロ以外を与えた場合に発生する。
    DESCRIPTION_ACCESS_DENIED              = NEX_ERROR_BIT   |  17, //!< SYSTEMERROR_GEN_ACCESS_DENIED に対応。              （違法改造などにより）システムのオペレーション実行が拒絶された事を示す。
    //  DESCRIPTION_TRANSPORT_GENERAL_RELIABLE = NEX_ERROR_BIT   |  18, //!< SYSTEMERROR_TRANSPORT_GENERAL_RELIABLE に対応。     このコードをシグナルしようとしている箇所は見あたらないため、除去。
    //  DESCRIPTION_TRANSPORT_UDS              = NEX_ERROR_BIT   |  19, //!< SYSTEMERROR_TRANSPORT_UDS に対応。                  このコードをシグナルしようとしている箇所は見あたらないため、除去。
    DESCRIPTION_ROLE_MISMATCH              = NEX_ERROR_BIT   |  20, //!< SYSTEMERROR_TRANSPORT_UDS_INVALID_STATE に対応。
    DESCRIPTION_INVALID_OPERATION_WARNING  = NEX_WARNING_BIT |  21, //!< SYSTEMERROR_GEN_INVALID_OPERATION_WARNING に対応。  {Secret}
    //  DESCRIPTION_RELIEABLE_SEND_FAILURE     = NEX_ERROR_BIT   |  22, //!< SYSTEMERROR_OBJDUP_RELIABLE_SEND_FAILURE に対応。   除去。
    DESCRIPTION_RELIEABLE_SEND_WARNING     = NEX_WARNING_BIT |  23, //!< SYSTEMERROR_OBJDUP_RELIABLE_SEND_WARNING に対応。   {Secret}
    DESCRIPTION_OBJDUP_DO_NOT_FOUND        = NEX_WARNING_BIT |  24, //!< SYSTEMERROR_OBJDUP_DO_NOT_FOUND に対応。            Description: The duplicated object was not found. 無視可能。
    //                                             // 欠番。
    DESCRIPTION_OUT_OF_MEMORY              = NEX_ERROR_BIT   |  26, //!<                                             // メモリ不足。
    DESCRIPTION_SOCKET_SEND_WARNING        = NEX_WARNING_BIT |  27, //!< SYSTEMERROR_TRANSPORT_SOCKET_SEND_WARNING に対応。  // 未使用。無視可能。
    DESCRIPTION_INVALID_NEW_STATION        = NEX_ERROR_BIT   |  28, //                                             // Description: ExecChangeDupSet was called with invalid new station.
    DESCRIPTION_HOST_MIGRATION_FAILURE     = NEX_ERROR_BIT   |  29, //!< ホストマイグレーションに失敗しました。

    DESCRIPTION_DC_CLUSTER_REDEFINITION        = NEX_ERROR_BIT   | 30, //!< SYSTEMERROR_DC_CLUSTER_REDEFINITION に対応。
    DESCRIPTION_DC_CLUSTER_JOIN_DENIED         = NEX_ERROR_BIT   | 31, //!< SYSTEMERROR_DC_CLUSTER_JOIN_DENIED に対応。
    DESCRIPTION_DC_CLUSTER_JOIN_FAILED         = NEX_ERROR_BIT   | 32, //!< SYSTEMERROR_DC_CLUSTER_JOIN_FAILED に対応。
    DESCRIPTION_DC_CLUSTER_NOT_INITIALIZED     = NEX_ERROR_BIT   | 33, //!< SYSTEMERROR_DC_CLUSTER_NOT_INITIALIZED に対応。
    DESCRIPTION_DC_NOT_CLUSTER_MASTER          = NEX_ERROR_BIT   | 34, //!< SYSTEMERROR_DC_NOT_CLUSTER_MASTER に対応。
    DESCRIPTION_DC_INVALID_STATE_TRANSITION    = NEX_ERROR_BIT   | 35, //!< SYSTEMERROR_DC_INVALID_STATE_TRANSITION に対応。
    DESCRIPTION_DC_CLUSTER_ALREADY_INITIALIZED = NEX_ERROR_BIT   | 36  //!< SYSTEMERROR_DC_CLUSTER_ALREADY_INITIALIZED に対応。

            //n1769: ここにDESCRIPTIONを追加した場合は、ErrorCodeConverterの方もケアして下さい。
};
#endif



// SYSTEMERROR_GEN_NOT_ENOUGH_MEM   //n1769 Not used[2010/10/01 16:04:17]

// SYSTEMERROR_GEN_ILLEGAL_ACCESS   //n1769 Not used[2010/10/01 11:59:21]

// Description: An underlying WIN_FAMILY operation has failed.  The WIN_FAMILY error code can be
// retrieved with SystemError::GetLastExt
// SYSTEMERROR_GEN_WIN32_ERROR  //n1769 CTR TARGETでは使われないはずだが、使っている箇所があった！


// SYSTEMERROR_GEN_INDEX_OOB
/*!
    :category   Result
    @class nn::nex::ResultOutOfBoundary
    @brief 添え字アクセスが範囲外だったことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultOutOfBoundary,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_OUT_OF_BOUNDARY);


// n1769:memo セレクションの重複？  Selectionクラスでしか使われている気配がない。
// SYSTEMERROR_GEN_DUPLICATED_KEY
/*!
    :category   Result
    @class nn::nex::ResultDuplicatedKey
    @brief Selectionに追加しようとしたDOが既に存在していたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDuplicatedKey,
    Result::LEVEL_STATUS,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DUPLICATED_KEY);


// SYSTEMERROR_GEN_ITEM_NOT_DELETED  //n1769:memo Not used[2010/10/01 13:22:35]

// SYSTEMERROR_GEN_ITEM_NOT_FOUND
/*!
    :category   Result
    @class nn::nex::ResultItemNotFound
    @brief 探そうとしていた要素が見つからなかったことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultItemNotFound,
    Result::LEVEL_STATUS,
    Result::SUMMARY_NOT_FOUND,
    DESCRIPTION_ITEM_NOT_FOUND);


// SYSTEMERROR_GEN_END_OF_COLLECTION
/*!
    :category   Result
    @class nn::nex::ResultEndOfCollection
    @brief DOのイテレータにおいて、既にポインタが末端に達していたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultEndOfCollection,
    Result::LEVEL_STATUS,
    Result::SUMMARY_NOT_FOUND,
    DESCRIPTION_END_OF_COLLECTION);

// SYSTEMERROR_GEN_OBJ_NOT_INIT
/*!
    :category   Result
    @class nn::nex::ResultObjectIsNotInitialized
    @brief オブジェクトが未初期化であることを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultObjectIsNotInitialized,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_OBJ_NOT_INIT);

// SYSTEMERROR_GEN_NOT_IMPLEMENTED // n1769:memo Not used[2010/10/01 13:56:59] 当然、使われていてはまずい…。
/*!
    :category   Result
    @class nn::nex::ResultNotImplemented
    @brief 機能が未実装であることを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultNotImplemented,
    Result::LEVEL_FATAL,
    Result::SUMMARY_NOT_SUPPORTED,
    DESCRIPTION_NOT_IMPLEMENTED);


// SYSTEMERROR_GEN_OUT_OF_ID //n1769:memo Not used[2010/10/01 13:59:10]

// SYSTEMERROR_GEN_INTERNAL //n1769:memo Not used[2010/10/01 13:59:10]


// Description: The delay allowed for an operation to occur has expired.
// SYSTEMERROR_GEN_TIMEOUT
/*!
    :category   Result
    @class nn::nex::ResultTimeout
    @brief 許容される遅延時間を超過（タイムアウト）したことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultTimeout,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_TIMEOUT);


//n1769: このシステムエラーはNEX_MULTIPROCESS_SUPPORT環境でのみ有効。
//そもそもNEX_CLIENT_OPTIMIZATION環境では、このエラーを投げるコードは
//コンパイル対象から外されている。よってnn::Result値も作成しない。
//SYSTEMERROR_GEN_CHILDPROCESS_FAILED


// Description: An operation was called in an invalid context.
// SYSTEMERROR_GEN_INVALID_OPERATION
/*!
    :category   Result
    @class nn::nex::ResultInvalidOperation
    @brief 不適切なコンテクストでオペレーションが実行されようとしたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidOperation,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_OPERATION);


// Description: An attempt was made at instantiating more than one $PRODUCTNAME$ object in the same process.
// SYSTEMERROR_GEN_MULTIPLE_INSTANCES
/*!
    :category   Result
    @class nn::nex::ResultMultipleInstances
    @brief NetZオブジェクトを複数個作成しようとしたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultMultipleInstances,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_MULTIPLE_INSTANCES);

// Description: An attempt to register more than one ExceptionHandler was made.
// SYSTEMERROR_GEN_MULTIPLE_EXCEPTION_HANDLER //n1769: memo Not used[2010/10/01 14:17:23]


//n1769: CTRではプロダクトキーの検証は不要のため、このシステムエラーは使われない。
// Description: An attempt to use an expired version of $PRODUCTNAME$ was made.
// SYSTEMERROR_GEN_EXPIRED


// Description: A method was called with an out-of-range parameter.
// SYSTEMERROR_GEN_OUT_OF_RANGE
/*!
    :category   Result
    @class nn::nex::ResultOutOfRange
    @brief 許容範囲外のパラメータをメソッドに渡したことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultOutOfRange,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_ARGUMENT,
    DESCRIPTION_OUT_OF_RANGE);

// SYSTEMERROR_GEN_ASSERTION_FAILED //n1769: memo Not used[2010/10/01 14:32:19]

// SYSTEMERROR_GEN_DEADLOCK_DETECTED //n1769: memo Not used[2010/10/01 14:32:42]

// Description: $PRODUCTNAME$ has not been initialized.
// SYSTEMERROR_GEN_NOT_INITIALIZED   //n1769: memo Not used[2010/10/01 15:11:09]

// Description: An invalid parameter, which is typically NULL, was passed to a method.
// SYSTEMERROR_GEN_INVALID_PARAMETER
/*!
    :category   Result
    @class nn::nex::ResultInvalidParameter
    @brief 例えばNULLポインタのような、無効なパラメータがメソッドに渡されたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidParameter,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_ARGUMENT,
    DESCRIPTION_INVALID_PARAMETER);

// n1769:memo ↓のDescriptionは正しくない気がする。ファイル操作(I/O)が失敗したときに
// このエラーコードが使われているようだ。
// このコードが実際に使われているのはFileクラス。しかもWindows, UNIX環境に限定されている模様。
// NEX_CLIENT_OPTIMIZATIONであれば、"Not used"の気がする。
// Description: An invalid input was passed to a method.
// SYSTEMERROR_GEN_IO_ERROR


//n1769:memo CTR環境ではプロダクトキーの検証はしていない。よってnn::Result値は作成しない。
// Description: The product key used for $PRODUCTNAME$ does not exist or is invalid.
// SYSTEMERROR_GEN_INVALID_PRODUCT_KEY


// n1769:memo 非常に使用箇所が限定されている（Buffer.cppのCopyContentのみ）。
// Description: A buffer extraction tried to go beyond the end of the buffer.
// SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW
/*!
    :category   Result
    @class nn::nex::ResultBufferExtractionOverflow
    @brief Bufferオブジェクトのキャパシティを超過するコピーがなされようとしたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultBufferExtractionOverflow,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_ARGUMENT,
    DESCRIPTION_BUFFER_EXTRACTION_OVERFLOW);

//n1769:memo タイムアウト値が無効。
// Description: The timeout value set is invalid.
// Remarks: This error is returned if DuplicatedObject::Create or DuplicatedObject::Publish are called
// in a singlethread context with a timeout value other than 0.
// See Also: DuplicatedObject::Create, DuplicatedObject::Publish
// SYSTEMERROR_GEN_INVALID_WAIT
/*!
    :category   Result
    @class nn::nex::ResultInvalidWait
    @brief タイムアウト値が無効であることを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidWait,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_WAIT);

// n1769:これも実際に使用されている箇所は少ない。
// DuplicatedObject::ExecuteOperation()ただ一つだけのようだ…。
// Description: Access to the requested resource was refused.
// SYSTEMERROR_GEN_ACCESS_DENIED
/*!
    :category   Result
    @class nn::nex::ResultAccessDenied
    @brief 実行しようとしたシステム操作が拒絶されたことを示します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultAccessDenied,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_WRONG_ARGUMENT,
    DESCRIPTION_ACCESS_DENIED);

// Description: The operation was denied.
// SYSTEMERROR_GEN_OP_DENIED  //n1769: このコードを「セット」しようとしている箇所は見つからなかった。

//n1769:未使用
// SYSTEMERROR_TRANSPORT_UDS
//NN_DEFINE_NEX_RESULT_CONST(
//    ResultUdsFailure,
//    Result::LEVEL_STATUS,
//    Result::SUMMARY_CANCELLED,
//    DESCRIPTION_TRANSPORT_UDS);

//n1769: UDS Clientがセッションを作成しようとしたときや、
// 逆にUDS Masterがセッションに合流しようとしたときに発せられるエラー。
// SYSTEMERROR_TRANSPORT_UDS_INVALID_STATE
/*!
    :category   Result
    @class nn::nex::ResultInvalidUdsStatus
    @brief UDS Clientがセッションを作成しようとしたり、逆にUDS Masterがセッションに合流しようとしたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidUdsStatus,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_ROLE_MISMATCH);

//n1769: UDS Clientがセッションを作成しようとしたときや、
// 逆にUDS Masterがセッションに合流しようとしたときに発せられるエラー。
// SYSTEMERROR_TRANSPORT_UDS_INVALID_STATE
/*!
    :category   Result
    @class nn::nex::ResultInvalidVirtualNetworkStatus
    @brief UDS Clientがセッションを作成しようとしたり、逆にUDS Masterがセッションに合流しようとしたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidVirtualNetworkStatus,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_ROLE_MISMATCH);


// SYSTEMERROR_GEN_INVALID_OPERATION_WARNING
// {Secret}
/*!
    :category   Result
    @class nn::nex::ResultInvalidOperationWarning
    @brief {Secret}
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidOperationWarning,
    Result::LEVEL_STATUS,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_OPERATION_WARNING);

// SYSTEMERROR_OBJDUP_RELIABLE_SEND_FAILURE
//NN_DEFINE_NEX_RESULT_CONST(
//    ResultObjDupReliableSendFailure,
//    Result::LEVEL_STATUS,
//    Result::SUMMARY_CANCELLED,
//    DESCRIPTION_RELIEABLE_SEND_FAILURE);


// SYSTEMERROR_OBJDUP_RELIABLE_SEND_WARNING
// {Secret}
/*!
    :category   Result
    @class nn::nex::ResultObjDupReliableSendWarning
    @brief {Secret}
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultObjDupReliableSendWarning,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_RELIEABLE_SEND_WARNING);


// SYSTEMERROR_OBJDUP_DO_NOT_FOUND
/*!
    :category   Result
    @class nn::nex::ResultObjDupDoNotFound
    @brief 複製オブジェクトが見つからなかったことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultObjDupDoNotFound,
    Result::LEVEL_STATUS,
    Result::SUMMARY_NOT_FOUND,
    DESCRIPTION_OBJDUP_DO_NOT_FOUND);

/*!
    :category   Result
    @class nn::nex::ResultOutOfMemory
    @brief メモリ不足であることを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultOutOfMemory,
    Result::LEVEL_STATUS,
    Result::SUMMARY_OUT_OF_RESOURCE,
    DESCRIPTION_OUT_OF_MEMORY);


// SYSTEMERROR_TRANSPORT_SOCKET_SEND_WARNING
/*!
    :category   Result
    @class nn::nex::ResultSocketSendWarning
    @brief 未使用
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultSocketSendWarning,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_SOCKET_SEND_WARNING);


// Description: ExecChangeDupSet was called with invalid new station.
/*!
    :category   Result
    @class nn::nex::ResultInvalidNewStation
    @brief 無効な新ステーションでExecChangeDupSet()が呼ばれたことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidNewStation,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_NEW_STATION);


/*!
    :category   Result
    @class nn::nex::ResultUdsHostMigrationFailure
    @brief ホストマイグレーション（UDSの親交代機能）に失敗したことを表します。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultUdsHostMigrationFailure,
    Result::LEVEL_STATUS,
    Result::SUMMARY_STATUS_CHANGED,
    DESCRIPTION_HOST_MIGRATION_FAILURE);




/*
Description: An attempt was made to create or join the session more than once.
See Also: Session::CreateSession, Session::JoinSession.
*/
// SYSTEMERROR_DC_CLUSTER_REDEFINITION
/*!
    :category   Result
    @class nn::nex::ResultDcClusterRedefinition
    @brief セッションの作成または参加を、複数回試みようとしました。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterRedefinition,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_REDEFINITION);

/*
Description: An attempt was made to join a session and the station hosting the session
denied the join request.
See Also: Session::JoinSession.
*/
// SYSTEMERROR_DC_CLUSTER_JOIN_DENIED
/*!
    :category   Result
    @class nn::nex::ResultDcClusterJoinDenied
    @brief セッションへの参加を試みましたが、セッションをホストしているステーションが参加要求を拒絶しました。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterJoinDenied,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_DC_CLUSTER_JOIN_DENIED);


/*
Description: The join operation failed due to a transient network failure.
See Also: Session::JoinSession.
*/
// SYSTEMERROR_DC_CLUSTER_JOIN_FAILED
/*!
    :category   Result
    @class nn::nex::ResultDcClusterJoinFailed
    @brief ネットワークの一時的な障害により、参加処理が失敗しました。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterJoinFailed,
    Result::LEVEL_STATUS,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_JOIN_FAILED);


/*
Description: An operation that requires the session to be initialized was called, but
the session had not been initialized beforehand.
See Also: Session::CreateSession, Session::JoinSession
*/
// SYSTEMERROR_DC_CLUSTER_NOT_INITIALIZED
/*!
    :category   Result
    @class nn::nex::ResultDcClusterNotInitialized
    @brief セッションが初期化されていないにもかかわらず、事前にセッションの初期化が必要なオペレーションを呼び出しました。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterNotInitialized,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_NOT_INITIALIZED);


// Description: An operation that must be called on the session master was called on a secondary node.
// SYSTEMERROR_DC_NOT_CLUSTER_MASTER
/*!
    :category   Result
    @class nn::nex::ResultDcNotClusterMaster
    @brief セッションマスターで呼び出されるべきオペレーションが、セッションマスター以外で呼び出されました。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcNotClusterMaster,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_NOT_CLUSTER_MASTER);


// Description: An invalid system state transition occurred.
// SYSTEMERROR_DC_INVALID_STATE_TRANSITION
/*!
    :category   Result
    @class nn::nex::ResultDcInvalidStateTransition
    @brief 無効なシステム状態の遷移が発生しました。
           （注：現時点では、このエラーがライブラリから通知されることは無いはずです。
           もしこのエラーを発見されましたら、弊社まで連絡をお願いいたします）
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcInvalidStateTransition,
    Result::LEVEL_FATAL,
    Result::SUMMARY_NOT_SUPPORTED,
    DESCRIPTION_DC_INVALID_STATE_TRANSITION);


/*
Description: A method that must be called prior to calling Session::CreateSession, Session::JoinSession, or
Session::CreateOrJoinSession was called after any one of these three methods.
See Also: Session::CreateSession, Session::JoinSession, Session::CreateOrJoinSession
*/
// SYSTEMERROR_DC_CLUSTER_ALREADY_INITIALIZED
/*!
    :category   Result
    @class nn::nex::ResultDcClusterAlreadyInitialized
    @brief Session::CreateSession や Session::JoinSession または Session::CreateOrJoinSession の呼び出し前に
           呼び出すべき関数を、これらの関数の実行後に呼び出しました。
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterAlreadyInitialized,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_ALREADY_INITIALIZED);



/*!
    @brief CTRのリザルトコード（nn::Result型）を扱うためのクラスです。
 */
class SystemErrorResult
{
public:
    /*!
    @brief コンストラクタです。特に何もしません。
     */
    SystemErrorResult(void);


    /*!
    @brief デストラクタです。特に何もしません。
     */
    ~SystemErrorResult(void);


    /*!
    @brief nn::Result型への変換を行います。
    @param[in] err  @ref SystemError::GetLast() で取得した値。
    @return 対応するnn::Result型の値が返されます。対応するものがなかった場合は、@ref ResultNotImplemented が返されます。
     */
    static nn::Result Convert(qUnsignedInt32 err);


    /*!
    @brief エラーレベルのリザルトコードかどうかを判定します。
    @param[in] result （NEX APIの）リザルトコード。
    @return NEXリザルトコードかつエラーレベルのresult値であれば、trueが返されます。そうでなければ、falseが返ります。
     */
    static bool IsNexError(nn::Result result);


    /*!
    @brief 警告レベルのリザルトコードかどうかを判定します。
    @param[in] result （NEX APIの）リザルトコード。
    @return NEXリザルトコードかつ警告レベルのresult値であれば、trueが返されます。そうでなければ、falseが返ります。
     */
    static bool IsNexWarning(nn::Result result);


    /*!
    @brief NetZオブジェクトの再初期化が必要なレベルのリザルトコードかどうかを判定します。
    @param[in] result （NEX APIの）リザルトコード。
    @return NEXリザルトコードかつ再初期化レベルのresult値であれば、trueが返されます。そうでなければ、falseが返ります。
     */
    static bool IsReinitializeRequired(nn::Result result);


    /*!
    @brief CTR本体のリセット、あるいは電源Offが必要なレベルのリザルトコードかどうかを判定します。
    @param[in] result （NEX APIの）リザルトコード。
    @return NEXリザルトコードかつCTR本体リセット／電源Offレベルのresult値であれば、trueが返されます。そうでなければ、falseが返ります。
     */
    static bool IsResetRequired(nn::Result result);


    /*!
    @brief アプリケーション、またはライブラリのプログラミングエラーが疑われるリザルトコードかどうかを判定します。
    @param[in] result （NEX APIの）リザルトコード。
    @return NEXリザルトコードかつプログラミングエラーが疑われるresult値であれば、trueが返されます。そうでなければ、falseが返ります。
     */
    static bool IsProgrammingError(nn::Result result);


private:
    // コピーコンストラクタ、代入演算子は封印します。
    SystemErrorResult           (const SystemErrorResult&);
    SystemErrorResult& operator=(const SystemErrorResult&);
};


END_NAMESPACE_NEX


#endif // #ifdef CTR_TARGET // [NEX1][LINUS]
