﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

/* NOTE: This file is read in two different contextes: Imakefile and CPP files. */
/* Regular CPP comments cannot be used in it */

#pragma once

/* This file is used to define optional macros to enable or disable
 some specific functionnality, given the capabilities and target
 selection macros (DEBUG, INTERNAL_VERSION, UNICODE).
 */


/* undef support or enable some optional functionality */

// n2494:Debugビルド時にNEX_DEBUG_VERSIONを定義する
#if defined(NN_BUILD_NOOPT) && !defined(NEX_DEBUG_VERSION)
#define NEX_DEBUG_VERSION
#endif

#if defined(NON_SUPPORT_B_TARGET) || defined(NON_SUPPORT_D_FAMILY) || defined(NON_SUPPORT_A_TARGET)
#undef NEX_GLOBALNEW_DETECTION_ENABLED
#endif

#ifdef NEX_RELEASE_VERSION
#undef NEX_THREAD_SCRAMBLER_SUPPORT
#endif

#if defined(NEX_DEBUG_VERSION) || defined(NEX_DEVELOPMENT_VERSION)
#define NEX_SYSTEMCHECK_ENABLED
#endif

#if defined(NEX_GLOBAL_HEAP_CHECK_ONLY) && defined(NEX_LOCK_RESTRICTIONS_ENABLED)
#undef NEX_LOCK_RESTRICTIONS_ENABLED
#endif

#ifdef SN_COMPILER
#undef NEX_LOCK_RESTRICTIONS_ENABLED
#endif

#ifdef COMPILING_Q_PLATFORM
#define APPLY_PATCHES
#endif

// [NEX1][LINUS] No patches.

#if defined(NEX_WATERMARK_SUPPORT) && !defined(NEX_CLIENT_OPTIMIZATION)
#define NEX_WATERMARK_ENABLED
#endif

#if defined(NEX_GLOBAL_HEAP_CHECK_ONLY)
#undef NEX_WATERMARK_ENABLED
#endif

#if !defined(NEX_CLIENT_OPTIMIZATION)
#define NEX_PERFCOUNTER_ENABLED
#endif

#if defined(CTR_SHIPPING_VERSION) || defined(NEX_GLOBAL_HEAP_CHECK_ONLY)
#undef NEX_PERFCOUNTER_ENABLED
#endif

/* n1589:the following define enable to trace in !NEX_DEBUG_VERSION
#define NEX_ENABLE_TRACES
*/

#if defined(NEX_DEBUG_VERSION) || defined(NEX_ENABLE_TRACES)
#define NEX_TRACE_ENABLED
#endif

#define NEX_LOG_ENABLED

// 製品実機はQLOGを無効にする
#if (defined(CTR_TARGET) || defined(CAFE_TARGET) || defined(NEX_NX_TARGET)) && defined(NEX_RELEASE_VERSION)
#undef NEX_LOG_ENABLED
#endif

#if defined(NO_LOG_SUPPORT)
#undef NEX_LOG_ENABLED
#endif

/* we want to have more traces on the backend. */
#if defined(NEX_TRACE_ENABLED) || defined (LIVE_VERSION) || NEX_DISABLE_USING_CTR_PROFILING_UNIT == 0
#define NEX_JOB_TRACE_INFO_ENABLED
#endif

#ifdef NEX_DEBUG_VERSION
/* It may cause a leak, I believe, in some tests. */
/* n1589 */
/* #define DEBUG_REFERENCES */
#endif


#if defined(NEX_DEBUG_VERSION) && !defined(NON_SUPPORT_A_TARGET) && !defined(NON_SUPPORT_B_TARGET) && !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !defined(CAFEIOP_TARGET) && !defined(NEX_NX_TARGET)
// CTRやCafeではデバッグビルド時にMemoryManager::Headerが大きく増えてしまうのを避けるためヒープデバッグは無効にする
#ifndef NEX_HEAP_DEBUGGING_ENABLED
#define NEX_HEAP_DEBUGGING_ENABLED
#endif
#endif

#if defined(NEX_EXCEPTION_HANDLER_SUPPORT)
#define NEX_EXCEPTION_TRAPPING_ENABLED
#endif

// CafeIOP環境でアプリからビルドするときはメモリ削減用defineを定義
#if defined(CAFEIOP_TARGET) && !defined(COMPILING_Q_PLATFORM)
#ifndef NEX_IOP_SIZE_OPTIMIZE
#define NEX_IOP_SIZE_OPTIMIZE
#endif
#endif

// デバッグビルドにおいて、Python.hインクルード時にデバッグ用のpython用ライブラリ(python27_d.lib)をリンクするのを抑制する
#define SWIG_PYTHON_INTERPRETER_NO_DEBUG

// CTR、Cafe の場合は PrincipalID は 32bit 固定
// NEX_SIGLO_FAMILYは PrincipalID は 64bit 固定
#if defined(CTR_TARGET) || defined(CAFE_TARGET)
#ifdef NEX_USE_PRINCIPALID_64BIT
#error define NEX_USE_PRINCIPALID_64BIT
#endif
#elif defined(NEX_SIGLO_FAMILY)
#ifndef NEX_USE_PRINCIPALID_64BIT
#define NEX_USE_PRINCIPALID_64BIT
#endif
#endif

#if defined(NEX_SIGLO_FAMILY)
#define NEX_USE_WEBSOCKET
#endif


// TCPもしくはWebSocket対応だったらPRUDPLiteをサポートする
#if defined(NEX_TRANSPORT_PROTOCOL_TYPE_TCP_SUPPORT) || defined(NEX_TRANSPORT_PROTOCOL_TYPE_WEB_SOCKET_SUPPORT)
#define NEX_PRUDP_LITE_SUPPORT
#endif

#if defined(LINUX_TARGET)
#define NEX_USE_CLOCK_GETTIME
#endif
