﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#pragma once 

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#ifdef NEX_SIGLO_FAMILY
#include <nn/mem/mem_StandardAllocator.h>
#endif

BEGIN_NAMESPACE_NEX

class ThreadSafeMemoryAllocator: public RootObject
{
public:
    ThreadSafeMemoryAllocator()
#ifdef NEX_SIGLO_FAMILY
        : m_pMemoryBlock(NULL)
#endif
    {}
    ~ThreadSafeMemoryAllocator()
    {
        Finalize();
    }
    void Initialize(size_t size);
    void Finalize();

    void* Allocate(size_t size)
    {
#ifdef NEX_SIGLO_FAMILY
        return m_standardAllocator.Allocate(size);
#else
        return malloc(size);
#endif
    }

    void Free(void* addr)
    {
#ifdef NEX_SIGLO_FAMILY
        return m_standardAllocator.Free(addr);
#else
        return free(addr);
#endif
    }

private:
#ifdef NEX_SIGLO_FAMILY
    nn::mem::StandardAllocator m_standardAllocator;
    void* m_pMemoryBlock;
#endif
};

END_NAMESPACE_NEX
