﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

// For NEX Developer See jwiki:MemoryAllocation for an overview of memory allocation in
// the system.

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/MemAllocator.h>

// STL compiles with level 4 warnings...
#ifdef WIN_FAMILY
#pragma warning( push, 1 )
#endif

#include <map>

#ifdef WIN_FAMILY
#pragma warning( pop ) /* restore previous warning */
#endif

BEGIN_NAMESPACE_NEX

//$ifdef SPARK$
//Group=Lists
//$else$
//Group=Data Types
//$endif$

/*
Description: A map container template.
Remarks: This map inherits all operations from the STL map container template,
std::map, the only difference being that qMap allocates memory using
the Quazal memory allocator.
*/

/*!
    @brief  Map 型コンテナのテンプレートクラスです

    std&#58&#58mapを継承していますが qMap はアロケータに MemoryManager で指定したメモリ確保・開放関数を用いる点が異なります。

    @tparam Key Map 型コンテナのKeyとなるオブジェクト。
    @tparam T Map 型コンテナのValueとなるオブジェクト。
*/
template <typename Key, typename T> class qMap
#if defined(NEX_STD_TREE_PAIR_ALLOCATOR_SUPPORT)
    : public std::map<Key, T, std::less<Key>, MemAllocator<std::pair<const Key,T> > >, public RootObject

#elif defined(NEX_ALLOCATOR_SUPPORT)
    : public std::map<Key, T, std::less<Key>, MemAllocator<T> >, public RootObject

#else
    : public std::map<Key, T>, public RootObject

#endif
{
public:
};

END_NAMESPACE_NEX

