﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/MemAllocator.h>

// STL compiles with level 4 warnings...
// STL compiles with level 4 warnings...
#ifdef WIN_FAMILY
#pragma warning( push, 1 )
#endif

#include <deque>

#ifdef WIN_FAMILY
#pragma warning( pop ) /* restore previous warning */
#endif

BEGIN_NAMESPACE_NEX

//==============================================================================
// Inherits all operations from the std::deque using the appropriate allocator.
//==============================================================================
template <typename T> class qQueue
#ifdef NEX_ALLOCATOR_SUPPORT
    : public std::deque<T, MemAllocator<T> >, public RootObject
{
#else
    :
public std::deque<T>, public RootObject
{
#endif
public:
    qQueue()
#ifdef NEX_ALLOCATOR_SUPPORT
        : std::deque<T, MemAllocator<T> >()
#else
        : std::deque<T>()
#endif
    {
    }

    explicit qQueue(typename std::deque<T>::size_type size)
#ifdef NEX_ALLOCATOR_SUPPORT
        : std::deque<T, MemAllocator<T> >(size)
#else
        : std::deque<T>(size)
#endif
    {
    }

#ifdef NEX_ALLOCATOR_SUPPORT
    typedef typename std::deque<T, MemAllocator<T> >::value_type value_type;
#else
    typedef typename std::deque<T>::value_type value_type;
#endif

    value_type& top()
    {
        return qQueue::front();
    }

    const value_type& top() const
    {
        return qQueue::front();
    }

    void push(const value_type& x)
    {
        qQueue::push_back( x );
    }

    void pop()
    {
        qQueue::pop_front();
    }
};

END_NAMESPACE_NEX

