﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/Socket.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#if defined(NEX_SIGLO_STACK)
// SIGLOのスタック利用時はwinsock2.hをインクルードしない。
#elif defined(WIN_FAMILY)
#ifdef WINSOCK2_STACK_SUPPORT
#include <winsock2.h>
#else
#include <winsock.h>
#endif
#endif

#define IOCC_IDLE                   (qByte)0
#define IOCC_IO_PENDING             (qByte)1
#define IOCC_READY_FOR_COMPLETION   (qByte)2

BEGIN_NAMESPACE_NEX

class IOCompletionContext: public RootObject
{
    friend class IOCompletionNotifier;
    friend StringStream& operator<<(StringStream& oStr, const IOCompletionContext& oComplContext);

protected:

    //n1589
#if 0
    IOCompletionContext()
    {
        Reset();
    }
#else
    IOCompletionContext();
    ~IOCompletionContext();
#endif

    enum { IOTYPE_NONE, IOTYPE_CONNECTING, IOTYPE_SENDING, IOTYPE_RECEIVING };


public:

    void Reset();

    qUnsignedInt32  GetIOSize()
    {
        return m_uiIOSize;
    }

    qUnsignedInt32  *GetIOSizeAddress()
    {
        return &m_uiIOSize;
    }

    void SetIOSize(qUnsignedInt32 uiIOSize)
    {
        m_uiIOSize = uiIOSize;
    }

    Socket::CALL_STATE GetReturnCode()
    {
        return m_uiReturnCode;
    }

    void SetReturnCode(Socket::CALL_STATE uiReturnCode)
    {
        m_uiReturnCode = uiReturnCode;
    }

    qByte *GetBuffer()
    {
        return m_pBuffer;
    }

    qUnsignedInt GetBufferSize()
    {
        return m_uiBufferSize;
    }

    void SetBuffer(qByte *pBuffer, qUnsignedInt uiBufferSize)
    {
        m_pBuffer = pBuffer;
        m_uiBufferSize = uiBufferSize;
#ifdef WINSOCK2_STACK_SUPPORT
        if (pBuffer != NULL)
        {
            m_aWsaBufs[0].buf = (char*)pBuffer;
            m_aWsaBufs[0].len = uiBufferSize;
        }
#endif
    }

    //  IOTYPE accessors
    void  ResetIOType()
    {
        m_byIOType = IOTYPE_NONE;
    }

    qBool IsSending()
    {
        return m_byIOType == IOTYPE_SENDING;
    }

    void SetSending()
    {
        m_byIOType = IOTYPE_SENDING;
    }

    qBool IsConnecting()
    {
        return m_byIOType == IOTYPE_CONNECTING;
    }

    void SetConnecting()
    {
        m_byIOType = IOTYPE_CONNECTING;
    }

    qBool IsReceiving()
    {
        return m_byIOType == IOTYPE_RECEIVING;
    }

    void SetReceiving()
    {
        m_byIOType = IOTYPE_RECEIVING;
    }

    void SetSendAddress(InetAddress *pInetAddress)
    {
        m_InetAddressSend = *pInetAddress;
    }

    void SetReceiveAddress(InetAddress *pInetAddress)
    {
        m_pInetAddressReceive = pInetAddress;
    }

    qByte GetIOType()
    {
        return m_byIOType;
    }

    InetAddress *GetSendAddress()
    {
        return &m_InetAddressSend;
    }

    InetAddress *GetReceiveAddress()
    {
        return m_pInetAddressReceive;
    }

    qBool Pending()
    {
        return m_byState == IOCC_IO_PENDING;
    }
    void SetPending()
    {
        m_byState = IOCC_IO_PENDING;
    }
    qBool ReadyForCompletion()
    {
        return m_byState == IOCC_READY_FOR_COMPLETION;
    }
    void SetReadyForCompletion()
    {
        m_byState = IOCC_READY_FOR_COMPLETION;
        ResetIOType();
    }
    qBool Idle()
    {
        return m_byState == IOCC_IDLE;
    }
    void SetIdle()
    {
        m_byState = IOCC_IDLE;
    }

    UserContext *GetUserContext()
    {
        return &m_oUserContext;
    }

    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

#ifdef WINSOCK2_STACK_SUPPORT
    WSAEVENT        GetEvent()
    {
        return m_wsaOverlapped.hEvent;
    }
    void            SetEvent(WSAEVENT pEvent)
    {
        m_wsaOverlapped.hEvent = pEvent;
    }
    WSAOVERLAPPED   *GetOverlapped()
    {
        return &m_wsaOverlapped;
    }
    WSABUF          *GetWsaBuf()
    {
        return m_aWsaBufs;
    }
#endif

#ifdef BERKELEY_STACK_SUPPORT
    Socket      *GetSocket()
    {
        return m_pSocket;
    }
    void        SetSocket(Socket* pSocket)
    {
        m_pSocket = pSocket;
    }
#endif

private:
    qByte           *m_pBuffer;
    qUnsignedInt    m_uiBufferSize;
    qUnsignedInt32  m_uiIOSize;
    Socket::CALL_STATE  m_uiReturnCode;
    InetAddress     m_InetAddressSend;
    InetAddress     *m_pInetAddressReceive;
    qByte           m_byState;
    qByte           m_byIOType;
    UserContext     m_oUserContext;

#ifdef WINSOCK2_STACK_SUPPORT
    WSAOVERLAPPED   m_wsaOverlapped;
    WSABUF          m_aWsaBufs[1];
#endif
#ifdef BERKELEY_STACK_SUPPORT
    Socket          *m_pSocket;
#endif
};

StringStream& operator<<(StringStream& oStr, const IOCompletionContext& oComplContext);

END_NAMESPACE_NEX

