﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/*!
 :private

 @brief    トランスポートプロトコル種別を扱うクラスです。
 */
class TransportProtocol
{
public:

    /*!
     :private

     @brief    トランスポートプロトコル種別です。

     */
    enum Type
    {
        TYPE_UDP         = 0,            //!< UDP
        TYPE_TCP         = 1,            //!< TCP
        TYPE_WEB_SOCKET  = 2,            //!< WebSocket
        TYPE_COUNT,                      //!< Typeの定義数
        TYPE_UNKNOWN     = 0xffffffff    //!< 無効値
    };

    static const qChar* GetName(Type type)
    {
        SYSTEMCHECK(type < TYPE_COUNT);
        return s_name[type];
    }

    // トランスポートプロトコルが、(Listenしていない)同一のローカルポートから異なる宛先に送信可能かどうかを返す。
    static qBool CanShareLocalPort(Type type)
    {
        SYSTEMCHECK(type < TYPE_COUNT);
        return type == TYPE_UDP;
    }

    // トランスポートプロトコルがコネクション指向かどうか(Connect/Listenが必要かどうか)を返す。
    static qBool IsConnectionOriented(Type type)
    {
        SYSTEMCHECK(type < TYPE_COUNT);
        return type != TYPE_UDP;
    }

    // トランスポートプロトコルがデータグラムによる通信(必ず送信したサイズで受信する)をサポートしているかどうかを返す。
    static qBool IsDatagramSupported(Type type)
    {
        SYSTEMCHECK(type < TYPE_COUNT);
        return type != TYPE_TCP;
    }

    // トランスポートプロトコルが到達保証を持つかどうかを返す。
    static qBool IsReliable(Type type)
    {
        SYSTEMCHECK(type < TYPE_COUNT);
        return type != TYPE_UDP;
    }

    // PRUDPLiteを使うかどうかを返す。
    static qBool IsUsePRUDPLite(Type type)
    {
        SYSTEMCHECK(type < TYPE_COUNT);
        return type != TYPE_UDP;
    }

    static qBool IsNatTraversalSupported(Type type)
    {
        SYSTEMCHECK(type < TYPE_COUNT || type == TYPE_UNKNOWN);

        // TYPE_UNKNOWN(プロトコル指定なし)が実際に使われる事はないが、テストで使われているので有効にする。
        return type == TYPE_UDP || type == TYPE_UNKNOWN;
    }
private:
    static const qChar* s_name[TYPE_COUNT];

};

END_NAMESPACE_NEX
