﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>

BEGIN_NAMESPACE_NEX

class CallContext;

/*!
    @brief      DNS 解決を行うためのクライアントです。
*/
class DnsClient : public RootObject, public NonCopyable
{
public:
    DnsClient();
    virtual ~DnsClient();

    /*!
        @brief      DNS 解決を開始します

        @param[in]  pCC 呼び出し情報。
        @param[in]  domain 解決するドメイン名
        @param[out] resolvedAddress 解決されたアドレスを格納するStringへのポインタ

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    qBool Resolve(CallContext* pCC, const String& domain, String* resolvedAddress);
private:
    qBool GenericDnsResolveMethod();

    String m_domain;
    String* m_pResolvedAddress;
};

END_NAMESPACE_NEX

