﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>

#include <OnlineCore/src/Platform/Core/qList.h>



BEGIN_NAMESPACE_NEX

class EndPoint;
class ConnectionOrientedStream;
class Buffer;

class EndPointGroup: public RootObject
{

    friend class ConnectionOrientedStream;

public:
    typedef void (*Callback)(EndPointGroup*, Buffer *pBuffer, UserContext *pContext);
    typedef qList<EndPointGroup*>           EndPointGroupList;
    typedef qList<EndPointGroup*>::iterator EndPointGroupIterator;
    typedef qList<EndPoint*>                EndPointList;
    typedef qList<EndPoint*>::iterator      EndPointIterator;

    EndPointGroup();
    virtual ~EndPointGroup();

    qResult     AddEndPoint(EndPoint *pEndPoint, qUnsignedInt32 uiTimeout = 30000);
    qResult     AddEndPoint(EndPoint *pEndPoint,
                            const EndPointGroup::Callback *pCallback, const UserContext &oContext,
                            qUnsignedInt32 uiTimeout = 30000);

    qResult     RemoveEndPoint(EndPoint *pEndPoint, qUnsignedInt32 uiTimeout = 30000);
    qResult     RemoveEndPoint(EndPoint *pEndPoint,
                               const EndPointGroup::Callback *pCallback, const UserContext &oContext,
                               qUnsignedInt32 uiTimeout = 30000);


    qResult     AddEndPointGroup(EndPointGroup *pEndPointGroup, qUnsignedInt32 uiTimeout = 30000);
    qResult     AddEndPointGroup(EndPointGroup *pEndPointGroup,
                                 const EndPointGroup::Callback *pCallback, const UserContext &oContext,
                                 qUnsignedInt32 uiTimeout = 30000);

    qResult     RemoveEndPointGroup(EndPointGroup *pEndPointGroup, qUnsignedInt32 uiTimeout = 30000);
    qResult     RemoveEndPointGroup(EndPointGroup *pEndPointGroup,
                                    const EndPointGroup::Callback *pCallback, const UserContext &oContext,
                                    qUnsignedInt32 uiTimeout = 30000);


    qResult     Send(Buffer *pBuffer, qUnsignedInt32 uiFlags = 0);
    qResult     Send(Buffer *pBuffer,
                     const EndPointGroup::Callback *pCallback, const UserContext &oContext,
                     qUnsignedInt32 uiFlags = 0);

    void            Lock();
    void            Unlock();

    EndPointGroupIterator   GetEndPointGroupIterator();
    EndPointIterator        GetEndPointIterator();

protected:
    EndPointGroup(ConnectionOrientedStream *pStream);

protected:
    EndPointList        m_oEndPointList;
    EndPointGroupList   m_oEndPointGroupList;
    CriticalSection     m_csLock;
    ConnectionOrientedStream *          m_pAssociatedStream;
};


END_NAMESPACE_NEX

