﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/NetFaults.h>

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief NetFaultReasonをqResultに変換するコンバータです。
 */
class FaultReasonConverter : public RootObject
{
public:
    /*!
        @internal
        @brief NetFaultReasonをqResultに変換します。

        @param faultReason 変換元。
        @param defaultResult 変換テーブルにない値(NET_FAULT_UNKNOWNなど)だった場合の変換後の値。
        @return 変換されたqResult。
     */
    static qResult ConvertFaultReasonToQresult(NetFaultReason faultReason, const qResult& defaultResult);

    /*!
        @internal
        @brief NetFaultReasonをqResultに変換します。(一般用途向け)

        @param faultReason 変換元。
        @return 変換されたqResult。
     */
    static qResult ConvertFaultReasonToQresultForGeneral(NetFaultReason faultReason);

    /*!
        @internal
        @brief NetFaultReasonをqResultに変換します。(RV)

        @param faultReason 変換元。
        @return 変換されたqResult。
     */
    static qResult ConvertFaultReasonToQresultForRv(NetFaultReason faultReason);
};

END_NAMESPACE_NEX
