﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Core/SystemPorts.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Core/LogicalClock.h>

BEGIN_NAMESPACE_NEX

class RootTransport;
class StationURL;
class Buffer;
class PRUDPMessageInterface;
class PacketEncDecInf;
class InetAddress;
class PacketOut;
class SignatureBytes;

class HighLevelStream: public Stream
{
public:
    HighLevelStream(Stream::Type oType, RootTransport *pTransport = NULL);
    virtual ~HighLevelStream();

    qBool Listen(Stream::ID idStream);

    virtual qBool ReceiveIncomingPacket(qUnsignedInt16 uiPort, Stream::ID idStream, Packet *pPacket);
    virtual void DoWork(const Time*) {}

    virtual void ReceiveIncomingMsg(const InetAddress &oSourceAddress, Stream::Type oType, Stream::ID idStream, Buffer *pBuffer, Time ioTime, qUnsignedInt16 ui16Port);

    virtual qResult Send(const InetAddress &oDestinationAddress, Stream::Type oType, Stream::ID idStream, Buffer *pBuffer, qUnsignedInt8 uiTTL, qUnsignedInt16 ui16Port = 0);

    RootTransport *GetTransport()
    {
        return m_pTransport;
    }
    Stream::ID  GetID()
    {
        return m_idStream;
    };

    qBool IsListened();

protected:
    qByte m_version;
    qBool m_isEnabledSignature;
    qBool m_isSignatureHeaderOnly;
    RootTransport *m_pTransport;
private:
    void CalcSignature(PacketOut *pPacket);
    qBool ExpectedSignature(Packet *pPacket, SignatureBytes *expectedSign);
    Stream::ID m_idStream;
    PRUDPMessageInterface* m_pPRUDPMessageInterface;
    PacketEncDecInf* m_pPacketEncDec;
    qByte m_sessionId;
    LogicalClock16 m_seqId;
};

END_NAMESPACE_NEX

