﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>
#include <OnlineCore/src/Core/CallContext.h>

BEGIN_NAMESPACE_NEX

class JobResolveRelayInfo;

class JobStartNATSession : public StepSequenceJob, public NonCopyable
{
public:
    static const qUnsignedInt32 DEFAULT_JOB_DELAY_INTERNAL_THREAD = 10;
    JobStartNATSession(CallContext* associatedCallContext);
    virtual ~JobStartNATSession();

    //Jobをセットしてスタートさせる。falseの場合は、RELEASEREFでこのオブジェクトを削除 trueの場合は、Schedulerが削除するので、削除は不要
    //スタート前にGetCallContext()で取得できるCallContextがすでに初期化されてCallContextRegisterに登録されている必要がある。
    qBool StartJob();

    void CompleteJob(const qResult& result);
    void StepReady();
    void StepCallCompletion();
    void StepDNSCallCompletion();
    void StepFinish();

    //newしたあと、このGetCallContext()を使って、CallContextを初期化する必要がある
    CallContext* GetCallContext() { return &m_callContext;}


private:

    CallContext* m_AssociatedCallContext;
    CallContext m_callContext;
    qUnsignedInt32 m_AssociatedCallID;

    CallContext m_resolveRelayCallContext;
    JobResolveRelayInfo* m_pJobResloveRelayInfo;
    CallContext m_dnsResolveCallContext;

};

END_NAMESPACE_NEX


