﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/EmulationDevice.h>


BEGIN_NAMESPACE_NEX

//Group = Network Emulation

/*
Summary: This class defines an output EmulationDevice.
Remarks: An output emulation device objct can be referenced using
RootTransport::GetOutputEmulationDevice. For example,

    RootTransport::GetInstance()->GetOutputEmulationDevice();

You must call EmulationDevice::Enable before using an emulation device.

The OutputEmulationDevice class has no specific methods of its own
but inherits all methods from the EmulationDevice class.

See Also: EmulationDevice, InputEmulationDevice, RootTransport, /Network Emulation/
in the /Net-Z User Guide/ for an example of how to use this class.
*/


/*!
 * @brief   出力ネットワークデバイスを定義しているクラスです。デバッグ用機能です。
 *
 * 出力ネットワークデバイスは、RootTransport::GetOutputEmulationDeviceで参照可能で、
 * RootTransport::GetInstance()->GetOutputEmulationDevice()とすることで取得可能です。
 *
 * エミュレーションデバイスを使う前には、
 * RootTransport::GetInstance()->GetOutputEmulationDevice()->Enable()
 * を呼び出す必要があります。
 *
 * このクラスは、EmulationDeviceの派生クラスで、特別なメソッドは定義されておりません。
 *
 * @see EmulationDevice, OutputEmulationDevice
 * @see RootTransport::GetOutputEmulationDevice
 */

class OutputEmulationDevice: public EmulationDevice
{

};


END_NAMESPACE_NEX

