﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Core/Buffer.h>

BEGIN_NAMESPACE_NEX

//group=Transport Protocol

/*
Description: This class enables non-$PRODUCTNAME$ packets to be accepted by the system rather than
automatically being dropped.
Remarks: In order for non-$PRODUCTNAME$ packets to be accepted by the system rather than automatically
being dropped, you need to sub class this class and then implement the AcceptIncomingPacket method to
indicate which non-$PRODUCTNAME$ packets should be accepted and those which should be still dropped.
Once implemented, your PacketRecognizer class must then be registered to the RootTransport via
RootTransport::RegisterPacketRecognizer.

An example of when you may need to implement a PacketRecognizer is if you wish to do NAT negotiation
with a lobby other than Quazal's Rendez-Vous. For details on how to send, as opposed to receive,
non-$PRODUCTNAME$ packets, refer to RootTransport::GetSocketHandle.
See Also: RootTransport
*/
class PacketRecognizer: public RootObject
{
public:
    PacketRecognizer() {}
    virtual ~PacketRecognizer() {}

    //group=Methods

    /*
    Description: Dictates which incoming packets will be accepted by this PacketRecognizer.
    Parameters:
    pBuffer: A pointer to a buffer that contains the packet data.
    pStationURL: A pointer to the StationURL of the Station that sent the packet.
    Returns: true if the incoming packet is accepted, false if not or an error occurred.
    Remarks: For non-$PRODUCTNAME$ packets to be accepted by the system rather than automatically
    being dropped this method must return true. For the system to use a PacketRecognizer, once implemented
    it must then be registered to the RootTransport via RootTransport::RegisterPacketRecognizer.
    See Also: RootTransport::RegisterPacketRecognizer
    */
    virtual qBool AcceptIncomingPacket(Buffer *pBuffer, StationURL *pStationURL) = 0;
};

END_NAMESPACE_NEX

