﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Core/Key.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Transport/Interface/PacketEncDec.h>

BEGIN_NAMESPACE_NEX

class Packet;
class EncryptionAlgorithm;

class RC4PacketEncDec : public PacketEncDecInf
{
    friend class OnlineCore_Transport_Interface_RC4PacketEncDec_Test;
public:
    RC4PacketEncDec();
    ~RC4PacketEncDec();

    virtual qBool Encode(Packet* pPacket, Stream::Type streamType, qBool needCopy = true);
    virtual void Decode(Packet* pPacket);
    virtual void SetEncryptionKey(const Key &oKey);
    virtual void SetReliableEncryptionKey(const Key &oKey, qByte sessionID, qByte peerSessionID);
    virtual void SetUnreliableEncryptionKey(const Key &oKey);
    virtual Key *GetEncryptionKey();

    virtual void DecideMaxUserReliableSubStreams(qUnsignedInt16 maxUserReliableSubStreams);

    virtual void ReinitUserReliableSubStreams();


private:
    void GenerateKey(const Key &baseKey,
                     const qByte* additionalKey1, const qByte* additionalKey2, Key *targetKey);


    qVector<EncryptionAlgorithm*> m_encoder;
    qVector<EncryptionAlgorithm*> m_decoder;
    EncryptionAlgorithm* m_pUnreliableEncoderDecoder;
    Key m_oKey;
    Key m_oUnreliableEncryptionKey;
    Key m_oUnreliableBroadcastEncryptionKey;
    qBool m_isMultiChunk;
    qUnsignedInt16 m_maxUserReliableSubStreams;

};

END_NAMESPACE_NEX

