﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n2788

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
//#include <Platform/Stack/Core/SocketDriver.h>
#include <OnlineCore/src/Transport/Interface/VirtualSocketDriver.h>

BEGIN_NAMESPACE_NEX

class TransportAdapter;

/*!
@brief  ローカル通信連携機能を使用するための基底クラスです。このクラスをそのまま使用することはできません。

        このクラスはプラットフォームごとに異なるローカル通信機能を抽象化したクラスです。
        実際にアプリケーションが使用するのは、プラットフォームごとに用意される
        「 VirtualNetwork を継承した派生クラス」になります。

        「 VirtualNetwork 派生クラス」のインスタンスを @ref VirtualSocketDriver に登録することで
        NEX の通信が全てローカル通信上で行われることになり、インターネット P2P 通信と同様に
        P2P 通信機能が使用できるようになります。
*/
class VirtualNetwork: public RootObject
{
public:
    /*!
    @brief  コンストラクタ【使用不可】

            このクラスを直接使用しないでください。

            実際にローカル通信連携機能を使用する場合は「 VirtualNetwork を継承した派生クラス」の
            インスタンスを生成してください。

            「 VirtualNetwork 派生クラス」のインスタンスを @ref VirtualSocketDriver に登録することで
            NEX の通信が全てローカル通信上で行われるように設定されます。
    */
    VirtualNetwork() {};

    /*!
    @brief  デストラクタ【使用不可】

            このクラスを直接使用しないでください。

            実際にローカル通信連携機能を使用する場合は「 VirtualNetwork を継承した派生クラス」の
            インスタンスを生成してください。
    */
    virtual ~VirtualNetwork() {};

    virtual TransportAdapter* GetTransportAdapter()=0;
    virtual void DestroyTransportAdapter()=0;

    virtual qBool ReceiveFrom(
        qByte* pBuffer,
        qUnsignedInt uiBufferSize,
        SocketDriver::InetAddress* pSourceAddress,
        qUnsignedInt* puiNbBytesRecvd,
        SocketDriver::SocketFlag flags = SocketDriver::NonBlock
    )=0;

    virtual qBool SendTo(
        qByte* pBuffer,
        qUnsignedInt uiBufferSize,
        const SocketDriver::InetAddress &refDestinationAddress,
        qUnsignedInt* puiNbBytesSent
    )=0;

    virtual qBool Shutdown()=0;

    virtual VirtualMessageIn* ProcessReceiveMessage()=0;
    virtual qBool HandlingMessage(
        VirtualMessageIn* pMessage,
        SocketDriver::InetAddress* pSourceAddress
    )=0;

    virtual qUnsignedInt16 GetReceivePort()=0;

    virtual qBool CanBecomeSessionMaster()=0;
    virtual qBool CanBecomeSessionClient()=0;

    virtual qBool IsFatalReliableSendBufferFull(NODE_ID_TYPE nodeId) = 0;
    virtual qBool IsDisconnected()const
    {
        return false;
    }
};

END_NAMESPACE_NEX

