﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/RandomNumberGenerator.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

class Packet;
class PacketIn;
class PacketOut;
class Key;
class ByteStream;
class TransportSignatureGenerator;

class PRUDPMessageV1 : public PRUDPMessageInterface, public RootObject
{
    friend class  OnlineCore_Transport_PRUDPMessageV1_OptionUnPackTest_Test; //For UnitTest
public:
    static const size_t SIGNATURE_SIZE = 16; /*! 署名サイズ */
    static const size_t HEADER_SIZE = 30; /*! ヘッダサイズ */
    static const size_t MAX_OPTION_SIZE = 22; /*! 最大オプションサイズ */
    static const size_t MAX_SYN_CONNECT_OPTION_SIZE = 36; /*! SynやConnect時の最大オプションサイズ */

    enum OptionType
    {
        SUPPORTED_FUNCTIONS = (qByte)0x00,
        CONNECTION_SIGNATURE = (qByte)0x01,
        PARTNO = (qByte)0x02,
        UNRELIABLE_SEQ = (qByte)0x03,
        MAX_RELIABLE_USER_SUBSTREAMS = (qByte)0x04,
        DISCONNECT_REASON = (qByte)0x05,
        PADDING  = (qByte)0xff
    };
    
    static const qUnsignedInt32 SUPPORTED_FLAG_DISCONNECT_REASON_MINOR_VERSION = 5;

    PRUDPMessageV1();

    virtual ~PRUDPMessageV1();

    virtual qBool CalcSignature(PacketOut *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                                TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    virtual void CalcSignature(Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                               TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    virtual SignatureMethod DecideSignatureMethod(const Packet *pPacket, Stream::Type type, qBool hasEndPoint);

    virtual SignatureMatchPattern CalcExpectedSignature(const Packet *pPacket, SignatureMethod method, const Key *pKey,
            TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &expectedSignature);;

    virtual qBool Unpack(ByteStream *pByteStream, PacketIn *pPacketIn, size_t *pSizeUnpacked = NULL);

    virtual qBool FastUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, size_t *pPayloadSize = NULL);

    virtual qBool Pack(PacketOut *pPacketOut, ByteStream *pByteStream);

    struct SubHeader
    {
        qByte mSourceVPort;
        qByte mDestinationVPort;
        qUnsignedInt16 mPacketTypeFlags;
        qByte mSessionID;
        qByte mSubStreamID;
        qUnsignedInt16 mSeqId;
    };

    struct Header //署名算出用Headerクラス Protocol IDと署名フィールドを含まず リトルエンディアン
    {
        qByte mVersion;
        qByte mOptionSize;
        qUnsignedInt16 mPayloadSize;
        SubHeader mSubHeader;
    };

private:
    qBool OptionUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt optionSize);
    qBool OptionBuilder(PacketOut *pPacketOut);
    qBool OptionSubHeaderBuilder(PacketOut *pPacketOut);
    void SubHeaderBuilder(PacketOut *pPacketOut);
    qBool HeaderParse(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt &optionSize, qUnsignedInt &payloadSize);
    void CalcSignatureHelper(const Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                             const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    RandomNumberGenerator mRng;
    CriticalSection mCsRng;
};


END_NAMESPACE_NEX

