﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n1589

#pragma once

#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>

BEGIN_NAMESPACE_NEX

// ライブラリ内で使用する型
typedef void (*PRUDPUserReceiveCallback)(ConnectionID sourceId, qByte* buffer, size_t size);//DirectTransport互換用
typedef void (*PRUDPUserPrivateReceiveCallback)(ConnectionID sourceId, Buffer* pBuffer, qUnsignedInt32 uiFlags, qByte subStreamID);

class PRUDPUserProtocol: public EndPointEventHandler
{
public:
    static void AcquireInstance();
    static void ReleaseInstance();

    static PRUDPUserProtocol* GetInstance()
    {
        InstanceControl* pInstance = InstanceControl::FindInstance(IC_TYPE_PRUDPUSERPROTOCOL, PseudoSingleton::GetCurrentContext());
        if (pInstance != NULL)
        {
            return static_cast<PRUDPUserProtocol*>(pInstance->GetDelegatorInstance());
        }
        else
        {
            return NULL;
        }
    }

    PRUDPUserProtocol();
    virtual ~PRUDPUserProtocol();

    void RegisterUserReceiveCallback( PRUDPUserReceiveCallback callback );
    void UnregisterUserReceiveCallback();
    // 内部コールバック登録・解除
    /*
    @brief PrivateReceiveCallbackを登録します。非PrivateReceiveCallbackとは、どちらかでしかコールバックされません。NEXライブラリ内でのみ使用します。
    @param[in] callback 登録するPrivateReceiveCallback
    */
    void RegisterUserPrivateReceiveCallback( PRUDPUserPrivateReceiveCallback callback );
    /*
    @brief PrivateReceiveCallbackを登録解除します。NEXライブラリ内でのみ使用します。
    */
    void UnregisterUserPrivateReceiveCallback();

    virtual void FaultDetection(EndPoint *pEndPoint, qUnsignedInt32 uiFault);
    virtual void PeerDisconnected(EndPoint *pEndPoint);
    virtual void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qByte subStreamID);
    // パケットのTypeFlagsを追加でもらう受信メソッド
    /*
    @brief パケット受信時にコールされるメソッドです。同名メソッドとの違いは、パケットのTypeFlagsを受け取るように拡張しています。
    */
    virtual void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qUnsignedInt32 uiPacketTypeFlags, qByte subStreamID);
    qResult Send(EndPoint* pEndPoint, const qByte* buffer, size_t size, qUnsignedInt32 uiFlags, qByte subStreamID, qUnsignedInt32 uiCallID = 0);

#ifdef NEX_TRACE_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif

protected:
    PseudoSingleton m_psInstance;
    //      static PRUDPUserProtocol* s_pUserProtocolInstance;
    PRUDPUserReceiveCallback m_pfReceiveCallback;
    // PrivateReceiveCallbackの格納先 Register/UnregisterPrivateReceiveCallback()で操作する。
    PRUDPUserPrivateReceiveCallback m_pfPrivateReceiveCallback;
};

END_NAMESPACE_NEX

