﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>

#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Core/LogicalClock.h>

BEGIN_NAMESPACE_NEX

class PacketIn;

class PacketDispatchQueue: public RootObject
{

public:
    typedef qMap<LogicalClock16,PacketIn*> PacketMap;

    PacketDispatchQueue();
    ~PacketDispatchQueue();

    void        Purge();

    void        Queue(PacketIn *pPacket);
    PacketIn    *GetNextToDispatch();
    void        Dispatched(PacketIn *pPacket);

    const PacketMap& GetPacketMap()
    {
        return m_oPacketMap;
    }

    const LogicalClock16& GetNextIdToPop()
    {
        return m_lcNextIdToPop;
    }

#if NEX_ENABLE_COLLATED_ACK
    qBool Complete(LogicalClock16 lcId);
#endif
#ifdef NEX_TRACE_ENABLED // n2370
    void  Trace();
#else
    void  Trace() {}
#endif // NEX_TRACE_ENABLED

    qBool       IsAnyPacketProcessed()
    {
        return (m_lcNextIdToPop != LogicalClock16(1));
    }

public:
    PacketMap   m_oPacketMap;
    LogicalClock16                      m_lcNextIdToPop;
#ifdef NEX_WATERMARK_ENABLED
    static WaterMark                s_wmDispatchQueue;
#endif
};

END_NAMESPACE_NEX

