﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Core/LogicalClock.h>

BEGIN_NAMESPACE_NEX

//n1589:test:off
#define NEX_EXPERIMENT_SLIDING_WINDOW_ALWAYS_PUSH_FAIL 0

class PacketOut;
class PacketEncDec;
class Stream;

class SlidingWindow: public RootObject
{

public:
    typedef qMap<LogicalClock16,PacketOut*>::iterator Iterator;

#if NEX_EXPERIMENT_SLIDING_WINDOW_ALWAYS_PUSH_FAIL
    static qBool s_bEnableAlwaysPushFail;
#endif

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    SlidingWindow(qUnsignedInt16 ui16WindowSize, size_t ui32MaxMapSize);
#else
    SlidingWindow(qUnsignedInt16 ui16WindowSize);
#endif
    ~SlidingWindow();
    void                Purge();

    //n2690

    qBool               Push(PacketOut *pPacket);
    void                Acknowledged(LogicalClock16 lcSeqId, qMap<qUnsignedInt32, qUnsignedInt32>& unackedCount);
    PacketOut           *GetNextToSend();
    PacketOut           *GetPacket(LogicalClock16 lcSeqId);
    qBool               ReadyToSend();
    qBool               Empty();

    //n1589:memo:DataPending
    //  The purpose of keeping track of number of DATA packets
    //  is to make sure that we send all reliable data prior
    //  to send a DISCONNECT packet.
    qBool               DataPending();
    qUnsignedInt32      NbDataPending();
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    /*! @internal
     @brief スライディングウィンドウに積められるパケットの残り数を返します。
            返り値が0以下の時、@ref Push はエラーになります。
     */
    qInt32              GetFreeBufferNum() const; // Reliableパケットを残り何個載せられるか
#endif // NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION

    Iterator            AcquireIterator();
    PacketOut           *GetPacket(Iterator it);
    void                ReleaseIterator();
    LogicalClock16      LowestSeq() const
    {
        return m_lcLeftEdge;
    };

    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);

    //n1589
    size_t GetCurrentPacketCount() const
    {
        return m_oPacketMap.size();
    }

    //n1589
protected:
    qMap<LogicalClock16,PacketOut*> m_oPacketMap;
    qUnsignedInt16                  m_ui16WindowSize;
    //n1589:memo:次にPushするシーケンスID = Mapの位置
    LogicalClock16                  m_lcNextSeqId;
    LogicalClock16                  m_lcRightEdge;
    LogicalClock16                  m_lcLeftEdge;
    qUnsignedInt32                  m_uiNbDataPacketsPending;

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    size_t                  m_uiMaxMapSize;
#endif

#ifdef NEX_WATERMARK_ENABLED
    static WaterMark                            s_wmSlidingWindow;

private:

    qUnsignedInt32                              m_uiSlidingWindowID;

    static void CheckThreshold(qChar* szCallerName);
#endif

private:
    // ?? for some reason the compiler is trying to generate this method and
    //fails. This prevents it from generating it in the first place.
    SlidingWindow& operator= (SlidingWindow& oOther);
};

END_NAMESPACE_NEX

