﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/qSet.h>
#include <OnlineCore/src/Platform/Core/qMultiMap.h>
#include <OnlineCore/src/Platform/Core/WaterMark.h>
#include <OnlineCore/src/Transport/UDP/Timeout.h>
#include <OnlineCore/src/Transport/UDP/PacketDefines.h>

BEGIN_NAMESPACE_NEX

class PacketOut;
class EndPoint;

class TimeoutManager: public RootObject
{
    typedef qMultiMap<Time,PacketOut*>  TimeoutMap;
public:
    TimeoutManager();
    ~TimeoutManager();

    void            SchedulePacketTimeout(PacketOut *pPacket);
    qBool           CancelPacketTimeout(PacketOut *pPacket);
    void            ServicePacketTimeouts(const Time& currentTime);
#ifdef NEX_SYSTEMCHECK_ENABLED
    void            CheckEndPointPackets(EndPoint *pEndPoint);
#endif
    void SetQuickTimeout(); // タイムアウトを早める状態にする
    void SetPacketQuickTimeout(PacketType); // 特定の種別のパケットのタイムアウトを早める
    void ForceSetPacketTimeouts(Time time);
#ifdef NEX_TRACE_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags);
#endif  // NEX_TRACE_ENABLED

private:
    qSet<PacketOut*>                        m_setPacket;
    TimeoutMap                              m_mmapTimeout;
#ifdef NEX_WATERMARK_ENABLED
    WaterMark                               m_wmTimeoutQueue;
#endif
    qBool m_quickTimeout;

    // タイムアウトしたPacketOutを一時的に退避するvector
    // ServicePacketTimeouts()で毎回宣言してしまうと頻繁にメモリ確保が発生するのでメンバに持つ
    qVector<PacketOut*> m_tempPacketList;
};

END_NAMESPACE_NEX

