﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>


BEGIN_NAMESPACE_NEX

class RootTransport;
class Buffer;
class StationURL;
class Network;
class RelayStream;
struct RelayPingProtocolHeader;

class Relay : public RootObject
{
public:
    Relay();
    virtual ~Relay();

    void SetTransport( RootTransport *pTransport );
    void EnableRelay(qBool bEnable = true);
    qUnsignedInt32 GetMTU();
    qBool HasRelay(const InetAddress *pAddress, const InetAddress *pRelayAddr = NULL);

    qBool ShouldRelay(const InetAddress *pAddress);
    qBool RelayMessage( Buffer *pBuffer, const InetAddress *pAddress, qUnsignedInt16 uiPort);
    qBool PingMessage(Buffer *pBuffer, const RelayPingProtocolHeader* pHeader,const InetAddress *pAddress, qUnsignedInt16 uiPort);

    //n1589:memo
    qBool AddRelay( const InetAddress &rDest, const InetAddress &rRelayAddr, qBool bOverride = false );

    RelayStream &GetStream()
    {
        return *m_pRelayStream;
    }


    qBool IsEnabledRelay()
    {
        return m_bRelayEnabled;
    }

    void SetSrcPublicAddress(const InetAddress &addr)
    {
        m_srcPublicAddress = addr;
    }

    void SetSrcPublicAddress(const StationURL &url);

    const InetAddress& GetSrcPublicAddress()
    {
        return m_srcPublicAddress;
    }

    void SetGameServerID(qUnsignedInt32 gameServerID);
    qUnsignedInt32 GetGameServerID();
    void SetRelayGatheringID(qUnsignedInt32 ID);
    qUnsignedInt32 GetRelayGatheringID();
    void Clean();
    qBool Exists();

private:
    // Cannot copy
    Relay( const Relay & );
    Relay &operator=(const Relay &);

    void UnregisterStream();

    RelayStream* m_pRelayStream;
    qBool m_bRelayEnabled;
    qBool m_bStreamRegistered;

    Network* m_pNetwork;

    InetAddress m_srcPublicAddress;
};

END_NAMESPACE_NEX

