﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/UDP/PacketOut.h>
#include <OnlineCore/src/Transport/UDP/Timeout.h>

#ifdef NEX_CLIENT_PBPOOL // クライアントのみ

BEGIN_NAMESPACE_NEX

/*! @internal
 @brief Timeoutオブジェクトをパケットバッファプールでメモリ確保するPacketOutクラスです。
        qNew(RootObject::POOL_PB_MANAGEMENT)で確保します。
        パケットバッファプールで確保するパケットについてはPacketOutでなく、こちらのクラスを使用します。
        new後のインスタンスがNULLまたはPacketOut::IsValid()==falseの場合はメモリ確保失敗として、枯渇エラーとする処理が必要です。
 */

class PacketBufferPacketOut: public PacketOut
{
public:
    PacketBufferPacketOut(PRUDPEndPoint *pEndPoint,
                          PacketType byPacketType,
                          qUnsignedInt16 uiFlags = FLAG_NONE,
                          Buffer *pBuffer = NULL,
                          qUnsignedInt32 uiCallID = 0
                         ) : PacketOut()
    {
        // PacketOutとは異なり、パケットバッファプールで確保したTimeoutクラスをセットする
        Timeout* pTimeout = (uiFlags & FLAG_ACK_WANTED) ? (qNewMayFail(RootObject::POOL_PB_MANAGEMENT) Timeout()) : NULL;
        Initialize(pEndPoint, byPacketType, uiFlags, pBuffer, uiCallID, pTimeout);
    }
    
    PacketBufferPacketOut(qByte version, RelayType byPacketType,
                          qUnsignedInt16 uiFlags = FLAG_NONE,
                          Buffer *pBuffer = NULL
                         ) : PacketOut()
    {
        SetVersion(version);
        Initialize(NULL, static_cast<PacketType>(byPacketType), uiFlags, pBuffer, 0, NULL);
    }
    
    virtual ~PacketBufferPacketOut()
    {
    }
};

END_NAMESPACE_NEX

#endif // NEX_CLIENT_PBPOOL // クライアントのみ

