﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/TransportBufferSet.h>
#include <OnlineCore/src/Transport/UDP/PacketQueue.h>
#include <OnlineCore/src/Transport/UDP/PacketHistoryChainDefine.h>
#include <OnlineCore/src/Transport/UDP/NetworkEmulator.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/Socket.h>
#include <OnlineCore/src/Platform/Stack/Core/IOCompletionContext.h>
#include <OnlineCore/src/Platform/Stack/Core/IOCompletionNotifier.h>

BEGIN_NAMESPACE_NEX

class PacketIn;
class PacketOut;
class RootTransport;
class ByteStream;
class PRUDPMessageInterface;

class QueuingSocket: public Socket
{
public:
    QueuingSocket(TransportProtocol::Type transportProtocolType, RootTransport *pTransport, qUnsignedInt32 uiMtu);
    virtual ~QueuingSocket();

    virtual qBool Bind(InetAddress *pBindAddress, qBool bReuse = false);
    virtual qInt Send(const Buffer& sendBuffer, const InetAddress& inetAddress);
    void DequeueAndSendOutgoingBuffer();
    void RecvAndQueueIncomingBuffer(qUnsignedInt32 mtu);
    Buffer *CreateBufferFromPacketQueue(PacketQueue *pPacketQueue, PacketHistoryChain *pHistoryPacketQueue, qUnsignedInt32 uiMTU );
    static void RetransmitFromHistoryPacketQueue(PacketQueue *pPacketQueue, PacketHistoryChain *pHistoryPacketChain, qUnsignedInt16 localPort);
    virtual qInt Recv(Buffer* receivedBuffer, InetAddress* pInetAddress);   // 未実装
    virtual qInt RecvNonBlock(Buffer* receivedBuffer, InetAddress* pInetAddress);
    virtual qBool IsRecvDataExists() const;

    qUnsignedInt FillPacketQueueFromBuffer(Buffer *pBuffer, const InetAddress *pSourceAddress, PacketQueue *pQueue, TransportStreamManager *pTransportStreamMnager, qBool* pIsNoBuffer);
    qUnsignedInt32 QueueSize() const;
    qBool InputFlooded();
    qBool OutputFlooded();

    qUnsignedInt32 GetRefCount()
    {
        return m_uiRefCount;
    }
    void IncreaseRefCount()
    {
        m_uiRefCount++;
    }
    void DecreaseRefCount()
    {
        m_uiRefCount--;
    }

    /*! @internal
     @brief 受信パケットについて、パケットバッファプールで確保するかを指定する。
            サーバー通信に使用するポートに対してfalseを設定する。
            デフォルトtrue。
     */
    void SetUsePacketBufferPool(qBool flag)
    {
        m_usePacketBufferPool = flag;
    }
    qBool GetUsePacketBufferPool() const
    {
        return m_usePacketBufferPool;
    }

protected:
    virtual qInt SendBuffer(const Buffer& sendBuffer, const InetAddress& inetAddress);
    virtual qInt RecvBuffer(Buffer* receivedBuffer, InetAddress* pInetAddress);
    PacketIn* ExtractIncomingPacket(TransportStreamManager *pTransportStreamManager,
                                    const InetAddress *pSourceAddress,
                                    Buffer *pBuffer,
                                    qBool* pIsNoBuffer);
    qInt OutputQueue(const Buffer& sendBuffer, const InetAddress& inetAddress);
    void InputQueue(Buffer *pBuffer, const InetAddress& oSourceAddress)
    {
        m_oInputEmulator.Queue(pBuffer, oSourceAddress);
    }
    void InputDequeue(Buffer *pBuffer)
    {
        m_oInputEmulator.Dequeue(pBuffer);
    }
    void OutputDequeue(Buffer *pBuffer)
    {
        m_oOutputEmulator.Dequeue(pBuffer);
    }

    qUnsignedInt32 m_uiRefCount;
    NetworkEmulator m_oOutputEmulator;
    NetworkEmulator m_oInputEmulator;

    PRUDPMessageInterface* m_pPRUDPMessageInterface;

    PacketQueue m_sendingPacketQueue; //n2967: PacketQueue中のWaterMarkがスレッドアンセーフであるため一時領域をメンバに確保

    qBool m_usePacketBufferPool; // 受信パケットをパケットバッファプールで確保するか

};

//n2967:過去履歴バンドリング用
void PacketHistoryChainPurge(PacketHistoryChain* chain);
PacketHistoryChain::iterator PacketHistoryChainDequeue(
    PacketHistoryChain *chain, PacketHistoryChain::iterator it);
void PacketHistoryChainQueueFront(PacketHistoryChain *chain, PacketOut *pPacket);


END_NAMESPACE_NEX

