﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/DataHolder.h>
#include <RendezVous/Core/src/Client/BackEndServices.h>

/** \internal */
BEGIN_NAMESPACE_NEX

// Child instances.

class QuazalAuthenticationPlugin;
class TicketGrantingProtocolClient;
class KerberosEncryption;
class TicketManager;
class Credentials;
class ProtocolCallContext;
class RVConnectionData;

/**
\ingroup group_rdv_core_services_authentication_client
\brief This service manages player log-ins and ensures the integrity and confidentiality
of data sent across the network.
\remarks Access to this service is achieved via the RendezVous::GetAuthenticationClient method,
which is only valid once a Principal has logged in to the lobby via the RendezVous::Login method.
*/
class OSDK_API AuthenticationClient: public ServiceClient
{
public:

    AuthenticationClient();
    virtual ~AuthenticationClient();

    /** \internal */
    qBool ValidateAndAcquireSecureConnectionData(ProtocolCallContext *pContext, qResult *pqResult, const String& strName, const qChar8* szPassword,
                                   RVConnectionData* pConnectionData, String *pstrReturnMsg, DataHolder *pAnyData=NULL);

    /**
    \brief Returns the PrincipalID of the Principal with the specified user name.

    \param pContext     A pointer to the ProtocolCallContext for this call.
    \param pidUserID    A pointer to the PrincipalID of the Principal with the specified strUserName.
    \param strUserName  The user name, as a string, of the Principal whos PrincipalID we wish to know.
    \returns true if the call was successfully initiated, false if not or an error occurred.
    If this method returns true then pidUserID will point to the PrincipalID of the Principal with the
    specified user name. The user name of a Principal is set when the Principal creates or updates his account.
    \sa AccountManagementClient::GetName
    */
    qBool GetPID(ProtocolCallContext *pContext, PrincipalID *pidUserID, String strUserName);

    /** \internal */
    qBool GetName(ProtocolCallContext *pContext, String* pstrUserName, PrincipalID pidUser);

public:
    /**
    For internal use.
    \internal */
    PrincipalID GetLocalPrincipal() const
    {
        return m_idLocalPrincipal;
    }
    
    void ClearLocalPrincipal()
    {
        m_idLocalPrincipal = INVALID_PRINCIPALID;
    }

    /** \internal */
    TicketGrantingProtocolClient *GetTicketGrantingProtocol()
    {
        return m_pTicketGrantingProtocolClient;
    }
    /** \internal */
    KerberosEncryption *GetKerberosEncryption()
    {
        return m_pEncryption;
    }
    /** \internal */
    TicketManager *GetTicketManager()
    {
        return m_pTicketManager;
    }
    /** \internal */
    qBool Rebind(Credentials* pCredentials);

    Time GetLoginDeadline() const
    {
        return m_tLoginDeadline;
    };

protected:
    /** \internal */
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

private:
    TicketGrantingProtocolClient*   m_pTicketGrantingProtocolClient;
    KerberosEncryption*             m_pEncryption;
    TicketManager*                  m_pTicketManager;
    PrincipalID                     m_idLocalPrincipal;
    Time                            m_tLoginDeadline;
};

/** \internal */
END_NAMESPACE_NEX


