﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <RendezVous/Core/src/Common/ProtocolCallContext.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/NintendoHTTP/HttpConnection.h>

// GetObjectのdefine回避開始 #29450
#ifdef WIN32_TARGET
#ifdef GetObject
#define NEX_PREVIOUS_GETOBJECT_DEFINED
#undef GetObject
#endif
#endif

BEGIN_NAMESPACE_NEX

class HttpEventListener;

class FileServerClient : public RootObject, public NonCopyable
{
public:
    FileServerClient();
    virtual ~FileServerClient();

    /*!
        @brief      ストレージからデータを取得します。

        
        本関数呼び出し前に nn::http::Initialize() 呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        RMCが失敗したとき、特に下記のqResultが格納される可能性があります。

        @li QERROR(Transport, NotFound) :  対象のデータが見つからなかった。
        @li QERROR(Transport, PermanentServerError) : 恒久的なサーバーのエラー。
        @li QERROR(Transport, TemporaryServerError) : 一時的なサーバーのエラー。

        @param[in]  pContext 呼び出し情報。
        @param[in]  url 詳細情報。
        @param[in]  eventListener ダウンロードされたデータを受け取るリスナーオブジェクト。
        @param[in]  relayBufferSize HttpEventListener::ProcessResponse() において一度にバッファに読み書きする最大サイズ。
                                    この関数に渡される最大バッファサイズを指定します。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    qResult GetObject(CallContext* pCC, const String& url, HttpEventListener* eventListener,
                      size_t relayBufferSize=HttpConnection::DEFAULT_RELAY_BUFFER_SIZE);

#ifdef WIN32_TARGET
    // GetObjectのdefine問題回避用定義
    qResult GetObjectA(CallContext* pCC, const String& url, HttpEventListener* eventListener,
                      size_t relayBufferSize=HttpConnection::DEFAULT_RELAY_BUFFER_SIZE)
    {
        return GetObject(pCC, url, eventListener, relayBufferSize);
    }

    // GetObjectのdefine問題回避用定義
    qResult GetObjectW(CallContext* pCC, const String& url, HttpEventListener* eventListener,
                      size_t relayBufferSize=HttpConnection::DEFAULT_RELAY_BUFFER_SIZE)
    {
        return GetObject(pCC, url, eventListener, relayBufferSize);
    }
#endif  // WIN32_TARGET

    /*!
        @brief      ストレージにデータをアップロードします。

        
        本関数呼び出し前に nn::http::Initialize() 呼び出し、HTTP ライブラリを初期化する必要があります。
        
        
        RMCが失敗したとき、特に下記のqResultが格納される可能性があります。

        @li QERROR(Transport, PermanentServerError) : 恒久的なサーバーのエラー。
        @li QERROR(Transport, TemporaryServerError) : 一時的なサーバーのエラー。

        @param[in]  pContext 呼び出し情報。
        @param[in]  url 詳細情報。
        @param[in]  eventListener アップロードするデータ指定するリスナーオブジェクト。
        @param[in]  relayBufferSize HttpEventListener::PostChunkedBuffer() において一度にバッファに読み書きする最大サイズ。
                                    この関数に渡される最大バッファサイズを指定します。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    qResult PostObject(CallContext* pCC, const String& url, HttpEventListener* eventListener,
                       size_t relayBufferSize=HttpConnection::DEFAULT_RELAY_BUFFER_SIZE);

protected:
    qResult ApiImpl(CallContext* pCC, const String& url, HttpEventListener* eventListener, HttpConnection::Method method, size_t relayBufferSize);
};

END_NAMESPACE_NEX

// GetObjectのdefine回避終了 #29450
#ifdef WIN32_TARGET
#ifdef NEX_PREVIOUS_GETOBJECT_DEFINED
#ifdef UNICODE
#define GetObject GetObjectW
#else
#define GetObject GetObjectA
#endif
#undef NEX_PREVIOUS_GETOBJECT_DEFINED
#endif
#endif

