﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <RendezVous/Core/src/Common/Protocol.h>

BEGIN_NAMESPACE_NEX

class OSDK_API ProtocolRegistry : public RootObject
{

public:
    ProtocolRegistry();
    virtual ~ProtocolRegistry();

public:
    void TraceProtocols(qUnsignedInt32 uiFlags);
    void Trace(qUnsignedInt32 uiFlags=TRACE_ALWAYS)
    {
        TraceProtocols(uiFlags);
    }

    qBool RegisterProtocol(ProtocolID uiProtocolID, Protocol *pProtocol);

    Protocol *GetProtocol(ProtocolID uiProtocolID, Protocol::Type eType);
    ServerProtocol *GetServerProtocol(ProtocolID uiProtocolID);

private:
    qBool RegisterProtocol(Protocol *pProtocol);

    // Block default copy construction and assignement
    ProtocolRegistry(const ProtocolRegistry &);
    ProtocolRegistry & operator=(const ProtocolRegistry &);

private:
    typedef qMap<ProtocolID, Protocol*> ProtocolMap;
    ProtocolMap& GetProtocolMap(Protocol::Type eType)
    {
        return (eType == Protocol::Server)?m_mServerProtocols:m_mClientProtocols;
    }
    OSDK_DISABLE_EXPORT_WARNING(ProtocolMap m_mClientProtocols);
    OSDK_DISABLE_EXPORT_WARNING(ProtocolMap m_mServerProtocols);
};

END_NAMESPACE_NEX


