﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <RendezVous/Core/src/Common/ProtocolRequestBroker.h>
#include <OnlineCore/src/Platform/Core/EpochTime.h>

BEGIN_NAMESPACE_NEX

class ProtocolRegistry;
class ServerProcotol;
class NotificationEventManager;
class GlobalNotificationEventManager;

#define _NEX_MANAGEINSTANCE(TYPE,NAME)      \
    public:                             \
        const TYPE* Get##NAME() const {return m_p##NAME;}       \
        TYPE* Get##NAME() {return m_p##NAME;}               \
    private:                                \
        TYPE *m_p##NAME;

#define _NEX_MANAGEGLOBALINSTANCE(TYPE,NAME)    \
    public:                                 \
        static TYPE* Get##NAME() {return s_p##NAME;}                \
    private:                                \
        static TYPE *s_p##NAME;


#define NEX_MANAGEINSTANCE(C)   _NEX_MANAGEINSTANCE(C,C)

/**
\ingroup group_rdv_core_client
\brief This abstract class is the common back-end services base class for all platforms.
*/
class OSDK_API RVClientCore: public RefCountedObject
{
public:

    /** \internal */
    static void AcquireInstance();
    /** \internal */
    static void ReleaseInstance();

    /** \internal */
    static RVClientCore* GetInstance()
    {
        InstanceControl* pInstance = InstanceControl::FindInstance(IC_TYPE_RVCLIENTCORE, PseudoSingleton::GetCurrentContext());
        if (pInstance!=NULL)
        {
            return static_cast<RVClientCore*>(pInstance->GetDelegatorInstance());
        }
        else
        {
            return NULL;
        }
    }

    qBool RegisterGlobalNotificationEventManager(NotificationEventManager* pNotificationEventManager);
    qBool UnregisterGlobalNotificationEventManager(NotificationEventManager* pNotificationEventManager);

    qBool RegisterServerProtocol(ProtocolID uiProtocolID, ServerProtocol* pServerProtocol);
    qBool UnregisterServerProtocol(ProtocolID uiProtocolID);
    ServerProtocol *GetServerProtocol(ProtocolID uiProtocolID);
    qBool GetGameServerTime(DateTime* pServerTime) const;
    void SetGameServerTime(DateTime& serverTime);


    NEX_MANAGEINSTANCE(ProtocolRegistry)

private:
    /**
    \remarks A Core object should never be directly instantiated. Instead, it is created
    by the system before \productname is created.
    \internal */
    RVClientCore();

    /**
    \remarks A Core object should never be directly destroyed. It is destroyed by
    \productname when the session terminates.
    \internal */
    ~RVClientCore();

    /** Cannot copy
    \internal */
    RVClientCore(const RVClientCore &);
    /** \internal */
    RVClientCore &operator=(const RVClientCore &);

    /** Settings...
    \internal */
    PseudoSingleton m_psInstance;

    GlobalNotificationEventManager* m_pEventManager;
    typedef qMap<ProtocolID, ServerProtocol*> ProtocolMap;
    OSDK_DISABLE_EXPORT_WARNING(ProtocolMap m_mProtocols);

    EpochTime m_serverTime;
};

// Cleanup
#undef NEX_MANAGEINSTANCE
#undef NEX_MANAGEGLOBALINSTANCE
#undef _NEX_MANAGEINSTANCE
#undef _NEX_MANAGEGLOBALINSTANCE

END_NAMESPACE_NEX


