﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Operation.h>

BEGIN_NAMESPACE_NEX

class BackEndServices;
class Credentials;
class CallContext;

/**
\ingroup group_rdv_core_client
\brief A RendezVousOperation encapsulates operations
performed on RendezVous.  RendezVousOperation are managed
exclusively from the OperationManager from the BackEndServices
class.

\sa OperationManager, OperationCallback
*/
class OSDK_API RendezVousOperation : public Operation
{
public:

    /** \internal */
    explicit RendezVousOperation(BackEndServices& be);
    /** \internal */
    virtual ~RendezVousOperation();

    /**
    \brief Return the current BackEndServices.
    \returns A reference to the BackEndServices instance.
    */
    BackEndServices& GetBackEndServices();

    /**
    \brief Return a pointer to the credentials
    \returns A pointer to credentials (can be null).
    \remarks The function returns null if the user is not
    connected.
    */
    Credentials* GetCredentials();

private:
    /** \internal */
    RendezVousOperation(const RendezVousOperation& rvo);
    /** \internal */
    RendezVousOperation& operator=(const RendezVousOperation& rvo);

    /** Implement Operation virtual methods
    \internal */
    void TraceImpl(Event event, qUnsignedInt64 traceFlags) const;

    BackEndServices&  m_be;
};

END_NAMESPACE_NEX

