﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/ProtocolFlags.h>
#include <RendezVous/Core/src/Common/Protocol.h>

BEGIN_NAMESPACE_NEX

class Credentials;

//==============================================================================
// Abstract base class for client-side protocol classes. The DDL compiler
// generates C++ classes that inherits from this class along with ServerProtocol
// The following link describes an overview of the related design:
// For NEX Developer http://jwiki/cgi-bin/wiki.cgi?RendezVousProtocol
//==============================================================================
class OSDK_API ClientProtocol: public Protocol
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(ClientProtocol,Protocol);

    ClientProtocol(qUnsignedInt32 uiFlags = ProtocolFlags::External);
    virtual ~ClientProtocol();

    static void SetCallError(qResult rCallResult);

    virtual Protocol::Type GetProtocolType() const
    {
        return Protocol::Client;
    }

    qBool SendRMCMessage(ProtocolCallContext *pContext, Message* pMessage);
    qBool SendOverLocalLoopback(ProtocolCallContext *pContext, Message* pMessage);

    void ProcessResponse(Message *pMessage, EndPoint * pEndPoint);

    virtual void ExtractCallSpecificResults(Message* pMessage, ProtocolCallContext* pContext)=0;

    virtual ClientProtocol* CreateResponder() const = 0;
    virtual void SetDefaultCredentials(Credentials *pCredentials);
    Credentials *GetDefaultCredentials() const
    {
        return m_pCredentials;
    }

private:
    Credentials* m_pCredentials;
};


END_NAMESPACE_NEX

