﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/**
    \ingroup group_rdv_core_common
    \brief Namespace for enum BuiltInPrincipals.
*/
#ifdef SWIG
class BuiltInPrincipals
{
public:
#else
namespace BuiltInPrincipals
{
#endif

    /**
    \ingroup group_rdv_core_common
    \brief Enumerates the valid built-in PrincipalID s.
    \remarks PrincipalIDs ranging from 1 to 99 are reserved for Quazal processes and cannot be assigned by the
    developer. IDs above and including 100 can be assigned by the developer however, IDs from 100 to 999 are
    reserved for server processes.

    \note In order to access these types you must include the RendezVous.h file.
    \sa Principal, PrincipalID
    */

    /**
    An invalid PrincipalID.
    */
    static const PrincipalID InvalidPID         = INVALID_PRINCIPALID;

#ifdef NEX_USE_PRINCIPALID_64BIT
    /* ascending order */
    static const PrincipalID Authentication     = 0x036150e4122bbfb2ULL;
    static const PrincipalID RendezVous         = 0x03913919df86eeb1ULL;
    static const PrincipalID SandboxManager     = 0x0551760e6f55d90fULL;
    static const PrincipalID Guest              = 0x07d4e76f6d1470efULL;
    static const PrincipalID Administrator      = 0x081c704917e83fd9ULL;
    static const PrincipalID Eval               = 0x089fede848142a39ULL;
    static const PrincipalID SandboxProbe       = 0x099eb38c267017d0ULL;
    static const PrincipalID LobbyAdministrator = 0x11c20aaf7d8c4700ULL;
    static const PrincipalID Anonymous          = 0x12dfbf61e96fde23ULL;
    static const PrincipalID BackgroundPlayer   = 0x137ff1ae1717db48ULL;
    static const PrincipalID OfficialUser       = 0x14d2a5bbc079da4bULL;
    static const PrincipalID FirstUserAccount   = 0xfeb81b91b037d8dfULL;
#else
    /* ascending order */
    /** \internal */
    static const PrincipalID Authentication     = 1;
    /**
    The Rendez-Vous daemon.
    */
    static const PrincipalID RendezVous         = 2;
    static const PrincipalID SandboxManager     = 3;
    /** \internal */
    static const PrincipalID Guest              = 100;
    /** \internal */
    static const PrincipalID Administrator      = 101;
    /** \internal */
    static const PrincipalID Eval               = 102;
    /** \internal */
    static const PrincipalID SandboxProbe       = 103;
    /** \internal */
    static const PrincipalID LobbyAdministrator = 104;
    /** \internal */
    static const PrincipalID Anonymous          = 106;
    //{secret}
    static const PrincipalID BackgroundPlayer   = 110;
    /** \internal */
    static const PrincipalID OfficialUser       = 111;
    /**
    The first user-configurable account.
    */
    static const PrincipalID FirstUserAccount   = 1000;
#endif

#ifdef SWIG
private:
#endif
    extern const PrincipalID SystemUsers[];
    qInt CmpPrincipalId(const void *a, const void *b);

#ifdef SWIG
public:
    static qBool IsSystemUser(const PrincipalID pid);
#else
    qBool IsSystemUser(const PrincipalID pid);
#endif

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

