﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/*
\brief This class acts like a namespace for enum Types.
*/

/*!
 *
 * @brief メッセージタイプを定義します
 *
 * @see UserMessage, UserMessage::GetRecipientID(), UserMessage::GetRecipientType()
 *
 */
class IdentifierTypes
{
public:
    /*
    \ingroup group_rdv_core_common
    \brief Enumerates the valid built-in IdentifierType s.
    \remarks Types ranging from 1 to 99 are reserved for Quazal processes and cannot be assigned by the
    developer. IDs above (and including) 100 can be assigned by the developer.

    \note In order to access these types you must include the RendezVous.h file.
    */

    /*!
     * @brief メッセージタイプの列挙型を定義します
     *
     * 値の1から99がNEXの予約値で、100以上がユーザー用の値です。
     *
     * @see UserMessage, UserMessage::GetRecipientID(), UserMessage::GetRecipientType()
     */
    enum Types
    {
        /**
        An invalid type.
        */
        InvalidType = 0, //!< 無効なタイプです。
        /**
        The Principal Identifier (PrincipalID) type.
        */
        PIDType = 1, //!< PrincipalID
        /**
        The Gathering Identifier (GatheringID) type.
        */
        GIDType = 2, //!< GatheringID
        /**
        The first user-configurable account.
        */
        FirstUserIdType = 100, //!< User用TypeIDの最小値(100)です。
        MaxTypeID = QUNSIGNEDINT32_MAX //!< IDの最大値

    };
};

END_NAMESPACE_NEX

