﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/DataDDL.h>

BEGIN_NAMESPACE_NEX

class OSDK_API DDLCLASS(Data);
/*
\ingroup group_rdv_core_common
\brief This is the base class for custom data.
\remarks Any custom data classes or hierarchy of classes must be declared in the DDL file as inheriting from
this class. Before a custom Data class can be used it must first be registered via the AnyDataAdapter to ensure
that the derived class is correctly instantiated when required and that data is correctly streamed between a client
and the \productname back end. Refer to Custom Data for further details.
\sa Custom Data
*/

/*!
 @brief    UserMessage のベースクラスです。
 */
class OSDK_API Data: public DDLCLASS(Data)
{
public:
    /*!
     * @brief コンストラクタです
     */
    Data();
    /*!
     * @brief デストラクタです
     */
    virtual ~Data();
#ifdef OSDK_DLL_SUPPORT
    bool operator ==(const Data&other) const
    {
        return DDLCLASS(Data)::operator ==(other);
    }
    bool operator <(const Data&) const
    {
        // This is required for exporting STL containers of this type
        QLOG(EventLog::Error,NEX_T("This operation is unimplemented. This is just to permit exporting Data-derived classes in the OSDK DLL."));
        SYSTEMCHECK(true);
        return true;
    }
#endif
#ifdef FOR_DOC_PURPOSE
    /*!
     * @brief   Dataクラスポインタの動的キャストを行います。
     *
     * 本関数は、DDLコンパイラによってDDLで定義されたDataクラスの派生クラスに自動的に作成されます。
     *
     * 各クラスで定義された同名関数を呼び出すことで、引数で指定されたポインタを各クラスの型のポインタに動的にキャストします。
     * たとえば、@ref UserMessage::DynamicCast() であれば、引数で指定したData*型のポインタをUserMessage*に動的にキャストします。
     * キャスト可能であれば有効なポインタを返し、キャスト不可能であればNULLを返します。
     *
     * 本関数による動的キャストは独自に実装しているので、RTTIを有効にする必要はありません。
     *
     * @param[in]   pData Dataクラスの派生クラスのポインタを指定します。
     * @return  pDataで指定したポインタを、呼び出し時のクラスのポインタとして返します。
     * @retval  NULL    pDataで指定されたポインタが、呼び出し時のクラスの派生クラスでない場合に返ります。
     *                  たとえば、@ref UserMessage::DynamicCast() を呼び出したときに、pDataの実体がUserMessageの派生クラスでなければNULLが返ります。
     */
    static Data* DynamicCast(Data* pData)
    {
        // This method is provided by specific classes.
        SYSTEMCHECK(false);
        return NULL;
    }
    static const Data* DynamicCast(const Data* pData)
    {
        // This method is provided by specific classes.
        SYSTEMCHECK(false);
        return NULL;
    }
#endif
};

END_NAMESPACE_NEX

