﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Core/src/Common/PropertyDDL.h>

BEGIN_NAMESPACE_NEX

/**
   \brief A unique positive integer used to identify a Property.
   \see Property
*/
typedef qUnsignedInt32 PropertyID;

/**
   \brief A unique positive integer used to store the property value.
   \see Property
*/
typedef qInt32 PropertyValue;  //TODO: should be a variant when supported

/**
   \ingroup group_rdv_services_gamesession_client
   \brief A property is used when defining the fields you have defined in the service through the configuration tool. It can also be used to pass arguments when calling a query.

   \see GameSession Service
*/

class OSDK_API DDLCLASS(Property);
class OSDK_API Property: public DDLCLASS(Property)
{
public:

    /**
       \brief Default constructor.
    */
    Property();

    /**
       \brief Destructor.
    */
    virtual ~Property();

    /**
       \brief Returns the unique identifier for this property.
       \returns The unique identifier for this property.
       \see SetID
    */
    PropertyID GetID() const;

    /**
       \brief Sets the unique identifier for this property.
       \param[in] propertyID The unique identifier for this property.
       \see GetID
    */
    void SetID(const PropertyID& propertyID);

    /**
       \brief Returns the property value.
       \returns The property value.
       \see SetValue
    */
    PropertyValue GetValue() const;

    /**
       \brief Sets the property value.
       \param[in] propertyValue The property value.
       \see GetValue
    */
    void SetValue(const PropertyValue& propertyValue);

};

END_NAMESPACE_NEX

