﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/**
\brief Namespace for enum ProtocolList.
*/
#ifdef SWIG
class ProtocolList
{
public:
#else
namespace ProtocolList
{
#endif

    /**
    \ingroup group_rdv_core_common
    \brief Defines the ID of user-defined protocols.
    \remarks Any custom protocol that you define should be given an ID that corresponds to this type definition.
    \note In order to access these types you must include the RendezVous.h file.
    */
    enum ProtocolList
    {
        /** \internal */
        Unknown              = 0,

        /**
        Extensions protocols
        \internal */
        RemoteLogDevice      = 1,
        ControlProtocol      = 2,
        NATTraversal         = 3,

        /**
        Core Protocols services
        \internal */
        TicketGranting       = 10,
        SecureConnection     = 11,
        BackEndManagement    = 12,
        ServiceManagement    = 13,
        Notification         = 14,
        CoreService          = 15,
        SandboxManagement    = 16, // Obsolete
        Siege                = 17,
        Health               = 18,
        Monitoring           = 19,

        /**
        Regular services protocols
        \internal */
        Friends              = 20,
        MatchMaking          = 21,
        Peer                 = 22,
        Messaging            = 23,
        PersistentStore      = 24,
        AccountManagement    = 25,
        Competition          = 26,
        MessageDelivery      = 27,
        ZZZAccountManagement = 29,
        CoreTestingFramework = 34,
        SubAccountManagement = 44,

        /**
        Extension protocols to regular services
        \internal */
        MatchMakingExt       = 50,

        //{secret}
        CDKeyPackage         = 80,

        /** \internal */
        SiegeAdmin           = 84,

        /** \internal */
        OAuth                = 85,

        /** \internal */
        ContentSharing       = 86,

        // 汎用のテスト向けID
        GeneralTest          = 99,

        /**
        The ID of the first user-defined protocol, which by default is equal to 100.
        */
        FirstUserProtocol   = 100,

        // Nintendo defined protocols
        NintendoNotificationEvent   = 100,
        NintendoFriendPresence  = 101,

        NintendoMatchmake      = 109,

        //n2494
        NintendoUtility        = 110,
        //NintendoStorage        = 111,
        NintendoRanking        = 112,
        NintendoMessaging      = 113,
        NintendoReporting      = 114,
        //n2852
        NintendoDataStore      = 115,
        NintendoDebug   = 116,
        //n2850
        NintendoSubscription   = 117,
        NintendoRating         = 118,
        NintendoServiceItem    = 119,
        NintendoMatchmakeReferee  = 120,
        NintendoSubscriber     = 121,
        NintendoRanking2       = 122,
        NintendoAAUser         = 123,

        // Custom reserved
        NintendoPlugin         = 200,
        NintendoCustom1        = 201,
        NintendoCustom2        = 202,
        NintendoCustom3        = 203,
        NintendoCustom4        = 204,
        NintendoCustom5        = 205,
        NintendoCustom6        = 206,
        NintendoCustom7        = 207,
        NintendoCustom8        = 208,
        NintendoCustom9        = 209,
        NintendoCustom10       = 210,
        NintendoCustom11       = 211,
        NintendoCustom12       = 212,
        NintendoCustom13       = 213,
        NintendoCustom14       = 214,
        NintendoCustom15       = 215,
        NintendoCustom16       = 216,

        Last = 1000    // no comma
    };

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

