﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>
#include <RendezVous/Core/src/Common/Protocol.h>
#include <OnlineCore/src/Core/OperationManager.h>
#include <OnlineCore/src/Core/SecurityContextManager.h>
#include <OnlineCore/src/Platform/Core/BandwidthCounter.h>

BEGIN_NAMESPACE_NEX

class Message;

// The ProtocolRequestBroker is a base class for both Client and Server RequestBroker. the name
// interface was kept instead of just ProtocolRequestBroker because we wanted to minimize changes
// to the Current Server implementation of ProtocolrequestBroker. when we will revisit the server
// implementation, the first change wiill be to rename the class to ServerProtocolRequestBroker.

class OSDK_API ProtocolRequestBrokerInterface : public RootObject
{

public:
    ProtocolRequestBrokerInterface();
    virtual ~ProtocolRequestBrokerInterface();

public:
    static void InitMessage(Message *msg, ProtocolID uiProtocolID, Protocol::Command eCommand);

    // Called by the JobProcessProtocolMessage.
    virtual void ProcessMessage(CallProtocolMethodOperation* pOp, EndPoint *pEndPoint, Buffer *pBuffer) = 0;
    virtual void ProcessCallRequest(ProtocolID uiProtocolID, Message* pMessageReceived, Message* pmsgResponse, qBool *pbReturnsResponse, EndPoint *pEndPoint) = 0;
    virtual void ProcessCallResponse(ProtocolID uiProtocolID, Message* pMessageReceived, EndPoint *pEndPoint) = 0;

    // Used by MessagingService, NATTraversalRelayProtocol, AuthenticationService, HealthService,
    // SecureConnectionService, RemoteLogDeviceClient, ServiceHost
    virtual qBool RegisterProtocol(Protocol *pProtocol) = 0;
    virtual void UnregisterProtocol(Protocol *pProtocol) = 0;
    virtual Protocol *GetProtocol(ProtocolID uiProtocolID, Protocol::Type eType) = 0;

    // Used by the Dispatcher
    virtual qBool CloneProtocols() = 0;
    void CloneProtocols(qBool /*bDummy*/)
    {
        CloneProtocols();
    }

    OperationManager& GetOperationManager()
    {
        return m_oManager;
    }

    virtual qBool RegisterServerProtocol(ProtocolID, ServerProtocol*) = 0;
    virtual qBool UnregisterServerProtocol(ProtocolID) = 0;
    virtual ServerProtocol *GetServerProtocol(ProtocolID) = 0;

    void Receive(EndPoint *pEndPoint, Buffer *pBuffer);

    // Used for local dispatching in :
    // - ClientProtocol::SendOverLocalLoopback (which is used by ClientProtocol::SendRMCMessage !)
    // - TicketGrantingServer
    // - SecureConnectionServer
    // - Service::BlockingCallOnServiceControlScripts
    // - Service::NonBlockingCallOnServiceControlScripts
    virtual qResult DispatchLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint) = 0;
    virtual qResult DispatchBlockingLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint) = 0;
    virtual qResult DispatchThreadSafeLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint) = 0;


private:
    // Block default copy construction and assignement
    ProtocolRequestBrokerInterface(const ProtocolRequestBrokerInterface &);
    ProtocolRequestBrokerInterface & operator=(const ProtocolRequestBrokerInterface &);

protected:
    BandwidthCounterMap m_bandwidthCounters;

private:
    OperationManager m_oManager;
    static qBool s_bOutputFormatIsRegistered;
};

END_NAMESPACE_NEX


