﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreEventListener.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreClientInterface.h>

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      DataStoreのAPIバージョンです。
*/
enum DATASTORE_API_VERSION
{
    DATASTORE_API_VERSION_1,
    DATASTORE_API_VERSION_LATEST
};

/*!
    @internal
    @brief      DataStoreのAPIバージョンごとの特性を表すクラスです。
*/
template<DATASTORE_API_VERSION>
struct DataStoreApiVersionTraits;

class DataStorePrepareGetParamV1;
class DataStoreReqGetInfoV1;
class DataStorePreparePostParamV1;
class DataStoreReqPostInfoV1;
class DataStoreCompletePostParamV1;
class DataStoreNotificationV1;
class DataStoreGetSpecificMetaParamV1;
class DataStoreSpecificMetaInfoV1;

class DataStorePrepareGetParam;
class DataStoreReqGetInfo;
class DataStorePreparePostParam;
class DataStoreReqPostInfo;
class DataStoreCompletePostParam;
class DataStoreNotification;
class DataStoreGetSpecificMetaParam;
class DataStoreSpecificMetaInfo;

/*!
    @internal
    @brief      DataStoreのAPIバージョンごとの特性を表すクラスです。V1の特殊化です。
*/
template<>
struct DataStoreApiVersionTraits<DATASTORE_API_VERSION_1>
{
    // 対応するデータIDの型
    typedef qUnsignedInt32 TDataId;

    // 対応するDataStoreReqGetInfoの型
    typedef DataStoreReqGetInfoV1 TReqGetInfo;

    // 対応するPrepareGetParamの型
    typedef DataStorePrepareGetParamV1 TPrepareGetParam;

    // 対応するPreparePostParamの型
    typedef DataStorePreparePostParamV1 TPreparePostParam;

    // 対応するReqPostInfoの型
    typedef DataStoreReqPostInfoV1 TReqPostInfo;

    // 対応するCompletePostParamの型
    typedef DataStoreCompletePostParamV1 TCompletePostParam;

    // 対応するNotificationの型
    typedef DataStoreNotificationV1 TNotification;

    // 対応するGetSpecificMetaParamの型
    typedef DataStoreGetSpecificMetaParamV1 TGetSpecificMetaParam;

    // 対応するSpecificMetaInfoの型
    typedef DataStoreSpecificMetaInfoV1 TSpecificMetaInfo;

    // 対応するGetObjectEventListenerの型
    typedef DataStoreGetObjectEventListenerTemplate<TDataId> TGetObjectEventListener;

    // 対応するReceiveNewArrivedObjectsEventListenerの型
    typedef DataStoreReceiveNewArrivedObjectsEventListenerTemplate<TDataId> TReceiveNewArrivedObjectsEventListener;

    // 対応するDataStoreClientInterface::GetObject関数ポインタの型と変数。JobReceiveNewArrivedObjectsで使用する。
    // 関数名を変更できるようにするため関数ポインタとして持つ
    typedef qBool (DataStoreClientInterface::*GetObjectFunction)(ProtocolCallContext *pContext, const TPrepareGetParam& param, TGetObjectEventListener* eventListener);
    static GetObjectFunction pGetObject;
};

/*!
    @internal
    @brief      DataStoreのAPIバージョンごとの特性を表すクラスです。最新バージョンの特殊化です。
*/
template<>
struct DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST>
{
    // 対応するデータIDの型
    typedef qUnsignedInt64 TDataId;

    // 対応するPrepareGetParamの型
    typedef DataStorePrepareGetParam TPrepareGetParam;

    // 対応するDataStoreReqGetInfoの型
    typedef DataStoreReqGetInfo TReqGetInfo;

    // 対応するPreparePostParamの型
    typedef DataStorePreparePostParam TPreparePostParam;

    // 対応するReqPostInfoの型
    typedef DataStoreReqPostInfo TReqPostInfo;

    // 対応するCompletePostParamの型
    typedef DataStoreCompletePostParam TCompletePostParam;

    // 対応するNotificationの型
    typedef DataStoreNotification TNotification;

    // 対応するGetSpecificMetaParamの型
    typedef DataStoreGetSpecificMetaParam TGetSpecificMetaParam;

    // 対応するSpecificMetaInfoの型
    typedef DataStoreSpecificMetaInfo TSpecificMetaInfo;

    // 対応するGetObjectEventListenerの型
    typedef DataStoreGetObjectEventListenerTemplate<TDataId> TGetObjectEventListener;

    // 対応するReceiveNewArrivedObjectsEventListenerの型
    typedef DataStoreReceiveNewArrivedObjectsEventListenerTemplate<TDataId> TReceiveNewArrivedObjectsEventListener;

    // 対応するDataStoreClientInterface::GetObject関数ポインタの型と変数。JobReceiveNewArrivedObjectsで使用する。
    // 関数名を変更できるようにするため関数ポインタとして持つ
    typedef qBool (DataStoreClientInterface::*GetObjectFunction)(ProtocolCallContext *pContext, const TPrepareGetParam& param, TGetObjectEventListener* eventListener);
    static GetObjectFunction pGetObject;
};

END_NAMESPACE_NEX

