﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreProtocolDDL.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreLogicServerClientInterface.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreEventListener.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief DataStore サーバーとの通信を行うためのAPIインタフェースです。
*/
class DataStoreClientInterface
{
public:
    DataStoreClientInterface() {};
    virtual ~DataStoreClientInterface() {};
    virtual qBool GetObject(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener) = 0;
    virtual qBool GetObjectOrMetaBinary(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener) = 0;
    virtual qBool GetObjectV1(ProtocolCallContext *pContext, const DataStoreGetParamV1& param, DataStoreGetObjectEventListenerV1* eventListener) = 0;
    virtual qBool GetObject(ProtocolCallContext *pContext, const DataStoreGetParam& param, void* buffer, size_t bufferSize, size_t* pWrittenSize) = 0;
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId, DataStorePostObjectEventListener* eventListener) = 0;
    virtual qBool PostObjectV1(ProtocolCallContext *pContext, const DataStorePostParamV1& param, qUnsignedInt32* dataId, DataStorePostObjectEventListener* eventListener) = 0;
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId, const void* buffer, qBool useInternalBuffer=true) = 0;
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId) = 0;
    virtual qBool PostObject(ProtocolCallContext *pContext, const qUnsignedInt64 dataId, const DataStorePostParam& param) = 0;
    virtual qBool PostObject(ProtocolCallContext *pContext, const qVector< std::pair<qUnsignedInt64, DataStorePostParam> >& params, const qBool transactional=true, qVector<qResult>* pResults=NULL) = 0;
    virtual qBool UpdateObject(ProtocolCallContext *pContext, const DataStoreUpdateParam& param, DataStorePostObjectEventListener* eventListener) = 0;
    virtual qBool UpdateObject(ProtocolCallContext *pContext, const DataStoreUpdateParam& param, const void* buffer, qBool useInternalBuffer=true) = 0;
    virtual qBool CheckNotification(ProtocolCallContext *pContext,
                                    const String& urlIn,
                                    String* urlOut,
                                    qUnsignedInt64 notificationIdIn,
                                    qUnsignedInt64* notificationIdOut,
                                    qUnsignedInt32 sandboxInstanceIdIn,
                                    qUnsignedInt32* sandboxInstanceIdOut,
                                    PrincipalID principalId,
                                    qBool* isFailedRetrieveNotificationUrl,
                                    qBool forceRetrieveUrl=false
                                   ) = 0;
    virtual qResult ReceiveNewArrivedObjects(ProtocolCallContext* pCC,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            qVector<DataStoreGetObjectEventListener*> getObjectEventListeners,
            DataStoreReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
            qBool* hasNext
                                            ) = 0;
    virtual qResult ReceiveNewArrivedObjectsV1(ProtocolCallContext* pCC,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            qVector<DataStoreGetObjectEventListenerV1*> getObjectEventListeners,
            DataStoreReceiveNewArrivedObjectsEventListenerV1* receiveNewArrivedObjectsEventListener,
            qBool* hasNext
                                              ) = 0;
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating,
                            qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qVector< DataStoreRatingInfoWithSlot > > *pRatings,
                            qVector<qResult>* pResults=NULL, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool GetRating(ProtocolCallContext* pContext, const qUnsignedInt64 dataId, qMap<qInt8, DataStoreRatingInfo> *pRatings,
                            qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qMap<qInt8, DataStoreRatingInfo> > *pRatings,
                            qVector<qResult>* pResults=NULL) = 0;
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating,
                            DataStoreRatingLog* pRatingLog, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool GetObjectByInfo(ProtocolCallContext* pContext, const DataStoreReqGetInfo& info, DataStoreGetObjectEventListener* pEventListener) = 0;
    virtual qBool GetObjectByInfo(ProtocolCallContext* pContext, const DataStoreReqGetInfo& info, void* buffer, size_t bufferSize, size_t* pWrittenSize) = 0;

};

END_NAMESPACE_NEX

 
