﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreDDF.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreProtocolDDL.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreConstants.h>

BEGIN_NAMESPACE_NEX

/// データのダウンロードに関するパラメータを表すクラスです。
typedef DataStorePrepareGetParam DataStoreGetParam;
typedef DataStorePrepareGetParamV1 DataStoreGetParamV1;

/// データのアップロードに関するパラメータを表すクラスです。
typedef DataStorePreparePostParam DataStorePostParam;
typedef DataStorePreparePostParamV1 DataStorePostParamV1;

/// データの更新に関するパラメータを表すクラスです。
typedef DataStorePrepareUpdateParam DataStoreUpdateParam;

/*!
    @brief DataStoreサーバーとの通信を行うためのインタフェースです。
*/
class DataStoreLogicServerClientInterface
{
public:
    DataStoreLogicServerClientInterface() {};
    virtual ~DataStoreLogicServerClientInterface() {};
    virtual qBool PrepareGetObject(ProtocolCallContext* pContext, const DataStorePrepareGetParam& param, DataStoreReqGetInfo* pReqGetInfo) = 0;
    virtual qBool PrepareGetObject(ProtocolCallContext* pContext, const DataStorePrepareGetParamV1& param, DataStoreReqGetInfoV1* pReqGetInfo) = 0;
    virtual qBool PrepareGetObjectOrMetaBinary(ProtocolCallContext *pContext, const DataStorePrepareGetParam& param, DataStoreReqGetInfo* pReqGetInfo, DataStoreReqGetAdditionalMeta* pReqGetAdditionalMeta) = 0;
    virtual qBool PrepareGetObjectOrMetaBinary(ProtocolCallContext *pContext, const DataStorePrepareGetParamV1& param, DataStoreReqGetInfoV1* pReqGetInfo, DataStoreReqGetAdditionalMeta* pReqGetAdditionalMeta) = 0;
    virtual qBool PreparePostObject(ProtocolCallContext* pContext, const DataStorePreparePostParam& param, DataStoreReqPostInfo* pReqPostInfo) = 0;
    virtual qBool PreparePostObject(ProtocolCallContext* pContext, const DataStorePreparePostParamV1& param, DataStoreReqPostInfoV1* pReqPostInfo) = 0;
    virtual qBool CompletePostObject(ProtocolCallContext* pContext, const DataStoreCompletePostParam& param) = 0;
    virtual qBool CompletePostObject(ProtocolCallContext* pContext, const DataStoreCompletePostParamV1& param) = 0;
    virtual qBool CompleteSuspendedPostObject(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds) = 0;
    virtual qBool PrepareUpdateObject(ProtocolCallContext* pContext, const DataStorePrepareUpdateParam& param, DataStoreReqUpdateInfo* pReqUpdateInfo) = 0;
    virtual qBool CompleteUpdateObject(ProtocolCallContext* pContext, const DataStoreCompleteUpdateParam& param) = 0;
    virtual qBool DeleteObject(ProtocolCallContext* pContext, const DataStoreDeleteParam& param) = 0;
    virtual qBool ChangeMeta(ProtocolCallContext* pContext, const DataStoreChangeMetaParam& param) = 0;
    virtual qBool GetMeta(ProtocolCallContext* pContext, const DataStoreGetMetaParam& param, DataStoreMetaInfo* pMetaInfo) = 0;
    virtual qBool GetMeta(ProtocolCallContext *pContext, const qVector<qUnsignedInt64>& dataIds, const DataStoreGetMetaParam& param, qVector<DataStoreMetaInfo>* pMetaInfos, qVector<qResult>* pResults = NULL) = 0;
    virtual qBool GetMeta(ProtocolCallContext *pContext, const qVector<DataStoreGetMetaParam>& params, qVector<DataStoreMetaInfo>* pMetaInfos, qVector<qResult>* pResults = NULL) = 0;
    virtual qBool SearchObject(ProtocolCallContext* pContext, const DataStoreSearchParam& param, DataStoreSearchResult* pSearchResult) = 0;
    virtual qBool SearchObjectLight(ProtocolCallContext* pContext, const DataStoreSearchParam& param, DataStoreSearchResult* pSearchResult) = 0;
    virtual qBool GetNotificationUrl(ProtocolCallContext* pContext, const DataStoreGetNotificationUrlParam& param, DataStoreReqGetNotificationUrlInfo* info) = 0;
    virtual qBool GetNewArrivedNotifications(ProtocolCallContext* pContext, const DataStoreGetNewArrivedNotificationsParam& param, qVector<DataStoreNotification>* pResult, qBool* pHasNext) = 0;
    virtual qBool GetNewArrivedNotifications(ProtocolCallContext* pContext, const DataStoreGetNewArrivedNotificationsParam& param, qVector<DataStoreNotificationV1>* pResult, qBool* pHasNext) = 0;
    virtual qBool RateObject(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, const DataStoreRateObjectParam& param, DataStoreRatingInfo* pRating) = 0;
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qVector< DataStoreRatingInfoWithSlot > > *pRatings,
                            qVector<qResult>* pResults=NULL, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool ResetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, qUnsignedInt64 updatePassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool ResetRating(ProtocolCallContext* pContext, const qUnsignedInt64& dataId) = 0;
    virtual qBool ResetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, const qBool transactional = true,  qVector<qResult>* pResults = NULL) = 0;
    virtual qBool GetSpecificMeta(ProtocolCallContext* pContext, const DataStoreGetSpecificMetaParam& param, qVector<DataStoreSpecificMetaInfo>* pMetaInfos) = 0;
    virtual qBool GetSpecificMeta(ProtocolCallContext* pContext, const DataStoreGetSpecificMetaParamV1& param, qVector<DataStoreSpecificMetaInfoV1>* pMetaInfos) = 0;
    virtual qBool PostMetaBinary(ProtocolCallContext* pContext, const DataStorePreparePostParam& param, qUnsignedInt64* pDataId) = 0;
    virtual qBool PostMetaBinaryWithDataId(ProtocolCallContext *pContext, const qUnsignedInt64 dataId, const DataStorePreparePostParam& param) = 0;
    virtual qBool PostMetaBinaryWithDataIds(ProtocolCallContext *pContext, const qVector< qUnsignedInt64 >& dataId, const DataStorePreparePostParam& param, const qBool transactional = true, qVector<qResult>* pResults = NULL) = 0;
    virtual qBool PostMetaBinariesWithDataId(ProtocolCallContext *pContext, const qVector< std::pair<qUnsignedInt64, DataStorePreparePostParam> >& params, const qBool transactional = true, qVector<qResult>* pResults = NULL) = 0;
    virtual qBool TouchObject(ProtocolCallContext* pContext, const DataStoreTouchObjectParam& param) = 0;
    virtual qBool GetRatingWithLog(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, DataStoreRatingLog* pRatingLog, qUnsignedInt64 updatePassword=DataStoreConstants::INVALID_PASSWORD) = 0;
    virtual qBool GetPersistenceInfo(ProtocolCallContext* pContext, PrincipalID ownerId, qUnsignedInt16 persistenceSlotId, DataStorePersistenceInfo* pPersistenceInfo) = 0;
    virtual qBool GetPersistenceInfo(ProtocolCallContext* pContext, PrincipalID ownerId, qVector<qUnsignedInt16> persistenceSlotId, qVector<DataStorePersistenceInfo>* pPersistenceInfo, qVector<qResult>* pResults) = 0;
    virtual qBool PerpetuateObject(ProtocolCallContext* pContext, qUnsignedInt16 persistenceSlotId, qUnsignedInt64 dataId, qBool deleteLastObject = true) = 0;
    virtual qBool UnperpetuateObject(ProtocolCallContext* pContext, qUnsignedInt16 persistenceSlotId, qBool deleteLastObject = false) = 0;
    virtual qBool GetPasswordInfo(ProtocolCallContext* pContext, qUnsignedInt64 dataId, DataStorePasswordInfo* pPasswordInfo) = 0;
    virtual qBool GetPasswordInfo(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector<DataStorePasswordInfo>* pPasswordInfos, qVector<qResult>* pResults) = 0;
    virtual qBool GetObjectInfo(ProtocolCallContext * pContext, const qVector<qUnsignedInt64>& dataIds, qVector<DataStoreReqGetInfo>* pInfos, qVector<qResult>* pResults = NULL) = 0;

protected:
    virtual PrincipalID GetMyPrincipalId() const = 0;
    virtual qBool IsCredentialsNeeded() const = 0;
};

END_NAMESPACE_NEX

