﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreCompletePostParamDDL.h"
#include "DataStorePreparePostParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      CompletePostObjectで使用するパラメータを表すクラスです。
*/
class DataStoreCompletePostParam: public DDLCLASS(DataStoreCompletePostParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreCompletePostParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreCompletePostParam() {}

    void SetDataId(qUnsignedInt64 dataId_)
    {
        this->dataId = dataId_;
    }
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }
    void SetIsSuccess(qBool isSuccess_)
    {
        this->isSuccess = isSuccess_;
    }
    qBool GetIsSuccess() const
    {
        return this->isSuccess;
    }

    /*!
        @internal
        @brief      対応するPreparePostParamを受け取って必要な情報をセットする

        @param      preparePostParam 対応するPreparePostObjectで使用したオブジェクト
    */
    void SetPreparePostParam(const DataStorePreparePostParam& preparePostParam)
    {
        (void)preparePostParam;
    }

    /*!
        @internal
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID);
        SetIsSuccess(false);
    }

    /*!
        @internal
        @brief      DataStoreCompletePostParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStoreCompletePostParam: (")
               << NEX_T("dataId=") << dataId
               << NEX_T(", isSuccess=") << isSuccess
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

