﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief DataStore に関する定数が定義された名前空間です。
*/
namespace DataStoreConstants
{
/// 無効な日時です。 DateTime(9999, 12, 31, 23, 59, 59) として定義されています。
extern const DateTime& INVALID_DATE_TIME;

/// 永続化されたデータに設定される有効期限です。 DateTime(9999, 12, 31, 0, 0, 0) として定義されています。 DateTime::Never とは異なることにご注意ください。
extern const DateTime& PERMANENT_DATE_TIME;

/// 無効なデータIDです。
static const qUnsignedInt64 INVALID_DATAID = 0;


/// データコードの最大文字列長です。
static const size_t MAX_DATACODE_STR_LEN = 14; // LOG(18,446,744,073,709,551,615, 28) = 13.3

// 無効なデータIDです。(V1用API)
static const qUnsignedInt32 INVALID_DATAID_V1 = 0;

/// 無効なデータタイプです。
static const qUnsignedInt16 INVALID_DATA_TYPE = 0xffff;

/// デフォルトの有効日数です。デフォルトは90日です。
static const qUnsignedInt16 DEFAULT_PERIOD = 90;

/// データの送受信に使用される HTTP 通信スレッドの優先度のデフォルト値です。
static const qUnsignedInt32 DEFAULT_HTTP_THREAD_PRIORITY = 16;

// 新着チェックのために使用されるトークンのサイズ。
static const qUnsignedInt32 NEW_ARRIVAL_CHECK_TOKEN_SIZE = 352;

/// 評価スロットの最大数です。
static const qInt8 RATING_SLOT_MAX = 15;

/// 評価スロットの個数です。
static const qInt8 NUM_RATING_SLOT = RATING_SLOT_MAX + 1;

// HPP に対しての通信タイムアウト値です。
static const qUnsignedInt32 DEFAULT_HPP_TRANSFER_TIMEOUT = 60 * 1000;

/// ストレージサーバーからダウンロードするときの通信速度によるタイムアウト設定です。(バイト毎秒) 100KBの送受信が10分かかっても終わらない場合にタイムアウトします。
static const TimeInterval DEFAULT_DATA_TRANSFER_TIMEOUT_BYTES_PER_SECOND = 167;

/// ストレージサーバーからダウンロードするときのデフォルトの最低タイムアウト値です。(ミリ秒)
static const qUnsignedInt32 DEFAULT_DATA_TRANSFER_MINIMUM_TIMEOUT = 60 * 1000;

/// データを受け渡す際の最大サイズのデフォルト値です。
static const size_t DEFAULT_RELAY_BUFFER_SIZE = 16 * 1024;

/// HTTP 通信で使用されるバッファサイズのデフォルト値です。
static const size_t DEFAULT_HTTP_BUFFER_SIZE = 32 * 1024;

// 無効な通知IDです。
static const qUnsignedInt64 INVALID_NOTIFICATIONID = 0;

// 無効なSandboxInstanceIDです。
static const qUnsignedInt32 INVALID_SANDBOX_INSTANCEID = 0;

// SandboxInstanceIDの初期値です。
static const qUnsignedInt32 INITIAL_SANDBOX_INSTANCEID = 1;

/// 無効な永続化スロットIDです。
static const qUnsignedInt16 INVALID_PERSISTENCE_SLOT_ID = 0xFFFF;

/// 永続化スロットの個数です。
static const qUnsignedInt16 NUM_PERSISTENCE_SLOT = 16;

/// パスワードの無効値です。この値がパスワードに設定されることはありません。
static const qUnsignedInt64 INVALID_PASSWORD = 0;

/// 一括処理を行う関数で一度に指定可能なデータIDの最大個数です。
static const size_t BATCH_PROCESSING_CAPACITY = 100;

/// 一括 POST 処理で一度に指定可能なデータIDの最大個数です。
static const size_t BATCH_PROCESSING_CAPACITY_POST_OBJECT = 16;

/// 一度に取得可能な検索結果の最大数です。
static const qUnsignedInt32 MAX_SEARCH_RESULT_SIZE = 100;

/// RESULTRANGE_ANY_OFFSET による無作為な検索で一度に取得可能な検索結果の最大数です。
static const qUnsignedInt32 MAX_SEARCH_ANY_RESULT_SIZE = 20;

/// タグの個数です。
static const qUnsignedInt32 NUM_TAG_SLOT = 16;

/// タグに指定できる文字数の最大値です。
static const size_t MAX_TAG_LENGTH = 24;

/// タイトル(要約情報)に指定できる文字数の最大値です。
static const size_t MAX_NAME_LENGTH = 64;

/// 検索条件に指定できるデータタイプ数の最大値です。
static const qUnsignedInt32 MAX_SEARCH_DATA_TYPE_SIZE = 10;

#ifdef NEX_SIGLO_FAMILY
/// ストレージへの HTTP 通信で使用するソケットの送信バッファサイズの初期値です。
static const size_t DEFAULT_HTTP_SEND_SOCKET_BUFFER_SIZE = 64 * 1024;

/// ストレージへの HTTP 通信で使用するソケットの受信バッファサイズの初期値です。
static const size_t DEFAULT_HTTP_RECV_SOCKET_BUFFER_SIZE = 64 * 1024;
#else
static const size_t DEFAULT_HTTP_SEND_SOCKET_BUFFER_SIZE = 0;
static const size_t DEFAULT_HTTP_RECV_SOCKET_BUFFER_SIZE = 0;
#endif

/*!
    @brief データのパーミッションです。参照権限と更新権限の両方で使用します。
*/
enum Permission
{
    PERMISSION_PUBLIC           = 0, //!< 全てのプリンシパルIDに対して許可されます
    PERMISSION_FRIEND           = 1, //!< 本人とフレンド関係にあるプリンシパルIDのみが許可されます
    PERMISSION_SPECIFIED        = 2, //!< 指定した複数のプリンシパルIDに対して許可されます
    PERMISSION_PRIVATE          = 3, //!< 本人のみに許可されます
    PERMISSION_SPECIFIED_FRIEND = 4  //!< 指定した複数のプリンシパルIDかつフレンド関係にあるプリンシパルIDに対して許可されます。なお、フレンド関係にあるプリンシパルIDはサーバー側で確認されます。
};

/*!
    @brief アップロードに関する設定フラグです。
*/
enum DataFlag
{
    DATA_FLAG_NONE                      = 0x0000, //!< 設定無し
    DATA_FLAG_NEED_REVIEW               = 0x0001, //!< 運用者によるデータレビューが必要
    DATA_FLAG_PERIOD_FROM_LAST_REFERRED = 0x0002, //!< データの参照、更新時に有効期限を伸びるようにする。仕様の詳細についてはプログラミングマニュアルを参照してください。
    DATA_FLAG_USE_READ_LOCK             = 0x0004, //!< 使用不可
    DATA_FLAG_USE_NOTIFICATION_ON_POST  = 0x0008, //!< DataStoreClient::PostObject() 実行時に受信者に対して通知する。アップロードしたデータをいつの間に通信で受信できるようになります。3DS 、 Wii U でのみ利用可能です。
    DATA_FLAG_USE_NOTIFICATION_ON_UPDATE= 0x0010, //!< アップロード後、そのデータを DataStoreClient::UpdateObject() で更新したときに受信者に対して通知する。更新したデータをいつの間に通信で受信できるようになります。Wii U でのみ利用可能です。
    DATA_FLAG_NOT_USE_FILESERVER        = 0x0020, //!< 使用不可
    DATA_FLAG_NEED_COMPLETION           = 0x0040  //!< DataStoreClient::PostObject() 完了後に DataStoreClient::CompleteSuspendedPostObject() を実行するまでサーバー上のデータを有効にしない。
};

/*!
    @brief 変更するメタ情報を設定するフラグです。
*/
enum ModificationFlag
{
    MODIFICATION_FLAG_NONE              = 0x0000, //!< 設定無し
    MODIFICATION_FLAG_NAME              = 0x0001, //!< タイトルを変更
    MODIFICATION_FLAG_ACCESS_PERMISSION = 0x0002, //!< 参照権限を変更
    MODIFICATION_FLAG_UPDATE_PERMISSION = 0x0004, //!< 更新権限を変更
    MODIFICATION_FLAG_DELETE_PERMISSION = MODIFICATION_FLAG_UPDATE_PERMISSION, //!< 更新権限を変更
    MODIFICATION_FLAG_PERIOD            = 0x0008, //!< 有効日数を変更
    MODIFICATION_FLAG_METABINARY        = 0x0010, //!< バイナリデータを変更
    MODIFICATION_FLAG_TAGS              = 0x0020, //!< タグを変更
    MODIFICATION_FLAG_UPDATED_TIME      = 0x0040, //!< 更新時間を現在時刻に更新する
    MODIFICATION_FLAG_DATA_TYPE         = 0x0080, //!< データタイプを変更
#ifndef FOR_DOC_PURPOSE
    MODIFICATION_FLAG_REFERRED_CNT      = 0x0100, // 参照回数を変更。廃止されるため使用できません
#endif  // FOR_DOC_PURPOSE
    MODIFICATION_FLAG_STATUS            = 0x0200  //!< ステータスを変更
};

/*!
    @brief 比較するメタ情報を設定するフラグです。
*/
enum ComparisonFlag
{
    COMPARISON_FLAG_NONE              = 0x0000, //!< 設定無し
    COMPARISON_FLAG_NAME              = 0x0001, //!< タイトルを比較
    COMPARISON_FLAG_ACCESS_PERMISSION = 0x0002, //!< 参照権限を比較
    COMPARISON_FLAG_UPDATE_PERMISSION = 0x0004, //!< 更新権限を比較
    COMPARISON_FLAG_PERIOD            = 0x0008, //!< 有効日数を比較
    COMPARISON_FLAG_METABINARY        = 0x0010, //!< バイナリデータを比較
    COMPARISON_FLAG_TAGS              = 0x0020, //!< タグを比較
    COMPARISON_FLAG_DATA_TYPE         = 0x0040, //!< データタイプを比較
#ifndef FOR_DOC_PURPOSE
    COMPARISON_FLAG_REFERRED_CNT      = 0x0080, // 参照回数を比較。廃止されるため使用できません
#endif  // FOR_DOC_PURPOSE
    COMPARISON_FLAG_STATUS            = 0x0100, //!< ステータスを比較
    COMPARISON_FLAG_ALL               = 0xFFFF  //!< すべて比較
};

/*!
    @brief  検索タイプです。
*/
enum SearchType
{
    SEARCH_TYPE_PUBLIC                =  1, //!< 参照権限が DataStoreConstants::PERMISSION_PUBLIC のもの
    SEARCH_TYPE_SEND_FRIEND           =  2, //!< 自分がオーナーかつ参照権限が DataStoreConstants::PERMISSION_FRIEND のもの
    SEARCH_TYPE_SEND_SPECIFIED        =  3, //!< 自分がオーナーかつ参照権限が DataStoreConstants::PERMISSION_SPECIFIED のもの
    SEARCH_TYPE_SEND_SPECIFIED_FRIEND =  4, //!< 自分がオーナーかつ参照権限が DataStoreConstants::PERMISSION_SPECIFIED_FRIEND のもの
    SEARCH_TYPE_SEND                  =  5, //!< 自分がオーナーかつ参照権限が DataStoreConstants::PERMISSION_FRIEND, DataStoreConstants::PERMISSION_SPECIFIED, DataStoreConstants::PERMISSION_SPECIFIED_FRIEND のいずれかであるもの
    SEARCH_TYPE_FRIEND                =  6, //!< 自分のフレンドが参照権限を DataStoreConstants::PERMISSION_FRIEND としてアップロードしたもの
    SEARCH_TYPE_RECEIVED_SPECIFIED    =  7, //!< 参照権限が DataStoreConstants::PERMISSION_SPECIFIED もしくは DataStoreConstants::PERMISSION_SPECIFIED_FRIEND で、自分が受け取ったもの
    SEARCH_TYPE_RECEIVED              =  8, //!< 参照権限が DataStoreConstants::PERMISSION_SPECIFIED もしくは DataStoreConstants::PERMISSION_SPECIFIED_FRIEND で自分が受け取ったもの、もしくは自分のフレンドが参照権限を DataStoreConstants::PERMISSION_FRIEND としてアップロードしたもの
    SEARCH_TYPE_PRIVATE               =  9, //!< 自分がオーナーかつ参照権限が DataStoreConstants::PERMISSION_PRIVATE のもの
    SEARCH_TYPE_OWN                   = 10, //!< 自分がオーナーかつ DataStoreConstants::DataStatus が DataStoreConstants::DATA_STATUS_NONE のもの
#ifndef FOR_DOC_PURPOSE
    SEARCH_TYPE_PUBLIC_EXCLUDE_FRIEND = 11, // フレンドと自分以外がアップロードしたものかつ参照権限が DataStoreConstants::PERMISSION_PUBLIC のもの
#endif  // FOR_DOC_PURPOSE
    SEARCH_TYPE_OWN_PENDING           = 12, //!< 自分がオーナーかつ DataStoreConstants::DataStatus が DataStoreConstants::DATA_STATUS_PENDING のもの
    SEARCH_TYPE_OWN_REJECTED          = 13, //!< 自分がオーナーかつ DataStoreConstants::DataStatus が DataStoreConstants::DATA_STATUS_REJECTED のもの
    SEARCH_TYPE_OWN_ALL               = 14  //!< 自分がオーナーかつ DataStoreConstants::DataStatus が DataStoreConstants::DATA_STATUS_NONE, DataStoreConstants::DATA_STATUS_PENDING, DataStoreConstants::DATA_STATUS_REJECTED のいずれかのもの
};

/*!
    @brief  検索する対象の設定です。
*/
enum SearchTarget
{
    SEARCH_TARGET_ANYBODY               = 0,    //!< すべての人を対象にする
    SEARCH_TARGET_FRIEND                = 1,    //!< フレンドのみを対象にする
#ifndef FOR_DOC_PURPOSE
    SEARCH_TARGET_EXCLUDE_SPECIFIED_IDS = 2     // 指定ID以外を対象にする
#endif  // FOR_DOC_PURPOSE
};

/*!
    @brief  検索結果のソート対象とするカラムです。
*/
enum SearchSortColumn
{
    SEARCH_SORT_COLUMN_DATAID           = 0, //!< データID
#ifndef FOR_DOC_PURPOSE
    SEARCH_SORT_COLUMN_SIZE_            = 1, // サイズ。現在は使用できません
    SEARCH_SORT_COLUMN_NAME_            = 2, // タイトル。現在は使用できません
    SEARCH_SORT_COLUMN_DATA_TYPE_       = 3, // データタイプ。現在は使用できません
    SEARCH_SORT_COLUMN_REFERRED_COUNT_  = 4, // 参照回数。現在は使用できません
#endif  // FOR_DOC_PURPOSE
    SEARCH_SORT_COLUMN_CREATED_TIME     = 5, //!< 作成日時
    SEARCH_SORT_COLUMN_UPDATED_TIME     = 6, //!< 更新日時

    SEARCH_SORT_COLUMN_RATING0          = 0x40, //!< 評価スロット0の評価合計値
    SEARCH_SORT_COLUMN_RATING1          = 0x41, //!< 評価スロット1の評価合計値
    SEARCH_SORT_COLUMN_RATING2          = 0x42, //!< 評価スロット2の評価合計値
    SEARCH_SORT_COLUMN_RATING3          = 0x43, //!< 評価スロット3の評価合計値
    SEARCH_SORT_COLUMN_RATING4          = 0x44, //!< 評価スロット4の評価合計値
    SEARCH_SORT_COLUMN_RATING5          = 0x45, //!< 評価スロット5の評価合計値
    SEARCH_SORT_COLUMN_RATING6          = 0x46, //!< 評価スロット6の評価合計値
    SEARCH_SORT_COLUMN_RATING7          = 0x47, //!< 評価スロット7の評価合計値
    SEARCH_SORT_COLUMN_RATING8          = 0x48, //!< 評価スロット8の評価合計値
    SEARCH_SORT_COLUMN_RATING9          = 0x49, //!< 評価スロット9の評価合計値
    SEARCH_SORT_COLUMN_RATING10         = 0x4a, //!< 評価スロット10の評価合計値
    SEARCH_SORT_COLUMN_RATING11         = 0x4b, //!< 評価スロット11の評価合計値
    SEARCH_SORT_COLUMN_RATING12         = 0x4c, //!< 評価スロット12の評価合計値
    SEARCH_SORT_COLUMN_RATING13         = 0x4d, //!< 評価スロット13の評価合計値
    SEARCH_SORT_COLUMN_RATING14         = 0x4e, //!< 評価スロット14の評価合計値
    SEARCH_SORT_COLUMN_RATING15         = 0x4f, //!< 評価スロット15の評価合計値

    SEARCH_SORT_COLUMN_RATING_AVERAGE0  = 0x60, //!< 評価スロット0の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE1  = 0x61, //!< 評価スロット1の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE2  = 0x62, //!< 評価スロット2の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE3  = 0x63, //!< 評価スロット3の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE4  = 0x64, //!< 評価スロット4の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE5  = 0x65, //!< 評価スロット5の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE6  = 0x66, //!< 評価スロット6の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE7  = 0x67, //!< 評価スロット7の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE8  = 0x68, //!< 評価スロット8の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE9  = 0x69, //!< 評価スロット9の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE10 = 0x6a, //!< 評価スロット10の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE11 = 0x6b, //!< 評価スロット11の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE12 = 0x6c, //!< 評価スロット12の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE13 = 0x6d, //!< 評価スロット13の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE14 = 0x6e, //!< 評価スロット14の評価平均値
    SEARCH_SORT_COLUMN_RATING_AVERAGE15 = 0x6f  //!< 評価スロット15の評価平均値
};

/*!
    @brief  検索結果のソート順です。
*/
enum SearchSortOrder
{
    SEARCH_SORT_ORDER_ASC    = 0,    //!< 昇順
    SEARCH_SORT_ORDER_DESC   = 1     //!< 降順
};

/*!
    @brief どのメタ情報を取得するかを選択する際に使用するフラグです。
*/
enum ResultFlag
{
    RESULT_FLAG_TAGS            = 0x0001, //!< タグ
    RESULT_FLAG_RATINGS         = 0x0002, //!< 評価情報
    RESULT_FLAG_METABINARY      = 0x0004, //!< バイナリデータ
    RESULT_FLAG_PERMITTED_IDS   = 0x0008  //!< 権限を持ったプリンシパルID。DataStoreClient::SearchObject() や DataStoreClient::SearchObjectLight() を使用する際は指定できません。
};


/*!
    @brief DataStoreRatingInitParam::SetLock() で指定する重複ロックのタイプです。
*/
enum RatingLockType
{
    RATING_LOCK_NONE      = 0,   //!< 重複ロックをしない
    RATING_LOCK_INTERVAL  = 1,   //!< 間隔によるロック
    RATING_LOCK_PERIOD    = 2,   //!< 期限によるロック
    RATING_LOCK_PERMANENT = 3    //!< 永久ロック
};

/*!
    @brief DataStoreRatingInitParam::SetFlag() で指定するビットフラグです。

        DataStoreConstants::RATING_FLAG_MODIFIABLEを指定した場合、再評価時には以前の評価値が合計値から引かれ、新たな評価値が合計値に足される動作となります。

*/
enum RatingFlag
{
    RATING_FLAG_MODIFIABLE          = 0x04,   //!< 変更可能にする
    RATING_FLAG_ROUND_MINUS         = 0x08,   //!< 加減算の結果負数になる場合0に丸める
    RATING_FLAG_DISABLE_SELF_RATING = 0x10    //!< 自己評価をできなくする
};

#ifndef FOR_DOC_PURPOSE
/*!
    @internal
    @brief DataStoreRatingInitParam の内部フラグです
*/
enum RatingInternalFlag
{
    RATING_INTERNAL_FLAG_USE_RANGE_MIN       = 0x02,   //!< 一度に評価可能な評価値の最低値を設定する。
    RATING_INTERNAL_FLAG_USE_RANGE_MAX       = 0x04    //!< 一度に評価可能な評価値の最大値を設定する。
};
#endif

/*!
    @brief DataStoreRatingLockInitParam::SetPeriodicLock() で指定する重複ロック期限です。
*/
enum RatingLockPeriod
{
    RATING_LOCK_PERIOD_MON  = -0x01,    //!< 毎月曜日
    RATING_LOCK_PERIOD_TUE  = -0x02,    //!< 毎火曜日
    RATING_LOCK_PERIOD_WED  = -0x03,    //!< 毎水曜日
    RATING_LOCK_PERIOD_THU  = -0x04,    //!< 毎木曜日
    RATING_LOCK_PERIOD_FRI  = -0x05,    //!< 毎金曜日
    RATING_LOCK_PERIOD_SAT  = -0x06,    //!< 毎土曜日
    RATING_LOCK_PERIOD_SUN  = -0x07,    //!< 毎日曜日
    RATING_LOCK_PERIOD_DAY1 = -0x11     //!< 毎月初1日
};

/*!
    @brief 検索一致件数の情報

        各タイプの仕様の詳細についてはプログラミングマニュアルを参照してください。

*/
enum SearchResultTotalCountType
{
    SEARCH_RESULT_TOTAL_EXACT    = 0x00,    //!< 正確な一致件数
    SEARCH_RESULT_TOTAL_MINIMUM  = 0x01,    //!< 返却した件数以上データがある
    SEARCH_RESULT_TOTAL_ESTIMATE = 0x02,    //!< 大凡の値である
    SEARCH_RESULT_TOTAL_DISABLED = 0x03     //!< 検索一致件数の取得が無効になっている
};

/*!
    @brief データのステータスです

        DataStoreConstants::DATA_FLAG_NEED_REVIEW を指定したデータはアップロード後に DataStoreConstants::DATA_STATUS_PENDING となります。
        DataStoreConstants::DATA_FLAG_NEED_REVIEW を指定しないデータはアップロード後に DataStoreConstants::DATA_STATUS_NONE となります。
        DataStoreConstants::DATA_STATUS_PENDING , DataStoreConstants::DATA_STATUS_REJECTED では権限設定によらずオーナーのみ参照・更新が可能です。
        DataStoreConstants::DATA_STATUS_NONE での参照・更新の可否は権限設定によります。

*/
enum DataStatus
{
    DATA_STATUS_NONE       = 0,    //!< 通常状態
    DATA_STATUS_PENDING    = 2,    //!< 審査中
    DATA_STATUS_REJECTED   = 5     //!< 審査で却下された
};

}

END_NAMESPACE_NEX

