﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreDeleteParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      データの削除に関するパラメータを表すクラスです。
*/
class DataStoreDeleteParam: public DDLCLASS(DataStoreDeleteParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreDeleteParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreDeleteParam() {}

    /*!
        @brief      データIDをセットします。

        削除するデータのデータIDを指定します。本パラメータは必ず指定する必要があります。

        @param[in]  dataId_ 削除するデータのデータID。
    */
    void SetDataId(qUnsignedInt64 dataId_)
    {
        this->dataId = dataId_;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      更新パスワードを設定します。

        更新権限を持たないデータを操作する場合に更新パスワードを指定します。
        データのオーナーから受け取った更新パスワードを指定してください。
        更新権限を持っている場合は更新パスワードは無視され、常に操作可能です。
        DataStoreConstants::INVALID_PASSWORD を指定した場合は更新パスワードでの認証を行いません。
        本関数で指定しない場合は DataStoreConstants::INVALID_PASSWORD が使用されます。

        @param[in]  updatePassword_ 更新パスワード
    */
    void SetUpdatePassword(qUnsignedInt64 updatePassword_)
    {
        this->updatePassword = updatePassword_;
    }

    /*!
        @brief      更新パスワードを取得します。

        @return     更新パスワード。
    */
    qUnsignedInt64 GetUpdatePassword() const
    {
        return this->updatePassword;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID);
        SetUpdatePassword(DataStoreConstants::INVALID_PASSWORD);
    }

    /*!
        @brief      DataStoreDeleteParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStoreDeleteParam: (")
               << NEX_T("dataId=") << dataId
               << NEX_T("updatePassword=") << updatePassword
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

