﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePersistenceInfoDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      永続化データ情報を表すクラスです。
*/
class DataStorePersistenceInfo: public DDLCLASS(DataStorePersistenceInfo)
{
public:
    /*!
        @brief      コンストラクタです。無効な永続化データ情報を作成します。
    */
    DataStorePersistenceInfo()
    {
        // ユーザーがこのクラスを生成したときにIsValid()がfalseを返すようにする
        dataId = DataStoreConstants::INVALID_DATAID;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePersistenceInfo() {}


    /*!
        @brief      プリンシパルIDを取得します。

        @return     プリンシパルID
    */
    PrincipalID GetPrincipalId() const
    {
        return this->ownerId;
    }

    /*!
        @brief      永続化スロットIDを取得します。

        @return     永続化スロットID
    */
    qUnsignedInt16 GetPersistenceSlotId() const
    {
        return this->persistenceSlotId;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      このオブジェクトが有効かどうかを調べます。

        返値が false だった場合は内部パラメータは無効ですので値を取得しないでください。

        @return     このオブジェクトが有効なら true 、無効なら false 。
    */
    qBool IsValid() const
    {
        return dataId != DataStoreConstants::INVALID_DATAID;
    }
};

END_NAMESPACE_NEX

