﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreRatingInfoDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      評価情報を表すクラスです。
*/
class DataStoreRatingInfo: public DDLCLASS(DataStoreRatingInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreRatingInfo() {}

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreRatingInfo() {}

    /*!
        @brief      評価合計値を取得します。

        取得した合計値には初期値も含みます。

        @return     評価合計値。
    */
    qInt64 GetTotalValue() const
    {
        return this->totalValue;
    }

    /*!
        @brief      評価された回数を取得します。

        @return     評価された回数。
    */
    qUnsignedInt32 GetCount() const
    {
        return this->count;
    }

    /*!
        @brief      評価合計値の初期値を取得します。

        @return     評価合計値の初期値。
    */
    qInt64 GetInitialValue() const
    {
        return this->initialValue;
    }

    /*!
        @brief      DataStoreRatingInfoオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド定義です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const DataStoreRatingInfo& oRatingInfo);
};

inline StringStream& operator<<(StringStream& oStr, const DataStoreRatingInfo& oRatingInfo)
{
    oStr << NEX_T("DataStoreRatingInfo: (")
         << NEX_T("totalValue=") << oRatingInfo.totalValue
         << NEX_T(", count=") << oRatingInfo.count
         << NEX_T(", initialValue=") << oRatingInfo.initialValue
         << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

