﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreReqGetInfoDDL.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreConstants.h>

BEGIN_NAMESPACE_NEX

/*
    @brief      ファイルサーバーからデータを取得するための情報を格納するクラスです。
*/
class DataStoreReqGetInfo: public DDLCLASS(DataStoreReqGetInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreReqGetInfo()
    {
        // ユーザーがこのクラスを生成したときにIsValid()がfalseを返すようにする
        dataId = DataStoreConstants::INVALID_DATAID;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreReqGetInfo() {}


    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      データサイズを取得します。

        @return     データサイズ。
    */
    qUnsignedInt32 GetSize() const
    {
        return this->size;
    }

    /*!
        @brief      このオブジェクトが有効かどうかを調べます。

        返値が false だった場合は内部パラメータは無効ですので値を取得しないでください。

        @return     このオブジェクトが有効なら true 、無効なら false 。
    */
    qBool IsValid() const
    {
        return dataId != DataStoreConstants::INVALID_DATAID;
    }
};

END_NAMESPACE_NEX

