﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreSearchResultDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      検索結果を表すクラスです。
*/
class DataStoreSearchResult: public DDLCLASS(DataStoreSearchResult)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreSearchResult() {}

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreSearchResult() {}

    /*!
        @brief      検索条件に一致した総数を取得します。

        DataStoreSearchResult::GetTotalCountType で取得した DataStoreConstants::SearchResultTotalCountType の値によって意味が異なります。

        @return     検索条件に一致した総数。
    */
    qUnsignedInt32 GetTotalCount() const
    {
        return this->totalCount;
    }

    /*!
        @brief      検索結果を取得します。

        @return     検索結果のリスト。

        検索結果レコードの参照権限(DataStoreMetaInfo::GetAccessPermission)、更新権限(DataStoreMetaInfo::GetUpdatePermission)の
        権限保有者の プリンシパルID は現状返却されず、常に空となっています。
    */
    const qVector<DataStoreMetaInfo>& GetResult() const
    {
        return this->result;
    }

    /*!
        @brief      検索一致件数が正確であるかどうかの情報を取得します。

        @return     検索一致件数が正確であるか
    */
    DataStoreConstants::SearchResultTotalCountType GetTotalCountType() const
    {
        return static_cast<DataStoreConstants::SearchResultTotalCountType>(this->totalCountType);
    }

};

END_NAMESPACE_NEX

