﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/ApiCallSummaryDDL.h>
#include <RendezVous/Services/Jugem/Debug/src/Common/DebugConstants.h>
BEGIN_NAMESPACE_NEX

/*!
    @brief      APIの呼び出し情報を格納したクラスです。
*/
#ifdef FOR_DOC_PURPOSE
class ApiCallSummary
#else
class ApiCallSummary: public DDLCLASS(ApiCallSummary)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ApiCallSummary()
    {
        methodName = NEX_T("");
        result = DebugConstants::ApiCallSummaryResult_Normal;
        period = 0;
        limit = 0;
        callStartTime = DateTime::Never;
        count = 0;
        totalCount = 0;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ApiCallSummary() {}

    /*!
        @brief      関数名を取得します

        @return     呼び出し関数名
    */
    const String& GetMethodName() const
    {
        return methodName;
    }

    /*!
        @brief      呼び出し頻度制限チェック結果を返します

        @return     呼び出し頻度制限チェック結果
    */
    DebugConstants::ApiCallSummaryResult GetResult() const
    {
        return static_cast<DebugConstants::ApiCallSummaryResult>(result);
    }

    /*!
        @brief      ガイドラインで規定されている呼び出し頻度の基準期間(秒)を返します

        ガイドラインで規定されている呼び出し頻度の基準期間(○秒間に×回、の○秒)を返します。

        @return     ガイドラインで規定されている呼び出し頻度の基準期間(秒)
    */
    qUnsignedInt32 GetPeriod() const
    {
        return period;
    }

    /*!
        @brief      ガイドラインで規定されている呼び出し頻度の基準回数を返します

        ガイドラインで規定されている呼び出し頻度の目安(○秒間に×回、の×回)を返します。

        @return     ガイドラインで規定されている呼び出し頻度の基準回数
     */
    qUnsignedInt32 GetLimit() const
    {
        return limit;
    }

    /*!
        @brief      呼び出し頻度の目安を超える最初の時刻を返します

        関数の呼び出しがガイドラインで規定されている頻度の目安を超える場合に、その一連の呼び出しの最初の時刻を返します。
        目安を超えていない場合は、最も多く呼び出されている箇所の最初の時刻を返します。

        @return      呼び出し頻度の目安を超える最初の時刻
     */
    const DateTime& GetCallStartTime() const
    {
        return callStartTime;
    }

    /*!
        @brief      目安を超えた際の呼び出し回数を返します

        関数の呼び出しがガイドラインで規定されている頻度の目安を超える場合に、その一連の呼び出しの呼び出し回数を返します。
        目安を超えていない場合は、最も多く呼び出されている箇所の呼び出し回数を返します。

        @return      目安を超えた際の呼び出し回数
     */
    qUnsignedInt32 GetCount() const
    {
        return count;
    }

    /*!
        @brief      トータルの呼び出し回数を返します

        DebugClient::GetApiCallSummary() で指定した期間内のトータル呼び出し回数を返します

        @return      トータルの呼び出し回数を返します
     */
    qUnsignedInt32 GetTotalCount() const
    {
        return totalCount;
    }
};

END_NAMESPACE_NEX

