﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTE: Values of each element of the following enumation must be
// specify so that translating this file for Python scripts works as expected.
// Also, the description of elements must use C-style /* */ comments where
// the opening and closing syntax must be on their own lines of code.
////////////////////////////////////////////////////////////////////////////////

#ifndef _FpdErrors_H_
#define _FpdErrors_H_

BEGIN_NAMESPACE_NEX

/*!
 @brief    フレンドモジュール(nn::friends)のサクセスコードを定義する名前空間です。
 */
#ifdef SWIG
class FPDSuccessCode
{
public:
#else
namespace FPDSuccessCode
{
#endif
/*!
 @brief    FPD モジュールのサクセスコード。
*/
enum SuccessCode
{
    Success         = 1,    //!<    成功
    RmcNotCalled   = 2    //!<    成功(非同期処理は呼ばれなかった)
};
#ifdef SWIG
};
#else
}
#endif

/*!
 @brief    フレンドモジュール(nn::friends)のエラーコードを定義する名前空間です。
 */
#ifdef SWIG
class FPDErrorCode
{
public:
#else
namespace FPDErrorCode
{
#endif

/*!
 @brief    フレンドモジュールのエラーコード。
*/
enum ErrorCode
{
    START = 0,
    NotInitialized = 0,    //!<    フレンドライブラリが初期化されていない。
    AlreadyInitialized,    //!<    フレンドライブラリが初期化ずみ。
    NotConnected,    //!<    NFS にログインしていない
    Connected,    //!<    NFS にログイン済み
    InitializationFailure,    //!<    NGS との接続に失敗
    OutOfMemory,    //!<    メモリ確保に失敗(使用されません)
    RmcFailed,    //!<    非同期処理に失敗(使用されません)
    InvalidArgument,    //!<    引数が不正(使用されません)
    InvalidLocalAccountID,    //!<    不正なローカルアカウント ID
    InvalidPrincipalID,    //!<    不正なプリンシパル ID
    InvalidLocalFriendCode,    //!<    不正なローカルフレンドコード
    LocalAccountNotExists,    //!<    ローカルアカウントが存在しない(使用されません)
    LocalAccountNotLoaded,    //!<    ローカルアカウントがロードされていない(使用されません)
    LocalAccountAlreadyLoaded,    //!<    ローカルアカウントがロード済み(使用されません)
    FriendAlreadyExists,    //!<    既にフレンド関係が構築されている(使用されません)
    FriendNotExists,    //!<    フレンドが存在しない(使用されません)
    FriendNumMax,    //!<    フレンド数の上限に到達(使用されません)
    NotFriend,    //!<    フレンドではない(使用されません)
    FileIO,    //!<    ファイル IO エラー(使用されません)
    P2PInternetProhibited,    //!<    P2P による他者との通信が禁止されている(使用されません)
    Unknown    //!<    未定義のエラー(使用されません)
};
//note: 新規result追加時には、OnlineCore/src/Platform/Core/ResulDefine.hのNEX_ERROR_RESULT_DEFINEの変更も必要
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif
