﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Matchmake/src/Common/FindMatchmakeSessionByParticipantParamDDL.h>

#if defined(NEX_SIGLO_FAMILY)
#include <nn/account.h>
#endif

BEGIN_NAMESPACE_NEX
/*!
@brief      マッチメイクセッションの検索条件を格納するデータクラスです。

MatchmakeExtensionClient::FindMatchmakeSessionByParticipant における検索条件として利用します

*/
class FindMatchmakeSessionByParticipantParam : public DDLCLASS(FindMatchmakeSessionByParticipantParam)
{
public:
    /*!
    @brief      コンストラクタです。
    */
    FindMatchmakeSessionByParticipantParam()
    {
        Reset();
    }

    /*!
    @brief      デストラクタです。
    */
    virtual ~FindMatchmakeSessionByParticipantParam() {}

    /*!
     @brief    検索対象となるプリンシパルIDのリストを設定します

     @param[in]     principalIdList    プリンシパルIDのリスト
     */
    void SetPrincipalIds(const qList<PrincipalID>& principalIdList)
    {
        m_principalIdList = principalIdList;
    }

    /*!
     @brief    検索結果として返す FindMatchmakeSessionResult の形式を設定します

     @param[in]     resultOptions    FindMatchmakeSessionResultOption のビットフラグ

     @see     nn::nex::FindMatchmakeSessionResultOption
     */
    void SetResultOptions(qUnsignedInt32 resultOptions)
    {
        m_resultOptions = resultOptions;
    }
    /*!
     @brief    ブロックリストの使用方法を設定します

     @param[in]     blockListParam    ブロックリストの挙動を指定するパラメータ
     */
    void SetBlockListParam(const MatchmakeBlockListParam& blockListParam)
    {
        m_blockListParam = blockListParam;
    }

    /*!
     @brief    検索対象となるプリンシパルIDのリストを返します

     @return   プリンシパルIDのリスト
     */
    const qList<PrincipalID>& GetPrincipalIds() const
    {
        return m_principalIdList;
    }

    /*!
     @brief    設定された FindMatchmakeSessionResultOption の値を返します

     @return   FindMatchmakeSessionResultOption の値

     @see     nn::nex::FindMatchmakeSessionResultOption
     */
    qUnsignedInt32 GetResultOptions() const
    {
        return m_resultOptions;
    }

    /*!
     @brief    設定された MatchmakeBlockListParam を返します

     @return   MatchmakeBlockListParam
     */
    const MatchmakeBlockListParam& GetBlockListParam() const
    {
        return m_blockListParam;
    }

#if defined(NEX_SIGLO_FAMILY)
    /*!
     @brief    NetworkServiceAccountId から検索対象となるPrincipalIDのリストを設定します

     @param[in]     pNetworkServiceAccountIds    NetworkServiceAccountID のリスト
     @param[in]     length                       リストに含まれる NetworkServiceAccountID の数
     */
    void SetPrincipalIdListByNetworkServiceAccountId(const nn::account::NetworkServiceAccountId *pNetworkServiceAccountIds, int length);
#endif

    /*!
    @brief    オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        m_resultOptions = FIND_MATCHMAKESESSION_RESULT_OPTION_NONE;
        m_blockListParam.Reset();
        m_principalIdList.clear();
    }
};
END_NAMESPACE_NEX

