﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/BuiltInParticipationPolicies.h>

BEGIN_NAMESPACE_NEX

/// マッチメイクセッションに指定可能な属性の数
static const qUnsignedInt32 NUM_MATCHMAKE_SESSION_ATTRIBUTES = 6;

/// 条件に最も近いマッチメイクセッションを選ぶ際の基準とする属性値のインデックス
static const qUnsignedInt32 MATCHMAKE_SESSION_NEAREST_NEIGHBOR_ATTRIBUTE_INDEX = 1;

/// レンジ拡大形式でマッチメイクセッションを選ぶ際の基準とする属性値のインデックス
static const qUnsignedInt32 MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX = MATCHMAKE_SESSION_NEAREST_NEIGHBOR_ATTRIBUTE_INDEX;

/// 無効な永続ギャザリングコード
static const qUnsignedInt64 INVALID_PERSISTENT_GATHERING_CODE = 0;

/// 一人あたりの永続ギャザリング登録上限
static const qUnsignedInt32 PERSISTENT_GATHERING_CREATION_MAX = 4;

/// 一人あたりの永続ギャザリング参加上限
static const qUnsignedInt32 PERSISTENT_GATHERING_PARTICIPATION_MAX = 16;

/// 永続ギャザリング内での最大チャット人数、この人数以下に最大参加人数をセットする必要があります。
static const qUnsignedInt64 PERSISTENT_GATHERING_CHAT_PARTICIPANTS_MAX = 32;

#if defined(CTR_TARGET) || !defined(FOR_DOC_PURPOSE)
/// 無効なコミュニティコード
static const qUnsignedInt64 INVALID_COMMUNITY_CODE = INVALID_PERSISTENT_GATHERING_CODE;

/// 一人あたりのコミュニティ登録上限
static const qUnsignedInt32 COMMUNITY_CREATION_MAX = PERSISTENT_GATHERING_CREATION_MAX;

/// 一人あたりのコミュニティ参加上限
static const qUnsignedInt32 COMMUNITY_PARTICIPATION_MAX = PERSISTENT_GATHERING_PARTICIPATION_MAX;

/// コミュニティ内での最大チャット人数、この人数以下に最大参加人数をセットする必要があります。
static const qUnsignedInt64 COMMUNITY_CHAT_PARTICIPANTS_MAX = PERSISTENT_GATHERING_CHAT_PARTICIPANTS_MAX;
#endif

/// マッチメイクセッションを複数の属性値で検索するとき、属性値の要素数の最大値
static const qUnsignedInt32 SETATTIBUTE_VECTOR_SIZE_MAX = 100;

/// ゲーム進行度の最大値
static const qUnsignedInt8 MAX_PROGRESS_SCORE = 100;

/// ゲーム進行度の最小アップデート間隔（秒）
static const qUnsignedInt32 UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME = 30;

/// マッチメイクセッションユーザーパスワードの最大長
static const qUnsignedInt32 MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH = 32;

// マッチメイクセッションシステムパスワードの長さ
static const qUnsignedInt32 MATCHMAKE_SESSION_SYSTEM_PASSWORD_LENGTH = 16;

/// MatchmakeExtensionClient::BrowseMatchmakeSession() で検索可能な最大サイズ。もしくは MatchmakeExtensionClient::FindMatchmakeSessionByGatheringId() で指定可能なギャザリング ID リストの最大サイズ
static const qUnsignedInt32 MAX_MATCHMAKE_BROWSE_SIZE = 100;

/// MatchmakeExtensionClient::FindMatchmakeSessionByParticipant() もしくは MatchmakeExtensionClient::GetPlayingSession() で指定可能なプリンシパル ID  のリストの最大サイズ
static const qUnsignedInt32 MAX_PRINCIPALID_SIZE_TO_FIND_MATCHMAKE_SESSION = 300;

// MatchmakeExtensionClient::FindMatchmakeSessionByParticipant() で指定可能な ID リストの最大サイズ
static const qUnsignedInt32 MAX_MATCHMAKE_BROWSE_SIZE_BY_PARTICIPANT = MAX_PRINCIPALID_SIZE_TO_FIND_MATCHMAKE_SESSION;

/// MatchmakeExtensionClient::FindMatchmakeSessionByParticipant() で返却される最大セッション数
static const qUnsignedInt32 MAX_MATCHMAKE_SESSION_BY_PARTICIPANT = 1000;


extern const qChar* MATCHMAKE_PARAM_KEY_SETTING_INDEX;
extern const qChar* MATCHMAKE_PARAM_KEY_NINTENDO_COUNTRY_CODE;
extern const qChar* MATCHMAKE_PARAM_KEY_OVERRIDE_IP_ADDRESS;
extern const qChar* MATCHMAKE_PARAM_KEY_USE_GEO_IP;
extern const qChar* MATCHMAKE_PARAM_KEY_COUNTRY_CODE;
extern const qChar* MATCHMAKE_PARAM_KEY_RATING_VALUE;
extern const qChar* MATCHMAKE_PARAM_KEY_DISCONNECTION_RATE;
extern const qChar* MATCHMAKE_PARAM_KEY_VIOLATION_RATE;
extern const qChar* MATCHMAKE_PARAM_KEY_LATITUDE;
extern const qChar* MATCHMAKE_PARAM_KEY_LONGITUDE;
extern const qChar* MATCHMAKE_PARAM_KEY_DISTANCE;
extern const qChar* MATCHMAKE_PARAM_KEY_TOTAL_SCORE;
extern const qChar* MATCHMAKE_PARAM_KEY_GEO_IP_RESULT;
extern const qChar* MATCHMAKE_PARAM_KEY_UPDATE_GEO_IP;
extern const qChar* MATCHMAKE_PARAM_KEY_SERVER_RETURNED;
extern const qChar* MATCHMAKE_PARAM_KEY_LAST_GID_FOR_PARTICIPATION_CHECK;

// 追加枠人数に設定できる最大数
static const qUnsignedInt32 MAX_EXTRA_PARTICIPANTS = 4;

/*!
 @brief    マッチメイクの種別を表す列挙子

 現状では、 MATCHMAKE_SYSTEM_TYPE_ANYBODY, MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING (MATCHMAKE_SYSTEM_TYPE_COMMUNITY) が指定可能です。
 */
typedef enum
{
    MATCHMAKE_SYSTEM_TYPE_INVALID,         //!< 無効
    MATCHMAKE_SYSTEM_TYPE_ANYBODY,         //!< 誰でも参加可能
#if !defined(FOR_DOC_PURPOSE)
    // フレンドマッチは廃止予定なので deprecated 扱いとする
    MATCHMAKE_SYSTEM_TYPE_FRIEND,          //!< オーナーと友達の場合のみ参加可能
    MATCHMAKE_SYSTEM_TYPE_FRIEND_INVITE,   //!< 招待されていて、かつオーナーと友達の場合のみ参加可能(使用不可)
#endif
    MATCHMAKE_SYSTEM_TYPE_ANYBODY_INVITE,  //!< 招待されている場合のみ参加可能(使用不可)
    // Community->PersistentGatheringの移行。 コミュニティマッチと同じです。
    MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING, //!< 永続ギャザリングマッチ。互換性のために存在します。
    // CTRの場合(ドキュメント生成含む) || CTR以外ではドキュメント生成時以外
#if defined(CTR_TARGET) || !defined(FOR_DOC_PURPOSE)
    MATCHMAKE_SYSTEM_TYPE_COMMUNITY = MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING, //!< コミュニティマッチ(コミュニティは廃止予定であるため、永続ギャザリングの使用を推奨)
#endif
    // Cafeの場合(ドキュメント生成含む) || Cafe以外ではドキュメント生成時以外
#if defined(CAFE_TARGET) || !defined(FOR_DOC_PURPOSE)
    MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY,   //!< Miiverseコミュニティマッチ。互換性のために存在します。
#endif
    MATCHMAKE_SYSTEM_TYPE_END              //!< 列挙子の末尾(使用不可)
}
MatchmakeSystemType;

/*!
 @brief    検索したマッチメイクセッションから参加するものを選ぶ方法

 オートマッチメイク機能を利用するとき、検索条件に一致するマッチメイクセッションが複数存在する場合、
 ゲームサーバーは MatchmakeSessionSearchCriteria::SetSelectionMethod()
 で指定した選び方に基づいて参加するマッチメイクセッションを決定します。デフォルトはランダムです。
 MatchmakeSessionSearchCriteria::SetSelectionMethod() では MatchmakeSelectionMethod の値を指定できます。
 検索条件に一致するマッチメイクセッションが50件を超える場合は、任意の50件に対して
 MatchmakeSessionSearchCriteria::SetSelectionMethod() で指定した選び方が適用されます。

 MATCHMAKE_SELECTION_METHOD_RANDOM を指定した場合、ランダムに選択します。

 MATCHMAKE_SELECTION_METHOD_NEAREST_NEIGHBOR を指定した場合、マッチメイクセッションの属性値が
 MatchmakeExtensionClient::AutoMatchmake() の引数に与えた MatchmakeSession オブジェクトの属性値に
 最も近いマッチメイクセッションを選びます。
 属性値は、インデックス MATCHMAKE_SELECTION_METHOD_NEAREST_NEIGHBOR の属性値が使用されます。
 属性値が同じマッチメイクセッションが複数存在する場合、選ばれるマッチメイクセッションはギャザリングID順となります。
 検索条件に一致するマッチメイクセッションが50件を超える場合は、
 必ずしも属性値が最も近いマッチメイクセッションが選択されるとは限りません。

 MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE を指定した場合、レンジ拡大形式でマッチメイクセッションを選びます。
 レンジとは、連続する整数を任意の閾値で何段階かに区切ったとき、その区切られた一区間のことをいいます。
 デフォルトでは、ELOレーティングを使うことを想定して、0から10000までの整数が、
 0～1100、1101～1240、1241～1400、1401～1520、1521～1620、1621～1720、
 1721～1815、1816～1925、1926～2040、2041～2180、2181～2300、2301～10000の12段階に区切られており、
 区切られた各区間がレンジとなります。
 レンジ拡大形式では、検索条件に合致するマッチメイクセッションの中から、マッチメイクセッションの属性値が
 MatchmakeExtensionClient::AutoMatchmake() の引数に与えた MatchmakeSession オブジェクトの属性値と
 同じレンジにあるマッチメイクセッションをランダムで選びます。
 属性値は、インデックス MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX の属性値が使用されます。
 同じレンジにマッチメイクセッションが見つからなかった場合は、
 現在のレンジに隣接する任意のレンジまで範囲を拡大し、拡大後の範囲内にある検索条件に合致するマッチメイクセッションの中から、
 インデックス MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX の属性値が最も近いマッチメイクセッションを選びます。
 デフォルトでは、現在のレンジに隣接する上下2レンジまで範囲を拡大します。
 先の例では、例えば1401～1520のレンジでマッチメイクセッションを探して見つからなかった場合、1101～1720まで範囲を拡大します。
 この機能は、例えばレーティングスコアによってランク付けがされるようなゲームの場合、
 インデックス MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX の属性値をレーティングスコアに、
 レンジをランクに対応させることで、近いランクの人とのマッチメイクが可能になります。

 MATCHMAKE_SELECTION_METHOD_PROGRESS_SCORE を指定した場合、
 検索条件に合致するマッチメイクセッションの中から、
 MatchmakeSession::SetProgressScore() もしくは MatchmakeExtensionClient::UpdateProgressScore() で設定した
 ゲーム進行度が最も大きいマッチメイクセッションを選びます。
 例えば、対戦が開始されても引き続き新たな参加者を受け付けるマッチメイクセッションにおいて、
 セッションの新たな参加者は次の対戦から参加することができる場合に、この機能は有用です。
 このとき、対戦が開始して間もないセッションに参加するよりも、対戦が終了しそうなセッションに参加した方が、
 次の対戦までの待ち時間が少なくて済みます。
 これの実現には、対戦の開始時にはセッションのゲーム進行度を 0 に、
 対戦が進むにつれ MatchmakeExtensionClient::UpdateProgressScore() で定期的にゲーム進行度を上げていきます。
 なお、 MatchmakeExtensionClient::UpdateProgressScore() を頻繁に呼ぶとサーバーに負荷がかかります。
 最小アップデート間隔 UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒以上の間隔でアップデートするようにしてください。

 MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE を指定した場合、
 レンジ拡大形式とゲーム進行度を組み合わせてマッチメイクセッションを選択します。
 検索条件に合致するマッチメイクセッションの中から、マッチメイクセッションの属性値が
 MatchmakeExtensionClient::AutoMatchmake() の引数に与えた MatchmakeSession オブジェクトの属性値と
 同じレンジにあるマッチメイクセッションのうち、
 ゲーム進行度が最も大きいマッチメイクセッションを選びます。
 属性値は、インデックス MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX の属性値が使用されます。
 同じレンジにマッチメイクセッションが見つからなかった場合は、
 現在のレンジに隣接するレンジまで範囲を拡大し、拡大後の範囲内にある検索条件に合致するマッチメイクセッションの中から、
 ゲーム進行度が最も大きいマッチメイクセッションを選びます。

 MATCHMAKE_SELECTION_METHOD_SCORE_BASED を指定した場合、
 様々なパラメータをスコア付けして最適なマッチメイクセッションを選択します。
 あらかじめ各種パラメータの値に対するスコアをスコア換算設定に定義しておくことにより、
 検索条件に合致するマッチメイクセッションのパラメータと自身のパラメータと比較することでそのトータルスコアが
 もっとも高いマッチメイクセッションを選びます。
 使用できるパラメータは以下の通りです。

 @li レーティング値の差
 @li 切断率の差
 @li 違反率の差
 @li インターネット経路を考慮した距離
 @li 国の一致
 @li 距離情報が無いときの国が一致
 @li マッチメイクセッションの存続時間
 @li マッチメイクセッションの進行度

 スコア換算設定は設定インデックスを切り替えることで複数種類使用することができます。
 例えばゲームモードによって設定インデックスを切り替えてスコアの割り振りを変化させることができます。
 MatchmakeSessionSearchCriteria を複数指定するときに一番目の条件には厳しめのスコア換算設定を用い、
 二番目の条件には緩めのスコア換算設定を用いることでより適したマッチメイクセッションを探すこともできます。
 本機能を利用するには MatchmakeParam を引数に取る MatchmakeSessionSearchCriteria::SetSelectionMethod()
 のオーバーロード関数を利用する必要があります。また、この関数に指定する MatchmakeParam オブジェクトを
 MatchmakeSession::SetMatchmakeParam() を使い MatchmakeSession にも指定する必要があります。
 自身のレーティング値、切断率、違反率、国といった情報はアプリケーションが
 MatchmakeParam に対してセットする必要があります。
 アプリケーションでレーティング値、切断率、違反率を算出する場合は MatchmakeReferee の利用をご検討ください。

*/
typedef enum
{
    MATCHMAKE_SELECTION_METHOD_RANDOM = 0,        //!< ランダムに選択します
    MATCHMAKE_SELECTION_METHOD_NEAREST_NEIGHBOR,  //!< 属性値が最も近いマッチメイクセッションを選択します
    MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE,     //!< レンジ拡大形式で選択します
    MATCHMAKE_SELECTION_METHOD_PROGRESS_SCORE,    //!< ゲーム進行度が最も大きいマッチメイクセッションを選択します
    MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE,  //!< レンジ拡大形式とゲーム進行度を組み合わせて選択します
    MATCHMAKE_SELECTION_METHOD_SCORE_BASED        //!< 各種パラメータをスコア付けして最適なものを選択します
}
MatchmakeSelectionMethod;

/*!
 @brief    位置情報の取得結果
*/
typedef enum
{
    MATCHMAKE_GEO_IP_RESULT_INVALID = 0,    //!< 使用されません。無効値として利用できます。
    MATCHMAKE_GEO_IP_RESULT_FOUND,          //!< 位置情報が見つかった。
    MATCHMAKE_GEO_IP_RESULT_NOT_FOUND,      //!< 位置情報が見つからなかった。
    MATCHMAKE_GEO_IP_RESULT_UNUSED          //!< 位置情報を利用しない設定だった。
}
MATCHMAKE_GEO_IP_RESULT;


// CreateMatchmakeSessionOption, JoinMatchmakeSessionOption, AutoMatchmakeOption は MatchmakeOption で定義した値を参照し、同じ意味の物は同じ値をとるようにする。
#ifndef FOR_DOC_PURPOSE
/*!
 @internal
 @brief    マッチメイクセッションに関するオプションビットフラグ
*/
enum MatchmakeOption
{
    MATCHMAKE_OPTION_NONE                           = 0x00000000, //!< オプションを指定しません。
    MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = 0x00000001, //!< 前回参加チェック用に指定したギャザリング ID をマッチメイクセッションの MatchmakeParam に保存します。永続ギャザリングには記憶されません。
    MATCHMAKE_OPTION_RESERVED_1 = 0x00000002 //!< 予約済みの値です。
};
#endif

#ifndef FOR_DOC_PURPOSE
/*!
 @internal
 @brief    マッチメイクセッションを作成する際のオプションビットフラグ
*/
enum CreateMatchmakeSessionOption
{
    CREATE_MATCHMAKE_SESSION_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //!< オプションを指定しません。
    CREATE_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK //!< 前回参加チェック用に指定したギャザリング ID をマッチメイクセッションの MatchmakeParam に保存します。永続ギャザリングには記憶されません。
};
#endif

#ifndef FOR_DOC_PURPOSE
/*!
 @internal
 @brief    マッチメイクセッションに参加する際のオプションビットフラグ
*/
enum JoinMatchmakeSessionOption
{
    JOIN_MATCHMAKE_SESSION_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //!< オプションを指定しません。
    JOIN_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK //!< 前回参加チェック用に指定したギャザリング ID をマッチメイクセッションの MatchmakeParam に保存します。永続ギャザリングには記憶されません。
};
#endif

/*!
 @brief    オートマッチメイクする際のオプションビットフラグ
*/
enum AutoMatchmakeOption
{
    AUTO_MATCHMAKE_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //!< オプションを指定しません。
    AUTO_MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK, // 前回参加チェック用に指定したギャザリング ID を MatchmakeParam に保存します。
    AUTO_MATCHMAKE_OPTION_UNIQUE_GATHERING_BY_CODEWORD = MATCHMAKE_OPTION_RESERVED_1 //!< あいことばマッチメイクで 1 つのみしかギャザリングを作成しない設定です。
};

/*!
 @brief    ブロックリストの挙動を指定するビットフラグ
*/
enum MatchmakeBlockListOption
{
    MATCHMAKE_BLOCKLIST_OPTION_NONE                      = 0x00000000, //!< オプションを指定しません。
    MATCHMAKE_BLOCKLIST_OPTION_USE_MY_BLOCKLIST          = 0x00000001, //!< フラグが立っている場合、参加者に自分がブロックリストに登録しているユーザーがいるマッチメイクセッションには参加できません。
    MATCHMAKE_BLOCKLIST_OPTION_USE_BLOCKLIST_BY_OTHERS   = 0x00000002  //!< フラグが立っている場合、参加者に自分をブロックリストに登録しているユーザーがいるマッチメイクセッションには参加できません。
};

/*!
 @brief    FindMatchmakeSessionByParticipantResult の形式を指定するビットフラグ
*/
enum FindMatchmakeSessionResultOption
{
    FIND_MATCHMAKESESSION_RESULT_OPTION_NONE               = 0x00000000, //!< オプションを指定しません。
    FIND_MATCHMAKESESSION_RESULT_OPTION_APPLICATION_BUFFER = 0x00000001, //!< フラグが立っている場合、マッチメイクセッションに ApplicationBuffer を含めます
    FIND_MATCHMAKESESSION_RESULT_OPTION_MATCHMAKE_PARAM    = 0x00000002  //!< フラグが立っている場合、マッチメイクセッションに MatchmakeParam を含めます
};

/*!
 @brief    参加処理の振る舞いに関するフラグ
*/
enum JoinMatchmakeSessionBehavior
{
    JOIN_MATCHMAKE_SESSION_BEHAVIOR_JOIN_MYSELF,                      //!< 自身も参加します。すでに参加済みなら結果は QERROR(RendezVous, AlreadyParticipatedGathering) となります。
    JOIN_MATCHMAKE_SESSION_BEHAVIOR_IM_ALREADY_JOINED                 //!< 自身はすでに参加済みのため参加しません。もし参加していなかった場合は QERROR(RendezVous, NotParticipatedGathering) となります。自身はすでに参加済みのマッチメイクセッションに他人を参加させる場合に利用できます。
};

/*!
 @brief    どの項目を更新するかを示すビットフラグ
*/
enum MatchmakeSessionModificationFlag
{
    MATCHMAKE_SESSION_MODIFICATION_FLAG_NONE                        = 0x00000000,   //!< どの項目も更新しない。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_ATTRIBUTES                  = 0x00000001,   //!< 属性。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_OPEN_PARTICIPATION          = 0x00000002,   //!< 締め切り設定。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_APPLICATION_BUFFER          = 0x00000004,   //!< アプリケーション定義データ。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_PROGRESS_SCORE              = 0x00000008,   //!< 進行度。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_OPTION0                     = 0x00000010,   //   オプション0 (使用しません)。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM             = 0x00000020,   //!< マッチメイクパラメーター。全てのパラメーターを置き換えます。MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM_OVERRIDE と同時に使用することはできません。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM_OVERRIDE    = 0x00000040,   //!< マッチメイクパラメーター。既存のパラメーターは更新、新しいパラメーターは追加します。MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM と同時に使用することはできません。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_STARTED_TIME                = 0x00000080,   //!< 開始日時。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_USER_PASSWORD               = 0x00000100,   //!< ユーザーパスワード。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_GAME_MODE                   = 0x00000200,   //!< ゲームモード。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_DESCRIPTION                 = 0x00000400,   //!< 説明文字列。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MIN_PARTICIPANTS            = 0x00000800,   //!< 最小参加人数。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MAX_PARTICIPANTS            = 0x00001000,   //!< 最大参加人数。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_SYSTEM_TYPE       = 0x00002000,   //!< マッチメイクシステムタイプ。
    MATCHMAKE_SESSION_MODIFICATION_FLAG_CODEWORD                    = 0x00004000    //!< あいことば
};

namespace BuiltInParticipationPolicies
{
static const qUnsignedInt32 FriendOnlyParticipationPolicy = FirstUserPolicy - 2;        // フレンドのみ参加可能
static const qUnsignedInt32 CommunityParticipationPolicy = FirstUserPolicy - 3;         // コミュニティ
static const qUnsignedInt32 MatchmakeSessionParticipationPolicy = FirstUserPolicy - 4;  // マッチメイクセッション
static const qUnsignedInt32 AnybodyParticipationPolicy          = FirstUserPolicy - 5;  // 誰でも参加可能なマッチメイクセッション
}

/*!
 @brief    誰でも参加可能なマッチメイクセッション時に指定するオプション
 */
namespace AnybodyParticipationPolicyArgument
{
/*!
 @brief    誰でも参加可能なマッチメイクセッション時に指定するオプション
 */
typedef enum
{
    WithoutClose          = 0, //!< オーナーが GatheringFlags::MigrateOwner フラグにより自動的に変更されてもマッチメイクセッションの募集状態を変えない
    CloseOnOwnerMigration = 1, //!< オーナーが GatheringFlags::MigrateOwner フラグにより自動的に変更された時点で自動的にマッチメイクセッションを締め切る
    End
}
AnybodyParticipationPolicyArgument;
}

/*!
 @brief    フレンドマッチ時に指定するオプション
 */
namespace FriendOnlyParticipationPolicyArgument
{
/*!
 @brief    フレンドマッチ時に指定するオプション
 */
typedef enum
{
    CloseOnOwnerMigration, //!< オーナーが GatheringFlags::MigrateOwner フラグにより自動的に変更された時点で自動的にマッチメイクセッションを締め切る
    WithoutClose,          //!< オーナーが GatheringFlags::MigrateOwner フラグにより自動的に変更れてもマッチメイクセッションの募集状態を変えない
    End
}
FriendOnlyParticipationPolicyArgument;
}

END_NAMESPACE_NEX

