﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Matchmake/src/Common/PlayingSessionDDL.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringHolder.h>

BEGIN_NAMESPACE_NEX

class MatchmakeSession;

/*!
 @brief   プリンシパルIDとそのユーザーが参加しているマッチメイクセッションを格納するクラスです。

 
 @ref MatchmakeExtensionClient::GetPlayingSession() で使用されます。
 */
#ifdef FOR_DOC_PURPOSE
class PlayingSession: public RootObject
#else
class PlayingSession: public DDLCLASS(PlayingSession)
#endif
{
public:
    /*!
    @brief      コンストラクタです。
    */
    PlayingSession() {}

    /*!
    @brief      デストラクタです。
    */
    virtual ~PlayingSession() {}

    /*!
     @brief      プリンシパルIDを取得します

     @return   参加者のプリンシパルID
     */
    PrincipalID GetPrincipalID() const
    {
        return m_PrincipalId;
    }

    /*!
     @brief      参加しているマッチメイクセッションを取得します

     @return   @ref MatchmakeSession へのポインタ
     */
    MatchmakeSession* GetMatchmakeSession() const;

    /*!
     @brief    参加しているマッチメイクセッションを @ref GatheringHolder 形式で取得します

     @return   @ref GatheringHolder への参照
     */
    GatheringHolder& GetGatheringHolder()
    {
        return m_Gathering;
    }
};

END_NAMESPACE_NEX

