﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "MatchmakeRefereeStatsInitParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      個人統計の初期値を表すクラスです。
*/
class MatchmakeRefereeStatsInitParam: public DDLCLASS(MatchmakeRefereeStatsInitParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    MatchmakeRefereeStatsInitParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~MatchmakeRefereeStatsInitParam() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetInitialRatingValue(0);
    }

    /*!
        @brief      レートの初期値をセットします。

        @param[in] initialRatingValue_ レートの初期値。
    */
    void SetInitialRatingValue(qUnsignedInt32 initialRatingValue_)
    {
        this->initialRatingValue = initialRatingValue_;
    }

    /*!
        @brief      レートの初期値を取得します。

        @return レートの初期値。
    */
    qUnsignedInt32 GetInitialRatingValue() const
    {
        return this->initialRatingValue;
    }

    /*!
        @brief      カテゴリをセットします。

        @param[in] category_ カテゴリ。
    */
    void SetCategory(qUnsignedInt32 category_)
    {
        this->category = category_;
    }

    /*!
        @brief      カテゴリを取得します。

        @return カテゴリを。
    */
    qUnsignedInt32 GetCategory() const
    {
        return this->category;
    }
};

END_NAMESPACE_NEX

