﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "MatchmakeRefereeStatsTargetDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      個人統計を取得する際の対象を表すクラスです。
*/
class MatchmakeRefereeStatsTarget: public DDLCLASS(MatchmakeRefereeStatsTarget)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    MatchmakeRefereeStatsTarget()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~MatchmakeRefereeStatsTarget() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetPrincipalId(0);
        SetCategory(0);
    }

    /*!
        @brief      プリンシパル ID をセットします。

        @param[in] pid_ プリンシパル ID。
    */
    void SetPrincipalId(PrincipalID pid_)
    {
        this->pid = pid_;
    }

    /*!
        @brief      プリンシパル ID を取得します。

        @return プリンシパル ID。
    */
    PrincipalID GetPrincipalId() const
    {
        return this->pid;
    }

    /*!
        @brief      カテゴリをセットします。

        @param[in] category_ カテゴリ。
    */
    void SetCategory(qUnsignedInt32 category_)
    {
        this->category = category_;
    }

    /*!
        @brief      カテゴリを取得します。

        @return カテゴリ。
    */
    qUnsignedInt32 GetCategory() const
    {
        return this->category;
    }

};

END_NAMESPACE_NEX

