﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingChangeAttributesParamDDL.h>
#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingConstants.h>

BEGIN_NAMESPACE_NEX


#ifdef FOR_DOC_PURPOSE
/*!
    @brief      スコア属性値の変更に関するパラメータを表すクラスです。
*/
class RankingChangeAttributesParam
#else
class RankingChangeAttributesParam: public DDLCLASS(RankingChangeAttributesParam)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    RankingChangeAttributesParam()
    {
        SetModificationFlag(RankingConstants::MODIFICATION_FLAG_NONE);
        this->groups.assign(4,0);
        SetParam(0);
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~RankingChangeAttributesParam() {};

    /*!
        @brief      変更フラグをセットします。

        属性値を変更する箇所を表すフラグを指定します。本パラメータは必ず指定する必要があります。
        RankingConstants::ModificationFlag に対してビット単位の論理和を用いて複数指定することができます。

        @param[in]  flag 属性値を変更する箇所を表すビットフラグ。 RankingConstants::ModificationFlag に対するビット単位の論理和です。
    */
    void SetModificationFlag(qByte flag)
    {
        this->modificationFlag = flag;
    };

    /*!
        @brief      変更フラグを取得します。

        @return     属性値を変更する箇所を表すビットフラグ。 RankingConstants::ModificationFlag に対するビット単位の論理和です。
    */
    qByte GetModificationFlag() const
    {
        return this->modificationFlag;
    };

    /*!
        @brief      グループ0をセットします

        新しいグループ0の値を指定します。本パラメータの指定は必須ではありません。デフォルトは0です。
        変更フラグに RankingConstants::MODIFICATION_FLAG_GROUP0 を指定していない場合、グループ0の値は更新されません。

        @param[in]  group0  新しいグループ0の値。
    */
    void SetGroup0(qByte group0)
    {
        this->groups[0] = group0;
    };

    /*!
        @brief      グループ0を取得します。

        @return     グループ0。
    */
    qByte GetGroup0() const
    {
        return this->groups[0];
    };

    /*!
        @brief      グループ1をセットします

        新しいグループ1の値を指定します。本パラメータの指定は必須ではありません。デフォルトは0です。
        変更フラグに RankingConstants::MODIFICATION_FLAG_GROUP1 を指定していない場合、グループ1の値は更新されません。

        @param[in]  group1  新しいグループ1の値。
    */
    void SetGroup1(qByte group1)
    {
        this->groups[1] = group1;
    };

    /*!
        @brief      グループ1を取得します。

        @return     グループ1。
    */
    qByte GetGroup1() const
    {
        return this->groups[1];
    };

    void SetGroups(const qVector<qByte>& groups_)
    {
        this->groups = groups_;
    };
    const qVector<qByte>& GetGroups() const
    {
        return this->groups;
    };

    /*!
        @brief      パラメータをセットします

        新しいパラメータの値を指定します。本パラメータの指定は必須ではありません。デフォルトは0です。
        変更フラグに RankingConstants::MODIFICATION_FLAG_PARAM を指定していない場合、パラメータの値は更新されません。

        @param[in]  param_  新しいパラメータの値。
    */
    void SetParam(qUnsignedInt64 param_)
    {
        this->param = param_;
    };

    /*!
        @brief      パラメータを取得します。

        @return     パラメータ。
    */
    qUnsignedInt64 GetParam() const
    {
        return this->param;
    };

};

END_NAMESPACE_NEX

