﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingOrderParamDDL.h>
#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingConstants.h>

BEGIN_NAMESPACE_NEX


#ifdef FOR_DOC_PURPOSE
/*!
    @brief      順位計算に必要なパラメータです。
*/
class RankingOrderParam
#else
class RankingOrderParam: public DDLCLASS(RankingOrderParam)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    RankingOrderParam()
    {
        SetOrderCalculation(RankingConstants::ORDER_CALCULATION_113);
        SetTimeScope(RankingConstants::TIME_SCOPE_ALL);
        SetFilterGroupIndex(RankingConstants::FILTER_GROUP_INDEX_NONE);
        SetFilterGroupNum(0);
        SetOffset(0);
        SetLength(10);
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~RankingOrderParam() {};

    /*!
        @brief      順位の計算方法をセットします。

        順位の計算方法を指定します。デフォルトは RankingConstants::ORDER_CALCULATION_113 です。

        @param[in]  orderCalculation_  順位の計算方法を表す定数。
    */
    void SetOrderCalculation(RankingConstants::OrderCalculation orderCalculation_)
    {
        this->orderCalculation = static_cast<qByte>(orderCalculation_);
    };

    /*!
        @brief      順位の計算方法を取得します。

        @return     順位の計算方法を表す定数。
    */
    RankingConstants::OrderCalculation GetOrderCalculation() const
    {
        return static_cast<RankingConstants::OrderCalculation>(this->orderCalculation);
    };

    /*!
        @brief      フィルタリングするグループのインデックスをセットします。

        フィルタリングするグループを指定する定数を指定します。デフォルトは RankingConstants::FILTER_GROUP_INDEX_NONE (フィルタリングしない)です。

        @param[in]  groupIndex_  フィルタリングするグループを表す定数。
    */
    void SetFilterGroupIndex(RankingConstants::FilterGroupIndex groupIndex_)
    {
        this->groupIndex = static_cast<qByte>(groupIndex_);
    };

    /*!
        @brief      フィルタリングするグループのインデックスを取得します。

        @return     フィルタリングするグループを表す定数。
    */
    RankingConstants::FilterGroupIndex GetFilterGroupIndex() const
    {
        return static_cast<RankingConstants::FilterGroupIndex>(this->groupIndex);
    };

    /*!
        @brief      フィルタリングするグループの値をセットします。

        フィルタリングするグループの値をセットします。 RankingOrderParam::SetFilterGroupIndex 関数で指定したインデックスのグループについて、本関数でセットした値によるフィルタリングが行われます。RankingOrderParam::SetFilterGroupIndex 関数で RankingConstants::FILTER_GROUP_INDEX_NONE をセットした場合、本関数でセットした値は無効になります。デフォルトは 0 です。

        @param[in]  groupNum_  フィルタリングするグループの値。
    */
    void SetFilterGroupNum(qByte groupNum_)
    {
        this->groupNum = groupNum_;
    };

    /*!
        @brief      フィルタリングするグループの値を取得します。

        @return     フィルタリングするグループの値。
    */
    qByte GetFilterGroupNum() const
    {
        return this->groupNum;
    };

    /*!
        @brief      ランキングに表示するスコアの対象期間(タイムスコープ)をセットします。

        RankingConstants::TIME_SCOPE_ALL をセットした場合は全ての期間を対象とします。

        RankingConstants::TIME_SCOPE_CUSTOM_0 または RankingConstants::TIME_SCOPE_CUSTOM_1 をセットした場合は、NMAS で設定されたタイムスコープタイプの期間を対象とします。
        ただし、TIME_SCOPE_CUSTOM_0 および TIME_SCOPE_CUSTOM_1 は初期状態では無効に設定されており、スコア登録の前にあらかじめ設定しておく必要があります。

        本関数で設定しない場合のデフォルト値は RankingConstants::TIME_SCOPE_ALL です。

        @param[in]  timeScope_  タイムスコープを表す定数。
    */
    void SetTimeScope(RankingConstants::TimeScope timeScope_)
    {
        this->timeScope = static_cast<qByte>(timeScope_);
    };

    /*!
        @brief      ランキングに表示するスコアの対象期間(タイムスコープ)を取得します。

        @return     タイムスコープを表す定数。
    */
    qByte GetTimeScope() const
    {
        return static_cast<RankingConstants::TimeScope>(this->timeScope);
    };

    /*!
        @brief      取得するリストのオフセットをセットします

        取得するリストのオフセットを指定します。デフォルトは0です。

        @param[in]  offset_  スコアの値。
    */
    void SetOffset(qUnsignedInt32 offset_)
    {
        this->offset = offset_;
    };

    /*!
        @brief      オフセットを取得します。

        @return     オフセット。
    */
    qUnsignedInt32 GetOffset() const
    {
        return this->offset;
    };

    /*!
        @brief      取得するリストの長さをセットします

        取得するリストの長さを指定します。0 ～ 255 の範囲で指定できます。デフォルトは10です。

        @param[in]  length_  リストの長さ。
    */
    void SetLength(qByte length_)
    {
        this->length = length_;
    };

    /*!
        @brief      取得するリストの長さを取得します。

        @return     リストの長さ。
    */
    qByte GetLength() const
    {
        return this->length;
    };

};

END_NAMESPACE_NEX

