﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingRankDataDDL.h>
#include <RendezVous/Services/Jugem/Utility/src/Common/UniqueId.h>

BEGIN_NAMESPACE_NEX

#ifdef FOR_DOC_PURPOSE
/*!
    @brief      ランキングの情報が格納されるクラスです。
 */
class RankingRankData
#else
class RankingRankData: public DDLCLASS(RankingRankData)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    RankingRankData()
    {
        this->principalId = 0;
        this->uniqueId = 0;
        this->order = 0;
        this->category = 0;
        this->param = 0;
        this->score = 0;
        // ランキング共通データの消費メモリ量を設定する
        this->commonData.clear(); // (MAX_COMMON_DATA_SIZE + 1) / 2
        this->updateTime = DateTime::Never;
    };

    /*!
        @brief      デストラクタです。
    */
    virtual ~RankingRankData() {};

    /*!
        @brief      プリンシパル IDを取得します。

        @return     プリンシパル ID。
    */
    PrincipalID GetPrincipalId() const
    {
        return this->principalId;
    };

    /*!
        @brief      NEX ユニーク IDを取得します。

        @return     NEX ユニーク ID。
    */
    UniqueId GetUniqueId() const
    {
        return this->uniqueId;
    };

    /*!
        @brief      順位を取得します。

        @return     順位。最上位は 1。
    */
    qUnsignedInt32 GetOrder() const
    {
        return this->order;
    };

    /*!
        @brief      カテゴリを取得します。

        @return     カテゴリ。
    */
    qUnsignedInt32 GetCategory() const
    {
        return this->category;
    };

    /*!
        @brief      スコアを取得します。

        @return     スコア。
    */
    qUnsignedInt32 GetScore() const
    {
        return this->score;
    };

    const qVector<qByte>& GetGroups() const
    {
        return this->groups;
    };

    /*!
        @brief      グループ0を取得します。

        @return     グループ0。
    */
    qByte GetGroup0() const
    {
        if( this->groups.size() < 1) return 0;
        return this->groups[0];
    };

    /*!
        @brief      グループ1を取得します。

        @return     グループ1。
    */
    qByte GetGroup1() const
    {
        if( this->groups.size() < 2) return 0;
        return this->groups[1];
    };

    /*!
        @brief      パラメータを取得します。

        @return     パラメータ。
    */
    qUnsignedInt64 GetParam() const
    {
        return this->param;
    };

    /*!
        @brief      共通データを取得します。

        @return     共通データ。
    */
    const qVector<qByte>& GetCommonData() const
    {
        return this->commonData;
    };

    /*!
        @brief      更新時間(UTC時間)を取得します

        @return     更新時間
    */
    const DateTime& GetUpdateTime() const
    {
        return this->updateTime;
    };

};

END_NAMESPACE_NEX

