﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingResultDDL.h>
#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingRankData.h>

BEGIN_NAMESPACE_NEX

#ifdef FOR_DOC_PURPOSE
/*!
    @brief      ランキングの情報が格納されるクラスです。
 */
class RankingResult
#else
class RankingResult: public DDLCLASS(RankingResult)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    RankingResult()
    {
        this->totalCount = 0;
    };

    /*!
        @brief      デストラクタです。
    */
    virtual ~RankingResult() {};

    /*!
        @brief      条件に一致したランキングの総数を取得します。

        @return     ランキングの総数。
    */
    qUnsignedInt32 GetTotalCount() const
    {
        return this->totalCount;
    };


    /*!
        @brief      結果のランキングリストを取得します。

        @return     ランキングリスト。
    */
    const qVector<RankingRankData>& GetRankDataList() const
    {
        return this->rankDataList;
    };

    /*!
        @brief      ランキングの集計開始日時を取得します。

        @return     ランキングの集計開始日時。
    */
    DateTime GetSinceTime() const
    {
        return this->sinceTime;
    };

};

END_NAMESPACE_NEX

