﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingScoreDataDDL.h>
#include <RendezVous/Services/Jugem/Ranking/src/Common/RankingConstants.h>

BEGIN_NAMESPACE_NEX


#ifdef FOR_DOC_PURPOSE
/*!
    @brief      アップロードするスコアの情報を格納するクラスです。
*/
class RankingScoreData
#else
class RankingScoreData: public DDLCLASS(RankingScoreData)
#endif
{
public:
    /*!
        @brief      コンストラクタです。
    */
    RankingScoreData()
    {
        SetCategory(0);
        SetScore(0);
        SetOrderBy(RankingConstants::ORDER_BY_DESC);
        SetUpdateMode(RankingConstants::UPDATE_MODE_NORMAL);
        this->groups.assign(4,0);
        SetParam(0);
    };

    /*!
        @brief      デストラクタです。
    */
    virtual ~RankingScoreData() {};

    /*!
        @brief      カテゴリを取得します。

        @return     カテゴリ。
    */
    qUnsignedInt32 GetCategory() const
    {
        return this->category;
    };

    /*!
        @brief      カテゴリをセットします

        カテゴリの値を指定します。デフォルトは0です。

        @param[in]  category_  カテゴリの値。
    */
    void SetCategory(qUnsignedInt32 category_)
    {
        this->category = category_;
    };

    /*!
        @brief      スコアをセットします

        スコアの値を指定します。デフォルトは0です。

        @param[in]  score_  スコアの値。
    */
    void SetScore(qUnsignedInt32 score_)
    {
        this->score = score_;
    };

    /*!
        @brief      スコアを取得します。

        @return     スコア。
    */
    qUnsignedInt32 GetScore() const
    {
        return this->score;
    };

    /*!
        @brief      アップロード先カテゴリのスコアの並び順を指定します。

        アップロード先カテゴリのスコアの並び順を指定します。 @n
        デフォルトは RankingConstants::ORDER_BY_DESC (降順)です。 @n
        同じカテゴリには必ず同じ並び順を指定するようにして下さい。 @n
        同じカテゴリに異なる並び順でスコアをアップロードした場合、ランキング結果が異常になります。

        @param[in]  orderBy_  スコアの並び順を表す定数。
    */
    void SetOrderBy(RankingConstants::OrderBy orderBy_)
    {
        this->orderBy = static_cast<qByte>(orderBy_);
    };

    /*!
        @brief      スコアの並び順を取得します。

        @return     スコアの並び順を表す定数。
    */
    RankingConstants::OrderBy GetOrderBy() const
    {
        return static_cast<RankingConstants::OrderBy>(this->orderBy);
    };

    /*!
        @brief      スコアの更新モードを指定します。

        RankingConstants::UPDATE_MODE_NORMAL (デフォルト) を指定した場合、
        自分が以前そのカテゴリに対してアップロードしたスコアの記録は保持され、ランキング取得時には
        指定されたタイムスコープの期間内で最も上位のスコアがランキングに利用されます。
        指定されたタイムスコープの期間で最も上位のスコアが複数ある場合には、最新のスコアがランキングに利用されます。

        RankingConstants::UPDATE_MODE_DELETE_OLD を指定した場合、
        対象カテゴリの過去のスコアを削除した上で新しいスコアを登録します。
        スコアの良し悪しに関わらず常に最新のスコアを利用する場合に使用します。

        @param[in]  updateMode_  スコアの更新モード。
    */
    void SetUpdateMode(RankingConstants::UpdateMode updateMode_)
    {
        this->updateMode = static_cast<qByte>(updateMode_);
    };

    /*!
        @brief      スコアの更新モードを取得します。

        @return     スコアの更新モード。
    */
    RankingConstants::UpdateMode GetUpdateMode() const
    {
        return static_cast<RankingConstants::UpdateMode>(this->updateMode);
    };

    /*!
        @brief      グループ0をセットします

        グループ0の値を指定します。デフォルトは0です。

        @param[in]  group0  グループ0の値。
    */
    void SetGroup0(qByte group0)
    {
        this->groups[0] = group0;
    };

    /*!
        @brief      グループ0を取得します。

        @return     グループ0。
    */
    qByte GetGroup0() const
    {
        return this->groups[0];
    };

    /*!
        @brief      グループ1をセットします

        グループ1の値を指定します。デフォルトは0です。

        @param[in]  group1 グループ1の値。
    */
    void SetGroup1(qByte group1)
    {
        this->groups[1] = group1;
    };

    /*!
        @brief      グループ1を取得します。

        @return     グループ1。
    */
    qByte GetGroup1() const
    {
        return this->groups[1];
    };

    void SetGroups(const qVector<qByte>& groups_)
    {
        this->groups = groups_;
    };

    const qVector<qByte>& GetGroups() const
    {
        return this->groups;
    };

    /*!
        @brief      パラメータをセットします

        パラメータの値を指定します。デフォルトは0です。

        @param[in]  param_ パラメータの値。
    */
    void SetParam(qUnsignedInt64 param_)
    {
        this->param = param_;
    };

    /*!
        @brief      パラメータを取得します。

        @return     パラメータ。
    */
    qUnsignedInt64 GetParam() const
    {
        return this->param;
    };
};

END_NAMESPACE_NEX

