﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Utility/src/Common/UniqueId.h>
#include "Ranking2RankDataDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      ランキング内の１つのランクデータを表すクラスです。
*/
class Ranking2RankData: public DDLCLASS(Ranking2RankData)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    Ranking2RankData()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~Ranking2RankData() {}

    /*!
        @brief      順位を取得します。

        @return     順位の値。最高位は 1。
    */
    qUnsignedInt32 GetRank() const
    {
        return rank;
    };

    /*!
        @brief      スコアを取得します。

        @return     スコアの値。
    */
    qUnsignedInt32 GetScore() const
    {
        return score;
    };

    /*!
        @brief      プリンシパル ID を取得します。

        @return     プリンシパル ID の値。
    */
    PrincipalID GetPrincipalId() const
    {
        return principalId;
    };

    /*!
        @brief      NEX ユニーク IDを取得します。

        @return     NEX ユニーク ID の値。
    */
    UniqueId GetNexUniqueId() const
    {
        return nexUniqueId;
    };

    /*!
        @brief      ユーザー共通データを取得します。

        @return     ユーザー共通データの値。
    */
    const Ranking2CommonData& GetCommonData() const
    {
        return commonData;
    };

    /*!
        @brief      misc を取得します。

        @return     misc。
    */
    qUnsignedInt64 GetMisc() const
    {
        return misc;
    };

    /*!
        @brief      Ranking2RankData オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2RankData& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2RankData& rhs)
{
    oStr << NEX_T("Ranking2RankData: (")
               << NEX_T("rank=") << rhs.rank
               << NEX_T(", score=") << rhs.score
               << NEX_T(", principalId=") << rhs.principalId
               << NEX_T(", nexUniqueId=") << rhs.nexUniqueId
               << NEX_T(", misc=") << rhs.misc;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX


