﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "Ranking2ScoreDataDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      スコアの登録に関するパラメータを表すクラスです。
*/
class Ranking2ScoreData: public DDLCLASS(Ranking2ScoreData)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    Ranking2ScoreData()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~Ranking2ScoreData() {}

    /*!
        @brief      カテゴリを取得します。

        @return     カテゴリの値。
    */
    qUnsignedInt32 GetCategory() const
    {
        return category;
    };

    /*!
        @brief      カテゴリをセットします。

        @param[in]  category_  カテゴリの値。デフォルトは 0。
    */
    void SetCategory(qUnsignedInt32 category_)
    {
        this->category = category_;
    };

    /*!
        @brief      スコアを取得します。

        @return     スコアの値。
    */
    qUnsignedInt32 GetScore() const
    {
        return score;
    };

    /*!
        @brief      スコアをセットします。

        @param[in]  score_  スコアの値。デフォルトは 0。
    */
    void SetScore(qUnsignedInt32 score_)
    {
        this->score = score_;
    };

    /*!
        @brief      misc を取得します。

        @return     misc。
    */
    qUnsignedInt64 GetMisc() const
    {
        return misc;
    };

    /*!
        @brief      misc をセットします。

        @param[in]  misc_  misc。デフォルトは 0。
    */
    void SetMisc(qUnsignedInt64 misc_)
    {
        this->misc = misc_;
    };


    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetCategory(0);
        SetScore(0);
        SetMisc(0);
    }

    /*!
        @brief      Ranking2ScoreData オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2ScoreData& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2ScoreData& rhs)
{
    oStr << NEX_T("Ranking2ScoreData: (")
               << NEX_T("category=") << rhs.category
               << NEX_T(", score=") << rhs.score
               << NEX_T(", misc=") << rhs.misc;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX


