﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef JOB_SERVICEITEM_ACQUIRE_EC_SERVICE_TOKEN_RMC_H__
#define JOB_SERVICEITEM_ACQUIRE_EC_SERVICE_TOKEN_RMC_H__

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>
#include <OnlineCore/src/Core/Chrono.h>
#include <RendezVous/Core/src/Common/ProtocolCallContext.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemPurchaseServiceItemResponse.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemPurchaseServiceItemParam.h>

#ifdef CAFE_TARGET
#include <nn/act.h>
#endif

BEGIN_NAMESPACE_NEX

class ServiceItemInternalClientInterface;
class ServiceItemClientInterface;

template<class TJobTraits>
class JobServiceItemAcquireEcServiceTokenRmc : public StepSequenceJob, public NonCopyable
{
public:
    JobServiceItemAcquireEcServiceTokenRmc();
    virtual ~JobServiceItemAcquireEcServiceTokenRmc();

    virtual qResult Initialize(
        qUnsignedInt32 uiAssociatedCallID,
        ServiceItemClientInterface* pServiceItemClient,
        ServiceItemInternalClientInterface* pServiceItemInternalClient,
        const typename TJobTraits::TParam& serviceItemParam,
        typename TJobTraits::TResponse* pServiceItemResponse
    );

protected:

#ifdef CAFE_TARGET
    static const qUnsignedInt32 s_EcServiceTokenSize = ACT_INDEPENDENT_SERVICE_TOKEN_SIZE;
#else
    static const qUnsignedInt32 s_EcServiceTokenSize = 513;
#endif

    qUnsignedInt32 m_AssociatedCallID;
    ServiceItemClientInterface* m_pServiceItemClient;
    ServiceItemInternalClientInterface* m_pServiceItemInternalClient;
    typename TJobTraits::TParam m_ServiceItemParam;
    typename TJobTraits::TResponse* m_pServiceItemResponseOut;
    typename TJobTraits::TResponse m_ServiceItemResponse;

    qChar8 m_EcServiceToken[s_EcServiceTokenSize];
    qResult m_AcquireEcServiceTokenResult;
    Chrono m_AcquireEcServiceTokenWaitingTime;
    ProtocolCallContext m_CallContext;
    qBool m_IsWarned;

    // トークン取得用の同期関数呼び出し用のスレッド
    ObjectThread<JobServiceItemAcquireEcServiceTokenRmc, void*>* m_pThread;

    void AcquireEcServiceTokenFunc(void*);

    void StepAcquireEcServiceToken();
    void StepWaitingForAcquireEcServiceToken();
    void StepRmc();
    void StepWaitingRmc();
    void CompleteJob(const qResult& result);
};

END_NAMESPACE_NEX

#endif /* JOB_SERVICEITEM_ACQUIRE_EC_SERVICE_TOKEN_RMC_H__ */
