﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAttributeDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      属性情報を表すクラスです。
*/
class ServiceItemAttribute: public DDLCLASS(ServiceItemAttribute)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAttribute()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAttribute() {}

    /*!
        @brief      属性名を取得します。

        @return     属性名。
    */
    const String& GetName() const { return name; }

    /*!
        @brief      属性値を取得します。

        @return     属性値。
    */
    const String& GetValue() const { return value; }

    /*!
        @brief      ServiceItemAttribute オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemAttribute(");
        ss << NEX_T("Name:") << GetName();
        ss << NEX_T(" Value:") << GetValue();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }

};


END_NAMESPACE_NEX

