﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAutoBillingPlansDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      継続課金アイテム情報の一覧を表すクラスです。
*/
class ServiceItemAutoBillingPlans: public DDLCLASS(ServiceItemAutoBillingPlans)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAutoBillingPlans()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAutoBillingPlans() {}

    /*!
        @brief      検索にマッチした継続課金アイテムの総数を取得します。

        @return     検索にマッチした継続課金アイテムの総数。
    */
    qUnsignedInt32 GetTotalSize() const { return totalSize; }

    /*!
        @brief      レスポンスに含まれる継続課金アイテムの数を取得します。

        @return     レスポンスに含まれる継続課金アイテムの数。
    */
    qUnsignedInt32 GetLength() const { return length; }

    /*!
        @brief      検索にマッチしたすべての継続課金アイテムの中から、リストに取得した継続課金アイテムのオフセット (開始位置) を取得します。

        @return     オフセット。
    */
    qUnsignedInt32 GetOffset() const { return offset; }

    /*!
        @brief      このタイトルの継続課金説明文を取得します。
                    Cafe でのみ利用されます。

        @return     このタイトルの継続課金説明文。
    */
    const String& GetDescription() const { return description; }

    /*!
        @brief      継続課金アイテムを取得します。

        @return     継続課金アイテム。
    */
    const qVector<ServiceItemAutoBillingPlan>& GetPlans() const { return plans; }

    /*!
        @brief      ServiceItemAutoBillingPlans オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemAutoBillingPlans(");
        ss << NEX_T(" totalSize:") << GetTotalSize();
        ss << NEX_T(" length:") << GetLength();
        ss << NEX_T(" offset:") << GetOffset();
        ss << NEX_T(" description:") << GetDescription();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());

        qInt i = 0;
        qVector<ServiceItemAutoBillingPlan>::const_iterator it = GetPlans().begin();
        for (; it != GetPlans().end(); ++it, ++i)
        {
            ss.Clear();
            ss << title << NEX_T("    plans[") << i << NEX_T("]");
            it->Trace(uiTraceFlags, ss.CStr());
        }
    }
};

END_NAMESPACE_NEX

