﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemGetSupportIdParamDDL.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      サポート ID を取得するためのパラメータを表すクラスです。
*/
class ServiceItemGetSupportIdParam: public DDLCLASS(ServiceItemGetSupportIdParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemGetSupportIdParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemGetSupportIdParam() {}

    /*!
        @internal
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @internal
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
    }

    /*!
        @brief      ServiceItemGetSupportIdParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemGetSupportIdParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        // 以下の理由によりユニークIDは必須にしない
        // ・サポート番号は管理ツールからも利用しているので、そちらとの互換性を維持したい
        // ・サポート番号は他のタイトルと重複しなければよいので、WUPとCTRでNGSを共有していてもどちらかのユニークIDから生成すればよい
//        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
//        {
//            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemGetSupportIdParam::SetUniqueId()."));
//            success = false;
//        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemGetSupportIdParam& rhs)
{
    oStr << NEX_T("ServiceItemGetSupportIdParam: (")
                << NEX_T(", uniqueId=") << rhs.uniqueId
                << NEX_T(")");
    (void)rhs;

    return oStr;
}

END_NAMESPACE_NEX

