﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemListAutoBillingServiceItemParamDDL.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      指定されたタイトルコードを持つゲームタイトルで定義された継続課金アイテムの一覧を取得します。
*/
class ServiceItemListAutoBillingServiceItemParam: public DDLCLASS(ServiceItemListAutoBillingServiceItemParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemListAutoBillingServiceItemParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemListAutoBillingServiceItemParam() {}

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId) { this->uniqueId = uniqueId; }

    /*!
        @brief      本体設定言語を取得します。

        @return     本体設定言語。
    */
    const String& GetLanguage() const { return language; }

    /*!
        @brief      [必須パラメタ] 言語をセットします。

        言語をセットします。通貨表記のフォーマットを決定するために使用します。
        ISO 639 で規定されている 2 文字の言語コードを使用します。
        nn::cfg::CTR::GetLanguageCodeA2() 関数で取得した文字列を指定してください。

        @param[in]  language 言語。
    */
    void SetLanguage(const String& language) { this->language = language; }

    /*!
        @brief      フィルタにマッチする検索結果からサービスアイテムを取得するオフセットを取得します。

        @return     フィルタにマッチする検索結果からサービスアイテムを取得するオフセット。
    */
    qUnsignedInt32 GetOffset() const { return offset; }

    /*!
        @brief      フィルタにマッチする検索結果からサービスアイテムを取得するオフセットをセットします。

        @param[in]  offset フィルタにマッチする検索結果からサービスアイテムを取得するオフセット。
    */
    void SetOffset(qUnsignedInt32 offset) { this->offset = offset; }

    /*!
        @brief      検索結果として取得するサービスアイテムの数を取得します。

        @return     検索結果として取得するサービスアイテムの数。
    */
    qUnsignedInt32 GetSize() const { return size; }

    /*!
        @brief      検索結果として取得するサービスアイテムの数をセットします。

        @param[in]  size 検索結果として取得するサービスアイテムの数。
    */
    void SetSize(qUnsignedInt32 size) { this->size = size; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        SetLanguage(ServiceItemConstants::INVALID_LANGUAGE);
        SetOffset(0);
        SetSize(20);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
    }
    /*!
        @brief      ServiceItemListAutoBillingServiceItemParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemListAutoBillingServiceItemParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemListAutoBillingServiceItemParam::SetUniqueId()."));
            success = false;
        }
        if(GetLanguage() == ServiceItemConstants::INVALID_LANGUAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid language. Please set language with ServiceItemListAutoBillingServiceItemParam::SetLanguage()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemListAutoBillingServiceItemParam& rhs)
{
    oStr << NEX_T("ServiceItemListAutoBillingServiceItemParam: (")
               << NEX_T("language=") << rhs.language
               << NEX_T(", offset=") << rhs.offset
               << NEX_T(", size=") << rhs.size
               << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX
