﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SERVICEITEM_LIST_REDEMPTION_ITEM_H
#define _SERVICEITEM_LIST_REDEMPTION_ITEM_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemListRedemptionItemDDL.h"
#include "ServiceItemAttribute.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      一つの引換アイテム情報を表すクラスです。
*/
class ServiceItemListRedemptionItem: public DDLCLASS(ServiceItemListRedemptionItem)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemListRedemptionItem()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemListRedemptionItem() {}

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      初回購入のみ可能かどうかを取得します。

            true の場合、あるリファレンス ID に対して既に権利を有している場合、該当のリファレンス ID に対してこのサービスアイテムを購入することができません。
            サービスアイテムのリストアップの際に権利情報も合わせて取得することで、リファレンス ID に対しての購入可否を判断することが可能です。

        @return     初回購入のみ可能かどうか。
    */
    qBool GetInitialPurchaseOnly() const { return initialPurchaseOnly; }

    /*!
        @brief      権利情報を取得します。

            ここで取得した権利情報の権利種別が ServiceItemConstants::LIMITATION_TYPE_ADDITIONAL_TIME の場合、権利種別に応じた値は有効期間 (秒数) となります。

        @return     権利情報。
    */
    const ServiceItemLimitation& GetLimitation() const { return limitation; }

    /*!
        @brief      属性情報のリストを取得します。

                サーバ側の変更に対応するため、本関数で取得した qVector の要素へはハードコードのインデックス等でのアクセスはせず、
                必ず REFERENCE_ID 等の文字列を検索してアクセスしてください。

        @return     属性情報。
    */
    const qVector<ServiceItemAttribute>& GetAttributes() const { return attributes; }

    /*!
        @brief      ServiceItemListRedemptionItem オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        {
            StringStream ss;
            ss << title << NEX_T("ServiceItemListRedemptionItem(");
            ss << NEX_T("ItemCode:") << GetItemCode();
            ss << NEX_T(" InitialPurchaseOnly:") << GetInitialPurchaseOnly();
            ss << NEX_T(")");
            QTRACE(uiTraceFlags, ss.CStr());
        }

        GetLimitation().Trace(uiTraceFlags, title + NEX_T("    "));

        qVector<ServiceItemAttribute>::const_iterator it = GetAttributes().begin();
        for (; it != GetAttributes().end(); ++it)
        {
            it->Trace(uiTraceFlags, title + NEX_T("    "));
        }
    }
};

END_NAMESPACE_NEX

#endif
