﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemPostRightBinaryResponseDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @extends    ServiceItemEShopResponse
    @brief      ServiceItemClient::PostRightBinaryByAccount() で使用する、レスポンスを表すクラスです。
*/
class ServiceItemPostRightBinaryResponse: public DDLCLASS(ServiceItemPostRightBinaryResponse)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemPostRightBinaryResponse()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemPostRightBinaryResponse() {}

    /*!
        @brief      eShop サーバーからのレスポンス内容を取得します。

        本関数は常に NULL を返します。

        @return      NULL 。
    */
    const RootObject* Get() const { return NULL; }

    /*!
        @brief      eShop サーバーから返されたネットワークエラーコードを取得します。

        @return     eShop サーバーから返されたネットワークエラーコード。
    */
    virtual qUnsignedInt32 GetErrorCode() const
    {
        // 正常時でもレスポンスは空なのでエラーにならないようにするためにオーバライドする
        return errorCode;
    }

    /*!
        @brief      ServiceItemPostRightBinaryResponse オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title;
        ss << *this;
        QTRACE(uiTraceFlags, ss.CStr());
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemPostRightBinaryResponse& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemPostRightBinaryResponse& rhs)
{
    oStr << NEX_T("ServiceItemPostRightBinaryResponse: (")
                << static_cast<const ServiceItemEShopResponse&>(rhs)
                << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

