﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemPurchaseInfoDDL.h"
#include "ServiceItemAmount.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      購入結果を表すクラスです。
*/
class ServiceItemPurchaseInfo: public DDLCLASS(ServiceItemPurchaseInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemPurchaseInfo()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemPurchaseInfo() {}

    /*!
        @brief      eShop サーバで発行されたトランザクション ID を取得します。

        @return     eShop サーバで発行されたトランザクション ID。
    */
    const String& GetTransactionId() const { return transactionId; }

    /*!
        @brief      NGS で発行されたトランザクション ID を取得します。

        @return     NGS で発行されたトランザクション ID。
    */
    const String& GetExtTransactionId() const { return extTransactionId; }

    /*!
        @brief      購入したサービスアイテムのアイテムコードを取得します。

        @return     購入したサービスアイテムのアイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      トランザクション実行後の残高情報を取得します。

        @return     トランザクション実行後の残高情報。
    */
    const ServiceItemAmount& GetPostBalance() const { return postBalance; }

    /*!
        @brief      ServiceItemPurchaseInfo オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemPurchaseInfo(");
        ss << NEX_T("TransactionId:") << GetTransactionId();
        ss << NEX_T(" ExtTransactionId:") << GetExtTransactionId();
        ss << NEX_T(" ItemCode:") << GetItemCode();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());

        GetPostBalance().Trace(uiTraceFlags, title + NEX_T("    PostBalance"));
    }
};

END_NAMESPACE_NEX

