﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SERVICEITEM_REDEMPTION_CATALOG_H
#define _SERVICEITEM_REDEMPTION_CATALOG_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemRedemptionCatalogDDL.h"
#include "ServiceItemListRedemptionItem.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      引換アイテム一覧を表すクラスです。
*/
class ServiceItemRedemptionCatalog: public DDLCLASS(ServiceItemRedemptionCatalog)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemRedemptionCatalog()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemRedemptionCatalog() {}

    /*!
        @brief      引換アイテムのリストを取得します。

        @return     引換アイテムのリスト。
    */
    const qVector<ServiceItemListRedemptionItem>& GetListItems() const { return listItems; }

    /*!
        @brief      ServiceItemRedemptionCatalog オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        {
            StringStream ss;
            ss << title << NEX_T("ServiceItemRedemptionCatalog");
            QTRACE(uiTraceFlags, ss.CStr());
        }

        qVector<ServiceItemListRedemptionItem>::const_iterator it = GetListItems().begin();
        for (qInt i = 0; it != GetListItems().end(); ++it, ++i)
        {
            QTRACE(uiTraceFlags, title + NEX_T("    ---- item[") << i << NEX_T("] ----"));
            it->Trace(uiTraceFlags, title + NEX_T("    "));
        }
    }
};

END_NAMESPACE_NEX

#endif
