﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include "ServiceItemRightConsumptionInfoDDL.h"
#include "ServiceItemAccountRightConsumption.h"

BEGIN_NAMESPACE_NEX

/*!
    @extends    ServiceItemRightInfo
    @brief      一つのリファレンス ID に対する消費型サービス利用券の権利情報を表すクラスです。
*/
class ServiceItemRightConsumptionInfo: public DDLCLASS(ServiceItemRightConsumptionInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemRightConsumptionInfo()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemRightConsumptionInfo() {}

    /*!
        @brief      プリンシパル ID ごとの権利情報のリストを取得します。

        @return     プリンシパル ID ごとの権利情報のリスト。
    */
    const qVector<ServiceItemAccountRightConsumption>& GetAccountRights() const { return accountRights; }

    /*!
        @brief      ServiceItemRightConsumptionInfo オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemRightConsumptionInfo(");
        ss  << NEX_T("ReferenceID:") << GetReferenceId();
        ss  << NEX_T(" ReferenceIDType:");
        switch (GetReferenceIdType())
        {
        case ServiceItemConstants::REFERENCE_ID_TYPE_PURCHASED:
            ss << NEX_T("PURCHASED");
            break;
        case ServiceItemConstants::REFERENCE_ID_TYPE_FREE:
            ss << NEX_T("FREE");
            break;
        default:
            ss << NEX_T("MIXED");
            break;
        }
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());

        qVector<ServiceItemAccountRightConsumption>::const_iterator it = GetAccountRights().begin();
        for (qInt i = 0; it != GetAccountRights().end(); ++it, ++i)
        {
            it->Trace(uiTraceFlags, title + NEX_T("    "));
        }
    }
};


END_NAMESPACE_NEX

