﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemRightInfoDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      一つのリファレンス ID に対する権利情報を表すクラスです。
*/
class ServiceItemRightInfo: public DDLCLASS(ServiceItemRightInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemRightInfo()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemRightInfo() {}

    /*!
        @brief      リファレンス ID を取得します。

        @return     リファレンス ID。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      リファレンス ID 種別を取得します。

        @return     リファレンス ID 種別。
    */
    ServiceItemConstants::ReferenceIdType GetReferenceIdType() const { return static_cast<ServiceItemConstants::ReferenceIdType>(referenceIdType); }

    /*!
        @brief      ServiceItemRightInfo オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemRightInfo& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemRightInfo& rhs)
{
    oStr << NEX_T("ServiceItemRightInfo: (")
                << NEX_T("referenceId=") << rhs.referenceId
                << NEX_T(", referenceIdType=") << rhs.referenceIdType
                << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

