﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include "ServiceItemRightInfosDDL.h"
#include "ServiceItemRightConsumptionInfo.h"
#include "ServiceItemRightTimeInfo.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      複数の権利情報を表すクラスです。
*/
class ServiceItemRightInfos: public DDLCLASS(ServiceItemRightInfos)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemRightInfos()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemRightInfos() {}

    /*!
        @brief      サポート ID を取得します。

        @return     サポート ID 。
    */
    const String& GetSupportId() const { return supportId; }

    /*!
        @internal
        @brief      初回購入のみ可能なサービスアイテムを購入済みかどうかを取得します。

        @retval true 購入済み。
        @retval false 未購入。
    */
    qBool GetAlreadyPurchasedInitialOnlyItem() const { return alreadyPurchasedInitialOnlyItem; }

    /*!
        @brief      消費型サービス利用券の権利情報のリストを取得します。

        @return     消費型サービス利用券の権利情報のリスト。
    */
    const qVector<ServiceItemRightConsumptionInfo>& GetConsumptionRightInfos() const { return consumptionRightInfos; }

    /*!
        @brief      期間券の権利情報のリストを取得します。

        @return     期間券の権利情報のリスト。
    */
    const qVector<ServiceItemRightTimeInfo>& GetAdditionalTimeRightInfos() const { return additionalTimeRightInfos; }

    /*!
        @brief      永久利用券の権利情報のリストを取得します。

        @return     永久利用券の権利情報のリスト。
    */
    const qVector<ServiceItemRightTimeInfo>& GetPermanentRightInfos() const { return permanentRightInfos; }

    /*!
        @brief      ServiceItemRightInfos オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemRightInfos(");
        ss << NEX_T("SupportID:") << GetSupportId();
        ss << NEX_T(" AlreadyPurchasedInitialOnlyItem:") << GetAlreadyPurchasedInitialOnlyItem();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
        {
            qVector<ServiceItemRightConsumptionInfo>::const_iterator it = GetConsumptionRightInfos().begin();
            for (qInt i = 0; it != GetConsumptionRightInfos().end(); ++it, ++i)
            {
                QTRACE(uiTraceFlags, title + NEX_T("    ---- consumption[") << i << NEX_T("] ----"));
                it->Trace(uiTraceFlags, title + NEX_T("    "));
            }
        }
        {
            qVector<ServiceItemRightTimeInfo>::const_iterator it = GetAdditionalTimeRightInfos().begin();
            for (qInt i = 0; it != GetAdditionalTimeRightInfos().end(); ++it, ++i)
            {
                QTRACE(uiTraceFlags, title + NEX_T("    ---- additionalTime[") << i << NEX_T("] ----"));
                it->Trace(uiTraceFlags, title + NEX_T("    "));
            }
        }
        {
            qVector<ServiceItemRightTimeInfo>::const_iterator it = GetPermanentRightInfos().begin();
            for (qInt i = 0; it != GetPermanentRightInfos().end(); ++it, ++i)
            {
                QTRACE(uiTraceFlags, title + NEX_T("    ---- permanent[") << i << NEX_T("] ----"));
                it->Trace(uiTraceFlags, title + NEX_T("    "));
            }
        }
    }
};

END_NAMESPACE_NEX

