﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemUsedInfoDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      消費後の消費型サービス利用券の権利情報を表すクラスです。
*/
class ServiceItemUsedInfo: public DDLCLASS(ServiceItemUsedInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemUsedInfo()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemUsedInfo() {}

    /*!
        @brief      累計の獲得個数を取得します。

        取得する値は、有料で購入した権利の個数と無料で獲得した権利の個数の和です。

        @return     累計の獲得個数。
    */
    qUnsignedInt32 GetAcquiredCount() const { return acquiredCount; }

    /*!
        @brief      累計の消費個数を取得します。

        取得する値には、無料で獲得した権利の消費個数も含まれます。

        @return     累計の消費個数。
    */
    qUnsignedInt32 GetUsedCount() const { return usedCount; }

    /*!
        @brief      ServiceItemUsedInfo オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemUsedInfo(");
        ss << NEX_T("Acquired Count:") << GetAcquiredCount();
        ss << NEX_T(" Used Count:") << GetUsedCount();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

