﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Subscriber/src/Common/SubscriberConstants.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief Subscriber に関する関数が定義された名前空間です。
*/
namespace SubscriberUtil
{
    /*!
        @brief      予約済みトピック番号からトピック文字列へ変換します。

        予約済みトピック番号から "r_<予約済みトピック番号>" というフォーマットの文字列に変換します。

        @param[in]  reservedTopicNum 予約済みトピック番号。

        @return     トピック文字列。
    */
    String ReservedTopicNumToTopicString(qUnsignedInt32 reservedTopicNum);

    /*!
        @brief      トピック文字列から予約済みトピック番号へ変換します。

        "r_<予約済みトピック番号>" というフォーマットの文字列から予約済みトピック番号に変換します。

        @param[in]  topic トピック文字列。

        @return     予約済みトピック番号。
    */
    qUnsignedInt32 TopicStringToReservedTopicNum(const String& topic);

}

END_NAMESPACE_NEX

