﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "SubscriberGetFollowerParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      フォロワーの取得に関するパラメータを表すクラスです。
*/
class SubscriberGetFollowerParam: public DDLCLASS(SubscriberGetFollowerParam)
{
public:
    /*!
        @brief      コンストラクタです。最低限 SubscriberGetFollowerParam::SetTopic() で取得するトピックを指定する必要があります。
    */
    SubscriberGetFollowerParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~SubscriberGetFollowerParam() {}

    /*!
    @brief      フォロワーを取得する対象のトピックを設定します。

    初期状態は空文字列 (無効なトピック名) です。

    @param[in] topic_ フォロワーを取得する対象のトピック。
    */
    void SetTopic(const String& topic_) { this->topic = topic_; }

    /*!
    @brief      フォロワーを取得する対象のトピックを取得します。

    @return フォロワーを取得する対象のトピック。
    */
    const String& GetTopic() const { return topic; }

    /*!
    @brief      取得するフォロワーの最大数を設定します。

    初期値は 20 です。一度に取得できるフォロワーの最大数は SubscriberConstants::MAX_GET_FOLLOWER_SIZE 件です。

    @param[in] size_ 取得するフォロワーの最大数。
    */
    void SetSize(qUnsignedInt32 size_) { this->size = size_; }

    /*!
    @brief      取得するフォロワーの最大数を取得します。

    @return 取得するフォロワーの最大数。
    */
    qUnsignedInt32 GetSize() const { return size; }

    /*!
    @brief      何件目から取得するかを表すオフセットを設定します。

    何件目から取得するかを表すオフセットを設定します。初期値は 0 です。

    @param[in] offset_ 何件目から取得するかを表すオフセット。
    */
    void SetOffset(qUnsignedInt32 offset_) { this->offset = offset_; }

    /*!
    @brief      何件目から取得するかを表すオフセットを取得します。

    @return 何件目から取得するかを表すオフセット。
    */
    qUnsignedInt32 GetOffset() const { return offset; }


    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetTopic(NEX_T(""));
        SetSize(20);
        SetOffset(0);
    }

    /*!
        @brief      SubscriberGetFollowerParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const SubscriberGetFollowerParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const SubscriberGetFollowerParam& rhs)
{
    oStr << NEX_T("SubscriberGetContentParam: (")
               << NEX_T("topic=") << rhs.topic
               << NEX_T(", size=") << rhs.size
               << NEX_T(", offset=") << rhs.offset;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

