﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringURLsDDL.h>

BEGIN_NAMESPACE_NEX

/*!
 @brief ギャザリングのホストのURLを格納するクラスです。
 */
#ifdef FOR_DOC_PURPOSE
class GatheringURLs: public RootObject
#else
class GatheringURLs: public DDLCLASS(GatheringURLs)
#endif
{
public:

    /*!
    @brief      コンストラクタです。
    */
    GatheringURLs()
    {
        m_gid = 0;
        m_lstStationURLs.clear();
    }

    /*!
    @brief      デストラクタです。
    */
    virtual ~GatheringURLs() {}

    /*!
     @brief    ギャザリングのIDを取得します
     */
    GatheringID GetGatheringID() const
    {
        return m_gid;
    }

    /*!
     @brief    ギャザリングのホストのURLを取得します
     */
    const qList<StationURL>& GetStationURLs() const
    {
        return m_lstStationURLs;
    }

    /*!
     @brief    ギャザリングのIDを設定します
     @param[in]  gid  ギャザリングID
     */
    void SetGatheringID(GatheringID gid)
    {
        m_gid = gid;
    }

    /*!
     @brief    ギャザリングのホストのURLを設定します
     @param[in]   stationURLList   ギャザリングのホストのURL
     */
    void SetStationURLs(const qList<StationURL>& stationURLList)
    {
        m_lstStationURLs = stationURLList;
    }
};

END_NAMESPACE_NEX

