﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

BEGIN_NAMESPACE_NEX

/**
    \ingroup group_rdv_services_gathering_client
    \brief   This class contains the enumeration of flags that may be assigned to a Principal's statistics for a given Gathering
*/
class StatisticFlags
{
public:
    /**
        \ingroup group_rdv_services_gathering_client
        \brief  Enumerates the various flags that may be assigned to a Principal's statistics for a given Gathering.

        These flags describe the manner in which a given Principal completes a gathering (typically a game session)
        that has statistics associated to it. They enable you to determine whether or not the reported game stats are
        valid and also to keep statistics on how many times a given player disconnects, abandons, or does not show up
        for game sessions. Each set of statistics reported via MatchMakingClient::ReportStats has a StatisticFlags
        associated to it.
    */
    enum Flags
    {
        /**
            The game session has started and Rendez-Vous is currently waiting for the Principal to complete the game session.
            This flag would be set for statistics reported while a game session is in progress, e.g. for when statistics
            are reported at several stages during a game session.
        */
        Pending = 0,
        /**
            The Principal Finished the game session in the regulatory manner and therefore, the reported stats are valid.
        */
        Finished = 1,
        /**
            The Principal Abandoned the game session but may still be connected to the lobby. Depending on your Competition logic
            implementation, any reported stats may or may not contain valid data.
        */
        Abandoned = 2,
        /**
            The Principal Disconnected from the game session and lobby. Depending on your Competition logic implementation,
            valid stats may have been reported prior to disconnection. Once statistics arbitration is complete, Principal's who's
            statistics are flagged Disconnected will have their disconnects field in the Arbitration database table incremented.
        */
        Disconnected = 3,
        /**
            The Principal Defaulted the game session, i.e. he did not show up for the scheduled match. It is up to your
            Competition logic to decide what to do for these Principal s.
        */
        Defaulted = 4
    };
};

END_NAMESPACE_NEX

