﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/DataHolder.h>


BEGIN_NAMESPACE_NEX

//group=Messaging Service

/*
Description: Abstract base class definition for incoming instant messages event handler.
*/

/*!
 @brief    インスタントメッセージ受信ハンドラの基底クラスです。

 インスタントメッセージ受信イベントをハンドリングするためには、@ref MessagingNotificationHandler::DeliverInstantMessage() を実装した
 MessagingNotificationHandler のサブクラスを MessagingClient に登録する必要があります。
 ハンドラーが登録されると、クライアントが通知メッセージを受信するたびに、
 メッセージがハンドラーへ送られ、DeliverInstantMessage コールバックが呼び出されます。

 @see MessagingClient
 */
class MessagingNotificationHandler: public RootObject
{
public:

    //group=Methods

    /*
    Description: Default constructor.
    */

    /*!
     @brief    コンストラクタです。
     */
    MessagingNotificationHandler();

    /*!
     @brief    デストラクタです。
     */
    virtual ~MessagingNotificationHandler();

    /*
    Description: Callback method to override. Called whenever a
    new message is received through instant messaging.
    Parameters:
    hMsg: Holder for message data, namely an object deriving from
    UserMessage.
    Remarks: Creates a class deriving from MessagingNotificationHandler,
    and overriding this method. Since all messages you receive derive from
    UserMessage, and they are delivered with MessagingClient::SendInstant or
    MessagingClient::SendChat, UserMessage::GetID() returns 0 for each of them.
    Example:
    UserMessage* pUserMsg = static_cast<UserMessage*>( hMsg.Get() );
    See also: MessagingClient, UserMessage
    */

    /*!
     @brief    インスタントメッセージ受信時に呼び出されるコールバック関数です。

     本関数をオーバーライドしたサブクラスを @ref MessagingClient に登録することで、
     クライアントが通知メッセージを受信するたびにコールバックが呼び出されます。

     @param[in]    hMsg    メッセージを格納するデータホルダー
     */
    virtual void DeliverInstantMessage(const DataHolder& hMsg) = 0;
};

END_NAMESPACE_NEX

