﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Services/NATTraversal/src/Client/NATTraversalRelayClient.h>
#include <RendezVous/Services/NATTraversal/src/Client/RVNATRelay.h>

BEGIN_NAMESPACE_NEX

class StreamManager;

//group=NAT Traversal Service

/*
Description: This service enables players to use NAT traversal in order to establish direct connections
with players located behind NAT devices.
Remarks: Once NAT traversal is invoked, the NAT Traversal Service performs all the necessary operations
to enable players to establish direct connections with players located behind NAT devices.

After successfully logging in to the lobby, NAT traversal is typically invoked as shown in the example below.

            // Create the NATTraversalClient
            NATTraversalClient  *pNATTraversalClient = qNew NATTraversalClient();

            // Bind a given player to the NATTraversalClient object
            if (!pNATTraversalClient->Bind(pCredentials)) {
                ConsoleIO::Print(NEX_T("Cannot bind to NATTraversal Service\n"));
            }

See Also: RootTransport, NATProperties, ConnectivityManager
*/

/*!
 @brief    NAT トラバーサル機能を利用するためのクラスです。

 クライアント間の P2P 通信を行うためには、事前に NATTraversalClient クラスオブジェクト
 に対して NATTraversalClient::Bind() を行う必要があります。
 */


// NATTraversalClient で行っていたことの多くを NATTraversalRelayClient に移しました。
// NATTraversalRelayClient で扱う NATRelayInterface を登録できるようにし、NEX と Pia の
// 疎結合化を図っています。そのため、NATTraversalClient で持っていた RVEcho などは
// RVNATRelay に移しています。

class NATTraversalClient: public ServiceClient
{
public:
    /*!
     @brief    コンストラクタです。
     */
    NATTraversalClient();
    /*!
     @brief    デストラクタです。
     */
    virtual ~NATTraversalClient();

    virtual qBool IsConnected() const;

protected:
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);
private:
    virtual void ConnectionStateHasChanged();

private:
    NATTraversalRelayClient m_oRelayClient;
    RVNATRelay              m_oRVNatRelay;
};

END_NAMESPACE_NEX

